/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>

#ifdef WIN32
#include <direct.h>
#endif

#ifdef UNIX
#include <dirent.h>
#include <unistd.h>
#endif

#include "CLcommon.h"
#include "CLTree.h"

static char* GetNextString(char** pBody, int nBodySize, int* nCharsRead, short* bError);

#define FILE_EXTENSION	".sos"
#define SERVER_DIRECTORY_NAME "servers"
#define SOS_FILE_NAME "database"

#define HEADER "$$$Version 3.0$$$"

void DeleteTreeNode(CLTreeNode* pNode)
{
	if (!pNode)
		return;

	if (pNode->m_strDeployDir)
	{
		free(pNode->m_strDeployDir);
		pNode->m_strDeployDir = NULL;
	}

	if (pNode->m_strWorkingDir)
	{
		free(pNode->m_strWorkingDir);
		pNode->m_strWorkingDir = NULL;
	}

	if (pNode->m_pFiles)
	{
		CLTreeLeafList* tmp = pNode->m_pFiles;

		while (tmp)
		{
			CLTreeLeafList* pCur = tmp;
			DeleteTreeLeaf(tmp->m_pLeaf);
			tmp = tmp->m_pNext;
			free(pCur);
		}

		pNode->m_pFiles = NULL;
	}

	if (pNode->m_pChildren)
	{
		CLTreeNodeList* tmp = pNode->m_pChildren;

		while (tmp)
		{
			CLTreeNodeList* pCur = tmp;

			DeleteTreeNode(tmp->m_pNode);

			tmp = tmp->m_pNext;

			free(pCur);
		}

		pNode->m_pChildren = NULL;
	}
}

void DeleteTreeLeaf(CLTreeLeaf* pLeaf)
{
	if (!pLeaf)
		return;
	
	if (pLeaf->m_name)
	{
		free(pLeaf->m_name);
		pLeaf->m_name = NULL;
	}

	free(pLeaf);
}

void TreeNodeAddLeaf(CLTreeNode* pNode, CLTreeLeaf* pLeaf)
{
	if (!pNode->m_pFiles)
	{
		pNode->m_pFiles = (CLTreeLeafList*) malloc(sizeof(CLTreeLeafList));
		pNode->m_pFilesTail = pNode->m_pFiles;
	}
	else
	{
		pNode->m_pFilesTail->m_pNext = (CLTreeLeafList*) malloc(sizeof(CLTreeLeafList));
		pNode->m_pFilesTail = pNode->m_pFilesTail->m_pNext;
	}

	pNode->m_pFilesTail->m_pNext = NULL;
	pNode->m_pFilesTail->m_pLeaf = pLeaf;
}

CLTreeLeaf* TreeLeafWithName(char* name)
{
	CLTreeLeaf* pLeaf = (CLTreeLeaf*) malloc(sizeof(CLTreeLeaf));

	memset(pLeaf, 0, sizeof(CLTreeLeaf));

	pLeaf->m_nLocalVersionNumber = -1;

	pLeaf->m_name = (char*) malloc(strlen(name) + 1);
	strcpy(pLeaf->m_name, name);

	return pLeaf;
}

CLTreeNode* TreeNodeWithName(char* name)
{
	CLTreeNode* pNode = (CLTreeNode*) malloc(sizeof(CLTreeNode));

	memset(pNode, 0, sizeof(CLTreeNode));

	pNode->m_name = (char*) malloc(strlen(name) + 1);
	strcpy(pNode->m_name, name);

	return pNode;
}

CLTreeLeaf* TreeNodeGetLeafByName(CLTreeNode* pNode, char* file)
{
	CLTreeLeaf *pLeaf;
	CLTreeLeafList* tmp = pNode->m_pFiles;

	while (tmp)
	{
		if (!strcmp(tmp->m_pLeaf->m_name, file))
			return tmp->m_pLeaf;

		tmp = tmp->m_pNext;
	}

	pLeaf = TreeLeafWithName(file);
	TreeNodeAddLeaf(pNode, pLeaf);

	return pLeaf;
}

CLTreeLeaf* GetTreeLeafFromProject(CLTreeNode* pTree, char* Project, char* File)
{
	CLTreeNode* pNode;
	
	pNode = GetTreeNodeFromProject(pTree, Project);

	if (!pNode)
		return NULL;

	return TreeNodeGetLeafByName(pNode, File);
}


CLTreeNode* TreeNodeGetTreeChildByName(CLTreeNode* pNode, char* name)
{
	CLTreeNodeList* tmp = pNode->m_pChildren;

	while (tmp)
	{
		if (!strcmp(tmp->m_pNode->m_name, name))
			return tmp->m_pNode;
		
		tmp = tmp->m_pNext;
	}

	return NULL;
}

void TreeNodeSetWorkingDirectory(CLTreeNode* pNode, char* pszWorkingDir)
{
	pNode->m_strWorkingDir = (char*) malloc(strlen(pszWorkingDir) + 1);
	strcpy(pNode->m_strWorkingDir, pszWorkingDir);
}

short TreeNodeDeleteMissingLeaves(CLTreeNode* pNode)
{
	return FALSE;
}

CLTreeNode* TreeNodeAddChild(CLTreeNode* pNode, char* name)
{
	if (!pNode->m_pChildren)
	{
		pNode->m_pChildren = (CLTreeNodeList*) malloc(sizeof(CLTreeNodeList));
		pNode->m_pChildrenTail = pNode->m_pChildren;
	}
	else
	{
		pNode->m_pChildrenTail->m_pNext = (CLTreeNodeList*) malloc(sizeof(CLTreeNodeList));
		pNode->m_pChildrenTail = pNode->m_pChildrenTail->m_pNext;
	}

	pNode->m_pChildrenTail->m_pNext = NULL;
	pNode->m_pChildrenTail->m_pNode = TreeNodeWithName(name);

	pNode->m_pChildrenTail->m_pNode->m_pParent = pNode;

	return pNode->m_pChildrenTail->m_pNode;
}

void TreeNodeDeleteLeaf(CLTreeNode* pNode, CLTreeLeaf* pLeaf)
{
	CLTreeLeafList* tmp = pNode->m_pFiles;

	CLTreeLeafList* pCur = NULL;

	while (tmp)
	{
		if (tmp->m_pLeaf == pLeaf)
		{
			/* first child being trashed ... */
			if (!pCur)
			{
				pNode->m_pFiles = pNode->m_pFiles->m_pNext;

				DeleteTreeLeaf(tmp->m_pLeaf);
				free(tmp);
				return;
			}
			else if (tmp == pNode->m_pFilesTail)
			{
				/* deleting the last child */

				pCur->m_pNext = NULL;
				pNode->m_pFilesTail = pCur;

				DeleteTreeLeaf(tmp->m_pLeaf);
				free(tmp);
				return;
			}
			else
			{
				/* somewhere in the middle */
				pCur->m_pNext = tmp->m_pNext;
				DeleteTreeLeaf(tmp->m_pLeaf);
				free(tmp);
				return;
			}
		}

		pCur = tmp;
		tmp = tmp->m_pNext;
	}
}


void TreeNodeDeleteChild(CLTreeNode* pNode, CLTreeNode* pChild)
{
	CLTreeNodeList* tmp = pNode->m_pChildren;

	CLTreeNodeList* pCur = NULL;

	while (tmp)
	{
		if (tmp->m_pNode == pChild)
		{
			/* first child being trashed ... */
			if (!pCur)
			{
				pNode->m_pChildren = pNode->m_pChildren->m_pNext;

				DeleteTreeNode(tmp->m_pNode);
				free(tmp);
				return;
			}
			else if (tmp == pNode->m_pChildrenTail)
			{
				/* deleting the last child */

				pCur->m_pNext = NULL;
				pNode->m_pChildrenTail = pCur;

				DeleteTreeNode(tmp->m_pNode);
				free(tmp);
				return;
			}
			else
			{
				/* somewhere in the middle */
				pCur->m_pNext = tmp->m_pNext;
				DeleteTreeNode(tmp->m_pNode);
				free(tmp);
				return;
			}
		}

		pCur = tmp;
		tmp = tmp->m_pNext;
	}
}

static char* GetNextString(char** pBody, int nBodySize, int* nCharsRead, short* bError)
{
	int i = 0;
	char* ret = NULL;

	while (((*pBody)[i] != '\r') && ((*nCharsRead + i + 1) < nBodySize))
		i++;

	if ((*pBody)[i] != '\r')
	{
		*bError = 1;
		return NULL;
	}

	(*pBody)[i] = '\0';

	ret = *pBody;
	
	*pBody = *pBody + (i + 2);

	*nCharsRead += (i + 2);

	return ret;
}

CLTreeLeaf* ParseFileInfo(char** pMessageBody, int nBodySize, int* nCharsRead)
{
	char* pBody = NULL;
	short bError = FALSE;
	CLTreeLeaf* pLeaf;
	char* name;
	char* verNum;
	char* strRemoteDate;
	char* strUser;
/*	char* strProject; */

	if ((!pMessageBody) || (!*pMessageBody) || (*nCharsRead >= nBodySize))
		return NULL;

	pBody = *pMessageBody;

	/* create a new tree leaf */

	pLeaf = (CLTreeLeaf*) malloc(sizeof(CLTreeLeaf));

	if (!pLeaf)
		return NULL;

	memset(pLeaf, 0, sizeof(CLTreeLeaf));

	pLeaf->m_nLocalVersionNumber = -1;

	/* parse out the filename */

	name = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		DeleteTreeLeaf(pLeaf);
		return NULL;
	}

	pLeaf->m_name = (char*) malloc(strlen(name) + 1);

	if (!pLeaf->m_name)
	{
		DeleteTreeLeaf(pLeaf);
		return NULL;
	}
	sprintf(pLeaf->m_name, "%s", name);

	/* parse out the version number */
	
	verNum = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		free(pLeaf);
		return NULL;
	}

	pLeaf->m_nRemoteVersionNumber = atoi(verNum);

	/* parse out the remote date */

	strRemoteDate = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		free(pLeaf);
		return NULL;
	}

	pLeaf->m_pRemoteVersionDate = ParseServerDate(strRemoteDate);

	/* parse out the user holding a lock, if any */

	strUser = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		free(pLeaf);
		return NULL;
	}

	pLeaf->m_lockedby = strUser;

	/* parse out the project where locked, if any */

/*	strProject = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		free(pLeaf);
		return NULL;
	}
*/

	return pLeaf;
}

time_t ParseServerDate(char* pDate)
{
	int nMonth, nDay, nYear, nHour, nMinute;
	char* pTmp;
    struct tm tm;

	if (!pDate)
		return 0;

	pTmp = pDate;

	/* parse the month */

	pTmp[2] = '\0';

	nMonth = atoi(pTmp);

	pTmp += 3;

	/* parse the day */

	pTmp[2] = '\0';

	nDay = atoi(pTmp);

	pTmp += 3;

	/* parse the year */

	pTmp[4] = '\0';

	nYear = atoi(pTmp);

	pTmp += 5;

	/* parse the hour */

	pTmp[2] = '\0';

	nHour = atoi(pTmp);

	pTmp += 3;

	/* parse the minute */

	nMinute = atoi(pTmp);

	/* create the new CTime object */

    memset(&tm, 0, sizeof(struct tm));
    tm.tm_min = nMinute;
    tm.tm_hour = nHour;

    if (nYear > 1900)
		nYear -= 1900;

    tm.tm_year = nYear;
    tm.tm_mday = nDay;
    tm.tm_mon = nMonth;

    return mktime(&tm);
}

short ParseFileList(CLTreeNode *pNode, char* pMessageBody, int nBodySize)
{
	int nCharsRead = 0;
	char* pParseBody;
	char* pBody = (char*)malloc(sizeof(char) * nBodySize);
	short bMadeChanges;
	
	if (!pBody)
		return FALSE;

	memcpy(pBody, pMessageBody, nBodySize);

	pParseBody = pBody;

	bMadeChanges = FALSE;

	while (nCharsRead < nBodySize)
	{
		CLTreeLeaf* pNewLeaf = ParseFileInfo(&pParseBody, nBodySize, &nCharsRead);
		CLTreeLeaf* pLeaf;
		
		if (!pNewLeaf)
			break;

		/* does the leaf already exist in the list? */

		pLeaf = TreeNodeGetLeafByName(pNode, pNewLeaf->m_name);

		if (pLeaf)
		{
			/* just update the leaf's info */

			if (pLeaf->m_nRemoteVersionNumber != pNewLeaf->m_nRemoteVersionNumber)
			{
				pLeaf->m_nRemoteVersionNumber = pNewLeaf->m_nRemoteVersionNumber;

				bMadeChanges = TRUE;
			}

			if (pLeaf->m_pRemoteVersionDate)
			{
				if (pLeaf->m_pRemoteVersionDate != pNewLeaf->m_pRemoteVersionDate)
				{
					pLeaf->m_pRemoteVersionDate = pNewLeaf->m_pRemoteVersionDate;

					bMadeChanges = TRUE;
				}
			}
			else
			{
				pLeaf->m_pRemoteVersionDate = pNewLeaf->m_pRemoteVersionDate;

				bMadeChanges = TRUE;
			}

			if (pNewLeaf->m_lockedby)
			{
				pLeaf->m_lockedby = (char *)malloc(strlen(pNewLeaf->m_lockedby));
				if (pLeaf->m_lockedby)
					memcpy(pLeaf->m_lockedby, pNewLeaf->m_lockedby, strlen(pNewLeaf->m_lockedby) + 1);
				bMadeChanges = TRUE;
			}
			
			pLeaf->m_bChecked = TRUE;

			DeleteTreeLeaf(pNewLeaf);
		}
		else
		{
			pNewLeaf->m_bChecked = TRUE;

			TreeNodeAddLeaf(pNode, pNewLeaf);

			bMadeChanges = TRUE;
		}
	}

	free(pBody);

	/* check to see if any of our leaves have been deleted from SS and
	** remove them
	*/

	if (TreeNodeDeleteMissingLeaves(pNode))
		bMadeChanges = TRUE;

	return bMadeChanges;
}

CLTreeNode* ParseTree(CLTreeNode* pNode, char* buf, int* pos, int maxPos)
{
	short bParseError;

	char* nodeName = ParseNameFromBuffer(buf, pos, maxPos, &bParseError);

	short bEntireTree = FALSE;

	/* this is the root node, not yet created */
	if (!pNode)
	{
		pNode = (CLTreeNode*) malloc(sizeof(CLTreeNode));
		memset(pNode, 0, sizeof(CLTreeNode));
		bEntireTree = TRUE;
	}

	while (nodeName != NULL)
	{
		CLTreeNode* pChildNode = TreeNodeAddChild(pNode, nodeName);

		ParseTree(pChildNode, buf, pos, maxPos);

		if (bEntireTree == TRUE)
			break;

		nodeName = ParseNameFromBuffer(buf, pos, maxPos, &bParseError);
	}

	return pNode;
}

char* ParseNameFromBuffer(char* buf, int *startPos, int maxPos, short* bParseError)
{
	int nInitPos = *startPos;
	int nSize = 0;
	
	*bParseError = FALSE;

	while ((buf[*startPos] != '\r') || (buf[(*startPos) + 1] != '\n'))
	{
		nSize++;
		(*startPos)++;

		if (*startPos > maxPos)
		{
			*bParseError = TRUE;
			return NULL;
		}
	}

	buf[(*startPos)++] = '\0';

	(*startPos)++;	/* jump past the \n */

	if (nSize == 0)
		return NULL;
	else
		return (buf + nInitPos);
}

void PruneTree(CLTreeNode* pNode)
{
	/* if this node was not checked off, then it should be removed */

	if (!pNode->m_bChecked)
	{
		TreeNodeDeleteChild(pNode->m_pParent, pNode);
	}
	else
	{
		pNode->m_bChecked = FALSE;		/* reset */

		/* iterate over all of the subprojects in this project */

		if (pNode->m_pChildren)
		{
			CLTreeNodeList* tmp = pNode->m_pChildren;

			while (tmp)
			{
				PruneTree(tmp->m_pNode);

				tmp = tmp->m_pNext;
			}
		}
	}
}

short ReadTreeNode(CLTreeNode* pNode, char* pBuffer, int* nFilePos, int maxPos, short bNewNode, short bConvertTimes)
{
	short bParseError;
	char* pszFileName;

	/* read all child nodes for this node */

	char* pszNodeName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);

	while (pszNodeName != NULL)
	{
		short bWebProject = FALSE;
		short bCloakedProject = FALSE;
		CLTreeNode* pChildNode = NULL;
		char* pszWorkingDir;

		/* is the node a web project? */

		if (!strncmp(pszNodeName, "$$$", 3))
		{
			bWebProject = TRUE;

			pszNodeName += 3;
		}

		if (!strncmp(pszNodeName, "$$", 2) )
		{
			bCloakedProject = TRUE;
			
			pszNodeName += 2;
		}


		if (!bNewNode)
			pChildNode = TreeNodeGetTreeChildByName(pNode, pszNodeName);

		if (!pChildNode)
		{
			pChildNode = TreeNodeAddChild(pNode, pszNodeName);
		}

		if (bWebProject)
			pChildNode->m_bWebProject = TRUE;

		if (bCloakedProject)
			pChildNode->m_bCloaked = TRUE;

		/* read this project's working directory */

		pszWorkingDir = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);

		if (pszWorkingDir)
		{
			pChildNode->m_strWorkingDir = (char*) malloc(strlen(pszWorkingDir) + 1);
			strcpy(pChildNode->m_strWorkingDir, pszWorkingDir);
		}
		else 
		{
			if (bParseError)
				return FALSE;
			else
				pChildNode->m_strWorkingDir = NULL;
		}

		/* recurse this child */

		if (!ReadTreeNode(pChildNode, pBuffer, nFilePos, maxPos, bNewNode, bConvertTimes))
			return FALSE;

		/* get the next child */

		pszNodeName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);
	}

	if (bParseError)
		return FALSE;

	pszFileName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);

	while (pszFileName != NULL)
	{
		CLTreeLeaf* pReadLeaf = (CLTreeLeaf*) malloc(sizeof(CLTreeLeaf));
		char* pszVerNum;
		char* pszLocalDate;
		char* pszUnused;
		CLTreeLeaf* pLeaf;

		memset(pReadLeaf, 0, sizeof(CLTreeLeaf));
		pReadLeaf->m_nLocalVersionNumber = -1;

		pReadLeaf->m_name = (char*) malloc(strlen(pszFileName) + 1);
		strcpy(pReadLeaf->m_name, pszFileName);

		/* parse out the version number */

		pszVerNum = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);

		if (pszVerNum)
			pReadLeaf->m_nLocalVersionNumber = atoi(pszVerNum);
		else if (bParseError)
			return FALSE;

		/* parse out the fetch date */

		pszLocalDate = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);

		if (pszLocalDate)
			pReadLeaf->m_pLocalVersionDate = atol(pszLocalDate);
		else if (bParseError)
			return FALSE;

		/* 2.0 stored times in local time - convert it to UTC */
		if (bConvertTimes && pszLocalDate)
		{
			time_t timet = (time_t) atol(pszLocalDate);

			pReadLeaf->m_pLocalVersionDate = mktime(gmtime(&timet));
		}


		/* parse out the unused field */

		pszUnused = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);

		/* does the leaf already exist in the node? */

		pLeaf = TreeNodeGetLeafByName(pNode, pReadLeaf->m_name);

		if (pLeaf)
		{
			/* just update the leaf's info */

			pLeaf->m_nLocalVersionNumber = pReadLeaf->m_nLocalVersionNumber;

			if (pLeaf->m_pLocalVersionDate)
				pLeaf->m_pLocalVersionDate = 0L;

			if (pReadLeaf->m_pLocalVersionDate)
				pLeaf->m_pLocalVersionDate = pReadLeaf->m_pLocalVersionDate;

			pLeaf->m_bChecked = TRUE;

			free(pReadLeaf);
		}
		else
		{
			pReadLeaf->m_bChecked = TRUE;

			TreeNodeAddLeaf(pNode, pReadLeaf);
		}

		/* read the next leaf */

		pszFileName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, &bParseError);
	}

	if (bParseError)
		return FALSE;

	/* check to see if any of our leaves have been deleted and remove them */

	TreeNodeDeleteMissingLeaves(pNode);

	return TRUE;
}

void WriteTreeNode(CLTreeNode* pNode, FILE* fp)
{
	/* add the name of this node */

	char* strName;
	int len = 0;
	CLTreeLeafList* pLeafs;

	if (pNode->m_bWebProject)
		len += 3;

	if (pNode->m_bCloaked)
		len += 2;

	len =+ strlen(pNode->m_name) + 3;

	strName = (char*) malloc(len);
	strName[0] = 0;

	if (pNode->m_bWebProject)
		strcat(strName, "$$$");

	if (pNode->m_bCloaked)
		strcat(strName, "$$");
	
	strcat(strName, pNode->m_name);
	strcat(strName, "\r\n");

	fprintf(fp, strName);

	/* add its working directory */

	if (pNode->m_strWorkingDir)
		fprintf(fp, "%s\r\n", pNode->m_strWorkingDir);
	else
		fprintf(fp, "\r\n");

	/* iterate over all of the subprojects in this project */

	if (pNode->m_pChildren)
	{
		CLTreeNodeList* tmp = pNode->m_pChildren;

		while (tmp)
		{
			WriteTreeNode(tmp->m_pNode, fp);

			tmp = tmp->m_pNext;
		}
	}

	/* no more subprojects */

	fprintf(fp, "\r\n");

	/* iterate over all of the files in this project */

	pLeafs = pNode->m_pFiles;

	while(pLeafs)
	{
		/* get the next leaf */

		CLTreeLeaf* pLeaf = pLeafs->m_pLeaf;

		/* add the name of this leaf */

		fprintf(fp, "%s\r\n", pLeaf->m_name);

		/* add its local version number */

		if (pLeaf->m_nLocalVersionNumber != -1)
		{
			fprintf(fp, "%i", pLeaf->m_nLocalVersionNumber);
		}

		fprintf(fp, "\r\n");

		/* add its fetch date */

		if (pLeaf->m_pLocalVersionDate)
		{
			fprintf(fp, "%ld", pLeaf->m_pLocalVersionDate);
		}

		fprintf(fp, "\r\n");

		/* the next field is currently unused - just write 0 */

		fprintf(fp, "0\r\n");

		pLeafs = pLeafs->m_pNext;
	}

	/* no more files */

	fprintf(fp, "\r\n");
}

void SaveTreeToFile(CLTreeNode* pNode, char* strCurrentProject, FILE* fp)
{
	if (!fp)
	{
		return;
	}

	/* rewind the file */
	fseek(fp, 0, SEEK_SET);

	/* dump out the version 3.0 header */
	fprintf(fp, "%s\r\n", HEADER);

	/* next write out the current project setting */
	if (!strCurrentProject)
		strCurrentProject = "$/";

	fprintf(fp, "%s\r\n", strCurrentProject);

	WriteTreeNode(pNode, fp);

	fclose(fp);
}

void CreateDatabaseEntryInIndex(char* strIndexFile, char* strDatabase, char** strSOSFile, int num)
{
	FILE* fp = fopen(strIndexFile, "ab");
	int len;
	char buf[10];

	fprintf(fp, "%s\r\n", strDatabase);
	fprintf(fp, "%s%d%s\r\n", SOS_FILE_NAME, num + 1, FILE_EXTENSION);

	fclose(fp);

	len = strlen(SOS_FILE_NAME) + 1 + strlen(FILE_EXTENSION) + 1;
	*strSOSFile = (char*) malloc(len);

	strcpy(*strSOSFile, SOS_FILE_NAME);

	sprintf(buf, "%d", num + 1);

	strcat(*strSOSFile, buf);
	strcat(*strSOSFile, FILE_EXTENSION);
}

/* Get the correct SOS filename for this server/database combo */

void GetSOSFileName(char* strDir, char* strServerNameRaw, char* strDatabase, char** strSOSFile)
{
	char* p = NULL;
	char* strServer;
	char* strServerName;
	int nDotIndex = 0;
	char buf[2048];
	char strIndexFile[2048];
	FILE* fp;
	int nFileSize;
	int nBufPos = 0;
	short bParseError;
	char* pFile = NULL;
	char* pBuffer;
	char* pDatabase;
	STAT st;

#ifdef WIN32
	strServerName = ConvertServerName(strServerNameRaw);

	p = strchr(strServerName, '.');
#else
	strServerName = strServerNameRaw;
#endif

	if (p)
		nDotIndex = p - strServerName;

	if (nDotIndex > 0)
	{
		strServer = (char*) malloc(nDotIndex + 1);
		strncpy(strServer, strServerName, nDotIndex);
	}
	else
	{
		strServer = (char*) malloc(strlen(strServerName) + 1);
		strcpy(strServer, strServerName);
	}

	sprintf(buf, "%s%c%s", strDir, SLASH, SERVER_DIRECTORY_NAME);

	memset(&st, 0, sizeof(STAT));

	/* create the "home" dir if it doesn't exist */
	if (stat(strDir, &st))
	{
#ifdef WIN32
		if (mkdir(strDir))
#else
		if (mkdir(strDir, 0755))
#endif /* WIN32 */
		{
			*strSOSFile = NULL;
			return;
		}
	}

	if (stat(buf, &st))
	{
#ifdef WIN32
		if (mkdir(buf))
#else
		if (mkdir(buf, 0755))
#endif /* WIN32 */

		{
			*strSOSFile = NULL;
			return;
		}
	}

	strcat(buf, "/");
	strcat(buf, strServer);

	free(strServer);

	if (stat(buf, &st))
	{
#ifdef WIN32
		if (mkdir(buf))
#else
		if (mkdir(buf, 0755))
#endif /* WIN32 */
		{
			*strSOSFile = NULL;
			return;
		}
	}

	strcat(buf, "/");

	/* attempt to open the index file */

	
	sprintf(strIndexFile, "%sindex.sos", buf);

	fp = fopen(strIndexFile, "rb");

	if (!fp)
	{
		char* strFullFile;

		CreateDatabaseEntryInIndex(strIndexFile, strDatabase, strSOSFile, 0);

		strFullFile = (char*)malloc(strlen(buf) + strlen(*strSOSFile) + 1);

		strcpy(strFullFile, buf);
		strcat(strFullFile, *strSOSFile);

		free(*strSOSFile);
		*strSOSFile = strFullFile;

		return;
	}

	/* read the index file and search for a match */

	fseek(fp, 0, SEEK_END);

	nFileSize = (int) ftell(fp);

	fseek(fp, 0, SEEK_SET);

	pBuffer = (char*) malloc(nFileSize);

	fread(pBuffer, sizeof(char), nFileSize, fp);

	fclose(fp);


	pDatabase = ParseNameFromBuffer(pBuffer, &nBufPos, nFileSize - 1, &bParseError);

	while (pDatabase)
	{
		pFile = ParseNameFromBuffer(pBuffer, &nBufPos, nFileSize - 1, &bParseError);

		if (!strcasecmp(strDatabase, pDatabase))
		{
			*strSOSFile = (char*) malloc(strlen(buf) + strlen(pFile) + 1);
			strcpy(*strSOSFile, buf);
			strcat(*strSOSFile, pFile);
			free(pBuffer);
			return;
		}

		pDatabase = ParseNameFromBuffer(pBuffer, &nBufPos, nFileSize - 1, &bParseError);
	}

	/* No entry found - create a new Entry */

	if (pFile)
	{
		int num;

		while (*pFile && (*pFile < '0' || *pFile > '9'))
			++pFile;

		num = atoi(pFile);

		/* grab it's number and pass that along to the creation func */

		CreateDatabaseEntryInIndex(strIndexFile, strDatabase, strSOSFile, num);
	}
	else /* no entries in the file */
	{
		CreateDatabaseEntryInIndex(strIndexFile, strDatabase, strSOSFile, 0);
	}


	*strSOSFile = (char*)malloc(strlen(buf) + strlen(*strSOSFile) + 1);

	strcpy(*strSOSFile, buf);
	strcat(*strSOSFile, *strSOSFile);

	free(pBuffer);
}

CLTreeNode* LoadTreeFromFile(char* strSOSHome, char* strServer, char* strDatabase, FILE** pSOSFile, short* bLoadTreeFromServer)
{
	char* strFileName;
	STAT st;
	FILE* fp = NULL;
	int nFileSize;
	char* pBuffer;
	int nFilePos;
	short bParseError;
	char* pszCurProject;
	short bConvertTimes = TRUE;
	char* pszRootNode;
	short bWebProject = FALSE;
	short bCloaked = FALSE;
	char* pszWorkingDir;
	CLTreeNode* pRootNode;

	*bLoadTreeFromServer = FALSE;

	*pSOSFile = NULL;

	/* get the name of the SOS file for this server/database */
	
	GetSOSFileName(strSOSHome, strServer, strDatabase, &strFileName);

	if (!strFileName)
		return NULL;

	/* does the file exist? */

	memset(&st, 0, sizeof(STAT));

	if (stat(strFileName, &st))
	{
		/* Version 1.0 and 1.1 dumped .sos files in the exepath dir - look there
		** and if we find one, use it
		*/

		if (!ConvertOldSOSFile(strSOSHome, strServer, strFileName))
		{
			/* create a new SOS file though it will be empty until exit */
			fp = fopen(strFileName, "wb");

			if (!fp)
			{
				return NULL;
			}
			else
			{
				*pSOSFile = fp;

				*bLoadTreeFromServer = TRUE;
			}

			return NULL;
		}
	}

	fp = fopen(strFileName, "r+b");

	if (!fp)
	{
		return NULL;
	}
	else
	{
		*pSOSFile = fp;
	}

	/* read the entire file into a buffer */

	fseek(fp, 0, SEEK_END);

	nFileSize = ftell(fp);

	fseek(fp, 0, SEEK_SET);

	if (nFileSize < 1)
	{
		if (*pSOSFile == NULL)
		{
			fclose(fp);
		}

		*bLoadTreeFromServer = TRUE;

		return NULL;
	}

	pBuffer = (char *)malloc(nFileSize);

	fread(pBuffer, sizeof(char), nFileSize, fp);

	if (*pSOSFile == NULL)
	{
		fclose(fp);
	}

	nFilePos = 0;

	/* next parse out the current project setting */

	pszCurProject = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, &bParseError);

	if (!pszCurProject)
	{
		free(pBuffer);
		return NULL;
	}


	/* This is a 3.0 file - don't convert times */
	if (!strcmp(pszCurProject, HEADER))
	{
		bConvertTimes = FALSE;

		pszCurProject = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, &bParseError);

		if (!pszCurProject)
		{
			free(pBuffer);
			return NULL;
		}
	}

	/* read the project tree starting with the root */

	pszRootNode = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, &bParseError);

	if (!pszRootNode)
	{
		free(pBuffer);
		return NULL;
	}

	/* is the root node a web project? */

	if (!strncmp(pszRootNode, "$$$", 3))
	{
		bWebProject = TRUE;
		
		pszRootNode += 3;
	}

	if (!strncmp(pszRootNode, "$$", 2))
	{
		bCloaked = TRUE;
		pszRootNode += 2;
	}

	pRootNode = TreeNodeWithName(pszRootNode);

	pszWorkingDir = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, &bParseError);

	if (pszWorkingDir)
		TreeNodeSetWorkingDirectory(pRootNode, pszWorkingDir);

	if (bWebProject)
		pRootNode->m_bWebProject = TRUE;

	else if (bParseError)
	{
		free(pBuffer);
		return NULL;
	}

	if (!ReadTreeNode(pRootNode, pBuffer, &nFilePos, nFileSize - 1, TRUE, bConvertTimes))
	{
		free(pBuffer);
		return NULL;
	}

	free(pBuffer);

#ifdef DEBUG
	printf("Loaded SOS Tree from file: %s\n", strFileName);
#endif

	return pRootNode;
}

short ConvertOldSOSFile(char* strDir, char* strServerName, char* strNewName)
{
	char strOldSOSFile[2048];
	STAT st;
	FILE* oldFile, *newFile;
	int nFileSize;
	char* pBuffer;
	int nFilePos = 0;
	short bParseError;
	char* pszDatabase;

	sprintf(strOldSOSFile, "%s/%s%s", strDir, strServerName, FILE_EXTENSION);

	memset(&st, 0, sizeof(STAT));

	if (stat(strOldSOSFile, &st))
		return FALSE;	/* an older version doesn't exist */

	/* we've located an old SOS file, so correct its format and move it to
	** its new location
	*/

	oldFile = fopen(strOldSOSFile, "rb");

	if (!oldFile)
		return FALSE;

	fseek(oldFile, SEEK_END, 0);

	nFileSize = ftell(oldFile);

	fseek(oldFile, SEEK_SET, 0);

	pBuffer = (char *) malloc(nFileSize);

	fread(pBuffer, sizeof(char), nFileSize, oldFile);

	fclose(oldFile);

	/* the database name is no longer used, so parse it out */


	pszDatabase = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, &bParseError);

	/* write out the new file */

	newFile = fopen(strNewName, "wb");

	if (!newFile)
	{
		free(pBuffer);
		return FALSE;
	}

	fwrite(pBuffer + nFilePos, sizeof(char), nFileSize - nFilePos, newFile);

	fclose(newFile);

	/* delete the old file */
	unlink(strOldSOSFile);

	free(pBuffer);

	return TRUE;
}

char* GetWorkingDirectoryFromProject(CLTreeNode* pRootNode, const char* pszProject)
{
	CLTreeNode* pNode = GetTreeNodeFromProject(pRootNode, pszProject);

	if (pNode)
		return GetWorkingDirectoryFromNode(pNode);
	else
		return "";
}

char* GetWorkingDirectoryFromNode(CLTreeNode* pNode)
{
	short bFoundDir = FALSE;
	char strWorkingDir[4096];
	CLTreeNode* pParent;
	int nLen = 0;

	if (!pNode)
		return NULL;

	if (pNode->m_strWorkingDir)
		return pNode->m_strWorkingDir;

	strcpy(strWorkingDir, pNode->m_name);

	pParent = pNode->m_pParent;

	while (pParent != NULL)
	{
		char strTmpPath[2048];
		char strTemp[2048];

		if (!pParent->m_strWorkingDir)
		{
			strcpy(strTmpPath, pParent->m_name);

			nLen = strlen(strTmpPath);
			if (strTmpPath[nLen] != SLASH)
			{
				strTmpPath[nLen] = SLASH;
				strTmpPath[nLen + 1] = '\0';
			}

			sprintf(strTemp, "%s%s", strTmpPath, strWorkingDir);
			sprintf(strWorkingDir, "%s", strTemp);

			pParent = pParent->m_pParent;
		}
		else
		{
			strcpy(strTmpPath, pParent->m_strWorkingDir);

			nLen = strlen(strTmpPath);
			if (strTmpPath[nLen] != SLASH)
			{
				strTmpPath[nLen] = SLASH;
				strTmpPath[nLen + 1] = '\0';
			}

			strcat(strTmpPath, strWorkingDir);

			strcpy(strWorkingDir, strTmpPath);

			bFoundDir = TRUE;

			break;
		}
	}

	if (bFoundDir)
	{
		char* p = (char*) malloc(strlen(strWorkingDir) + 1);
		strcpy(p, strWorkingDir);
		return p;
	}
	else
		return NULL;
}

CLTreeNode* GetTreeNodeFromProject(CLTreeNode* pRootNode, const char* pszProject)
{
	/* make a copy of the project for parsing */

	char* strProjectPath = (char*) malloc(strlen(pszProject) + 1);
	CLTreeNode* pNode = NULL;
	char* p;
	char* strProject = strProjectPath;
	CLTreeNode* pTemp = NULL;

	strcpy(strProjectPath, pszProject);

	p = strchr(strProjectPath, '/');

	strProjectPath = p + 1;

	if (strncmp(strProject, pRootNode->m_name, strlen(pRootNode->m_name)))
		return NULL;

	*p = 0;

	p = strchr(strProjectPath, '/');

	if (!p)
	{
		if (strlen(strProjectPath) == 0)  /* root node */
			return pRootNode;
		else
		{
			pTemp = TreeNodeGetTreeChildByName(pRootNode, strProjectPath);
			if (!pTemp)
				pTemp = TreeNodeAddChild(pRootNode, strProjectPath);
			return pTemp;
		}
	}

	pNode = pRootNode;

	while (p)
	{
		/* parse out the next node */

		strProject = strProjectPath;

		*p = 0;

		pTemp = TreeNodeGetTreeChildByName(pNode, strProject);
		if (!pTemp)
			pTemp = TreeNodeAddChild(pNode, strProject);
		pNode = pTemp;

		if (!pNode)
			return NULL;

		strProjectPath = p + 1;

		p = strchr(strProjectPath, '/');
	}

	pTemp = TreeNodeGetTreeChildByName(pNode, strProjectPath);
	if (!pTemp)
		pTemp = TreeNodeAddChild(pNode, strProjectPath);
	return pTemp;
}

void UpdateProjectTree(char* pBody, CLTreeNode* pTreeNode, int* pos, int maxPos)
{
	short bParseError;
	char* nodeName = ParseNameFromBuffer(pBody, pos, maxPos, &bParseError);

	pTreeNode->m_bChecked = TRUE;

	while (nodeName != NULL)
	{
		CLTreeNode* pChild = TreeNodeGetTreeChildByName(pTreeNode, nodeName);

		if (!pChild)
		{
			pChild = TreeNodeAddChild(pTreeNode, nodeName);

			pChild->m_bChecked = TRUE;
		}

		UpdateProjectTree(pBody, pChild, pos, maxPos);

		nodeName = ParseNameFromBuffer(pBody, pos, maxPos, &bParseError);
	}
}

void PrintTreeNode(CLTreeNode* pNode, int nDepth)
{
	CLTreeNodeList* pChild;
	int i = 0;

	for(i = 0;i < nDepth; i++)
		printf("    ");

	printf("%s\n", pNode->m_name);
	nDepth++;

	pChild = pNode->m_pChildren;

	while (pChild)
	{
		PrintTreeNode(pChild->m_pNode, nDepth);

		pChild = pChild->m_pNext;
	}

	nDepth--;
}

