/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#ifndef CLTREE_H
#define CLTREE_H

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE  1
#endif

/*
** Stat X-platform workaround
*/

#ifdef WIN32
#define STAT struct stat
#define stat stat
#define strcasecmp _stricmp
#define IFDIR _S_IFDIR
#else
#define STAT struct stat
#define IFDIR S_IFDIR
#endif

/*
** Classes needed to represent a SOS file/tree structure
*/
typedef struct _clTreeLeaf
{
	char* m_name;

	int m_nRemoteVersionNumber;
	long m_pRemoteVersionDate;

	int m_nLocalVersionNumber;
	long m_pLocalVersionDate;

	short m_bChecked;		// This is used when we get a file list from the server
							// to determine if this file still exists
	char* m_lockedby;

} CLTreeLeaf;

typedef struct _clTreeLeafList
{
	CLTreeLeaf* m_pLeaf;
	struct _clTreeLeafList* m_pNext;
} CLTreeLeafList;

typedef struct _clTreeNode
{
	char* m_name;
	short m_bWebProject;
	short m_bCloaked;

	char* m_strDeployDir;

	char* m_strWorkingDir;

	CLTreeLeafList* m_pFiles;
	CLTreeLeafList* m_pFilesTail;

	struct _clTreeNodeList* m_pChildren;
	struct _clTreeNodeList* m_pChildrenTail;

	struct _clTreeNode* m_pParent;
	short m_bChecked;	
} CLTreeNode;

typedef struct _clTreeNodeList
{
	CLTreeNode* m_pNode;
	struct _clTreeNodeList* m_pNext;
} CLTreeNodeList;

/*
** Functions
*/
void		DeleteTreeNode(CLTreeNode* pNode);
void		DeleteTreeLeaf(CLTreeLeaf* pLeaf);
CLTreeLeaf*	ParseFileInfo(char** pMessageBody, int nBodySize, int* nCharsRead);
short		ParseFileList(CLTreeNode *pNode, char* pMessageBody, int nBodySize);
void		TreeNodeAddLeaf(CLTreeNode* pNode, CLTreeLeaf* pLeaf);
CLTreeLeaf*	TreeNodeGetLeafByName(CLTreeNode* pNode, char* file);
short		TreeNodeDeleteMissingLeaves(CLTreeNode* pNode);
char*		ParseNameFromBuffer(char* buf, int *startPos, int maxPos, short* bParseError);
CLTreeNode*	TreeNodeGetTreeChildByName(CLTreeNode* pNode, char* name);
short		ReadTreeNode(CLTreeNode* pNode, char* pBuffer, int* nFilePos, int maxPos, short bNewNode, short bConvertTimes);
void		WriteTreeNode(CLTreeNode* pNode, FILE* fp);
void		SaveTreeToFile(CLTreeNode* pNode, char* strCurrentProject, FILE* fp);
void		GetSOSFileName(char* strDir, char* strServerName, char* strDatabase, char** strSOSFile);
CLTreeNode*	TreeNodeWithName(char* name);
void		TreeNodeSetWorkingDirectory(CLTreeNode* pNode, char* pszWorkingDir);
short		ConvertOldSOSFile(char* strDir, char* strServerName, char* strNewName);
char*		GetWorkingDirectoryFromProject(CLTreeNode* pRootNode, const char* pszProject);
char*		GetWorkingDirectoryFromNode(CLTreeNode* pNode);
CLTreeNode*	GetTreeNodeFromProject(CLTreeNode* pRootNode, const char* pszProject);
CLTreeNode*	LoadTreeFromFile(char* strSOSHome, char* strServer, char* strDatabase, FILE** pSOSFile, short* bLoadTreeFromServer);
CLTreeNode*	TreeNodeAddChild(CLTreeNode* pNode, char* name);
void		UpdateProjectTree(char* pBody, CLTreeNode* pTreeNode, int* pos, int maxPos);
time_t		ParseServerDate(char* pDate);
CLTreeLeaf*	TreeLeafWithName(char* name);
CLTreeLeaf*	GetTreeLeafFromProject(CLTreeNode* pTree, char* Project, char* File);
void		TreeNodeDeleteLeaf(CLTreeNode* pNode, CLTreeLeaf* pLeaf);
void		PrintTreeNode(CLTreeNode* pNode, int nDepth);
void		TreeNodeDeleteChild(CLTreeNode* pNode, CLTreeNode* pChild);

#endif // CLTREE_H

