/*******************************************************

	FILE: Unit.c

	DESC: This file contains unit tests for the CLClient.
 
        Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. 

*******************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "CLcommon.h"
#include "CLparams.h"
#ifndef UNIX
#include <direct.h>
#endif


#define SMALL_STR "This is a test"

/*----------------------------------------------
	Run unit tests on Dynamic Arrays

	Returns: 0 on success, negative number on error
----------------------------------------------*/
void RunDATests(int *numTests, int *numOK)
{
	DArray daTest;
	char szBuf[DARRAY_GROW_INCR*4];
	char saveBuf[DARRAY_GROW_INCR*4];
	int nSizeInGrowUnits;
	int i;
	int saveLen;
	char *pSavePtr;

	printf("\nRun Dynamic Array Tests\n");

	/* Initialize Test */
	printf("Initialization Test...");
	*numTests = *numTests + 1;
	DA_Initialize(&daTest);
	if (daTest.szArray == NULL && daTest.nCapacity == 0 && daTest.nCurLen == 0)
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Assign */
	printf("Assign small string to the array...");
	*numTests = *numTests + 1;
	strcpy(szBuf, SMALL_STR);
	nSizeInGrowUnits = (int)ceil((float)strlen(SMALL_STR) / (float)DARRAY_GROW_INCR);
	DA_Assign(&daTest, szBuf, -1);
	if ((!strcmp(daTest.szArray, SMALL_STR)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == strlen(SMALL_STR)))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Assign */
	printf("Assign portion of small string to the array...");
	*numTests = *numTests + 1;
	strcpy(szBuf, SMALL_STR);
	i = strlen(SMALL_STR) - 3;
	nSizeInGrowUnits = (int)ceil((float)i / (float)DARRAY_GROW_INCR);
	DA_Assign(&daTest, szBuf, i);
	if ((!strncmp(daTest.szArray, SMALL_STR, i)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == i))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Assign */
	printf("Assign string with exact capacity of array...");
	*numTests = *numTests + 1;
	for (i=0; i < daTest.nCapacity ; i++)
	{
		szBuf[i] = '-';
	}
	DA_Assign(&daTest, szBuf, i);
	if ((!strncmp(daTest.szArray, szBuf, i)) && 
		  (daTest.nCapacity == i) && 
		  (daTest.nCurLen == i))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Assign */
	printf("Assign string that requires array to grow...");
	*numTests = *numTests + 1;
	for (i=0; i < daTest.nCapacity + 8 ; i++)
	{
		szBuf[i] = '-';
	}
	nSizeInGrowUnits = (int)ceil((float)i / (float)DARRAY_GROW_INCR);
	DA_Assign(&daTest, szBuf, i);
	if ((!strncmp(daTest.szArray, szBuf, i)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == i))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Assign */
	printf("Assign small string again & verify capacity didn't shrink...");
	*numTests = *numTests + 1;
	strcpy(szBuf, SMALL_STR);
	i = daTest.nCapacity;
	pSavePtr = daTest.szArray;
	DA_Assign(&daTest, szBuf, -1);
	if ((!strcmp(daTest.szArray, SMALL_STR)) && 
		  (daTest.nCapacity == i) && 
		  (daTest.nCurLen == strlen(SMALL_STR)) &&
		  (pSavePtr == daTest.szArray))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Assign */
	printf("Assign string that requires multiple grows...");
	*numTests = *numTests + 1;
	for (i=0; i < (DARRAY_GROW_INCR*4)-3 ; i++)
	{
		szBuf[i] = '-';
	}
	nSizeInGrowUnits = (int)ceil((float)i / (float)DARRAY_GROW_INCR);
	DA_Assign(&daTest, szBuf, i);
	if ((!strncmp(daTest.szArray, szBuf, i)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == i))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Reset Test */
	printf("Reset Test...");
	*numTests = *numTests + 1;
	DA_Reset(&daTest);
	if (daTest.szArray == NULL && daTest.nCapacity == 0 && daTest.nCurLen == 0)
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("Failed\n");
	}

	/* Reset Test */
	printf("Reset Empty DArray...");
	*numTests = *numTests + 1;
	DA_Reset(&daTest);
	if (daTest.szArray == NULL && daTest.nCapacity == 0 && daTest.nCurLen == 0)
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("Failed\n");
	}

	/* Append */
	printf("Append small string to empty array...");
	*numTests = *numTests + 1;
	strcpy(szBuf, SMALL_STR);
	nSizeInGrowUnits = (int)ceil((float)strlen(SMALL_STR) / (float)DARRAY_GROW_INCR);
	DA_Append(&daTest, szBuf, -1);
	if ((!strcmp(daTest.szArray, SMALL_STR)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == strlen(SMALL_STR)))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Append */
	printf("Append portion of small string to the array...");
	*numTests = *numTests + 1;
	strcpy(szBuf, SMALL_STR);
	strncat(szBuf, SMALL_STR, strlen(SMALL_STR) - 3);
	i = strlen(szBuf);
	nSizeInGrowUnits = (int)ceil((float)i / (float)DARRAY_GROW_INCR);
	DA_Append(&daTest, SMALL_STR, strlen(SMALL_STR) - 3);
	if ((!strcmp(daTest.szArray, szBuf)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == i))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

	/* Append */
	printf("Append string to the exact capacity of array...");
	*numTests = *numTests + 1;
	for (i=0; i < daTest.nCapacity - daTest.nCurLen ; i++)
	{
		szBuf[i] = '-';
	}
	szBuf[i] = 0;
	saveLen = daTest.nCapacity;
	strcpy(saveBuf, daTest.szArray);
	strcat(saveBuf, szBuf);
	DA_Append(&daTest, szBuf, i);
	if ((!strcmp(daTest.szArray, saveBuf)) && 
		  (daTest.nCapacity == saveLen) && 
		  (daTest.nCurLen == saveLen))
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}



			/* Append */
	printf("Append that requires multiple grows...");
	*numTests = *numTests + 1;
	DA_Reset(&daTest);
	strcpy(szBuf, "H");
	DA_Assign(&daTest, szBuf, 1);
	for (i=0; i < (DARRAY_GROW_INCR*3)-3 ; i++)
	{
		szBuf[i] = '-';
	}
	szBuf[i] = 0;
	strcpy(saveBuf, daTest.szArray);
	strcat(saveBuf, szBuf);
	nSizeInGrowUnits = (int)ceil((float)strlen(saveBuf) / (float)DARRAY_GROW_INCR);
	DA_Append(&daTest, szBuf, -1);
	if ((!strcmp(daTest.szArray, saveBuf)) && 
		  (daTest.nCapacity == nSizeInGrowUnits*DARRAY_GROW_INCR) && 
		  (daTest.nCurLen == strlen(saveBuf)))
	{
		printf("%20s\n","OK"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

}


/*----------------------------------------------
	Run single unit test on a String compare function

	Returns: 0 on success, negative number on error
----------------------------------------------*/
void DoStrCmpTest(char *testFunc, 
				  char *s1, char *s2, int expectedResult,
				  int *numTests, int *numOK)
{
	int result;
	char *szEq;
	
	if (expectedResult == 0)
	{
		szEq = "!=";
	} else {
		szEq = "==";
	}

	printf("%s \"%s\" %s \"%s\" \t\tTest...", testFunc, s1, szEq, s2);
	*numTests = *numTests + 1;
	if (!strcmp(testFunc, "BeginsWithIgnoreCase"))
	{
		result = BeginsWithIgnoreCase(s1, s2);
	} else {
		result = EndsWithIgnoreCase(s1, s2);
	}

	if (result == expectedResult)
	{
		printf("OK\n"); 
		*numOK = *numOK + 1;
	} else {
		printf("****Failed****\n");
	}

}

/*----------------------------------------------
	Run unit tests on String compare functions

	Returns: 0 on success, negative number on error
----------------------------------------------*/
void RunStrCmpTests(int *numTests, int *numOK)
{
	printf("\nRun String Compare Tests\n");

	DoStrCmpTest("BeginsWithIgnoreCase", "ABCDEF", "ABC", 1,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "aBc", 1,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "aBce", 0,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "aBced", 0,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "a", 1,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "aBced", 0,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "AbCdEFa", 0,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "AbCdEF", "", 0,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "", "abb", 0,
		numTests, numOK);
	DoStrCmpTest("BeginsWithIgnoreCase", "", "", 0,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "ABCDEF", "dEf", 1,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "ABCDEF", "ABCDEF", 1,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "ABCDEF", "aDEF", 0,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "ABCDEF", "DEFa", 0,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "ABCDEF", "CDEa", 0,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "AbCdEF", "", 0,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "", "abb", 0,
		numTests, numOK);
	DoStrCmpTest("EndsWithIgnoreCase", "", "", 0,
		numTests, numOK);
}

void DoParamTest(RequestInfo *pExpReq, RequestInfo *pActualReq, 
				 int *pNumTests, int *pNumOK)
{

	*pNumTests = *pNumTests + 1;

	printf("\tCommand Num  Expect=%d \tActual=%d\n", pExpReq->eCommand, pActualReq->eCommand);
	printf("\tCommand Txt  Expect=%s \tActual=%s\n", pExpReq->szCommand, pActualReq->szCommand);
	printf("\tServer   Expect=%s \tActual=%s\n",	pExpReq->szServer,	pActualReq->szServer);
	printf("\tPort     Expect=%d \tActual=%d\n",	pExpReq->nPort,	pActualReq->nPort);
	printf("\tDatabase Expect=%s \tActual=%s\n", pExpReq->szDatabase, pActualReq->szDatabase);
	printf("\tName     Expect=%s \tActual=%s\n",	pExpReq->szName,	pActualReq->szName);
	printf("\tPassword Expect=%s \tActual=%s\n", pExpReq->szPassword, pActualReq->szPassword);
	printf("\tProject  Expect=%s \tActual=%s\n",	pExpReq->szProject,	pActualReq->szProject);
	printf("\tFile     Expect=%s \tActual=%s\n",	pExpReq->szFile,	pActualReq->szFile);
	printf("\tRevision Expect=%s \tActual=%s\n", pExpReq->szRevision, pActualReq->szRevision);
	printf("\tLog      Expect=%s \tActual=%s\n",	pExpReq->szLog,	pActualReq->szLog);
	printf("\tWorkdir  Expect=%s \tActual=%s\n",	pExpReq->szWorkdir,	pActualReq->szWorkdir);
	printf("\tLabel    Expect=%s \tActual=%s\n",	pExpReq->szLabel,	pActualReq->szLabel);
	printf("\tVerbose  Expect=%d \tActual=%d\n",	pExpReq->bVerbose,	pActualReq->bVerbose);

	if (pExpReq->eCommand == pActualReq->eCommand &&
		pExpReq->nPort == pActualReq->nPort &&
		pExpReq->bVerbose == pActualReq->bVerbose &&
		((pExpReq->szCommand == pActualReq->szCommand) || (pExpReq->szCommand && pActualReq->szCommand && !strcmp(pExpReq->szCommand, pActualReq->szCommand))) &&
		((pExpReq->szServer == pActualReq->szServer) || (pExpReq->szServer && pActualReq->szServer && !strcmp(pExpReq->szServer, pActualReq->szServer))) &&
		((pExpReq->szDatabase == pActualReq->szDatabase) || (pExpReq->szDatabase && pActualReq->szDatabase && !strcmp(pExpReq->szDatabase, pActualReq->szDatabase))) &&
		((pExpReq->szName == pActualReq->szName) || (pExpReq->szName && pActualReq->szName && !strcmp(pExpReq->szName, pActualReq->szName))) &&
		((pExpReq->szPassword == pActualReq->szPassword) || (pExpReq->szPassword && pActualReq->szPassword && !strcmp(pExpReq->szPassword, pActualReq->szPassword))) &&
		((pExpReq->szProject == pActualReq->szProject) || (pExpReq->szProject && pActualReq->szProject && !strcmp(pExpReq->szProject, pActualReq->szProject))) &&
		((pExpReq->szFile == pActualReq->szFile) || (pExpReq->szFile && pActualReq->szFile && !strcmp(pExpReq->szFile, pActualReq->szFile))) &&
		((pExpReq->szRevision == pActualReq->szRevision) || (pExpReq->szRevision && pActualReq->szRevision && !strcmp(pExpReq->szRevision, pActualReq->szRevision))) &&
		((pExpReq->szLog == pActualReq->szLog) || (pExpReq->szLog && pActualReq->szLog && !strcmp(pExpReq->szLog, pActualReq->szLog))) &&
		((pExpReq->szWorkdir == pActualReq->szWorkdir) || (pExpReq->szWorkdir && pActualReq->szWorkdir && !strcmp(pExpReq->szWorkdir, pActualReq->szWorkdir))) &&
		((pExpReq->szLabel == pActualReq->szLabel) || (pExpReq->szLabel && pActualReq->szLabel && !strcmp(pExpReq->szLabel, pActualReq->szLabel)))
		)
	{
		printf("Test OK\n"); 
		*pNumOK = *pNumOK + 1;
	} else {
		printf("****Test Failed****\n");
	}



}


void RunParseParamTests(int *pNumTests, int *pNumOK)
{
	RequestInfo expectReq;
	RequestInfo actualReq;
	char *argv[100];


	printf("Fully specified command line\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	argv[1] = "-command";	argv[2] = "GeTFilE";
	argv[3] = "-server";	argv[4] = "weasel:8888";
	argv[5] = "-database";	argv[6] = "ABC";
	argv[7] = "-name";		argv[8] = "henry";
	argv[9] = "-password";	argv[10] = "ford";
	argv[11] = "-project";	argv[12] = "model-t";
	argv[13] = "-file";		argv[14] = "in-order";
	argv[15] = "-revision";	argv[16] = "12.1";
	argv[17] = "-log";		argv[18] = "Stardate 2444 Captain's personal log";
	argv[19] = "-workdir";	argv[20] = "c:\\my\\dir";
	argv[21] = "-label";	argv[22] = "Cambell's soup";
	argv[23] = "-verbose";
	expectReq.eCommand = E_GetFile;
	expectReq.szCommand = MSG_GET_FILE;
	expectReq.szServer = "weasel";
	expectReq.nPort = 8888;
	expectReq.szDatabase = argv[6];
	expectReq.szName = argv[8];
	expectReq.szPassword = argv[10];
	expectReq.szProject = argv[12];
	expectReq.szFile = argv[14];
	expectReq.szRevision = argv[16];
	expectReq.szLog = argv[18];
	expectReq.szWorkdir = argv[20];
	expectReq.szLabel = argv[22];
	expectReq.bVerbose = 1;
	ParseParameters(24, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);


	printf("Fully specified command line with sort cuts for parameter designations\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	argv[1] = "-c";			argv[2] = "GetFile";
	argv[3] = "-s";			argv[4] = "weasel";
	argv[5] = "-da";		argv[6] = "ABC";
	argv[7] = "-n";			argv[8] = "henry";
	argv[9] = "-pa";		argv[10] = "ford";
	argv[11] = "-pr";		argv[12] = "model-t";
	argv[13] = "-f";		argv[14] = "in-order";
	argv[15] = "-r";		argv[16] = "12.1";
	argv[17] = "-lo";		argv[18] = "Stardate 2444 Captain's personal log";
	argv[19] = "-w";		argv[20] = "c:\\my\\dir";
	argv[21] = "-la";		argv[22] = "Cambell's soup";
	argv[23] = "-v";
	expectReq.eCommand = E_GetFile;
	expectReq.szCommand = argv[2];
	expectReq.szServer = "weasel";
	expectReq.nPort = -1;
	expectReq.szDatabase = argv[6];
	expectReq.szName = argv[8];
	expectReq.szPassword = argv[10];
	expectReq.szProject = argv[12];
	expectReq.szFile = argv[14];
	expectReq.szRevision = argv[16];
	expectReq.szLog = argv[18];
	expectReq.szWorkdir = argv[20];
	expectReq.szLabel = argv[22];
	expectReq.bVerbose = 1;
	ParseParameters(24, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);

	/* error conditions */

	printf("All parameter designations, no actual parameters.\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	argv[1] = "-c";	
	argv[2] = "-s";	
	argv[3] = "-da";
	argv[4] = "-n";	
	argv[5] = "-pa";
	argv[6] = "-pr";
	argv[7] = "-f";
	argv[8] = "-r";
	argv[9] = "-lo";
	argv[10] = "-la";
	ParseParameters(11, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);

	printf("No Parameters at all\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	ParseParameters(1, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);


	printf("Invalid server:port string\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	argv[1] = "-s";			
	argv[2] = "weasel:";
	expectReq.szServer = "weasel";
	expectReq.nPort = -1;
	ParseParameters(3, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);


	printf("Invalid server:port string\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	argv[1] = "-s";			
	argv[2] = ":8";
	expectReq.szServer = "";
	expectReq.nPort = 8;
	ParseParameters(3, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);

	printf("Random junk as input\n");
	RI_Initialize(&expectReq);
	argv[0] = "CLClient";
	argv[1] = "-c";			argv[2] = "asdf";
	argv[3] = "sssss";			argv[4] = "sss";
	argv[5] = "-da4444";		argv[6] = "333ABC";
	argv[7] = "-n";			argv[8] = "-henry";
	argv[9] = "-=a";		argv[10] = "-project";

	ParseParameters(11, argv, &actualReq);
	DoParamTest(&expectReq, &actualReq, pNumTests, pNumOK);

	/** TODO - more parameter input tests.  To be really anal, 
	input each command that a user can input and verify that it
	gets processed correctly.  Should definitely do this for 
	regression testing. ****/

}


void DoVerifyParamTest(RequestInfo *pReq, int testNum, int testShouldPass, 
				 int *pNumTests, int *pNumOK)
{
	int result;

	*pNumTests = *pNumTests + 1;


	printf("Running verify param test %d for command %s ", testNum,
		ConvertMsgNum2MsgStr(pReq->eCommand));
	if (!testShouldPass)
		printf("which should fail \n");
	result = VerifyRequestParams(pReq);
	if ((testShouldPass && result == 0) ||
		(!testShouldPass && result == -1))
	{
		printf("Test OK\n"); 
		*pNumOK = *pNumOK + 1;
	} else {
		printf("****Test Failed****\n");
	}
}




void RunVerifyParamTests(int *pNumTests, int *pNumOK)
{
	RequestInfo ReqInfo;
	int i;
	int numParams;

	/* For each command, run tests with each required
		parameter missing, and one with all required
		parameters present
	*/
	
	/* Commands that require only a server parameter */
	numParams=2;
	for (i=0; i < numParams+1; i++)
	{
		RI_Initialize(&ReqInfo);
		ReqInfo.eCommand=E_GetDatabases;
		if (i!=0)
			ReqInfo.szServer = "server"; 
		if (i!=1)
			ReqInfo.nPort = 8888;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
	}

	/* Commands that require server, name, password, database parameters */
	numParams=5;
	for (i=0; i < numParams+1; i++)
	{
		RI_Initialize(&ReqInfo);
		if (i!=0)
			ReqInfo.szServer = "server"; 
		if (i!=1)
			ReqInfo.nPort = 8888;
		if (i!=2)
			ReqInfo.szName = "buckaroo"; 
		if (i!=3)
			ReqInfo.szPassword = "banzai"; 
		if (i!=4)
			ReqInfo.szDatabase = "The Database"; 

		ReqInfo.eCommand=E_GetProjectTree;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
	}

	/* Commands that require server, name, password, database and project parameters */
	numParams=6;
	for (i=0; i < numParams+1; i++)
	{
		RI_Initialize(&ReqInfo);
		if (i!=0)
			ReqInfo.szServer = "server"; 
		if (i!=1)
			ReqInfo.nPort = 8888;
		if (i!=2)
			ReqInfo.szName = "buckaroo"; 
		if (i!=3)
			ReqInfo.szPassword = "banzai"; 
		if (i!=4)
			ReqInfo.szDatabase = "The Database"; 
		if (i!=5)
			ReqInfo.szProject = "Manhatten"; 

		ReqInfo.eCommand=E_GetFileList;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);

		ReqInfo.eCommand=E_CreateProject;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);

		ReqInfo.eCommand=E_GetProject;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
	}

	/* Commands that require server, name, password, database, project and file parameters */
	numParams=7;
	for (i=0; i < numParams+1; i++)
	{
		RI_Initialize(&ReqInfo);
		if (i!=0)
			ReqInfo.szServer = "server"; 
		if (i!=1)
			ReqInfo.nPort = 8888;
		if (i!=2)
			ReqInfo.szName = "buckaroo"; 
		if (i!=3)
			ReqInfo.szPassword = "banzai"; 
		if (i!=4)
			ReqInfo.szDatabase = "The Database"; 
		if (i!=5)
			ReqInfo.szProject = "Manhatten"; 
		if (i!=6)
			ReqInfo.szFile = "whoo.txt"; 

		ReqInfo.eCommand=E_AddFile;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
		ReqInfo.eCommand=E_GetFile;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
		ReqInfo.eCommand=E_CheckOutFile;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
		ReqInfo.eCommand=E_CheckInFile;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
		ReqInfo.eCommand=E_UndoCheckOut;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);
		ReqInfo.eCommand=E_GetFileHistory;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);

	}

	/* Commands that require server, name, password, database, project, file and label parameters */
	numParams=8;
	for (i=0; i < numParams+1; i++)
	{
		RI_Initialize(&ReqInfo);
		if (i!=0)
			ReqInfo.szServer = "server"; 
		if (i!=1)
			ReqInfo.nPort = 8888;
		if (i!=2)
			ReqInfo.szName = "buckaroo"; 
		if (i!=3)
			ReqInfo.szPassword = "banzai"; 
		if (i!=4)
			ReqInfo.szDatabase = "The Database"; 
		if (i!=5)
			ReqInfo.szProject = "Manhatten"; 
		if (i!=6)
			ReqInfo.szFile = "whoo.txt"; 
		if (i!=7)
			ReqInfo.szLabel = "Now is the time for all good countrymen to come to the aid of their country"; 

		ReqInfo.eCommand=E_AddLabel;
		DoVerifyParamTest(&ReqInfo, i, i == numParams, pNumTests, pNumOK);

	}




}

void PrintAndUpdateResult(int expectedResult, int *pNumTests, int *pNumOK)
{
	*pNumTests = *pNumTests + 1;
	if (expectedResult)
	{
		printf("OK\n"); 
		*pNumOK = *pNumOK + 1;
	} else {
		printf("****Failed****\n");
	}
}


void RunHeaderTests(int *pNumTests, int *pNumOK)
{
	DArray daHeaders;
	char *szValue;
	char *szRetVal;
	int nConLen;

	printf("\nRun Header Tests\n");

	DA_Initialize(&daHeaders);

	/* add a header and verify it is retrieved properly */
	szValue = "myproject";
	printf("Adding Header \"%s\"\twith value \"%s\" \n", H_CUR_PROJECT, szValue);
	AddHeader(&daHeaders, H_CUR_PROJECT, szValue);
	printf("Getting Header \"%s\"\tvalue=\"%s\" ... ", H_CUR_PROJECT, szValue);
	szRetVal = GetHeaderValue(&daHeaders, H_CUR_PROJECT);
	PrintAndUpdateResult( !strcmp(szRetVal, szValue), pNumTests ,pNumOK);

	/* add a bunch more headers */
	szValue = "the database";
	printf("Adding Header \"%s\"\twith value \"%s\" \n", H_DATABASE, szValue);
	AddHeader(&daHeaders, H_DATABASE, szValue);

	szValue = "c:\\thepath\\of\\the\\file";
	printf("Adding Header \"%s\"\twith value \"%s\" \n", H_FILE_NAME, szValue);
	AddHeader(&daHeaders, H_FILE_NAME, szValue);

	szValue = "45";
	printf("Adding Header \"%s\"\twith value \"%s\" \n", H_CONTENT_LEN, szValue);
	AddHeader(&daHeaders, H_CONTENT_LEN, szValue);

	/* Now get the values and verify */
	szRetVal = GetHeaderValue(&daHeaders, H_CUR_PROJECT);
	printf("Getting Header \"%s\"\tvalue=\"%s\" ... ", H_CUR_PROJECT, szRetVal);
	PrintAndUpdateResult( !strcmp(szRetVal, "myproject"), pNumTests ,pNumOK);
	szRetVal = GetHeaderValue(&daHeaders, H_DATABASE);
	printf("Getting Header \"%s\"\tvalue=\"%s\" ... ", H_DATABASE, szRetVal);
	PrintAndUpdateResult( !strcmp(szRetVal, "the database"), pNumTests ,pNumOK);
	szRetVal = GetHeaderValue(&daHeaders, H_FILE_NAME);
	printf("Getting Header \"%s\"\tvalue=\"%s\" ... ", H_FILE_NAME, szRetVal);
	PrintAndUpdateResult( !strcmp(szRetVal, "c:\\thepath\\of\\the\\file"), pNumTests ,pNumOK);
	szRetVal = GetHeaderValue(&daHeaders, H_CONTENT_LEN);
	printf("Getting Header \"%s\"\tvalue=\"%s\" ... ", H_CONTENT_LEN, szRetVal);
	PrintAndUpdateResult( !strcmp(szRetVal, "45"), pNumTests ,pNumOK);
	nConLen = GetContentLength(&daHeaders);
	printf("Call GetContentLength, returns %d... ", nConLen);
	PrintAndUpdateResult( (nConLen == 45), pNumTests ,pNumOK);

}




/*----------------------------------------------
	Run single Extract File Name Test

----------------------------------------------*/
void DoFileNameTest(char *filePath, char *expResult,
				  int *numTests, int *numOK)
{
	char *szFileName;
	
	if (filePath)
		printf("Extract file from \"%s\"...", filePath);
	else
		printf("Extract file from null string...");

	szFileName = ExtractFileName(filePath);

	if (szFileName)
	{
		printf("returned \"%s\"\t", szFileName);
		PrintAndUpdateResult(!strcmp(szFileName, expResult), numTests, numOK);
	} else {
		printf("returned null\t");
		PrintAndUpdateResult(szFileName == NULL, numTests, numOK);
	}


}

/*----------------------------------------------
	Run unit tests on String compare functions
----------------------------------------------*/
void RunExtractFileNameTests(int *numTests, int *numOK)
{
	printf("\nRun Extract FileName Tests\n");

	DoFileNameTest("c:\\myproj\\theFile.txt", "theFile.txt", numTests, numOK);
	DoFileNameTest("c:/myproj/theFile.txt", "theFile.txt", numTests, numOK);
	DoFileNameTest("c:/myproj/a", "a", numTests, numOK);
	DoFileNameTest(NULL, NULL, numTests, numOK);
	DoFileNameTest("c:/mydir/", "", numTests, numOK);
	DoFileNameTest("", "", numTests, numOK);
	DoFileNameTest("/ab.def", "ab.def", numTests, numOK);
	DoFileNameTest("ab.def", "ab.def", numTests, numOK);

}

/*----------------------------------------------
	Run single Change Dir Test

----------------------------------------------*/
void DoChangeDirTest(char *dirPath, char *expResult,
				  int *numTests, int *numOK)
{
	char buf[1024];

	printf("changing to dir: %s ", dirPath);
	ChangeDir(dirPath);

	getcwd(buf, 1024);

	printf("... current dir now %s\t", buf);
	PrintAndUpdateResult(!strcmp(buf, expResult), numTests, numOK);

}


/*----------------------------------------------
	Run unit tests on ChangeDir function
----------------------------------------------*/
void RunChangeDirTests(int *numTests, int *numOK)
{

#ifdef UNIX
	char dir1[512], dir2[512];
	char home[512];

	printf("\nRun ChangeDir Tests\n");
	strcpy(home, getenv("HOME"));
	if (strlen(home) < 1)
	{
		*numTests++;
		printf("*****Can't find $HOME env variable!!!\n");
	}
	sprintf(dir1, "%s/%s", home, "temp/sosjunk/dir1zxy/dir2");
	sprintf(dir2, "%s/%s", home, "temp/sosjunk/dir1zxy/dir2");
	DoChangeDirTest(dir1, dir2, numTests, numOK);

	strcpy(dir1, "dir3/dir4");
	strcat(dir2, "/dir3/dir4");
	DoChangeDirTest(dir1, dir2, numTests, numOK);

	sprintf(dir1, "%s/%s", home, "temp/sosjunk");
	sprintf(dir2, "%s/%s", home, "temp/sosjunk");
	DoChangeDirTest(dir1, dir2, numTests, numOK);

	strcpy(dir1, "morejunk");
	sprintf(dir2, "%s/%s", home, "temp/sosjunk/morejunk");
	DoChangeDirTest(dir1, dir2, numTests, numOK);
	
	sprintf(dir2, "rm -rf %s/%s", home, "temp/sosjunk");
	system(dir2);

/*
	DoChangeDirTest("dir3/dir4", "$HOME/temp/junk/dir1xyz/dir2/dir3/dir4", numTests, numOK);
	DoChangeDirTest("$HOME/temp/junk", "$HOME/temp/junk", numTests, numOK);
	DoChangeDirTest("morejunk", "$HOME/temp/junk/morejunk", numTests, numOK);
*/

#else
	printf("\nRun ChangeDir Tests\n");
	DoChangeDirTest("c:/temp/junk/dir1xyz/dir2", "c:\\temp\\junk\\dir1xyz\\dir2", numTests, numOK);
	DoChangeDirTest("dir3/dir4", "c:\\temp\\junk\\dir1xyz\\dir2\\dir3\\dir4", numTests, numOK);
	DoChangeDirTest("c:\\temp\\junk", "c:\\temp\\junk", numTests, numOK);
	DoChangeDirTest("morejunk", "c:\\temp\\junk\\morejunk", numTests, numOK);
#endif

}




void OutputAndUpdateResults(char *szTitle, int *pnCurTests, int *pnCurOK,
							int *pnTotalTests, int *pnTotalOK)
{
	printf("\nTotal %s Tests: %d ", szTitle, *pnCurTests);
	printf("Num Passed: %d\n\n", *pnCurOK);
	*pnTotalTests = *pnTotalTests + *pnCurTests;
	*pnTotalOK = *pnTotalOK + *pnCurOK;
}

int main(int argc, char **argv)
{
	int nNumTests=0, nNumOK=0;
	int nTotalTests=0, nTotalOK=0;

	/* Run Dynamic Array Tests */
	RunDATests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("Dynamic Array", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);

	/* Run String Compare Tests */
	nNumTests = nNumOK = 0;
	RunStrCmpTests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("String Compare", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);

	/* Run Header Tests */
	nNumTests = nNumOK = 0;
	RunHeaderTests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("Header", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);

	/* Run Parse Parameter Tests */
	nNumTests = nNumOK = 0;
	RunParseParamTests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("Parse Parameters", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);

	/* Run Verify Parameter Tests */
	nNumTests = nNumOK = 0;
	RunVerifyParamTests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("Verify Parameters", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);

	/* Run Extract File Name Test */
	nNumTests = nNumOK = 0;
	RunExtractFileNameTests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("Extract FileName", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);

	/* Run ChangeDir Test */
	nNumTests = nNumOK = 0;
	RunChangeDirTests(&nNumTests, &nNumOK);
	OutputAndUpdateResults("Change Dir", &nNumTests, &nNumOK, &nTotalTests, &nTotalOK);


	/* Print grand total results */
	printf("\n\nTotal Tests: %d ", nTotalTests);
	printf("Num Passed: %d\n\n", nTotalOK);

	return 0;

}
