#ifndef CLCOMMON_H
#define CLCOMMON_H

#ifndef BUILD_STANDARD
#include "blowfish.h"
#endif /* !BUILD_STANDARD */

/*******************************************************

	FILE:	CLcommon.h

	DESC:	This file contains the declarations for 
			structures and functions common to all parts 
			of the command line client
 
        Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. 

*******************************************************/


#ifdef WIN32
#define SLASH '\\'
#else
#define SLASH '/'
#endif



/* ---- D A T A T Y P E   D E F I N I T I O N S -------*/


/* String representation of all Protocol Header names */
#define PROT_VERSION			"2.0"
#define H_PROTOCOL				"Protocol-Version"
#define H_MESSAGE_ID			"Message-ID"
#define H_DATABASE				"Database"
#define H_USERNAME				"Username"
#define H_PASSWORD				"Password"
#define H_CUR_PROJECT			"Project"
#define H_CONTENT_LEN			"Content-Length"
#define H_FILE_NAME				"File"
#define H_REVISION				"Version"
#define H_LABEL					"Label"
#define H_LOG					"Log"
#define H_RECURSIVE				"Recursive"
#define H_RESPONSE_TO			"Response-To"
#define H_NAME					"Name"
#define H_PERMANENT				"Permanent"
#define H_DATE					"Date"
#define H_LOCKED_BY				"Locked-By"
#define H_UNIX_EOL				"Unix-EOL-Marker"
#define H_KEY_LEN				"Key Length"
#define H_USE_COMPRESSION		"Use-Compression"
#define H_ORIGINAL_SIZE			"Original-Size"
#define H_SHAREPROJECT			"Share-Project"
#define H_BRANCH_AFTER_SHARE	"Branch After Share"


/* Enumerate all possible Message Numbers 
	and server return codes */
enum MsgNum
{
	/* Non Protocol Commands */
	E_NoCommand					= 0,
	E_GetDatabases				= 1,
	E_CheckOutFile				= 2,
	E_UndoCheckOut				= 3,
	E_AddFolder					= 4,
	E_CheckInProject			= 5,
	E_UndoProjectCheckOut		= 6,
	E_BranchProject				= 7,

	/* from client to server */
	E_Login						= 100,
	E_GetProjectTree			= 101,
	E_GetFileList				= 102,
	E_CreateProject				= 103,
	E_AddFile					= 104,
	E_GetFile					= 105,
	E_LockFile					= 106,
	E_UnlockFile				= 107,
	E_GetFileHistory			= 108,
	E_AddLabel					= 109,
	E_CheckInFile				= 110,
	E_GetProject				= 111,
	E_GetFileInfo				= 112,
	E_Delete					= 113,
	E_Rename					= 114,
	E_CheckOutProject			= 115,
	E_ShareFile					= 116,
	E_GetProjectHistory			= 117,
	E_PinFile					= 118,
	E_UnpinFile					= 119,
	E_Deploy					= 120,
	E_BranchFile				= 121,
	E_Search					= 122,
	E_Noop						= 123,
	E_ChangePassword			= 124,
	E_ShareProject				= 125,
	E_Exit						= 199,

	/* From server to client */
	E_Challenge					= 200,
	E_ProjectTree				= 201,
	E_FileList					= 202,
	E_File						= 203,
	E_FileHistory				= 204,
	E_FileInfo					= 205,
	E_ProjectHistory			= 206,
	E_SearchResults				= 207,
	E_SecureChallenge			= 250,

	E_Success					= 300,
	E_CheckinComplete			= 301,
	E_FileUnlocked				= 302,
	E_FileAdded					= 303,
	E_FileLocked				= 304,

	E_OperationFailed			= 400,
	E_NoSuchFileOrProject		= 401,
	E_FileOrProjectNotLocked	= 402,
	E_AuthFailed				= 403,
	E_MissingHeader				= 404,
	E_InvalidLabel				= 405,
	E_InvalidVersion			= 406,
	E_UnrecognizedMsg			= 407,
	E_InvalidProtocolVersion	= 408,
	E_FileOrProjectAlreadyExists = 409,
	E_FileOrProjectAlreadyLocked = 410,
	E_UnathorizedUser			= 411,
	E_ServerExpired				= 412,
	E_KeyNotFound				= 413,
	E_KeyExpired				= 414,
	E_InvalidKey				= 415,
	E_BadIniFile				= 416,
	E_MissingSSAuto				= 417,
	E_MissingCrypto				= 418,
	E_CheckinMergeFailed		= 419,
	E_FailedDatabaseOpen		= 420

};

/* String representations for all Message Text */
#define	MSG_NO_COMMAND			"UnRecognizedCommand"
#define	MSG_GET_DATABASES		"GetDatabases"
#define	MSG_CHECK_OUT_FILE		"CheckOutFile"
#define	MSG_UNDO_CHECKOUT		"UndoCheckOut"
#define MSG_ADDFOLDER			"AddFolder"
#define MSG_CHECKINPROJECT		"CheckInProject"
#define MSG_UNDOPROJECTCHECKOUT	"UndoProjectCheckOut"
#define MSG_BRANCHPROJECT		"BranchProject"

#define	MSG_LOGIN				"Login"
#define	MSG_GET_PROJECT_TREE	"GetProjectTree"
#define	MSG_GET_FILE_LIST		"GetFileList"
#define	MSG_CREATE_PROJECT		"CreateProject"
#define	MSG_ADD_FILE			"AddFile"
#define	MSG_GET_FILE			"GetFile"
#define	MSG_GET_FILE_HISTORY	"GetFileHistory"
#define	MSG_ADD_LABEL			"AddLabel"
#define	MSG_CHECK_IN_FILE		"CheckInFile"
#define	MSG_GET_PROJECT			"GetProject"
#define	MSG_GET_FILE_INFO		"GetFileInfo"
#define	MSG_DELETE				"Delete"
#define	MSG_RENAME				"Rename"
#define MSG_CHECKOUTPROJECT		"CheckOutProject"
#define MSG_SHAREFILE			"ShareFile"
#define MSG_BRANCHFILE			"BranchFile"
#define MSG_SHAREPROJECT		"ShareProject"
#define	MSG_EXIT				"Exit"


#define	MSG_CHALLENGE			"Challenge"
#define MSG_SECURE_CHALLENGE	"SecureChallenge"
#define	MSG_PROJECT_TREE		"ProjectTree"
#define	MSG_FILE_LIST			"FileList"
#define	MSG_FILE				"File"
#define	MSG_FILE_HISTORY		"FileHistory"
#define	MSG_SUCCESS				"Success"
#define	MSG_CHECKIN_COMPLETE	"CheckinComplete"
#define	MSG_FILE_UNLOCKED		"FileUnlocked"
#define	MSG_FILE_ADDED			"FileAdded"
#define	MSG_FILE_LOCKED			"FileLocked"
#define	MSG_FILE_INFO			"FileInfo"
#define	MSG_NO_SUCH_FILE_OR_PROJECT		"NoSuchFileOrProject"
#define	MSG_FILE_OR_PROJECT_NOT_LOCKED	"FileOrProjectNotLocked"
#define	MSG_AUTH_FAILED			"AuthorizationFailed"
#define	MSG_MISSING_HEADER		"MissingHeader"
#define	MSG_INVALID_LABEL		"InvalidLabel"
#define	MSG_INVALID_VERSION		"InvalidVersion"
#define	MSG_INVALID_PROTOCOL_VERSION	"InvalidProtocolVersion"
#define	MSG_UNRECOGNIZED_MSG	"UnrecognizedMessage"
#define	MSG_FILE_OR_PROJECT_ALREADY_EXISTS	"FileOrProjectAlreadyExists"
#define	MSG_FILE_OR_PROJECT_ALREADY_LOCKED	"FileOrProjectAlreadyLocked"
#define	MSG_UNATHORIZED_USER	"UnathorizedUser"
#define	MSG_SERVER_EXPIRED		"ServerExpired"
#define	MSG_OPERATION_FAILED	"OperationFailed"
#define MSG_KEY_NOT_FOUND		"EncryptionKeyNotFound"
#define MSG_KEY_EXPIRED			"EncryptionKeyExpired"
#define MSG_INVALID_KEY			"InvalidEncryptionKey"
#define MSG_BAD_INI_FILE		"BadSourceSafeIniFile"
#define MSG_MISSING_AUTO		"MissingAutomationComponent"
#define MSG_MISSING_CRYPTO		"MissingCryptographyComponent"
#define MSG_CHECKIN_MERGE_FAILED	"CheckinMergeFailed"
#define MSG_FAILED_DATABASE_OPEN	"FailedDatabaseOpen"



typedef struct _TimeStamps
{
	long startClock;
	long startTime;

	long challengeClock;
	long challengeTime;

	long seedClock;
	long seedTime;
} TimeStamps;


/*
	 Structure containing all the parameters required
	 to make a request to the source server
*/
struct _RequestInfo
{
	enum MsgNum	eCommand;
	char	*szCommand;
	char	*szOrigInpCmd;	/* the command originally input by user */
	char	*szServer;
	int		nPort;
	char	*szDatabase;
	char	*szName;
	char	*szPassword;
	char	*szProject;
	char	*szFile;
	char	*szRevision;
	char	*szLog;
	char	*szWorkdir;
	char	*szLabel;
	char	*szNewName;
	char	*szSOSHome;
	short	bVerbose;
	short	bRecursive;
	short	bPermanent;
	short	bUseCompression;
	short	bAlwaysDownload;
	short	bWriteable;
	char	*szShareTo;
	short	bBranchAfterShare;
#ifndef BUILD_STANDARD
	short	bUseEncrypt;
	int		nKeyLength;
	BF_KEY	*bfKey;
	TimeStamps tsTimeStamps;
#endif /* !BUILD_STANDARD */
} ;

typedef struct _RequestInfo RequestInfo;


/* ------- Dynamic Array Structure and Functions ---------*/

/*  A DArray grows dynamically when strings are assigned
	or appended to it that are larger than its current
	capacity.
	IMPORTANT:  You MUST call DA_Initialize after creating
	a DArray to reset its contents.
*/

/* grow the arrays in this increment */
#define DARRAY_GROW_INCR 511

struct _DArray
{
	char	*szArray;	/* The array */
	int		nCapacity;	/* The capacity of the array */
	int		nCurLen;	/* The current length of the array */
} ;

typedef struct _DArray DArray;

/* Initialize the contents of a dynamic array */
void DA_Initialize(DArray *theArray);

/* Reset the allocated contents of a dynamic array */
void DA_Reset(DArray *theArray);

/* Destroy and reset the allocated memory in a dynamic array */
void DA_Destroy(DArray *theArray);

/*	Assign a string of characters to a dynamic array.  
	The array will grow in size if necessary to accomodate
	the input */
int DA_Assign(DArray *theArray, char *szBuf, int nBufSize);

/*	Append a string of characters to a dynamic array.
	The	array will grow in size if necessary to accomodate
	the input */
int DA_Append(DArray *theArray, char *szBuf, int nBufSize);

/*	Get the character in the dynamic array
	at the position specified by the input index. */
char DA_GetChar(DArray *theArray, int nIdx);

/*	Set the character in the dynamic array
	at the position specified by the input index. */
int DA_SetChar(DArray *theArray, int nIdx, char cChar);


/* -- Message Info Structure and Convenience Functions ---*/

struct _MessageInfo
{
	enum MsgNum eMsgNum;		/* The message number */
	DArray	daMsgText;			/* The message number string */
	DArray	daHeaders;			/* Message headers */
	char	*szMsgBody;			/* The message body */
	int		nBodySize;
	short	bUseCompression;
} ;

typedef struct _MessageInfo MessageInfo;

void MI_Initialize(MessageInfo *pMsgInfo);
void MI_Reset(MessageInfo *pMsgInfo);


/* ---- F U N C T I O N   P R O T O T Y P E S -------*/

/*	Append a header name and value to a header dynamic array.*/
void AddHeader(DArray *daHeaders, char *szHeaderName, char *szHeaderValue) ;

/* Get an arbitrary header value in a header array.*/
char *GetHeaderValue(DArray *daHeaders, char *szHeaderName) ;

/* Get an arbitrary header value in a header array, whose return value is an int */
int GetIntHeaderValue(DArray *daHeaders, char *szHeaderName) ;


/* Get the value of the content length header in the input header array.*/
int GetContentLength(DArray *daHeaders) ;

/*	Return 1 if s1 begins with s2, 0 otherwise */
int BeginsWithIgnoreCase(char *s1, char *s2) ;

/*	Return 1 if s1 ends with s2, 0 otherwise */
int EndsWithIgnoreCase(char *s1, char *s2) ;

/* Conversion routines */
enum MsgNum ConvertMsgStr2MsgNum(char *szCmd) ;
char *ConvertMsgNum2MsgStr(enum MsgNum eMsgNum) ;

/*	Returns the file name portion of a path */
char *ExtractFileName(char szFilePath[]) ;

/* Set file permissions, returning 0 on success, errno on error */
int MakeFileReadOnly(char *szFileName);
int MakeFileReadWrite(char *szFileName);

/* Return -1 and print out error if the file cannot be written
   in the directory specified, 0 if it can */
int TestFileAccess(char *szWorkDir, char *szFileName);

/* Copy the input file to the current working directory,
	then delete the input file from its old location.
	Returns 0 on success, errno on error */
int MoveFileToCurDir(char *szFromFile, char *szToFile);

/* Change to the input directory, creating intermediate
	directories as necessary */
int ChangeDir(char *szInDirPath);

#ifdef WIN32
char *GetFirstFile(long *pHandle, char *pFileSpec);
char *GetNextFile(long *pHandle);
#else
char *GetFirstFile(void **pHandle, char *pFileSpec);
char *GetNextFile(void **pHandle);
#endif // WIN32

char* ConvertServerName(const char* pServer);

#endif /* CLCOMMON_H */

