/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#ifndef BUILD_STANDARD

#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>

#include "md5.h"
#include "blowfish.h"
#include "rand.h"

#include "CLcrypto.h"

#include "CLcommon.h"

static char HexToDec(char* str);
DArray* DecryptKey(unsigned char* digest, char* pFileData, int nDataSize, int keyLength);

randctx rcCTX;


void InitCrypto(TimeStamps *pTimeStamps)
{
	int i = 0;
	for (i = 0; i < RANDSIZ; ++i)
		rcCTX.randrsl[i] = 0;

	if (pTimeStamps)
	{
		pTimeStamps->seedClock = clock();
		pTimeStamps->seedTime = time(NULL);

		rcCTX.randrsl[0] = pTimeStamps->startClock;
		rcCTX.randrsl[1] = pTimeStamps->startTime;
		rcCTX.randrsl[2] = pTimeStamps->challengeClock;
		rcCTX.randrsl[3] = pTimeStamps->challengeTime;
		rcCTX.randrsl[4] = pTimeStamps->seedClock;
		rcCTX.randrsl[5] = pTimeStamps->seedTime;
	}

	/* initialize the library */

	randinit(&rcCTX, TRUE);

}




int GetUserKey(RequestInfo *pReqInfo)
{
	int i = 0;
	int nLen = 0;
	DArray daWeakKey;
	char szKeyFile[1024];
	FILE *fpKeyFile;
	long lFileLen = 0;
	char *pFileData;

	MD5_CTX mdContext;
	
	DArray *daDecryptedFile;
	int nKeyLength = 0;
	char *pKeyStr;
	char *pFormat;
	unsigned char *pKey;
	char *pHashStr;
	unsigned char *pHash;

	char hexChar[3];

	unsigned char *pConcat;
	int nConcatLen = 0;
	MD5_CTX mdCTX;
	char* pszServer;

	/* open the user's key file */

	pszServer = ConvertServerName(pReqInfo->szServer);

	DA_Initialize(&daWeakKey);

	nLen = strcspn(pszServer, ".");

	if (nLen < 0)
		nLen = strlen(pszServer);

	for (i = 0; i < nLen; i++)
		pszServer[i] = tolower(pszServer[i]);

	DA_Assign(&daWeakKey, pszServer, nLen);

	sprintf(szKeyFile, "%s%c%s.key", pReqInfo->szSOSHome, SLASH, daWeakKey.szArray);

	nLen = strlen(pReqInfo->szName);
	for (i = 0; i < nLen; i++)
		pReqInfo->szName[i] = tolower(pReqInfo->szName[i]);

	DA_Append(&daWeakKey, pReqInfo->szName, -1);

	
	if ((fpKeyFile = fopen(szKeyFile, "rb")) == NULL)
	{
		printf("Unable to open key file: %s\n", szKeyFile);
		return -1;
	}

	/* read in the encrypted data in one big chunk */

	fseek(fpKeyFile, 0, SEEK_END);
	lFileLen = ftell(fpKeyFile);
	rewind(fpKeyFile);
	if ((pFileData = (char *)malloc(lFileLen)) == NULL)
	{
		printf("Out of memory reading key file\n");
		return -1;
	}

	fread(pFileData, 1, lFileLen, fpKeyFile);

	fclose(fpKeyFile);

	/* reconstruct the weak key used for encrypting the key file */

	MD5Init(&mdContext);
	MD5Update(&mdContext, (unsigned char*)daWeakKey.szArray, daWeakKey.nCurLen);
	MD5Final(&mdContext);

	daDecryptedFile = DecryptKey(mdContext.digest, pFileData, lFileLen, 16);

	/* if 128 bit decrypt fails, try again with 56 */
	if (daDecryptedFile == NULL)
	{
		daDecryptedFile = DecryptKey(mdContext.digest, pFileData, lFileLen, 7);

		if (!daDecryptedFile)
		{
			printf("Unable to decrypt user key: %s\n", szKeyFile);
			return -1;
		}
	}

	pKeyStr = GetHeaderValue(daDecryptedFile, KH_KEY);

	if (pKeyStr)
	{
		if (strlen(pKeyStr) == 34)
			nKeyLength = 16;
		else if (strlen(pKeyStr) == 16)
			nKeyLength = 7;
		else
			nKeyLength = 0;
	}

	/* make sure this key is of the correct format */

	pFormat = GetHeaderValue(daDecryptedFile, KH_KEY_FORMAT);

	if (nKeyLength == 0 || (!pFormat) || (atoi(pFormat) != KEY_FORMAT))
	{
		printf("Incompatible key format\n");

		return -1;
	}

	/* allocate space for the key, now that we know the key length */
	if ((pKey = (unsigned char*)malloc(nKeyLength)) == NULL)
	{
		printf("Out of memory converting key value\n");
		return -1;
	}


	/* convert the user key from a hex string to a byte array */

	pKeyStr += 2;

	hexChar[2] = '\0';

	for (i = 0; i < nKeyLength; i++)
	{
		hexChar[0] = *pKeyStr;
		pKeyStr++;
		hexChar[1] = *pKeyStr;
		pKeyStr++;

		pKey[i] = HexToDec(hexChar);

/*		sscanf(hexChar, "%x", (int)&((*key)[i])); */
	}

	/* convert the hash from a hex string to a byte array */

	pHashStr = GetHeaderValue(daDecryptedFile, KH_HASH);

	pHashStr += 2;

	if ((pHash = (unsigned char*)malloc(nKeyLength)) == NULL)
	{
		printf("Out of memory converting hash value\n");
		return -1;
	}

	for (i = 0; i < nKeyLength; i++)
	{
		hexChar[0] = *pHashStr;
		pHashStr++;
		hexChar[1] = *pHashStr;
		pHashStr++;

		pHash[i] = HexToDec(hexChar);

/*		sscanf(hexChar, "%x", &(hash[i])); */
	}

	/* verify the hash */

	nConcatLen = nKeyLength + strlen(pReqInfo->szName);

	pConcat = (unsigned char*)malloc(nConcatLen);
	memcpy(pConcat, pKey, nKeyLength);
	memcpy(pConcat+nKeyLength, pReqInfo->szName, strlen(pReqInfo->szName));

	MD5Init(&mdCTX);
	MD5Update(&mdCTX, pConcat, nConcatLen);
	MD5Final(&mdCTX);

	for (i = 0; i < nKeyLength; i++)
	{
		if (pHash[i] != mdCTX.digest[i])
		{
			printf("Bad key file\n");
			return -1;
		}
	}

	pReqInfo->bfKey = SetKey(pKey, nKeyLength);
	pReqInfo->nKeyLength = nKeyLength;

	return 0;

}





DArray* DecryptKey(unsigned char* digest, char* pFileData, int nDataSize, int nKeyLength)
{
	int i = 0;
	BF_KEY *pKey;
	char *decryptedData;
	int nSize;

	DArray *daDecryptedFile;

	char *pHeader;
	char size[CRYPTO_BLOCK_SIZE];
	char sizeBuffer[CRYPTO_BLOCK_SIZE];

	char* pData = pFileData;

	char IV[8];

	/* the first 8 bytes from the key file is the IV */

	for (i = 0; i < 8; i++)
	{
		IV[i] = *pData;
		pData++;
	}

	/* the next 8 bytes is the encrypted length of the file */

	for (i = 0; i < CRYPTO_BLOCK_SIZE; i++)
	{
		sizeBuffer[i] = *pData;
		pData++;
	}

	pKey = SetKey(digest, nKeyLength);

	Decrypt(sizeBuffer, size, CRYPTO_BLOCK_SIZE, pKey, IV);

	i = 0;

	while ((i < CRYPTO_BLOCK_SIZE) && (size[i] != PAD_CHAR))
		i++;

	size[i] = '\0';

	nSize = atoi(size);

	/* decrypt the rest of the key file data */
	if ((decryptedData = (char *)malloc(nDataSize - 16)) == NULL)
	{
		printf("Error decrypting key file data\n");
		return (DArray*)-1;
	}

	memset(decryptedData, 0, nDataSize - 16);

	Decrypt(pData, decryptedData, nDataSize - 16, pKey, IV);

	daDecryptedFile = (DArray *)malloc(sizeof(DArray));

	DA_Initialize(daDecryptedFile);

	DA_Assign(daDecryptedFile, decryptedData, nSize);

	pHeader = GetHeaderValue(daDecryptedFile, "Username");

	if (pHeader != NULL)
	{
		return daDecryptedFile;
	} else {
		return NULL;
	}
}








void GetRandomNumber(unsigned char *number, int nLength)
{
	/* create a random number nLength bytes long */

	int i = 0;
	int randNum;
	int nByte = 0;
	int nIterations;
		
	nIterations = nLength / 4;

	for (i = 0; i < nIterations; i++)
	{
		randNum = irand(&rcCTX);

		number[nByte++] = (char)((randNum << 24) >> 24);
		number[nByte++] = (char)((randNum << 16) >> 24);
		number[nByte++] = (char)((randNum << 8) >> 24);
		number[nByte++] = (char)(randNum >> 24);
	}
}






BF_KEY* SetKey(unsigned char* key, int nKeyLength)
{
	BF_KEY *pKey;

	pKey = (BF_KEY *)malloc(sizeof(BF_KEY));

	BF_set_key(pKey, nKeyLength, (unsigned char*)key);

	return pKey;
}



void Decrypt(char* in, char* out, int size, BF_KEY* pKey, char* IV)
{
	BF_cbc_encrypt((unsigned char*)in, (unsigned char*)out, size,
				   pKey, (unsigned char*)IV, BF_DECRYPT);
}



DArray* Encrypt(char* pStrData, int nLength, BF_KEY *pKey, unsigned char* IV)
{
	int i = 0;
	int padding = 0;
	int remainder;

	char *out = NULL;
	DArray *daOut;
	char *pEnc;

	/* do we need to pad the data? */

	if (nLength < CRYPTO_BLOCK_SIZE)
	{
		padding = CRYPTO_BLOCK_SIZE - nLength;
	}
	else
	{
		remainder = nLength % CRYPTO_BLOCK_SIZE;

		if (remainder > 0)
			padding = CRYPTO_BLOCK_SIZE - remainder;
	}

	if (padding != 0)
	{
		nLength += padding;
		pEnc = (char *)malloc(nLength);
		memcpy(pEnc, pStrData, nLength - padding);
		for (i = nLength; i < nLength; i++)
			pEnc[i] += PAD_CHAR;
	} else {
		pEnc = pStrData;
	}

	/* create a buffer to hold the encrypted result */

	out = (char *)malloc(nLength);

	if (out == NULL)
	{
		printf("Error encrypting data\n");
		return (DArray*)-1;
	}

	/* encrypt the data */

	BF_cbc_encrypt((unsigned char*)pEnc, (unsigned char*)out, nLength,
				   pKey, (unsigned char*)IV, BF_ENCRYPT);


	daOut = (DArray *)malloc(sizeof(DArray));

	DA_Initialize(daOut);

	DA_Assign(daOut, out, nLength);

	return daOut;
}





static char HexToDec(char* str)
{
	int i = 0;
	int mult = 1;
	int res = 0;

	for (i = 1; i >= 0; i--)
	{
		int num = 0;

		if (str[i] >= '0' && str[i] <= '9')
			num = str[i] - '0';
		else
			num = str[i] - 'a' + 10;

		res += (num * mult);

		mult *= 16;
	}

	return (char)res;
}

#endif /* !BUILD_STANDARD */
