
/*******************************************************

	FILE: CLparams.c

	DESC: This file contains definitions for functions 
		that parse and verify input parameters.
 
        Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. 

*******************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <time.h>
#include "CLcommon.h"
#include "CLparams.h"

#ifdef WIN32
#include <direct.h>
#endif /* WIN32 */

#ifdef UNIX
#include <dirent.h>
#include <unistd.h>
#endif /* UNIX */

/* Function prototypes local to this file */

void ParseServerPort(char *szInServerPort, 
				 char **szOutServer, int *nOutPort);



/*----------------------------------------------
	Parse out all the known command line parameters 
	from the input argc, argv, and fills the input 
	RequestInfo structure with the values it finds.

	Returns: Nothing
----------------------------------------------*/
void ParseParameters(
	int argc,				 /* the number of parameters in argv */
	char *argv[],			 /* the command line parameters */
	RequestInfo* pOutReqInfo /* the struct to place results into */
	)
{

	int i=0;

	RI_Initialize(pOutReqInfo);

	pOutReqInfo->szSOSHome = (char *)getenv("SOSHOME");

	/* loop through all parameters.  If one matches
		a known parameter, add it to the request
	*/
	for (i=0; i < argc; i++)
	{
		/*	Command parameter 
			Needs to be converted to its enum equivalent
		*/
		if (BeginsWithIgnoreCase(P_COMMAND, argv[i]))
		{
			if (++i < argc && *argv[i] != CMD_PREFIX)
			{
				/* Save input command */
				pOutReqInfo->szOrigInpCmd = argv[i];

				/* Convert to enum */
				pOutReqInfo->eCommand = ConvertMsgStr2MsgNum(argv[i]);

				/* Convert back to nomimal name of command */
				pOutReqInfo->szCommand = ConvertMsgNum2MsgStr(pOutReqInfo->eCommand);
			}	
		}
		/*	Server Parameter 
			Needs to be parsed into server and port
		*/
		else if (BeginsWithIgnoreCase(P_SERVER, argv[i]))
		{
			if (++i < argc && *argv[i] != CMD_PREFIX)
			{
				ParseServerPort(argv[i], &pOutReqInfo->szServer, &pOutReqInfo->nPort);

			}	
		}
		/*	Database Parameter	*/
		else if (BeginsWithIgnoreCase(P_DATABASE, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szDatabase = argv[i];	
			}
		}													
		/*	Name Parameter	*/
		else if (BeginsWithIgnoreCase(P_NAME, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szName = argv[i];
			}
		}													
		/*	Password Parameter	*/
		else if (BeginsWithIgnoreCase(P_PASSWORD, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szPassword = argv[i];
			}
		}													
		/*	Project Parameter	*/
		else if (BeginsWithIgnoreCase(P_PROJECT, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				char *szTemp = NULL;
				pOutReqInfo->szProject = argv[i];
				if (((pOutReqInfo->szProject[strlen(pOutReqInfo->szProject) - 1] == '/') ||
					 (pOutReqInfo->szProject[strlen(pOutReqInfo->szProject) - 1] == '\\')) &&
					(strlen(pOutReqInfo->szProject) > 2))
				{
					pOutReqInfo->szProject[strlen(pOutReqInfo->szProject) - 1] = 0;
				}

				if (pOutReqInfo->szProject[0] != '$')
				{
					if ((pOutReqInfo->szProject[0] != '/') && (pOutReqInfo->szProject[0] != '\\'))
					{
						szTemp = (char *)malloc(strlen(pOutReqInfo->szProject) + 3);
						if (szTemp)
						{
							sprintf(szTemp, "$/%s", pOutReqInfo->szProject);
							pOutReqInfo->szProject = szTemp;
						}
					}
					else
					{
						szTemp = (char *)malloc(strlen(pOutReqInfo->szProject) + 2);
						if (szTemp)
						{
							sprintf(szTemp, "$%s", pOutReqInfo->szProject);
							pOutReqInfo->szProject = szTemp;
						}
					}
				}

				szTemp = strchr(pOutReqInfo->szProject, '\\');
				while (szTemp)
				{
					szTemp[0] = '/';
					szTemp = strchr(pOutReqInfo->szProject, '\\');
				}

			}
		}													
		/*	File Parameter	*/
		else if (BeginsWithIgnoreCase(P_FILE, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szFile = argv[i];
			}
		}													
		/*	Revision Parameter	*/
		else if (BeginsWithIgnoreCase(P_REVISION, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szRevision = argv[i];
			}
		}													
		/*	Log Parameter	*/
		else if (BeginsWithIgnoreCase(P_LOG, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szLog = argv[i];
			}
		}													
		/*	Workdir Parameter	*/
		else if (BeginsWithIgnoreCase(P_WORKDIR, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szWorkdir = argv[i];
				if (pOutReqInfo->szWorkdir[strlen(pOutReqInfo->szWorkdir) - 1] == SLASH)
				{
					pOutReqInfo->szWorkdir[strlen(pOutReqInfo->szWorkdir) - 1] = 0;
				}

#ifdef WIN32
				if ((pOutReqInfo->szWorkdir[1] != ':') && (pOutReqInfo->szWorkdir[0] != SLASH))
#else
				if (pOutReqInfo->szWorkdir[0] != SLASH)
#endif
				{
					char tmpbuf[1024];
					char *szWork = NULL;
					getcwd(tmpbuf, 1024);

					szWork = (char *)malloc(strlen(tmpbuf) + strlen(argv[i]) + 2);
					if (szWork)
					{
						if (tmpbuf[strlen(tmpbuf) - 1] == SLASH)
							sprintf(szWork, "%s%s", tmpbuf, argv[i]);
						else
							sprintf(szWork, "%s%c%s", tmpbuf, SLASH, argv[i]);

						pOutReqInfo->szWorkdir = szWork;
					}
				}
			}
		}													
		/*	SOSHome Parameter	*/
		else if (BeginsWithIgnoreCase(P_SOSHOME, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szSOSHome = argv[i];
				if (pOutReqInfo->szSOSHome[strlen(pOutReqInfo->szSOSHome) - 1] == SLASH)
				{
					pOutReqInfo->szSOSHome[strlen(pOutReqInfo->szSOSHome) - 1] = 0;
				}

			}
		}													
		/*	Label Parameter	*/
		else if (BeginsWithIgnoreCase(P_LABEL, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szLabel = argv[i];
			}
		}													
		/*	NewName Parameter	*/
		else if (BeginsWithIgnoreCase(P_NEWNAME, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szNewName = argv[i];
			}
		}													
		/*	Recursive Parameter	*/
		else if (BeginsWithIgnoreCase(P_RECURSIVE, argv[i]))
		{
			pOutReqInfo->bRecursive = 1;
		}													
		/*  Verbose parameter
			if VERBOSE exists, just set the verbose flag 
		*/
		else if (BeginsWithIgnoreCase(P_VERBOSE, argv[i]))
		{
			pOutReqInfo->bVerbose = 1;
		}
		/*	Permanent Parameter	*/
		else if (BeginsWithIgnoreCase(P_PERMANENT, argv[i]))
		{
			pOutReqInfo->bPermanent= 1;
		}													
		/*	NoCompress Parameter	*/
		else if (BeginsWithIgnoreCase(P_NOCOMPRESS, argv[i]))
		{
			pOutReqInfo->bUseCompression = 0;
		}													
		/*	AlwaysDownload Parameter	*/
		else if (BeginsWithIgnoreCase(P_ALWAYSDOWNLOAD, argv[i]))
		{
			pOutReqInfo->bAlwaysDownload = 1;
		}													
		/*	Writeable Parameter	*/
		else if (BeginsWithIgnoreCase(P_WRITEABLE, argv[i]))
		{
			pOutReqInfo->bWriteable = 1;
		}													
		/*	ShareTo Parameter	*/
		else if (BeginsWithIgnoreCase(P_SHARETO, argv[i]))
		{
			if ((++i < argc) && (*argv[i] != CMD_PREFIX))
			{
				pOutReqInfo->szShareTo = argv[i];
			}
		}													
		/*	Branch After Share Parameter	*/
		else if (BeginsWithIgnoreCase(P_BRANCH, argv[i]))
		{
			pOutReqInfo->bBranchAfterShare = 1;
		}													


	}	
}




/*----------------------------------------------
	Parse out the server and port portions of an
	input string of the form "server:port"

	Returns: Nothing
----------------------------------------------*/

void ParseServerPort(
	char *szInServerPort, /* server and port num of the */
						  /* form "server:port_num" */
	char **szOutServer,	  /* place the server portion of string here */
						  /* is assigned NULL if error */
	int *pnOutPort		  /* place the port portion of string here. */
						  /* is assigned -1 if error */
	)
{
	unsigned int i=0;
	unsigned int serverPortLen = strlen(szInServerPort);
	char *newArray;

	/* find the separator character that specifies the port */
	while ((i < serverPortLen) && (*(szInServerPort+i) != ':'))
	{
		i++;
	}

	/* read the port number from the string */
	*pnOutPort=-1;
	sscanf(szInServerPort+i+1, "%d", pnOutPort);

	/* Allocate a new buffer for the server, and copy */
	/* the server portion into the buffer */

	newArray = (char *)malloc(i+2);
	if (newArray != NULL)
	{
		strncpy(newArray, szInServerPort, i);
		newArray[i] = '\0';
	}
	*szOutServer = newArray;

}


/*----------------------------------------------
	Verify that all the parameters for a given request
	are present and valid.

	Returns: 0 on success, -1 on error
----------------------------------------------*/
int VerifyRequestParams(RequestInfo* pReqInfo)
{
	int nRetVal = 0;;

	/*	Check that each command has the correct parameters.
		If not, print out an error message
	*/
	switch(pReqInfo->eCommand)
	{
		/* These commands require only the server parameter */
		case E_GetDatabases:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0)
			{
				nRetVal = -1;
				printf("The %s command requires a valid -server parameter.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require a database parameter and
			all its dependencies
		*/
		case E_GetProjectTree:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, and -database parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require a project parameter and
			all its dependencies
		*/
		case E_GetFileList:
		case E_CreateProject:
		case E_GetProject:
		case E_Delete:
		case E_Rename:
		case E_CheckOutProject:
		case E_CheckInProject:
		case E_BranchProject:
		case E_UndoProjectCheckOut:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL ||
				pReqInfo->szProject == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, -database and -project parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require a file parameter and
			all its dependencies
		*/
		case E_AddFile:
		case E_GetFile:
		case E_CheckOutFile:
		case E_CheckInFile:
		case E_UndoCheckOut:
		case E_GetFileHistory:
		case E_GetFileInfo:
		case E_BranchFile:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL ||
				pReqInfo->szProject == NULL ||
				pReqInfo->szFile == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, -database -project and -file parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require the label parameter and
			all its dependencies
		*/
		case E_AddLabel:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL ||
				pReqInfo->szProject == NULL ||
				pReqInfo->szLabel == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, -database, -project and -label parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require the shareto parameter and
			all its dependencies
		*/
		case E_ShareProject:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL ||
				pReqInfo->szProject == NULL ||
				pReqInfo->szShareTo == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, -database, -project and -shareto parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require the shareto and file parameters and
			all their dependencies
		*/
		case E_ShareFile:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL ||
				pReqInfo->szProject == NULL ||
				pReqInfo->szFile == NULL ||
				pReqInfo->szShareTo == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, -database, -project, -file and -shareto parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;

		/*	These commands require the workdir and project parameters and
			all their dependencies
		*/
		case E_AddFolder:
			if (pReqInfo->szServer == NULL || 
				pReqInfo->nPort <= 0 ||
				pReqInfo->szName == NULL || 
				pReqInfo->szPassword == NULL ||
				pReqInfo->szDatabase == NULL ||
				pReqInfo->szProject == NULL ||
				pReqInfo->szWorkdir == NULL )

			{
				nRetVal = -1;
				printf("The %s command requires valid -server, -name, -password, -database, -project and -workdir parameters.\n",
					ConvertMsgNum2MsgStr(pReqInfo->eCommand));
			}
			break;


		default:
			nRetVal = -1;
			if (pReqInfo->szOrigInpCmd != NULL && strlen(pReqInfo->szOrigInpCmd) > 0)
				printf("The SourceOffSite command \"%s\" was not recognized.\n", pReqInfo->szOrigInpCmd);
			else
				printf("No SourceOffSite command was entered.  Use the -command option to specify a command.\n");
			break;
	}


	return nRetVal;

}



/*----------------------------------------------
	Initialize a RequestInfo structure to default values

	Returns: Nothing
----------------------------------------------*/
void RI_Initialize(RequestInfo *pOutReqInfo)
{
		/* initialize request values */
	pOutReqInfo->eCommand = E_NoCommand;
	pOutReqInfo->szCommand = MSG_NO_COMMAND;
	pOutReqInfo->szOrigInpCmd = "";
	pOutReqInfo->szServer = NULL;
	pOutReqInfo->nPort = 0;
	pOutReqInfo->szDatabase = NULL;
	pOutReqInfo->szName = NULL;
	pOutReqInfo->szPassword = NULL;
	pOutReqInfo->szProject = NULL;
	pOutReqInfo->szFile = NULL;
	pOutReqInfo->szRevision = NULL;
	pOutReqInfo->szLog = NULL;
	pOutReqInfo->szWorkdir = NULL;
	pOutReqInfo->szSOSHome = NULL;
	pOutReqInfo->szLabel = NULL;
	pOutReqInfo->szNewName = NULL;
	pOutReqInfo->bVerbose = 0;
	pOutReqInfo->bRecursive = 0;
	pOutReqInfo->bPermanent = 0;
	pOutReqInfo->bUseCompression = 1;
	pOutReqInfo->bAlwaysDownload = 0;
	pOutReqInfo->bWriteable = 0;
	pOutReqInfo->szShareTo = NULL;
	pOutReqInfo->bBranchAfterShare = 0;
#ifndef BUILD_STANDARD
	pOutReqInfo->bUseEncrypt = 0;
	pOutReqInfo->tsTimeStamps.startClock = clock();
	pOutReqInfo->tsTimeStamps.startTime = time(NULL);
	pOutReqInfo->tsTimeStamps.challengeClock = 0;
	pOutReqInfo->tsTimeStamps.challengeTime = 0;
	pOutReqInfo->tsTimeStamps.seedClock = 0;
	pOutReqInfo->tsTimeStamps.seedTime = 0;
#endif /* !BUILD_STANDARD */

}
