/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

#include <stdio.h>
#include <gtk/gtk.h>

#include "Vermie.h"
#include "VermTreeNode.h"
#include "AddFilesDlg.h"

class DirEntry
{
public:
	String name;
	DirEntry* pNext;

	inline DirEntry() { pNext = NULL; }
	inline ~DirEntry() { };
};

static void NodeExpanded(GtkTreeItem* pNode, CAddFilesDlg* pDlg);
char* GetItemText(GtkWidget* w);
String GetDirectory(GtkWidget* pNode);
static void ExpandDirectories(GtkTreeItem* pNode, String& dir, CAddFilesDlg* pDlg);
static void NodeSelected(GtkWidget* pRoot, GtkWidget* pNode, CAddFilesDlg* pDlg);
static void FilesToAddDClick(GtkWidget* w, GdkEventButton* ev, CAddFilesDlg* pDlg);
static void FilesDClick(GtkWidget* w, GdkEventButton* ev, CAddFilesDlg* pDlg);

static void ListChildSelected(GtkWidget* list, GtkWidget* child, CAddFilesDlg* pDlg);
static void ListChildUnSelected(GtkWidget* list, GtkWidget* child, CAddFilesDlg* pDlg);

static void AddFiles(GtkWidget* w, CAddFilesDlg* pDlg);
static void RemoveFiles(GtkWidget* w, CAddFilesDlg* pDlg);

static void FilesToAddDClick(GtkWidget* w, GdkEventButton* ev, CAddFilesDlg* pDlg)
{
	if (ev && ev->type == GDK_2BUTTON_PRESS)
	{
		RemoveFiles(w, pDlg);
	}
}

static void FilesDClick(GtkWidget* w, GdkEventButton* ev, CAddFilesDlg* pDlg)
{
	if (ev && ev->type == GDK_2BUTTON_PRESS)
	{
		AddFiles(w, pDlg);
	}
}

static void AddFiles(GtkWidget*, CAddFilesDlg* pDlg)
{
	GList* pFiles = GTK_LIST(pDlg->files)->selection;
	GList* pClearList = NULL;
	
	if (!pFiles)
	{
		return;
	}

	while (pFiles)
	{
		GtkWidget *pItem = gtk_list_item_new_with_label(pDlg->GetItemText((GtkWidget*)pFiles->data));
		gtk_widget_show(pItem);
		
		gtk_container_add(GTK_CONTAINER(pDlg->filesToAdd), pItem);

		pClearList = g_list_prepend(pClearList, pFiles->data);
		pFiles = pFiles->next;
	}

	pClearList = g_list_reverse(pClearList);
	
	gtk_list_remove_items(GTK_LIST(pDlg->files), pClearList);

	gtk_widget_set_sensitive(pDlg->ok, TRUE);
}

static void RemoveFiles(GtkWidget*, CAddFilesDlg* pDlg)
{
	GList* pFiles = GTK_LIST(pDlg->filesToAdd)->selection;
	GList* pClearList = NULL;

	if (!pFiles)
	{
		return;
	}
	
	while (pFiles)
	{
		GtkWidget* pItem = gtk_list_item_new_with_label(pDlg->GetItemText((GtkWidget*)pFiles->data));
		gtk_widget_show(pItem);
		
		gtk_container_add(GTK_CONTAINER(pDlg->files), pItem);

		pClearList = g_list_prepend(pClearList, pFiles->data);
		
		pFiles = pFiles->next;
	}

	pClearList = g_list_reverse(pClearList);
	
	gtk_list_remove_items(GTK_LIST(pDlg->filesToAdd), pClearList);

	GList* children = gtk_container_children(GTK_CONTAINER(pDlg->filesToAdd));

	if (children)
		g_list_free(children);
	else
		gtk_widget_set_sensitive(pDlg->ok, FALSE);
}

static void ListChildSelected(GtkWidget* list, GtkWidget*, CAddFilesDlg* pDlg)
{
	if (list == GTK_WIDGET(pDlg->files))
	{
		if (pDlg->add)
			gtk_widget_set_sensitive(pDlg->add, 1);
	}
	else
	{
		if (pDlg->remove)
			gtk_widget_set_sensitive(pDlg->remove, 1);
	}
}

static void ListChildUnSelected(GtkWidget* list, GtkWidget*, CAddFilesDlg* pDlg)
{
	if (list == GTK_WIDGET(pDlg->files))
	{
		if (pDlg->files && !GTK_LIST(pDlg->files)->selection)
			gtk_widget_set_sensitive(pDlg->add, 0);
	}
	else
	{
		if (pDlg->filesToAdd && !GTK_LIST(pDlg->filesToAdd)->selection)
			gtk_widget_set_sensitive(pDlg->remove, 0);
	}
}

char* CAddFilesDlg::GetItemText(GtkWidget* w)
{
	char *name;
	GtkLabel* l;

	l = GTK_LABEL(GTK_BIN(w)->child);
	gtk_label_get(l, &name);

	return name;
}

String CAddFilesDlg::GetDirectory(GtkWidget* pNode)
{
	String dir;
	int first = 1;
	
	GtkTree* tree = GTK_TREE(pNode->parent);
	GtkTreeItem* item = GTK_TREE_ITEM(pNode);

	while (item)
	{
		char* curDir = GetItemText(GTK_WIDGET(item));
		if (first)
		{
			dir = curDir;
			first = 0;
		}
		else
		{
			if (!tree->tree_owner)
				dir = curDir + dir;
			else
			{
				dir = "/" + dir;
				dir = curDir + dir;
			}
		}
		
		if (tree->tree_owner != NULL)
		{
			item = GTK_TREE_ITEM(tree->tree_owner);
			tree = GTK_TREE(GTK_WIDGET(tree)->parent);
		}
		else
			item = NULL;
	}

	return dir;
}

static int compareStrings(const void* a, const void* b)
{
	return fcompare(*((String*)a), *((String*)b));
}

static void ExpandDirectories(GtkTreeItem* pNode, String& dir, CAddFilesDlg* pDlg)
{
	if (dir.empty())
		return;
	
	// Build up the current directory

	DIR* d = opendir(dir.chars());

	// something has gone horribly worng!
	if (!d)
		return;

	struct dirent* dent;

	GtkWidget* pSubtree = GTK_TREE_ITEM_SUBTREE(pNode);
		
	DirEntry *pHead, *pCur;

	pHead = NULL;

	pCur = pHead;
	
	int numDirs = 0;
	
//	gtk_widget_set_sensitive(pDlg->scrollWin1, FALSE);

	while ((dent = readdir(d)) != NULL)
	{
		if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
			continue;

		String strFile = dir + "/";
		strFile += dent->d_name;

		struct stat sb;

		stat(strFile.chars(), &sb);
		
		if (!S_ISDIR(sb.st_mode))
			continue;

		if (!pHead)
		{
			pHead = new DirEntry;
			pCur = pHead;
		}
		else
		{
			pCur->pNext = new DirEntry;
			pCur = pCur->pNext;
		}
		
		pCur->name = dent->d_name;
		++numDirs;
	}

	closedir(d);
	
	if (numDirs > 0)
	{
		String *directories = new String[numDirs];
		
		int i;
		
		for (i = 0; i < numDirs; i++)
		{
			directories[i] = pHead->name;
			pCur = pHead;
			pHead = pHead->pNext;
			delete pCur;
		}
		
		qsort(directories, numDirs, sizeof(String), compareStrings);
		
//		if (level == 0)
//		{
//			gtk_widget_set_sensitive(pDlg->scrollWin1, TRUE);
			for (i = 0; i < numDirs; i++)
			{
				GtkWidget* pItem = gtk_tree_item_new_with_label((char*)directories[i].chars());
				GTK_WIDGET_SET_FLAGS(pItem, GTK_CAN_FOCUS);
				gtk_widget_set_events(pItem, GDK_FOCUS_CHANGE_MASK);
				gtk_signal_connect(GTK_OBJECT(pItem), "expand", GtkSignalFunc(NodeExpanded), pDlg);
				GtkWidget* pItemSub = gtk_tree_new();
				gtk_signal_connect (GTK_OBJECT(pItemSub), "select_child",
									GTK_SIGNAL_FUNC(NodeSelected), pDlg);
				gtk_widget_show(pItemSub);
				gtk_tree_append(GTK_TREE(pSubtree), pItem);
				gtk_tree_item_set_subtree(GTK_TREE_ITEM(pItem), pItemSub);
				gtk_widget_show(pItem);
				
				String newDir = dir + "/" + directories[i];
				
//				ExpandDirectories(GTK_TREE_ITEM(pItem), newDir, pDlg, 1);
			}
//		}

		delete []directories;
	}
	else
	{
		gtk_tree_item_remove_subtree(pNode);
	}

//	gtk_widget_set_sensitive(pDlg->scrollWin1, TRUE);
}

static void NodeExpanded(GtkTreeItem* pNode, CAddFilesDlg* pDlg)
{
	// node has already been expanded
	if (gtk_object_get_user_data(GTK_OBJECT(pNode)))
		return;

	gtk_object_set_user_data(GTK_OBJECT(pNode), pDlg);

	String dir = pDlg->GetDirectory(GTK_WIDGET(pNode));

	ExpandDirectories(pNode, dir, pDlg);
}	


static void NodeSelected(GtkWidget*, GtkWidget* pNode, CAddFilesDlg* pDlg)
{
	gtk_list_clear_items (GTK_LIST(pDlg->files), 0, -1);
	gtk_list_clear_items (GTK_LIST(pDlg->filesToAdd), 0, -1);

	gtk_widget_set_sensitive(pDlg->ok, TRUE);
	
	pDlg->m_pNodeSelected = pNode;
	
	String dir = pDlg->GetDirectory(pNode);

	if (dir.empty())
		return;

	DIR *d = opendir(dir.chars());

	if (!d)
		return;
	
	struct dirent *dent;
	
	while ((dent = readdir(d)))
	{
		if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
			continue;

		String strFile = dir + "/";
		strFile += dent->d_name;

		struct stat sb;

		stat(strFile.chars(), &sb);

		if (!S_ISREG(sb.st_mode))
			continue;

		String file = dent->d_name;
		
		if (!pDlg->m_pNode->GetLeafByName(file))
		{
			GtkWidget* listItem = gtk_list_item_new_with_label(dent->d_name);
			gtk_widget_show(listItem);
			gtk_container_add(GTK_CONTAINER(pDlg->files), listItem);
		}
	}
}

CAddFilesDlg::CAddFilesDlg(CVermTreeNode* pNode) : CVermDlg("Add Files To Project", MB_OK | MB_CANCEL)
{
	GtkWidget *dialog_vbox1;
	GtkWidget *table1;
	GtkWidget *dialog_action_area1;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *scrollWin2;
	GtkWidget *scrollWin3;
  
	m_pNode = pNode;
  
	dialog_vbox1 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	table1 = gtk_table_new (7, 3, FALSE);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "table1", table1);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), table1, TRUE, TRUE, 0);

	label3 = gtk_label_new ("Files To Add:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label3", label3);
	gtk_widget_show (label3);
	gtk_table_attach (GTK_TABLE (table1), label3, 2, 3, 4, 5,
					  (GtkAttachOptions)0, (GtkAttachOptions)0, 5, 5);
	gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);

	label2 = gtk_label_new ("Files:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label2", label2);
	gtk_widget_show (label2);
	gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 4, 5,
					  (GtkAttachOptions)0, (GtkAttachOptions)0, 5, 6);
	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

	label1 = gtk_label_new ("Directories:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label1", label1);
	gtk_widget_show (label1);
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
					  (GtkAttachOptions)0, (GtkAttachOptions)0, 5, 5);
	gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

	scrollWin1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin1),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);

	gtk_widget_set_usize(scrollWin1, 200, 200);
	gtk_table_attach (GTK_TABLE (table1), scrollWin1, 0, 3, 1, 3,
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);

	keepCheckedOut = gtk_check_button_new_with_label ("Check out files");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "checkbutton4", keepCheckedOut);
	gtk_widget_show(keepCheckedOut);
	
	gtk_table_attach(GTK_TABLE(table1), keepCheckedOut, 0, 1, 3, 4, 
					 (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);
	
	directories = gtk_tree_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "directories", directories);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin1), directories);
	gtk_tree_set_selection_mode(GTK_TREE(directories), GTK_SELECTION_BROWSE);
	gtk_widget_show (directories);
	gtk_widget_show(scrollWin1);

	GtkWidget* pRoot = gtk_tree_item_new_with_label("/");
	GTK_WIDGET_SET_FLAGS(pRoot, GTK_CAN_FOCUS);
	gtk_widget_set_events(pRoot, GDK_FOCUS_CHANGE_MASK);
	GtkWidget* pSubtree = gtk_tree_new();
  
	gtk_signal_connect(GTK_OBJECT(pRoot), "expand", GtkSignalFunc(NodeExpanded), this);
	gtk_signal_connect (GTK_OBJECT(directories), "select_child",
						GTK_SIGNAL_FUNC(NodeSelected), this);

	gtk_signal_connect(GTK_OBJECT(pSubtree), "select_child",
					   GtkSignalFunc(NodeSelected), this);
  
	gtk_widget_show(pSubtree);
  
	gtk_tree_append(GTK_TREE(directories), pRoot);
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(pRoot), pSubtree);
  
	gtk_widget_show(pRoot);
	gtk_tree_item_expand(GTK_TREE_ITEM(pRoot));
  
	scrollWin2 = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin2),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);

	gtk_widget_set_usize(scrollWin2, 200, 200);
	gtk_widget_show(scrollWin2);
  
	gtk_table_attach (GTK_TABLE (table1), scrollWin2, 2, 3, 5, 7,
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);

	filesToAdd = gtk_list_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "filesToAdd", filesToAdd);
	gtk_widget_show (filesToAdd);
	gtk_signal_connect(GTK_OBJECT(filesToAdd), "select_child",
					   GtkSignalFunc(ListChildSelected), this);
	gtk_signal_connect(GTK_OBJECT(filesToAdd), "unselect_child",
					   GtkSignalFunc(ListChildUnSelected), this);
	gtk_signal_connect_after(GTK_OBJECT(filesToAdd), "button_press_event", GTK_SIGNAL_FUNC(FilesToAddDClick), this);
  
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin2), filesToAdd);
  
	gtk_list_set_selection_mode (GTK_LIST (filesToAdd), GTK_SELECTION_EXTENDED);

	remove = gtk_button_new_with_label ("Remove");
	gtk_signal_connect(GTK_OBJECT(remove), "clicked", GtkSignalFunc(RemoveFiles),
					   this);

	gtk_object_set_data (GTK_OBJECT (m_pDialog), "remove", remove);
	gtk_widget_show (remove);
	gtk_table_attach (GTK_TABLE (table1), remove, 1, 2, 6, 7,
					  (GtkAttachOptions)0, (GtkAttachOptions)0, 5, 5);

	add = gtk_button_new_with_label ("Add");
	gtk_signal_connect(GTK_OBJECT(add), "clicked", GtkSignalFunc(AddFiles),
					   this);
  
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "add", add);
	gtk_widget_show (add);
	gtk_table_attach (GTK_TABLE (table1), add, 1, 2, 5, 6,
					  (GtkAttachOptions)0, (GtkAttachOptions)0, 5, 5);

	gtk_widget_set_sensitive(remove, 0);
	gtk_widget_set_sensitive(add, 0);
  
	scrollWin3 = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin3),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);

	gtk_widget_set_usize(scrollWin3, 200, 200);
	gtk_widget_show(scrollWin3);
	gtk_table_attach (GTK_TABLE (table1), scrollWin3, 0, 1, 5, 7,
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);

	files = gtk_list_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "files", files);
	gtk_widget_show (files);
	gtk_signal_connect(GTK_OBJECT(files), "select_child",
					   GtkSignalFunc(ListChildSelected), this);
	gtk_signal_connect(GTK_OBJECT(files), "unselect_child",
					   GtkSignalFunc(ListChildUnSelected), this);
	gtk_signal_connect_after(GTK_OBJECT(files), "button_press_event", GTK_SIGNAL_FUNC(FilesDClick), this);
  
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin3), files);
  
	gtk_list_set_selection_mode (GTK_LIST (files), GTK_SELECTION_EXTENDED);

	dialog_action_area1 = GTK_DIALOG (m_pDialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

	gtk_widget_set_sensitive(ok, FALSE);
}

CAddFilesDlg::~CAddFilesDlg()
{
	gtk_widget_destroy(m_pDialog);
}

void CAddFilesDlg::SetOpenDirectory(String& dir)
{
	if (dir.empty())
		return;
		
	gtk_widget_map(directories);
	
	dir = dir.after(0);
	
	GList* rootChild = gtk_container_children(GTK_CONTAINER(directories));
	
	GtkWidget* item = GTK_WIDGET(rootChild->data);

	GtkWidget* node = GTK_TREE_ITEM(item)->subtree;

	g_list_free(rootChild);

	int i = dir.index('/');
	
	while (node)
	{
		String curItem;

		if (i < 0)
			curItem = dir;
		else
		{
			curItem = dir.before(i);
			dir = dir.after(i);
		}

		GList* c = gtk_container_children(GTK_CONTAINER(node));
		GList* children = c;

		node = NULL;
		
		while (children)
		{
			if (curItem == GetItemText(GTK_WIDGET(children->data)))
			{
				item = GTK_WIDGET(children->data);
				gtk_tree_item_expand(GTK_TREE_ITEM(children->data));
				node = GTK_TREE_ITEM(children->data)->subtree;
				break;
			}

			children = children->next;
		}

		g_list_free(c);
		
		i = dir.index('/');
	}

	if (item)
		gtk_tree_select_child(GTK_TREE(directories), item);
}
