/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "Vermie.h"
#include "VermTreeNode.h"
#include "DirectoryDlg.h"
#include "AfxMessageBox.h"

class DirEntry
{
public:
	String name;
	DirEntry* pNext;

	inline DirEntry() { pNext = NULL; }
	inline ~DirEntry() { };
};

static void NodeExpanded(GtkTreeItem* pNode, CDirectoryDlg* pDlg);
static void NodeSelected(GtkWidget*, GtkWidget* pNode, CDirectoryDlg* pDlg);

static int compareStrings(const void* a, const void* b)
{
	return fcompare(*((String*)a), *((String*)b));
}

static void ExpandDirectories(GtkTreeItem* pNode, String& dir, CDirectoryDlg* pDlg)
{
	if (dir.empty())
		return;
	
	// Build up the current directory

	DIR* d = opendir(dir.chars());

	// something has gone horribly worng!
	if (!d)
		return;

	struct dirent* dent;

	GtkWidget* pSubtree = GTK_TREE_ITEM_SUBTREE(pNode);
		
	DirEntry *pHead, *pCur;

	pHead = NULL;

	pCur = pHead;
	
	int numDirs = 0;
	
//	gtk_widget_set_sensitive(pDlg->scrollWin1, FALSE);

	while ((dent = readdir(d)) != NULL)
	{
		if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
			continue;

		String strFile = dir + "/";
		strFile += dent->d_name;

		struct stat sb;

		stat(strFile.chars(), &sb);
		
		if (!S_ISDIR(sb.st_mode))
			continue;

		if (!pHead)
		{
			pHead = new DirEntry;
			pCur = pHead;
		}
		else
		{
			pCur->pNext = new DirEntry;
			pCur = pCur->pNext;
		}
		
		pCur->name = dent->d_name;
		++numDirs;
	}

	closedir(d);
	
	if (numDirs > 0)
	{
		String *directories = new String[numDirs];
		
		int i;
		
		for (i = 0; i < numDirs; i++)
		{
			directories[i] = pHead->name;
			pCur = pHead;
			pHead = pHead->pNext;
			delete pCur;
		}
		
		qsort(directories, numDirs, sizeof(String), compareStrings);
		
		for (i = 0; i < numDirs; i++)
		{
			GtkWidget* pItem = gtk_tree_item_new_with_label((char*)directories[i].chars());
			GTK_WIDGET_SET_FLAGS(pItem, GTK_CAN_FOCUS);
			gtk_widget_set_events(pItem, GDK_FOCUS_CHANGE_MASK);
			gtk_signal_connect(GTK_OBJECT(pItem), "expand", GtkSignalFunc(NodeExpanded), pDlg);
			GtkWidget* pItemSub = gtk_tree_new();
			gtk_signal_connect (GTK_OBJECT(pItemSub), "select_child",
								GTK_SIGNAL_FUNC(NodeSelected), pDlg);
			gtk_widget_show(pItemSub);
			gtk_tree_append(GTK_TREE(pSubtree), pItem);
			gtk_tree_item_set_subtree(GTK_TREE_ITEM(pItem), pItemSub);
			gtk_widget_show(pItem);
			
			String newDir = dir + "/" + directories[i];
			
		}
		delete []directories;
	}
	else
	{
		gtk_tree_item_remove_subtree(pNode);
	}
}

char* CDirectoryDlg::GetItemText(GtkWidget* w)
{
	char *name;
	GtkLabel* l;

	l = GTK_LABEL(GTK_BIN(w)->child);
	gtk_label_get(l, &name);

	return name;
}

String CDirectoryDlg::GetDirectory(GtkWidget* pNode)
{
	String dir;
	int first = 1;
	
	GtkTree* tree = GTK_TREE(pNode->parent);
	GtkTreeItem* item = GTK_TREE_ITEM(pNode);

	while (item)
	{
		char* curDir = GetItemText(GTK_WIDGET(item));
		if (first)
		{
			dir = curDir;
			first = 0;
		}
		else
		{
			if (!tree->tree_owner)
				dir = curDir + dir;
			else
			{
				dir = "/" + dir;
				dir = curDir + dir;
			}
		}
		
		if (tree->tree_owner != NULL)
		{
			item = GTK_TREE_ITEM(tree->tree_owner);
			tree = GTK_TREE(GTK_WIDGET(tree)->parent);
		}
		else
			item = NULL;
	}

	return dir;
}

static void NodeExpanded(GtkTreeItem* pNode, CDirectoryDlg* pDlg)
{
	// node has already been expanded
	if (gtk_object_get_user_data(GTK_OBJECT(pNode)))
		return;

	gtk_object_set_user_data(GTK_OBJECT(pNode), pDlg);

	String dir = pDlg->GetDirectory(GTK_WIDGET(pNode));

	ExpandDirectories(pNode, dir, pDlg);
}	

static void NodeSelected(GtkWidget*, GtkWidget* pNode, CDirectoryDlg* pDlg)
{
	String dir = pDlg->GetDirectory(pNode);
	
	gtk_entry_set_text(GTK_ENTRY(pDlg->directory), dir.chars());
}

CDirectoryDlg::CDirectoryDlg() : CVermDlg("Set Working Directory", MB_OK|MB_CANCEL)
{
	GtkWidget *dialog_vbox2;
	GtkWidget *vbox1;
	GtkWidget *hbox2;
	GtkWidget *label3;
	GtkWidget *label2;
	GtkWidget *scrollWin;
	GtkWidget *dialog_action_area2;

	dialog_vbox2 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_vbox2", dialog_vbox2);
	gtk_widget_show (dialog_vbox2);

	vbox1 = gtk_vbox_new (FALSE, 5);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox1", vbox1);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox2), vbox1, TRUE, TRUE, 5);
	gtk_container_border_width (GTK_CONTAINER (vbox1), 5);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox2", hbox2);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 0);

	label3 = gtk_label_new ("Directory:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label3", label3);
	gtk_widget_show (label3);
	gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

	directory = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "directory", directory);
	gtk_widget_show (directory);
	gtk_box_pack_start (GTK_BOX (hbox2), directory, TRUE, TRUE, 0);

	label2 = gtk_label_new ("Directories:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label2", label2);
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (vbox1), label2, TRUE, TRUE, 0);
	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

	scrollWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	
	gtk_widget_set_usize(scrollWin, 200, 200);
	
	directories = gtk_tree_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "directories", directories);
	gtk_widget_show (directories);
	gtk_widget_show(scrollWin);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin), directories);
	
	gtk_box_pack_start (GTK_BOX (vbox1), scrollWin, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (scrollWin), 5);
	gtk_tree_set_selection_mode (GTK_TREE (directories), GTK_SELECTION_BROWSE);

	GtkWidget* pRoot = gtk_tree_item_new_with_label("/");
	GTK_WIDGET_SET_FLAGS(pRoot, GTK_CAN_FOCUS);
	gtk_widget_set_events(pRoot, GDK_FOCUS_CHANGE_MASK);
	GtkWidget* pSubtree = gtk_tree_new();
	
	gtk_signal_connect(GTK_OBJECT(pRoot), "expand", GtkSignalFunc(NodeExpanded), this);
	gtk_signal_connect (GTK_OBJECT(directories), "select_child",
						GTK_SIGNAL_FUNC(NodeSelected), this);
	
	gtk_signal_connect(GTK_OBJECT(pSubtree), "select_child",
					   GtkSignalFunc(NodeSelected), this);
	
	gtk_widget_show(pSubtree);
	
	gtk_tree_append(GTK_TREE(directories), pRoot);
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(pRoot), pSubtree);
	
	gtk_widget_show(pRoot);
	gtk_tree_item_expand(GTK_TREE_ITEM(pRoot));
	
	dialog_action_area2 = GTK_DIALOG (m_pDialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_action_area2", dialog_action_area2);
	gtk_widget_show (dialog_action_area2);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);
}

CDirectoryDlg::~CDirectoryDlg()
{
	gtk_widget_destroy(m_pDialog);
}
