/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <gtk/gtk.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include "Vermie.h"
#include "HistoryDlg.h"
#include "VermFrameWnd.h"
#include "VermProtocolMessage.h"
#include "VermUtils.h"
#include "AfxMessageBox.h"
#include "DetailsDlg.h"
#include "VermSocket.h"
#include "VermProjectView.h"
#include "VermTreeNode.h"
#include "VermTreeLeaf.h"
#include "VermTreeUtils.h"

typedef struct _HistoryDetails
{
	String strLabel;
	String strAction;
	String strComment;
} HistoryDetails;

static char* columns[] = {"Version", "User", "Date", "Comment"};

static void ListChildSelected(GtkWidget* pChild, int, int, GdkEventButton*,
							  CHistoryDlg* pDlg);

static void Diff(GtkWidget*, CHistoryDlg* pDlg);
static void Get(GtkWidget*, CHistoryDlg* pDlg);
static void Details(GtkWidget*, CHistoryDlg* pDlg);
static void Pin(GtkWidget*, CHistoryDlg* pDlg);

static void KillDialog(GtkWidget*, CHistoryDlg* pDlg)
{
	int nRows = GTK_CLIST(pDlg->list1)->rows;

	for (int i = 0; i < nRows; i++)
	{
		HistoryDetails* pDetails = (HistoryDetails*) gtk_clist_get_row_data(GTK_CLIST(pDlg->list1), i);

		delete pDetails;
	}

	if (!pDlg->m_bBeingDestroyed)
		pDlg->m_pParent->HistoryDlgGone();
}

static void ListChildSelected(GtkWidget*, int, int, GdkEventButton*,
							  CHistoryDlg* pDlg)
{
	GList* selection = GTK_CLIST(pDlg->list1)->selection;

	if (!selection)
	{
		// 0 items selected
		gtk_widget_set_sensitive(pDlg->buttonDetails, FALSE);
		gtk_widget_set_sensitive(pDlg->buttonGet, FALSE);
		gtk_widget_set_sensitive(pDlg->buttonDiff, FALSE);

		if (pDlg->buttonPin)
			gtk_widget_set_sensitive(pDlg->buttonPin, FALSE);
	}
	else if (!selection->next)
	{
		gtk_widget_set_sensitive(pDlg->buttonDetails, TRUE);

		// 1 item selected
		if (pDlg->m_bFileHistory)
		{
			gtk_widget_set_sensitive(pDlg->buttonGet, TRUE);
			gtk_widget_set_sensitive(pDlg->buttonDiff, FALSE);

			if (pDlg->buttonPin)
			{
				if (pDlg->m_nPinnedVersion != 0)
					gtk_widget_set_sensitive(pDlg->buttonPin, TRUE);	
				else
					gtk_widget_set_sensitive(pDlg->buttonPin, FALSE);
			}
		}
		else
		{
			int sel = (int) selection->data;
			HistoryDetails* pDetails = (HistoryDetails*) gtk_clist_get_row_data(GTK_CLIST(pDlg->list1), sel);

			if (pDlg->buttonPin)
			{
				if (pDlg->m_nPinnedVersion != 0)
					gtk_widget_set_sensitive(pDlg->buttonPin, TRUE);
				else
					gtk_widget_set_sensitive(pDlg->buttonPin, FALSE);
			}
			
			if (pDetails->strLabel.length())
			{
				gtk_widget_set_sensitive(pDlg->buttonGet, TRUE);
			}
			else
			{
				gtk_widget_set_sensitive(pDlg->buttonGet, FALSE);
			}
		}

	}
	else if (!selection->next->next)
	{
		if (pDlg->m_bFileHistory)
		{
			// 2 items selected
			gtk_widget_set_sensitive(pDlg->buttonDetails, FALSE);
			gtk_widget_set_sensitive(pDlg->buttonGet, FALSE);

			if (pDlg->buttonPin)
				gtk_widget_set_sensitive(pDlg->buttonPin, FALSE);
			
			if (!Vermie::m_strDiffProgram.empty() && pDlg->m_bFileHistory)
				gtk_widget_set_sensitive(pDlg->buttonDiff, TRUE);
			else
				gtk_widget_set_sensitive(pDlg->buttonDiff, FALSE);
		}
	}
	else
	{
		gtk_widget_set_sensitive(pDlg->buttonDetails, FALSE);
		gtk_widget_set_sensitive(pDlg->buttonGet, FALSE);
		gtk_widget_set_sensitive(pDlg->buttonDiff, FALSE);
		if (pDlg->buttonPin)
			gtk_widget_set_sensitive(pDlg->buttonPin, FALSE);
	}
}

static void Diff(GtkWidget*, CHistoryDlg* pDlg)
{
	GList* selection = GTK_CLIST(pDlg->list1)->selection;

	if (!selection)
		return;
	
	int sel1 = (int)selection->data;

	if (!selection->next)
		return;

	int sel2 = (int)selection->next->data;

	char *text = NULL;
	
	guint8 spacing;
	GdkPixmap* pix;
	GdkBitmap* bmp;

	gtk_clist_get_pixtext(GTK_CLIST(pDlg->list1), sel1, 0, &text, &spacing, &pix, &bmp);

	if (!text)
		gtk_clist_get_text(GTK_CLIST(pDlg->list1), sel1, 0, &text);

	String strVerOne = text;
	
	if (strVerOne.empty())
	{
		int index = sel1 + 1;
		while ((index < GTK_CLIST(pDlg->list1)->rows) && (strVerOne.empty()))
		{
			text = NULL;
			
			gtk_clist_get_pixtext(GTK_CLIST(pDlg->list1), sel1, 0, &text, &spacing, &pix, &bmp);

			if (!text)
				gtk_clist_get_text(GTK_CLIST(pDlg->list1), index++, 0, &text);
			
			strVerOne = text;
		}
	}
	text = NULL;
	
	gtk_clist_get_pixtext(GTK_CLIST(pDlg->list1), sel1, 0, &text, &spacing, &pix, &bmp);

	if (!text)
		gtk_clist_get_text(GTK_CLIST(pDlg->list1), sel2, 0, &text);
	
	String strVerTwo = text;

	if (strVerTwo.empty())
	{
		int index = sel2 + 1;

		while ((index < GTK_CLIST(pDlg->list1)->rows) && (strVerTwo.empty()))
		{
			text = NULL;
			
			gtk_clist_get_pixtext(GTK_CLIST(pDlg->list1), sel1, 0, &text, &spacing, &pix, &bmp);

			if (!text)
				gtk_clist_get_text(GTK_CLIST(pDlg->list1), index++, 0, &text);
			strVerTwo = text;
		}
	}

	if (strVerOne == strVerTwo)
	{
		String s("The files are identical");
		AfxMessageBox(s);
		return;
	}

	if (strVerOne.empty() || strVerTwo.empty())
	{
		return;
	}

	// get the temp directory

	String strTmpDir = tempnam(NULL, NULL);

	if (strTmpDir.empty())
	{
		String s("Unable to create a temporary directory");
		AfxMessageBox(s);
		return;
	}
	
	// create a unique directory in the temp directory

	if (mkdir(strTmpDir.chars(), 0700))
	{
		String s("Unable to create a temporary directory");
		AfxMessageBox(s);
		return;
	}

	if (strTmpDir[strTmpDir.length() - 1] != '/')
		strTmpDir += "/";
	
	// create a diff struct

	FileDiff* pFileDiff = new FileDiff;

	pFileDiff->bProjectDiff = 0;
	pFileDiff->bRecurse = 0;
	
	pFileDiff->strTmpDir = strTmpDir;

	pFileDiff->strFileOne = strTmpDir + pDlg->m_strFile;
	pFileDiff->strFileOneVersion = strVerOne;
	pFileDiff->bHaveFileOne = 0;
	pFileDiff->bOneIsTempFile = 1;
	
	pFileDiff->strFileTwo = strTmpDir + pDlg->m_strFile;
	pFileDiff->strFileTwoVersion = strVerTwo;
	pFileDiff->bHaveFileTwo = 0;
	pFileDiff->bTwoIsTempFile = 1;

	pDlg->m_pParent->GetDiffFiles(pFileDiff, pDlg->m_strProject);
}

static void Get(GtkWidget*, CHistoryDlg* pDlg)
{
	int row = (int) GTK_CLIST(pDlg->list1)->selection->data;

	if (pDlg->m_bFileHistory)
	{
		char *num = NULL;
		GdkPixmap* pix;
		GdkBitmap* bmp;
		guint8 spacing;
		
		gtk_clist_get_pixtext(GTK_CLIST(pDlg->list1), row, 0, &num, &spacing, &pix, &bmp);

		if (!num)
			gtk_clist_get_text(GTK_CLIST(pDlg->list1), row, 0, &num);
		
		int nVerNum = atoi(num);
		
		while (!nVerNum && nVerNum < GTK_CLIST(pDlg->list1)->rows)
		{
			num = NULL;
			
			gtk_clist_get_pixtext(GTK_CLIST(pDlg->list1), row, 0, &num, &spacing, &pix, &bmp);

			if (!num)
				gtk_clist_get_text(GTK_CLIST(pDlg->list1), ++row, 0, &num);

			nVerNum = atoi(num);
		}
		
		pDlg->m_pParent->GetFile(pDlg->m_strProject, pDlg->m_strFile,
								 !Vermie::m_bMakeFilesReadOnly, nVerNum);
	}
	else
	{
		HistoryDetails* pDetails = (HistoryDetails*)gtk_clist_get_row_data(GTK_CLIST(pDlg->list1), row);

		pDlg->m_pParent->GetProjectByLabel(pDlg->m_strProject,
										   pDetails->strLabel);
	}
}

static void Pin(GtkWidget*, CHistoryDlg* pDlg)
{
	if (pDlg->m_nPinnedVersion > 0)
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_UNPIN_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pDlg->m_strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pDlg->m_strFile));

		if (!pDlg->m_pParent->m_pSocket->SendMessage(pMessage))
			pDlg->m_pParent->SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
	else
	{
		int row = (int) GTK_CLIST(pDlg->list1)->selection->data;

		char *num;
		
		gtk_clist_get_text(GTK_CLIST(pDlg->list1), row, 0, &num);
		int nVerNum = atoi(num);
		
		while (!nVerNum && nVerNum < GTK_CLIST(pDlg->list1)->rows)
		{
			gtk_clist_get_text(GTK_CLIST(pDlg->list1), ++row, 0, &num);
			nVerNum = atoi(num);
		}

		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_PIN_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pDlg->m_strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pDlg->m_strFile));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_VERSION, num));

		if (!pDlg->m_pParent->m_pSocket->SendMessage(pMessage))
			pDlg->m_pParent->SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
	
}

static void Details(GtkWidget*, CHistoryDlg* pDlg)
{
	GList* selection = GTK_CLIST(pDlg->list1)->selection;

	if (!selection)
		return;
	
	int sel1 = (int)selection->data;

	HistoryDetails* pDetails = (HistoryDetails*)gtk_clist_get_row_data(GTK_CLIST(pDlg->list1), sel1);

	CDetailsDlg dlg;

	if (!pDetails->strComment.empty())
		gtk_text_insert(GTK_TEXT(dlg.comment), NULL, NULL, NULL,
						pDetails->strComment.chars(), -1);

	if (!pDetails->strLabel.empty())
		gtk_label_set(GTK_LABEL(dlg.label), pDetails->strLabel.chars());

	if (!pDetails->strAction.empty())
		gtk_label_set(GTK_LABEL(dlg.action), pDetails->strAction.chars());
	
	dlg.DoModal();
}

CHistoryDlg::CHistoryDlg(CVermFrameWnd* parent, CVermProtocolMessage* pMessage, int nPinnedVersion, int bPin /* = 0 */)
{
	buttonPin = NULL;

	m_bBeingDestroyed = 0;
	
	m_pParent = parent;
	
	m_strProject = pMessage->GetHeaderValue(HEADER_PROJECT);

	m_nPinnedVersion = nPinnedVersion;
	
	String strRev = reverse(m_strProject);
	int index = strRev.index('/');

	m_strProjectName = m_strProject.after((int)m_strProject.length() - index - 1);

	const char* pszFile = pMessage->GetHeaderValue(HEADER_FILE);

	if (pszFile)
	{
		m_strFile = pszFile;
		m_bFileHistory = 1;
	}
	else
	{
		m_bFileHistory = 0;
		columns[0] = "Name";
		columns[3] = "Action";

		m_nPinnedVersion = -1;
	}

	GtkWidget *dialog_vbox1;
//	GtkWidget *table1;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *dialog_action_area1;
	GtkWidget *scrolledWin;
	GtkWidget *hbuttonbox1;
	GtkWidget *button5;
	GtkWidget *button6;

	m_pDialog = gtk_dialog_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "m_pDialog", m_pDialog);
	gtk_window_set_title (GTK_WINDOW (m_pDialog), "File History");
	gtk_container_border_width(GTK_CONTAINER(m_pDialog), 5);
	gtk_widget_realize(m_pDialog);
	gtk_window_set_policy (GTK_WINDOW (m_pDialog), TRUE, TRUE, FALSE);
	
	dialog_vbox1 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);
	
	vbox = gtk_hbox_new(FALSE, 5);
	gtk_widget_show(vbox);

	gtk_box_pack_start(GTK_BOX(dialog_vbox1), vbox, TRUE, TRUE, 0);
	
	scrolledWin = gtk_scrolled_window_new(NULL, NULL);

	gtk_widget_set_usize(scrolledWin, 400, 400);
//	gtk_window_set_policy(GTK_WINDOW(scrolledWin), TRUE, TRUE, TRUE);
	
	list1 = gtk_clist_new_with_titles(4, columns);
	gtk_clist_set_column_width(GTK_CLIST(list1), 0, 50);
	gtk_clist_set_column_width(GTK_CLIST(list1), 1, 50);
	gtk_clist_set_column_width(GTK_CLIST(list1), 2, 75);
	gtk_clist_set_column_width(GTK_CLIST(list1), 3, 150);
	gtk_clist_set_selection_mode(GTK_CLIST(list1), GTK_SELECTION_EXTENDED);
	
	gtk_widget_show(scrolledWin);

	gtk_object_set_data (GTK_OBJECT (m_pDialog), "list1", list1);
	gtk_signal_connect(GTK_OBJECT(list1), "select_row",
					   (GtkSignalFunc)ListChildSelected, this);
	gtk_signal_connect(GTK_OBJECT(list1), "unselect_row",
					   (GtkSignalFunc)ListChildSelected, this);
	
	gtk_widget_show (list1);
	gtk_container_add(GTK_CONTAINER(scrolledWin), list1);

	gtk_box_pack_start(GTK_BOX(vbox), scrolledWin, TRUE, TRUE, 0);

	hbox = gtk_vbox_new(FALSE, 5);
	gtk_widget_show(hbox);

	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	
	buttonDiff = gtk_button_new_with_label ("Diff");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "buttonDiff", buttonDiff);
	gtk_widget_show (buttonDiff);
	gtk_signal_connect(GTK_OBJECT(buttonDiff), "clicked", (GtkSignalFunc)Diff,
					   this);

	gtk_widget_set_sensitive(buttonDiff, FALSE);
	
	gtk_box_pack_start(GTK_BOX(hbox), buttonDiff, FALSE, FALSE, 5);
	
//	gtk_table_attach (GTK_TABLE (table1), buttonDiff, 1, 2, 2, 3,
//					  (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 5, 5);
	
	buttonGet = gtk_button_new_with_label ("Get");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "buttonGet", buttonGet);
	gtk_widget_show (buttonGet);
	gtk_signal_connect(GTK_OBJECT(buttonGet), "clicked", (GtkSignalFunc)Get,
					  this);
	gtk_box_pack_start(GTK_BOX(hbox), buttonGet, FALSE, FALSE, 5);

	gtk_widget_set_sensitive(buttonGet, FALSE);
	
	buttonDetails = gtk_button_new_with_label ("Details");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "buttonDetails", buttonDetails);
	gtk_widget_show (buttonDetails);
	gtk_signal_connect(GTK_OBJECT(buttonDetails), "clicked", (GtkSignalFunc)Details,
					  this);
	gtk_box_pack_start(GTK_BOX(hbox), buttonDetails, FALSE, FALSE, 5);

	gtk_widget_set_sensitive(buttonDetails, FALSE);
	
	if (bPin)
	{
		if (m_nPinnedVersion <= 0)
			buttonPin = gtk_button_new_with_label ("Pin");	
		else
			buttonPin = gtk_button_new_with_label ("Un Pin");
		
		gtk_object_set_data (GTK_OBJECT (m_pDialog), "buttonPin", buttonPin);
		gtk_widget_show (buttonPin);
		gtk_signal_connect(GTK_OBJECT(buttonPin), "clicked", (GtkSignalFunc)Pin,
						   this);
		gtk_box_pack_start(GTK_BOX(hbox), buttonPin, FALSE, FALSE, 5);
		
		gtk_widget_set_sensitive(buttonPin, FALSE);
	}
	
	dialog_action_area1 = GTK_DIALOG (m_pDialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbuttonbox1", hbuttonbox1);
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);
	
	button5 = gtk_button_new_with_label ("OK");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "button5", button5);
	gtk_widget_show (button5);
	GTK_WIDGET_SET_FLAGS(button5, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(button5);

	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button5);

	gtk_signal_connect(GTK_OBJECT(button5), "clicked",
					   (GtkSignalFunc)KillDialog, this);
	
	button6 = gtk_button_new_with_label ("Cancel");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "button6", button6);
	gtk_widget_show (button6);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button6);

	gtk_signal_connect(GTK_OBJECT(button6), "clicked",
					   (GtkSignalFunc)KillDialog, this);
	
	m_destroySig = gtk_signal_connect(GTK_OBJECT(m_pDialog), "destroy",
									  (GtkSignalFunc)KillDialog, this);

	int bError = 0;

	int nPos = 0;
	int nIndex = 0;
	char *junk[4] = {"","","",""};
	
	
	char* pMessageBody = pMessage->m_body;

	while (nPos < pMessage->m_bodySize)
	{
		nIndex = gtk_clist_insert(GTK_CLIST(list1), nIndex, junk);

		char* pszName = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (pszName)
		{
			if (m_nPinnedVersion > 0)
			{
				if (atoi(pszName) == m_nPinnedVersion)
				{
					gtk_clist_set_pixtext(GTK_CLIST(list1), nIndex, 0, pszName, 3, Vermie::m_pPin, Vermie::m_pPinMask);
				}
				else
					gtk_clist_set_text(GTK_CLIST(list1), nIndex, 0, pszName);
			}
			else
				gtk_clist_set_text(GTK_CLIST(list1), nIndex, 0, pszName);
		}
		else
		{
			if (bError)
				break;
			
		}
		
		char* pszUser = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (!pszUser)
		{
			break;
		}			

		gtk_clist_set_text(GTK_CLIST(list1), nIndex, 1, pszUser);

		char* pszDate = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (!pszDate)
			break;

		gtk_clist_set_text(GTK_CLIST(list1), nIndex, 2, pszDate);

		char* pszComment = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (!pszComment && bError)
			break;


		char* pszLabel = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (!pszLabel && bError)
			break;

		char* pszLabelComment = NULL;

		if (Vermie::m_bUseVSS6Features)
		{
			pszLabelComment = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

			if (!pszLabelComment && bError)
				break;
		}
		
		char* pszAction = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (!pszAction && bError)
			break;

		if (m_bFileHistory)
		{
			if (!pszComment && pszLabelComment)
				gtk_clist_set_text(GTK_CLIST(list1), nIndex, 3, pszLabelComment);
			else
				gtk_clist_set_text(GTK_CLIST(list1), nIndex, 3, pszComment);
		}
		else
		{
			gtk_clist_set_text(GTK_CLIST(list1), nIndex, 3, pszAction);
		}			
			
		HistoryDetails* pDetails = new HistoryDetails;

		pDetails->strLabel = pszLabel;
		pDetails->strAction = pszAction;

		if (pszComment)
			pDetails->strComment = pszComment;
		else if (pszLabelComment)
			pDetails->strComment = pszLabelComment;

		gtk_clist_set_row_data(GTK_CLIST(list1), nIndex, pDetails);
		
		nIndex++;
	}

	gtk_clist_remove(GTK_CLIST(list1), nIndex);
}	

CHistoryDlg::~CHistoryDlg()
{
	m_bBeingDestroyed = 1;
	
	gtk_signal_disconnect(GTK_OBJECT(m_pDialog), m_destroySig);
	
	gtk_widget_destroy(m_pDialog);
}

void CHistoryDlg::PinUpdate(int nPinnedVersion)
{
	if (nPinnedVersion > 0)
	{
		// this file was successfully pinned

		gtk_label_set_text(GTK_LABEL(GTK_BUTTON(buttonPin)->child), "Unpin");

		for (int i = 0; i < GTK_CLIST(list1)->rows; i++)
		{
			char *name = NULL;
			gtk_clist_get_text(GTK_CLIST(list1), i, 0, &name);

			if (atoi(name) == nPinnedVersion)
			{
				char* foo = new char[strlen(name) + 1];
				strcpy(foo, name);
			
				gtk_clist_set_pixtext(GTK_CLIST(list1), i, 0, foo, 3, Vermie::m_pPin, Vermie::m_pPinMask);
			}
		}
	}
	else
	{
		// the file has been unpinned

		gtk_label_set_text(GTK_LABEL(GTK_BUTTON(buttonPin)->child), "Pin");

		for (int i = 0; i < GTK_CLIST(list1)->rows; i++)
		{
			char *name = NULL;
			gtk_clist_get_text(GTK_CLIST(list1), i, 0, &name);

			if (!name)
			{
				guint8 spacing;
				GdkPixmap *pix;
				GdkBitmap *bmp;
				
				gtk_clist_get_pixtext(GTK_CLIST(list1), i, 0, &name, &spacing, &pix, &bmp);
			}
			
			if (atoi(name) == m_nPinnedVersion)
			{
				char* foo = new char[strlen(name) + 1];
				strcpy(foo, name);
				gtk_clist_set_text(GTK_CLIST(list1), i, 0, foo);
			}
		}
	}

	m_nPinnedVersion = nPinnedVersion;			
}
