/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdio.h>

#include <gtk/gtk.h>
#include <gtk/gtkprivate.h>

#include "Vermie.h"
#include "OptionsDlg.h"

static short whichPressed = -1;
static 		GdkColor gdk_color;

static void ok_sel_clicked(GtkWidget* button, gpointer* foo)
{
	whichPressed = IDOK;

	gtk_main_quit();
}

static void ChooseColor(GtkWidget* button, COptionsDlg* pDlg)
{
	GtkWidget* colorseldlg = gtk_color_selection_dialog_new("Select new color");

	gtk_window_set_modal(GTK_WINDOW(colorseldlg), TRUE);

	gtk_window_set_transient_for(GTK_WINDOW(colorseldlg), GTK_WINDOW(pDlg->m_pDialog));

    gtk_signal_connect_object (GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorseldlg)->ok_button),
                               "clicked", GTK_SIGNAL_FUNC(ok_sel_clicked),
                               GTK_OBJECT (colorseldlg));
    gtk_signal_connect_object (GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colorseldlg)->cancel_button),
                               "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
                               GTK_OBJECT (colorseldlg));
	
	/* Show the dialog */

	gtk_widget_show(colorseldlg);	

	gtk_main();

	if (whichPressed == IDOK)
	{
		pDlg->colorChanged = 1;
		
		whichPressed = -1;

		GtkWidget* colorsel = GTK_COLOR_SELECTION_DIALOG(colorseldlg)->colorsel;

		gdouble color[3];
		GdkColormap *colormap;
		
		/* Get drawingarea colormap */
		
		colormap = gdk_window_get_colormap (pDlg->m_pDialog->window);
		
		/* Get current color */
		
		gtk_color_selection_get_color (GTK_COLOR_SELECTION(colorsel),color);
		
		/* Fit to a unsigned 16 bit integer (0..65535) and
		 * insert into the GdkColor structure */
		
		gdk_color.red = (guint16)(color[0]*65535.0);
		gdk_color.green = (guint16)(color[1]*65535.0);
		gdk_color.blue = (guint16)(color[2]*65535.0);
		
		/* Allocate color */
		
       gdk_color_alloc (colormap, &gdk_color);

	   GtkStyle* style;
	   GtkWidget** label;
	   GtkWidget* hbox;
	   GtkWidget** pButton;
	   char* str;
	   
	   if (button == pDlg->bothButton)
	   {
		   style = pDlg->pBothStyle;
		   label = &pDlg->cloakedDeployLabel;
		   hbox = pDlg->hbox14;
		   pButton = &pDlg->bothButton;
		   str = "Cloaked and Web Deploy Projects";
	   }
	   else if (button == pDlg->cloakedButton)
	   {
		   style = pDlg->pCloakedStyle;
		   label = &pDlg->cloakedLabel;
		   hbox = pDlg->hbox12;
		   pButton = &pDlg->cloakedButton;
		   str = "Cloaked Projects";
	   }
	   else
	   {
		   style = pDlg->pDeployStyle;
		   label = &pDlg->deployLabel;
		   hbox = pDlg->hbox13;
		   pButton = &pDlg->deployButton;
		   str = "Web Deploy Projects";
	   }
		   
	   for (int i = 0; i < 5; i++)
	   {
		   style->bg[i] = gdk_color;
		   style->fg[i] = gdk_color;
		   style->base[i] = gdk_color;
	   }

	   gtk_widget_destroy(*label);
	   gtk_widget_destroy(*pButton);

	   *label = gtk_label_new(str);
	   
	   gtk_widget_set_style(*label, style);

	   gtk_widget_show(*label);
	   
	   gtk_box_pack_start(GTK_BOX(hbox), *label, FALSE, TRUE, 2);

	   *pButton = gtk_button_new_with_label("Choose color");
	   gtk_widget_show(*pButton);
	   gtk_signal_connect(GTK_OBJECT(*pButton), "clicked",
						  GTK_SIGNAL_FUNC(ChooseColor), pDlg);
	   
	   gtk_box_pack_start(GTK_BOX(hbox), *pButton, FALSE, TRUE, 2);

	   gtk_widget_destroy(colorseldlg);
	}
}

static void CompressToggled(GtkWidget* button, COptionsDlg* pDlg)
{
	if (GTK_TOGGLE_BUTTON(button)->active)
	{
		if (button == pDlg->refresh)
			gtk_widget_set_sensitive(pDlg->timeToRefresh, 1);
		else
			gtk_widget_set_sensitive(pDlg->timeToKeepAlive, 1);
	}
	else
	{
		if (button == pDlg->refresh)
			gtk_widget_set_sensitive(pDlg->timeToRefresh, 0);
		else
			gtk_widget_set_sensitive(pDlg->timeToKeepAlive, 0);
	}
}

static void ProxyToggled(GtkWidget* button, COptionsDlg* pDlg)
{
	if (GTK_TOGGLE_BUTTON(button)->active)
	{
		gtk_widget_set_sensitive(pDlg->proxyServer, 1);
		gtk_widget_set_sensitive(pDlg->proxyPort, 1);		
	}
	else
	{
		gtk_widget_set_sensitive(pDlg->proxyServer, 0);
		gtk_widget_set_sensitive(pDlg->proxyPort, 0);		
	}
}

/* This is an internally used function to set notebook tab widgets. */
static void set_notebook_tab(GtkWidget* notebook, gint page_num,
							 GtkWidget* widget)
{
	GtkNotebookPage *page;
	GtkWidget *notebook_page;
	
	page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
	notebook_page = page->child;
	gtk_widget_ref (notebook_page);
	gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
	gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
							  widget, page_num);
	gtk_widget_unref (notebook_page);
}

COptionsDlg::COptionsDlg()  : CVermDlg("Options", MB_OK|MB_CANCEL)
{
	colorChanged = 0;
	
	pCloakedStyle = gtk_style_copy(Vermie::m_pCloakedStyle);
	pDeployStyle = gtk_style_copy(Vermie::m_pWebDeployStyle);
	pBothStyle = gtk_style_copy(Vermie::m_pCloakedDeployStyle);
	
	dialog_vbox1 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	notebook1 = gtk_notebook_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "notebook1", notebook1);
	gtk_widget_show (notebook1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), notebook1, TRUE, TRUE, 0);
	gtk_container_border_width (GTK_CONTAINER (notebook1), 5);

	frame1 = gtk_frame_new ((gchar*)NULL);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "frame1", frame1);
	gtk_widget_show (frame1);
	gtk_container_add (GTK_CONTAINER (notebook1), frame1);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox1", vbox1);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (frame1), vbox1);

	hbox11 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox11", hbox11);
	gtk_widget_show (hbox11);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox11, FALSE, TRUE, 0);

	keepAlive = gtk_check_button_new_with_label ("Automatically send keep-alive signal every");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "checkbutton4", keepAlive);
	gtk_signal_connect(GTK_OBJECT(keepAlive), "toggled", (GtkSignalFunc)CompressToggled, this);
  
	gtk_widget_show (keepAlive);
	gtk_box_pack_start (GTK_BOX (hbox11), keepAlive, FALSE, FALSE, 2);
	gtk_container_border_width (GTK_CONTAINER (keepAlive), 2);

	timeToKeepAlive = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "timeToKeepAlive", timeToKeepAlive);
	gtk_widget_show (timeToKeepAlive);
	gtk_box_pack_start (GTK_BOX (hbox11), timeToKeepAlive, FALSE, TRUE, 2);

	label8 = gtk_label_new ("minutes");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label8", label8);
	gtk_widget_show (label8);
	gtk_box_pack_start (GTK_BOX (hbox11), label8, FALSE, TRUE, 2);

	recurseProjects = gtk_check_button_new_with_label ("Act on projects recursively");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "recurseProjects", recurseProjects);
	gtk_widget_show (recurseProjects);
	gtk_box_pack_start (GTK_BOX (vbox1), recurseProjects, FALSE, TRUE, 2);

	compression = gtk_check_button_new_with_label ("Use compression for data transfers");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "compression", compression);
	gtk_widget_show (compression);
	gtk_box_pack_start (GTK_BOX (vbox1), compression, FALSE, TRUE, 2);

	hbox5 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox5", hbox5);
	gtk_widget_show (hbox5);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox5, FALSE, TRUE, 2);

	label11 = gtk_label_new ("Double-click on a file: ");
	gtk_object_set_data(GTK_OBJECT(m_pDialog), "label11", label11);
	gtk_widget_show(label11);
	gtk_box_pack_start(GTK_BOX(hbox5), label11, FALSE, TRUE, 0);

	GList *glist = NULL;

	glist = g_list_append(glist, (void*)"Ask");
	glist = g_list_append(glist, (void*)"View File");
	glist = g_list_append(glist, (void*)"Edit File");
  
	fileDClickCombo = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(fileDClickCombo), glist);
	gtk_widget_show(fileDClickCombo);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(fileDClickCombo)->entry), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox5), fileDClickCombo, FALSE, TRUE, 0);

	g_list_free(glist);
  
	hbox6 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox6", hbox6);
	gtk_widget_show (hbox6);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox6, FALSE, TRUE, 2);

	label12 = gtk_label_new ("File End-of-line termination: ");
	gtk_object_set_data(GTK_OBJECT(m_pDialog), "label12", label12);
	gtk_widget_show(label12);
	gtk_box_pack_start(GTK_BOX(hbox6), label12, FALSE, TRUE, 0);

	glist = NULL;

	glist = g_list_append(glist, (void*)UNIX_EOL);
	glist = g_list_append(glist, (void*)WINDOWS_EOL);
	glist = g_list_append(glist, (void*)MAC_EOL);
  
	fileEOLCombo = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(fileEOLCombo), glist);
	gtk_widget_show(fileEOLCombo);
	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(fileEOLCombo)->entry), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox6), fileEOLCombo, FALSE, TRUE, 0);

	g_list_free(glist);
  
	frame6 = gtk_frame_new((gchar*)NULL);
	gtk_widget_show(frame6);
	gtk_container_add(GTK_CONTAINER(notebook1), frame6);

	vbox7 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox7);
	gtk_container_add(GTK_CONTAINER(frame6), vbox7);

	frame7 = gtk_frame_new("Project Tree Label Colors");
	gtk_widget_show(frame7);
	gtk_box_pack_start(GTK_BOX(vbox7), frame7, FALSE, TRUE, 2);
	
	vbox8 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox8);
	gtk_container_add(GTK_CONTAINER(frame7), vbox8);

	hbox12 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show(hbox12);
	gtk_box_pack_start(GTK_BOX(vbox8), hbox12, FALSE, TRUE, 2);

	cloakedLabel = gtk_label_new("Cloaked Projects");
	gtk_widget_show(cloakedLabel);
	gtk_widget_set_style(cloakedLabel, Vermie::m_pCloakedStyle);
	gtk_box_pack_start(GTK_BOX(hbox12), cloakedLabel, FALSE, TRUE, 2);

	cloakedButton = gtk_button_new_with_label("Choose color");
	gtk_widget_show(cloakedButton);
	gtk_box_pack_start(GTK_BOX(hbox12), cloakedButton, FALSE, TRUE, 2);
	gtk_signal_connect(GTK_OBJECT(cloakedButton), "clicked",
					   GTK_SIGNAL_FUNC(ChooseColor), this);
	
	hbox13 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show(hbox13);
	gtk_box_pack_start(GTK_BOX(vbox8), hbox13, FALSE, TRUE, 2);

	deployLabel = gtk_label_new("Web Deploy Projects");
	gtk_widget_show(deployLabel);
	gtk_widget_set_style(deployLabel, Vermie::m_pWebDeployStyle);
	gtk_box_pack_start(GTK_BOX(hbox13), deployLabel, FALSE, TRUE, 2);

	deployButton = gtk_button_new_with_label("Choose color");
	gtk_widget_show(deployButton);
	gtk_box_pack_start(GTK_BOX(hbox13), deployButton, FALSE, TRUE, 2);
	gtk_signal_connect(GTK_OBJECT(deployButton), "clicked",
					   GTK_SIGNAL_FUNC(ChooseColor), this);
	
	hbox14 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show(hbox14);
	gtk_box_pack_start(GTK_BOX(vbox8), hbox14, FALSE, TRUE, 2);

	cloakedDeployLabel = gtk_label_new("Cloaked and Web Deploy Projects");
	gtk_widget_show(cloakedDeployLabel);
	gtk_widget_set_style(cloakedDeployLabel, Vermie::m_pCloakedDeployStyle);
	gtk_box_pack_start(GTK_BOX(hbox14), cloakedDeployLabel, FALSE, TRUE, 2);

	bothButton = gtk_button_new_with_label("Choose color");
	gtk_widget_show(bothButton);
	gtk_box_pack_start(GTK_BOX(hbox14), bothButton, FALSE, TRUE, 2);
	gtk_signal_connect(GTK_OBJECT(bothButton), "clicked",
					   GTK_SIGNAL_FUNC(ChooseColor), this);
	
	frame2 = gtk_frame_new ((gchar*)NULL);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "frame2", frame2);
	gtk_widget_show (frame2);
	gtk_container_add (GTK_CONTAINER (notebook1), frame2);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox2", vbox2);
	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame2), vbox2);

	hbox2 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox2", hbox2);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, TRUE, 2);

	label7 = gtk_label_new ("Number of lines to display in Status Window:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label7", label7);
	gtk_widget_show (label7);
	gtk_box_pack_start (GTK_BOX (hbox2), label7, FALSE, TRUE, 2);

	statusWindowLines = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "statusWindowLines", statusWindowLines);
	gtk_widget_show (statusWindowLines);
	gtk_box_pack_start (GTK_BOX (hbox2), statusWindowLines, FALSE, TRUE, 2);

	char numLines[2048];
	sprintf(numLines, "%d", Vermie::m_nLinesToDisplay);
	gtk_entry_set_text(GTK_ENTRY(statusWindowLines), numLines);
  
	hbox3 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox3", hbox3);
	gtk_widget_show (hbox3);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox3, FALSE, TRUE, 0);

	refresh = gtk_check_button_new_with_label ("Automatically refresh file list every");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "checkbutton4", refresh);
	gtk_signal_connect(GTK_OBJECT(refresh), "toggled", (GtkSignalFunc)CompressToggled, this);
  
	gtk_widget_show (refresh);
	gtk_box_pack_start (GTK_BOX (hbox3), refresh, FALSE, FALSE, 2);
	gtk_container_border_width (GTK_CONTAINER (refresh), 2);

	timeToRefresh = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "timeToRefresh", timeToRefresh);
	gtk_widget_show (timeToRefresh);
	gtk_box_pack_start (GTK_BOX (hbox3), timeToRefresh, FALSE, TRUE, 2);

	label10 = gtk_label_new ("minutes");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label10", label10);
	gtk_widget_show (label10);
	gtk_box_pack_start (GTK_BOX (hbox3), label10, FALSE, TRUE, 2);

	displayPinStatus = gtk_check_button_new_with_label ("Display Pin Status");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "checkbutton4", displayPinStatus);
	gtk_signal_connect(GTK_OBJECT(displayPinStatus), "toggled", (GtkSignalFunc)CompressToggled, this);
	gtk_widget_show (displayPinStatus);
	gtk_box_pack_start (GTK_BOX (vbox2), displayPinStatus, FALSE, FALSE, 2);
	gtk_container_border_width (GTK_CONTAINER (displayPinStatus), 2);

	frame3 = gtk_frame_new ((gchar*)NULL);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "frame3", frame3);
	gtk_widget_show (frame3);
	gtk_container_add (GTK_CONTAINER (notebook1), frame3);

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox4", vbox4);
	gtk_widget_show (vbox4);
	gtk_container_add (GTK_CONTAINER (frame3), vbox4);

	keepFilesCheckedOut = gtk_check_button_new_with_label ("Always keep files checked out");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "keepFilesCheckedOut", keepFilesCheckedOut);
	gtk_widget_show (keepFilesCheckedOut);
	gtk_box_pack_start (GTK_BOX (vbox4), keepFilesCheckedOut, FALSE, TRUE, 2);

	removeLocalCopy = gtk_check_button_new_with_label ("Remove local copy after checkin");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "removeLocalCopy", removeLocalCopy);
	gtk_widget_show (removeLocalCopy);
	gtk_box_pack_start (GTK_BOX (vbox4), removeLocalCopy, FALSE, TRUE, 2);

	dontReplaceLocalCopy = gtk_check_button_new_with_label ("Don't replace local copy during checkout");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dontReplaceLocalCopy", dontReplaceLocalCopy);
	gtk_widget_show (dontReplaceLocalCopy);
	gtk_box_pack_start (GTK_BOX (vbox4), dontReplaceLocalCopy, FALSE, TRUE, 2);

	dontReplaceAfterUndo = gtk_check_button_new_with_label ("Don't replace local copy during undo checkout");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dontReplaceAfterUndo", dontReplaceAfterUndo);
	gtk_widget_show (dontReplaceAfterUndo);
	gtk_box_pack_start (GTK_BOX (vbox4), dontReplaceAfterUndo, FALSE, TRUE, 2);

	makeFilesReadOnly = gtk_check_button_new_with_label("Use read-only flag for files that are not checked out");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "makefilesreadonly", makeFilesReadOnly);
	gtk_widget_show (makeFilesReadOnly);
	gtk_box_pack_start (GTK_BOX (vbox4), makeFilesReadOnly, FALSE, TRUE, 2);
  
	useChecksums = gtk_check_button_new_with_label("Use checksums to determine \"unknown\" file status");
	gtk_widget_show(useChecksums);
	gtk_box_pack_start (GTK_BOX (vbox4), useChecksums, FALSE, TRUE, 2);

	displayShareIcon = gtk_check_button_new_with_label("Display share icon for shared files");
	gtk_widget_show(displayShareIcon);
	gtk_box_pack_start (GTK_BOX (vbox4), displayShareIcon, FALSE, TRUE, 2);
	
	frame4 = gtk_frame_new ((gchar*)NULL);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "frame4", frame4);
	gtk_widget_show (frame4);
	gtk_container_add (GTK_CONTAINER (notebook1), frame4);

	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox5", vbox5);
	gtk_widget_show (vbox5);
	gtk_container_add (GTK_CONTAINER (frame4), vbox5);

	useHTTPProxy = gtk_check_button_new_with_label("Use an HTTP Proxy to connect to the remote SourceOffSite server");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "useHTTPProxy", useHTTPProxy);
	gtk_widget_show (useHTTPProxy);
	gtk_box_pack_start (GTK_BOX (vbox5), useHTTPProxy, FALSE, TRUE, 2);
	gtk_signal_connect(GTK_OBJECT(useHTTPProxy), "toggled", (GtkSignalFunc)ProxyToggled, this);

	hbox8 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox8", hbox8);
	gtk_widget_show (hbox8);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox8, FALSE, TRUE, 2);

	GtkWidget* l = gtk_label_new ("Proxy Server: ");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "l", l);
	gtk_widget_show (l);
	gtk_box_pack_start (GTK_BOX (hbox8), l, FALSE, TRUE, 2);

	proxyServer = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "proxyServer", proxyServer);
	gtk_widget_show (proxyServer);
	gtk_box_pack_start (GTK_BOX (hbox8), proxyServer, FALSE, TRUE, 2);
	gtk_widget_set_sensitive(proxyServer, 0);

	hbox9 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox9", hbox9);
	gtk_widget_show (hbox9);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox9, FALSE, TRUE, 2);

	l = gtk_label_new ("Proxy Port: ");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "l", l);
	gtk_widget_show (l);
	gtk_box_pack_start (GTK_BOX (hbox9), l, FALSE, TRUE, 2);

	proxyPort = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "proxyPort", proxyPort);
	gtk_widget_show (proxyPort);
	gtk_box_pack_start (GTK_BOX (hbox9), proxyPort, FALSE, TRUE, 2);
	gtk_widget_set_sensitive(proxyPort, 0);

	frame5 = gtk_frame_new ((gchar*)NULL);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "frame5", frame5);
	gtk_widget_show (frame5);
	gtk_container_add (GTK_CONTAINER (notebook1), frame5);

	vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox1", vbox1);
	gtk_widget_show (vbox6);
	gtk_container_add (GTK_CONTAINER (frame5), vbox6);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox1", hbox1);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox1, FALSE, TRUE, 2);

	label5 = gtk_label_new ("Application for comparing files:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label5", label5);
	gtk_widget_show (label5);
	gtk_box_pack_start (GTK_BOX (hbox1), label5, FALSE, TRUE, 0);

	diffProgram = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "entry1", diffProgram);
	gtk_widget_show (diffProgram);
	gtk_box_pack_start (GTK_BOX (hbox1), diffProgram, TRUE, TRUE, 0);

	hbox7 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox7", hbox7);
	gtk_widget_show (hbox7);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox7, FALSE, TRUE, 2);

	label6 = gtk_label_new ("Application for comparing directories:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label6", label6);
	gtk_widget_show (label6);
	gtk_box_pack_start (GTK_BOX (hbox7), label6, FALSE, TRUE, 0);

	projectDiffProgram = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "entry1", projectDiffProgram);
	gtk_widget_show (projectDiffProgram);
	gtk_box_pack_start (GTK_BOX (hbox7), projectDiffProgram, TRUE, TRUE, 0);

	hbox4 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox4", hbox4);
	gtk_widget_show (hbox4);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox4, FALSE, TRUE, 2);

	editLabel = gtk_label_new ("Application for editing files:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "editLabel", editLabel);
	gtk_widget_show (editLabel);
	gtk_box_pack_start (GTK_BOX (hbox4), editLabel, FALSE, TRUE, 0);

	editProgram = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "entry2", editProgram);
	gtk_widget_show (editProgram);
	gtk_box_pack_start (GTK_BOX (hbox4), editProgram, TRUE, TRUE, 0);

	hbox10 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox10", hbox10);
	gtk_widget_show (hbox10);
	gtk_box_pack_start (GTK_BOX (vbox6), hbox10, FALSE, TRUE, 2);

	label1 = gtk_label_new ("Application for merging files:");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "editLabel", label1);
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (hbox10), label1, FALSE, TRUE, 0);

	mergeProgram = gtk_entry_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "entry2", mergeProgram);
	gtk_widget_show (mergeProgram);
	gtk_box_pack_start (GTK_BOX (hbox10), mergeProgram, TRUE, TRUE, 0);

	frame7 = gtk_frame_new ((gchar*)NULL);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "frame7", frame7);
	gtk_widget_show (frame7);
	gtk_container_add (GTK_CONTAINER (notebook1), frame7);

	vbox9 = gtk_vbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "vbox9", vbox9);
	gtk_widget_show (vbox9);
	gtk_container_add (GTK_CONTAINER (frame7), vbox9);

	GtkWidget* f = gtk_frame_new((gchar*) NULL);
	gtk_widget_show(f);
	gtk_box_pack_start(GTK_BOX(vbox9), f, FALSE, TRUE, 0);

	GtkWidget* vb9 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vb9);
	gtk_container_add(GTK_CONTAINER(f), vb9);
	
	GtkWidget* l1 = gtk_label_new("Check a command, and an \"options\" dialog will be displayed each");
	gtk_widget_show(l1);
	gtk_box_pack_start(GTK_BOX(vb9), l1, FALSE, TRUE, 0);

	GtkWidget* l2 = gtk_label_new("time you execute that command.  Clear the checkbox and the");
	gtk_widget_show(l2);
	gtk_box_pack_start(GTK_BOX(vb9), l2, FALSE, TRUE, 0);
	
	GtkWidget* l3 = gtk_label_new("\"options\" dialog will only be displayed when you hold the Shift key");
	gtk_widget_show(l3);
	gtk_box_pack_start(GTK_BOX(vb9), l3, FALSE, TRUE, 0);

	GtkWidget* l4 = gtk_label_new("down.");
	gtk_widget_show(l4);
	gtk_box_pack_start(GTK_BOX(vb9), l4, FALSE, TRUE, 0);

	hbox15 = gtk_hbox_new (FALSE, 0);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbox15", hbox15);
	gtk_widget_show (hbox15);
	gtk_box_pack_start (GTK_BOX (vbox9), hbox15, FALSE, TRUE, 0);

	vbox10 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox10);
	gtk_box_pack_start(GTK_BOX(hbox15), vbox10, FALSE, TRUE, 10);

	GtkWidget* l5 = gtk_label_new("Commands used on files:");
	gtk_widget_show(l5);
	gtk_box_pack_start(GTK_BOX(vbox10), l5, FALSE, TRUE, 0);
	
	frame8 = gtk_frame_new((gchar*) NULL);
	gtk_widget_show(frame8);
	gtk_box_pack_start(GTK_BOX(vbox10), frame8, FALSE, TRUE, 0);

	GtkWidget* vb = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vb);
	gtk_container_add(GTK_CONTAINER(frame8), vb);
	
	fileGetLatest = gtk_check_button_new_with_label("Get Latest Version");
	gtk_widget_show(fileGetLatest);
	gtk_box_pack_start(GTK_BOX(vb), fileGetLatest, FALSE, TRUE, 0);

	fileCheckout = gtk_check_button_new_with_label("Check Out");
	gtk_widget_show(fileCheckout);
	gtk_box_pack_start(GTK_BOX(vb), fileCheckout, FALSE, TRUE, 0);

	fileUndoCheckout = gtk_check_button_new_with_label("Undo Check Out");
	gtk_widget_show(fileUndoCheckout);
	gtk_box_pack_start(GTK_BOX(vb), fileUndoCheckout, FALSE, TRUE, 0);

	fileEditAndView = gtk_check_button_new_with_label("Edit and View");
	gtk_widget_show(fileEditAndView);
	gtk_box_pack_start(GTK_BOX(vb), fileEditAndView, FALSE, TRUE, 0);

	vbox11 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox11);
	gtk_box_pack_start(GTK_BOX(hbox15), vbox11, FALSE, TRUE, 10);

	GtkWidget* l6 = gtk_label_new("Commands used on Projects:");
	gtk_widget_show(l6);
	gtk_box_pack_start(GTK_BOX(vbox11), l6, FALSE, TRUE, 0);
	
	frame9 = gtk_frame_new((gchar*) NULL);
	gtk_widget_show(frame9);
	gtk_box_pack_start(GTK_BOX(vbox11), frame9, FALSE, TRUE, 0);

	GtkWidget* vb2 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vb2);
	gtk_container_add(GTK_CONTAINER(frame9), vb2);
	
	projectGetLatest = gtk_check_button_new_with_label("Get Latest Version");
	gtk_widget_show(projectGetLatest);
	gtk_box_pack_start(GTK_BOX(vb2), projectGetLatest, FALSE, TRUE, 0);

	projectCheckout = gtk_check_button_new_with_label("Check Out");
	gtk_widget_show(projectCheckout);
	gtk_box_pack_start(GTK_BOX(vb2), projectCheckout, FALSE, TRUE, 0);

	projectUndoCheckout = gtk_check_button_new_with_label("Undo Check Out");
	gtk_widget_show(projectUndoCheckout);
	gtk_box_pack_start(GTK_BOX(vb2), projectUndoCheckout, FALSE, TRUE, 0);

	projectDeploy = gtk_check_button_new_with_label("Web Deploy");
	gtk_widget_show(projectDeploy);
	gtk_box_pack_start(GTK_BOX(vb2), projectDeploy, FALSE, TRUE, 0);

	General = gtk_label_new ("General");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "General", General);
	gtk_widget_show (General);
	set_notebook_tab (notebook1, 0, General);

	label14 = gtk_label_new("Project");
	gtk_widget_show(label14);
	set_notebook_tab(notebook1, 1, label14);
	
	label2 = gtk_label_new ("View");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label2", label2);
	gtk_widget_show (label2);
	set_notebook_tab (notebook1, 2, label2);

	lable3 = gtk_label_new ("File");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "lable3", lable3);
	gtk_widget_show (lable3);
	set_notebook_tab (notebook1, 3, lable3);

	label4 = gtk_label_new ("HTTP Proxy");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label4", label4);
	gtk_widget_show (label4);
	set_notebook_tab (notebook1, 4, label4);
  
	label13 = gtk_label_new ("External Programs");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label4", label13);
	gtk_widget_show (label13);
	set_notebook_tab (notebook1, 5, label13);
	
	CommandDlgs = gtk_label_new ("Command Dialogs");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "label4", CommandDlgs);
	gtk_widget_show (CommandDlgs);
	set_notebook_tab (notebook1, 6, CommandDlgs);

	dialog_action_area1 = GTK_DIALOG (m_pDialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);
}

COptionsDlg::~COptionsDlg()
{
	gtk_widget_destroy(m_pDialog);
}
