/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <gtk/gtk.h>

#include "SearchDlg.h"

static CSearchDlg* m_pDlg = (CSearchDlg*)NULL;

static void radioClicked(gpointer foo, GtkWidget* w)
{
	if (!GTK_TOGGLE_BUTTON(w)->active)
		return;
	
	if (w == m_pDlg->checkedOutTo)
	{
		if (m_pDlg->searchString)
			gtk_widget_set_sensitive(m_pDlg->searchString, TRUE);
		
		m_pDlg->bSearchAll = 0;
		m_pDlg->bSearchString = 1;
	}
	else if (w == m_pDlg->AllFiles)
	{
		if (m_pDlg->searchString)
			gtk_widget_set_sensitive(m_pDlg->searchString, FALSE);
		m_pDlg->bSearchAll = 1;
		m_pDlg->bSearchString = 0;
	}
	else if (w == m_pDlg->currentProj)
	{
		m_pDlg->bSearchCurProj = 1;
		m_pDlg->bSearchCurProjRecursively = 0;
		m_pDlg->bSearchAllProjects = 0;
	}
	else if (w == m_pDlg->currentProjRecursive)
	{
		m_pDlg->bSearchCurProj = 0;
		m_pDlg->bSearchCurProjRecursively = 1;
		m_pDlg->bSearchAllProjects = 0;
	}
	else if (w == m_pDlg->allProjects)
	{
		m_pDlg->bSearchCurProj = 0;
		m_pDlg->bSearchCurProjRecursively = 0;
		m_pDlg->bSearchAllProjects = 1;
	}
}

CSearchDlg::CSearchDlg(char* title) : CVermDlg(title, MB_OK | MB_CANCEL)
{
	m_pDlg = this;
	
	bSearchAll = 0;
	bSearchString = 1;

	bSearchCurProj = 0;
	bSearchCurProjRecursively = 1;
	bSearchAllProjects = 0;

	rgroup1_group = (GSList*)NULL;
	rgroup2_group = (GSList*)NULL;

	searchString = (GtkWidget*)NULL;
	
	vbox1 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "vbox1", vbox1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox2);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "vbox2", vbox2,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

	frame1 = gtk_frame_new ("Status Search");
	gtk_widget_ref (frame1);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "frame1", frame1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame1);
	gtk_box_pack_start (GTK_BOX (vbox2), frame1, TRUE, TRUE, 0);

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox3);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "vbox3", vbox3,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox3);
	gtk_container_add (GTK_CONTAINER (frame1), vbox3);
	gtk_container_set_border_width (GTK_CONTAINER (vbox3), 5);

	AllFiles = gtk_radio_button_new_with_label (rgroup1_group, "Display all checked out files");
	rgroup1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (AllFiles));
	gtk_widget_ref (AllFiles);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "AllFiles", AllFiles,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (AllFiles);
	gtk_box_pack_start (GTK_BOX (vbox3), AllFiles, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (AllFiles), 5);
	gtk_signal_connect_object (GTK_OBJECT (AllFiles), "toggled",
                               GTK_SIGNAL_FUNC(radioClicked),
                               (GtkObject*)NULL);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "hbox1", hbox1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox1, FALSE, FALSE, 0);

	checkedOutTo = gtk_radio_button_new_with_label (rgroup1_group, "Display files checked out to:");
	hbox1_group = gtk_radio_button_group (GTK_RADIO_BUTTON (checkedOutTo));
	gtk_widget_ref (checkedOutTo);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "checkedOutTo", checkedOutTo,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (checkedOutTo);
	gtk_box_pack_start (GTK_BOX (hbox1), checkedOutTo, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (checkedOutTo), 5);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkedOutTo), TRUE);
	gtk_signal_connect_object (GTK_OBJECT (checkedOutTo), "toggled",
                               GTK_SIGNAL_FUNC(radioClicked),
                               (GtkObject*)NULL);

	searchString = gtk_entry_new ();
	gtk_widget_ref (searchString);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "searchString", searchString,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (searchString);
	gtk_box_pack_start (GTK_BOX (hbox1), searchString, TRUE, TRUE, 0);
	
	frame2 = gtk_frame_new ("Search Area");
	gtk_widget_ref (frame2);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "frame2", frame2,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame2);
	gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox4);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "vbox4", vbox4,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox4);
	gtk_container_add (GTK_CONTAINER (frame2), vbox4);
	gtk_container_set_border_width (GTK_CONTAINER (vbox4), 5);

	currentProj = gtk_radio_button_new_with_label (rgroup2_group, "Search in current project");
	rgroup2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (currentProj));
	gtk_widget_ref (currentProj);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "currentProj", currentProj,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (currentProj);
	gtk_box_pack_start (GTK_BOX (vbox4), currentProj, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (currentProj), 5);
	gtk_signal_connect_object (GTK_OBJECT (currentProj), "toggled",
                               GTK_SIGNAL_FUNC(radioClicked),
                               (GtkObject*)NULL);

	currentProjRecursive = gtk_radio_button_new_with_label (rgroup2_group, "Search in current project and all subprojects");
	rgroup2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (currentProjRecursive));
	gtk_widget_ref (currentProjRecursive);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "currentProjRecursive", currentProjRecursive,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (currentProjRecursive);
	gtk_box_pack_start (GTK_BOX (vbox4), currentProjRecursive, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (currentProjRecursive), 5);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (currentProjRecursive), TRUE);
	gtk_signal_connect_object (GTK_OBJECT (currentProjRecursive), "toggled",
                               GTK_SIGNAL_FUNC(radioClicked),
                               (GtkObject*)NULL);

	allProjects = gtk_radio_button_new_with_label (rgroup2_group, "Search in all projects");
	rgroup2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (allProjects));
	gtk_widget_ref (allProjects);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "allProjects", allProjects,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (allProjects);
	gtk_box_pack_start (GTK_BOX (vbox4), allProjects, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (allProjects), 5);
	gtk_signal_connect_object (GTK_OBJECT (allProjects), "toggled",
                               GTK_SIGNAL_FUNC(radioClicked),
                               (GtkObject*)NULL);
}

CSearchDlg::~CSearchDlg(void)
{
	gtk_widget_destroy(m_pDialog);
}
