/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <gtk/gtk.h>
#include <sys/stat.h>

#include "SearchResultsDlg.h"
#include "VermProtocolMessage.h"
#include "VermUtils.h"
#include "VermFrameWnd.h"
#include "VermProjectView.h"
#include "Vermie.h"
#include "VermTreeUtils.h"
#include "VermSocket.h"
#include "YesNoDlg.h"

#include "CheckinDlg.h"

static void ColumnClicked(GtkCList* clist, int column, gpointer)
{
	if (column == clist->sort_column)
    {
		if (clist->sort_type == GTK_SORT_ASCENDING)
			clist->sort_type = GTK_SORT_DESCENDING;
		else
			clist->sort_type = GTK_SORT_ASCENDING;
    }
	else
	{
		clist->sort_type = GTK_SORT_ASCENDING;
		gtk_clist_set_sort_column (clist, column);
		Vermie::m_nSortColumn = column;
	}

	gtk_clist_sort(clist);
}

static int SortCompare(GtkCList* list, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row1 = (GtkCListRow *) p1;
	GtkCListRow *row2 = (GtkCListRow *) p2;

	char *t1 = GTK_CELL_TEXT(row1->cell[list->sort_column])->text;
	char *t2 = GTK_CELL_TEXT(row2->cell[list->sort_column])->text;

	return strcmp(t1, t2);
}

static void KillDialog(GtkWidget*, CSearchResultsDlg* pDlg)
{
	if (!pDlg->m_bBeingDestroyed)
		pDlg->m_pParent->SearchResultsDlgGone();
}

static void View(GtkWidget *widget, gpointer data)
{
	CSearchResultsDlg* pDlg = (CSearchResultsDlg*) data;

	GList* sel = GTK_CLIST(pDlg->m_pList)->selection;

	if (!sel)
		return;

	int row = (int) sel->data;
	
	char* text;
	
	gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), row, 1, &text);

	String strProject = text;

	int pos = strProject.ReverseFind('/');

	if (pos < 0)
		return;

	String strFile = strProject.after(pos);
	strProject = strProject.before(pos);

	pDlg->m_pParent->ViewFile(strProject, strFile);
}

static void Diff(GtkWidget *widget, gpointer data)
{
	CSearchResultsDlg* pDlg = (CSearchResultsDlg*) data;

	GList* sel = GTK_CLIST(pDlg->m_pList)->selection;

	if (!sel)
		return;

	int row = (int) sel->data;
	
	char* text;
	
	gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), row, 1, &text);

	String strProject = text;

	int pos = strProject.ReverseFind('/');

	if (pos < 0)
		return;

	String strFile = strProject.after(pos);
	strProject = strProject.before(pos);

	String strWorkingDir = GetWorkingDirectory(pDlg->m_pParent->m_pProjectView->GetRootItem(), strProject.chars());

	pDlg->m_pParent->DiffFile(strWorkingDir, strProject, strFile);
}

static void Checkin(GtkWidget *widget, gpointer data)
{
	CSearchResultsDlg* pDlg = (CSearchResultsDlg*) data;

	GList* sel = GTK_CLIST(pDlg->m_pList)->selection;

	if (!sel)
		return;

	CCheckinDlg dlg;

	if (dlg.DoModal() == IDCANCEL)
		return;

	String strComment = gtk_editable_get_chars(GTK_EDITABLE(dlg.comment), 0, -1);
	int bRemoveLocalCopy = GTK_TOGGLE_BUTTON(dlg.removeLocalCopy)->active;
	int bKeepCheckedOut = GTK_TOGGLE_BUTTON(dlg.keepCheckedOut)->active;

	while (sel)
	{
		int row = (int) sel->data;

		char* text;
	
		gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), row, 1, &text);

		String strProject = text;

		int pos = strProject.ReverseFind('/');

		if (pos < 0)
			return;

		String strFile = strProject.after(pos);
		strProject = strProject.before(pos);

		GtkWidget* pTree = pDlg->m_pParent->m_pProjectView->GetRootItem();

		CVermTreeNode* pNode = GetTreeNodeFromProject(pTree, strProject.chars());

		String strWorkingDir = GetWorkingDirectory(pDlg->m_pParent->m_pProjectView->GetRootItem(), strProject.chars());

	
		pDlg->m_pParent->CheckInFile(pNode, strProject, strWorkingDir, strFile,
									 strComment, bRemoveLocalCopy, bKeepCheckedOut, 1);

		sel = sel->next;

		gtk_clist_remove(GTK_CLIST(pDlg->m_pList), row);
	}		
}

static void UndoCheckout(GtkWidget *widget, gpointer data)
{
	CSearchResultsDlg* pDlg = (CSearchResultsDlg*) data;

	GList* sel = GTK_CLIST(pDlg->m_pList)->selection;

	int modstate = 0;
	
	while (sel)
	{
		int row = (int) sel->data;

		char* text;
	
		gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), row, 1, &text);

		String strProject = text;

		int pos = strProject.ReverseFind('/');

		if (pos < 0)
			return;

		String strFile = strProject.after(pos);
		strProject = strProject.before(pos);

		if (strProject == "$")
			strProject = "$/";

		GtkWidget* pRoot = pDlg->m_pParent->m_pProjectView->GetRootItem();

		String strWorkingDir = GetWorkingDirectory(pRoot, strProject.chars());

		CVermTreeNode* pNode = GetTreeNodeFromProject(pRoot, strProject.chars());

		short bDontGet = Vermie::m_bDontFetchFileOnUndoCheckout || strWorkingDir.empty();

		if (modstate != IDYESTOALL && modstate != IDNOTOALL && IsFileModified(pNode, strFile, strWorkingDir) != FILE_NOT_MODIFIED)
		{
			String s(strFile);
			s += " may have been modified, lose changes?";
			
			CYesNoDlg dlg(s);

			modstate = dlg.DoModal();

			if (modstate == IDCANCEL)
				return;
		}

		pDlg->m_pParent->SetWaitState(1);

		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_UNLOCK_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

		pDlg->m_pParent->m_pSocket->SendMessage(pMessage);

		// get a new copy if necessary

		if (bDontGet)
			continue;


		if (modstate == IDYES || modstate == IDYESTOALL)
		{
			pDlg->m_pParent->SetWaitState(1);
	
			CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();

			pNewMessage->m_number = PM_GET_FILE;

			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

			pDlg->m_pParent->m_pSocket->SendMessage(pNewMessage);
		}

		sel = sel->next;

		gtk_clist_remove(GTK_CLIST(pDlg->m_pList), row);
	}		
}

static void History(GtkWidget *widget, gpointer data)
{
	CSearchResultsDlg* pDlg = (CSearchResultsDlg*) data;

	GList* sel = GTK_CLIST(pDlg->m_pList)->selection;

	if (!sel)
		return;

	int row = (int) sel->data;
	
	char* text;
	
	gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), row, 1, &text);

	String strProject = text;

	int pos = strProject.ReverseFind('/');

	if (pos < 0)
		return;

	String strFile = strProject.after(pos);
	strProject = strProject.before(pos);

	pDlg->m_pParent->GetHistory(strProject, strFile);
}

static void select_row(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data)
{
	CSearchResultsDlg* pDlg = (CSearchResultsDlg*) data;

	GList* p = GTK_CLIST(pDlg->m_pList)->selection;

	// only 1 selected so far
	if (!p)
	{
		gtk_widget_set_sensitive(pDlg->m_pDiff, FALSE);
		gtk_widget_set_sensitive(pDlg->m_pView, FALSE);
		gtk_widget_set_sensitive(pDlg->m_pCheckin, FALSE);
		gtk_widget_set_sensitive(pDlg->m_pUndoCheckout, FALSE);
		gtk_widget_set_sensitive(pDlg->m_pHistory, FALSE);
	}
	else if (!p->next)
	{
		char* user = NULL;
		char* file = NULL;
		
		gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), (int)p->data, 0, &user);
		gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), (int)p->data, 1, &file);

		String sUser = user;
		
		int bSameUser = !fcompare(sUser, pDlg->m_pParent->m_username);
		String strFile = file;
		
		int slash = strFile.ReverseFind('/');
		
		String strProject = strFile.before(slash);
		
		if (strProject == "$")
			strProject = "$/";

		strFile = strFile.after(slash);

		// see if the file exists locally - if it doesn't, don't enable view/diff/checkin
		GtkWidget* cTreeCtrl = pDlg->m_pParent->m_pProjectView->GetRootItem();
		
		String strWorkingDir = GetWorkingDirectory(cTreeCtrl, strProject.chars());
		
		if (!strWorkingDir.empty())
		{
			String sFile = strWorkingDir;
			sFile += "/";
			sFile += strFile;
			struct stat st;
			
			gtk_widget_set_sensitive(pDlg->m_pView, TRUE);
			
			if (!stat(sFile.chars(), &st))
			{
				if (!Vermie::m_strDiffProgram.empty())
					gtk_widget_set_sensitive(pDlg->m_pDiff, TRUE);
				else
					gtk_widget_set_sensitive(pDlg->m_pDiff, FALSE);
				
				if (bSameUser)
					gtk_widget_set_sensitive(pDlg->m_pCheckin, TRUE);
				else
					gtk_widget_set_sensitive(pDlg->m_pCheckin, FALSE);
			}
			else
			{
				gtk_widget_set_sensitive(pDlg->m_pDiff, FALSE);
				gtk_widget_set_sensitive(pDlg->m_pCheckin, FALSE);
			}
		}
		else
		{
			gtk_widget_set_sensitive(pDlg->m_pDiff, FALSE);
			gtk_widget_set_sensitive(pDlg->m_pCheckin, FALSE);
		}
		
		gtk_widget_set_sensitive(pDlg->m_pHistory, TRUE);
		
		if (bSameUser)
			gtk_widget_set_sensitive(pDlg->m_pUndoCheckout, TRUE);
		else
			gtk_widget_set_sensitive(pDlg->m_pUndoCheckout, FALSE);
		
	}
	else
	{
		int bSameName = 1;
		int bExists = 1;
		
		while (p)
		{
			char* user = NULL;
			char* file = NULL;
			
			gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), (int)p->data, 0, &user);
			gtk_clist_get_text(GTK_CLIST(pDlg->m_pList), (int)p->data, 1, &file);
			
			String sUser = user;
			
			if (fcompare(sUser, pDlg->m_pParent->m_username))
				bSameName = 0;
			
			String strFile = file;
			
			int slash = strFile.ReverseFind('/');
			
			String strProject = strFile.before(slash);
			
			if (strProject == "$")
				strProject = "$/";
			
			strFile = strFile.after(slash);
			
			// see if the file exists locally - if it doesn't, don't enable view/diff/checkin
			GtkWidget* cTreeCtrl = pDlg->m_pParent->m_pProjectView->GetRootItem();
			
			String strWorkingDir = GetWorkingDirectory(cTreeCtrl, strProject.chars());
			
			if (!strWorkingDir.empty())
			{
				String sFile = strWorkingDir;
				sFile += "/";
				sFile += strFile;
				struct stat st;
				
				gtk_widget_set_sensitive(pDlg->m_pView, TRUE);
				
				if (stat(sFile.chars(), &st))
					bExists = 0;
			}

			p = p->next;
		}

		gtk_widget_set_sensitive(pDlg->m_pDiff, FALSE);
		gtk_widget_set_sensitive(pDlg->m_pView, FALSE);
		gtk_widget_set_sensitive(pDlg->m_pCheckin, bSameName && bExists);
		gtk_widget_set_sensitive(pDlg->m_pUndoCheckout, bSameName);
		gtk_widget_set_sensitive(pDlg->m_pHistory, FALSE);
	}
}

CSearchResultsDlg::CSearchResultsDlg(char* title, CVermProtocolMessage* pMessage, CVermFrameWnd* pParent)
{
	m_pParent = pParent;
	m_bBeingDestroyed = 0;
	
	GtkWidget *scrolledwindow1;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *hbuttonbox1;
	GtkWidget *button5;
	GtkWidget *dialog_action_area1;

	m_pDialog = gtk_dialog_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "m_pDialog", m_pDialog);
	gtk_window_set_title (GTK_WINDOW (m_pDialog), "Search Results");
	gtk_container_border_width(GTK_CONTAINER(m_pDialog), 5);
	gtk_widget_realize(m_pDialog);
	gtk_window_set_policy (GTK_WINDOW (m_pDialog), TRUE, TRUE, FALSE);

	GtkWidget* vbox1 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "vbox1", vbox1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	
	GtkWidget* hbox1 = gtk_hbox_new(0, 0);
	gtk_widget_show(hbox1);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, TRUE, TRUE, 5);
	
	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow1);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "scrolledm_pDialog", scrolledwindow1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow1);
	gtk_container_add (GTK_CONTAINER (hbox1), scrolledwindow1);
	gtk_widget_set_usize(scrolledwindow1, 400, 400);

	GtkWidget* vbox2 = gtk_vbutton_box_new();
	gtk_widget_show(vbox2);
	gtk_box_pack_start(GTK_BOX(hbox1), vbox2, TRUE, TRUE, 5);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(vbox2), 5);
	gtk_vbutton_box_set_layout_default(GTK_BUTTONBOX_START);
	
	m_pView = gtk_button_new_with_label("View");
	gtk_widget_show(m_pView);
	gtk_container_add(GTK_CONTAINER(vbox2), m_pView);
//	gtk_box_pack_start(GTK_BOX(vbox2), m_pView, TRUE, FALSE, 5);
	gtk_widget_set_sensitive(m_pView, FALSE);
	gtk_signal_connect (GTK_OBJECT (m_pView), "clicked",
						GTK_SIGNAL_FUNC (View), (gpointer) this);
	
	m_pCheckin = gtk_button_new_with_label("Check In ...");
	gtk_widget_show(m_pCheckin);
	gtk_container_add(GTK_CONTAINER(vbox2), m_pCheckin);
//	gtk_box_pack_start(GTK_BOX(vbox2), m_pCheckin, TRUE, FALSE, 5);
	gtk_widget_set_sensitive(m_pCheckin, FALSE);
	gtk_signal_connect (GTK_OBJECT (m_pCheckin), "clicked",
						GTK_SIGNAL_FUNC (Checkin), (gpointer) this);
	
	m_pUndoCheckout = gtk_button_new_with_label("Undo Check Out ...");
	gtk_widget_show(m_pUndoCheckout);
	gtk_container_add(GTK_CONTAINER(vbox2), m_pUndoCheckout);
//	gtk_box_pack_start(GTK_BOX(vbox2), m_pUndoCheckout, TRUE, FALSE, 5);
	gtk_widget_set_sensitive(m_pUndoCheckout, FALSE);
	gtk_signal_connect (GTK_OBJECT (m_pUndoCheckout), "clicked",
						GTK_SIGNAL_FUNC (UndoCheckout), (gpointer) this);
	
	m_pDiff = gtk_button_new_with_label("Diff");
	gtk_widget_show(m_pDiff);
	gtk_container_add(GTK_CONTAINER(vbox2), m_pDiff);
//	gtk_box_pack_start(GTK_BOX(vbox2), m_pDiff, TRUE, FALSE, 5);
	gtk_widget_set_sensitive(m_pDiff, FALSE);
	gtk_signal_connect (GTK_OBJECT (m_pDiff), "clicked",
						GTK_SIGNAL_FUNC (Diff), (gpointer) this);
	
	m_pHistory = gtk_button_new_with_label("History ...");
	gtk_widget_show(m_pHistory);
	gtk_container_add(GTK_CONTAINER(vbox2), m_pHistory);
//	gtk_box_pack_start(GTK_BOX(vbox2), m_pHistory, TRUE, FALSE, 5);
	gtk_widget_set_sensitive(m_pHistory, FALSE);
	gtk_signal_connect (GTK_OBJECT (m_pHistory), "clicked",
						GTK_SIGNAL_FUNC (History), (gpointer) this);
	
	m_pList = gtk_clist_new (2);
	gtk_widget_ref (m_pList);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "m_pList", m_pList,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (m_pList);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), m_pList);
	gtk_clist_set_column_width (GTK_CLIST (m_pList), 0, 80);
	gtk_clist_set_column_width (GTK_CLIST (m_pList), 1, 310);
	gtk_clist_column_titles_show (GTK_CLIST (m_pList));
	gtk_clist_set_selection_mode(GTK_CLIST(m_pList), GTK_SELECTION_EXTENDED);
	gtk_clist_set_compare_func(GTK_CLIST(m_pList), (GtkCListCompareFunc)SortCompare);
	gtk_clist_set_sort_column(GTK_CLIST(m_pList), 0);
	gtk_clist_set_sort_type(GTK_CLIST(m_pList), GTK_SORT_ASCENDING);
	gtk_widget_set_events(m_pList, GDK_BUTTON_PRESS);

	gtk_signal_connect(GTK_OBJECT(m_pList), "select_row", GTK_SIGNAL_FUNC(select_row), this);
	gtk_signal_connect(GTK_OBJECT(m_pList), "unselect_row", GTK_SIGNAL_FUNC(select_row), this);
	gtk_signal_connect(GTK_OBJECT(m_pList), "click_column", (GtkSignalFunc) ColumnClicked, this);
	
	label1 = gtk_label_new ("User");
	gtk_widget_ref (label1);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "label1", label1,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label1);
	gtk_clist_set_column_widget (GTK_CLIST (m_pList), 0, label1);

	label2 = gtk_label_new ("File");
	gtk_widget_ref (label2);
	gtk_object_set_data_full (GTK_OBJECT (m_pDialog), "label2", label2,
							  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label2);
	gtk_clist_set_column_widget (GTK_CLIST (m_pList), 1, label2);

	dialog_action_area1 = GTK_DIALOG (m_pDialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbuttonbox1", hbuttonbox1);
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);
	
	button5 = gtk_button_new_with_label ("OK");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "button5", button5);
	gtk_widget_show (button5);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button5);

	gtk_signal_connect(GTK_OBJECT(button5), "clicked",
					   (GtkSignalFunc)KillDialog, this);
	
	m_destroySig = gtk_signal_connect(GTK_OBJECT(m_pDialog), "destroy",
									  (GtkSignalFunc)KillDialog, this);

	int bError = 0;

	int nPos = 0;
	int nIndex = 0;

	char* pMessageBody = pMessage->m_body;

	while (nPos < pMessage->m_bodySize)
	{
		char *junk[2] = {"",""};

		// User name
		char* pszName = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (pszName)
		{
			junk[0] = pszName;
		}
		else
		{
			if (bError)
				break;
		}

		// File

		char* pszFile = ParseNameFromBuffer(pMessageBody, &nPos, pMessage->m_bodySize - 1, bError);

		if (!pszFile)
			break;

		String strFile = pszFile;
		
		if (!strncmp(pszFile, "$//", 3))
		{
			strFile = "$/";
			pszFile += 3;
			strFile += pszFile;
		}

		
		junk[1] = strFile.chars();
		
		gtk_clist_append(GTK_CLIST(m_pList), junk);

		nIndex++;
	}
	
}

CSearchResultsDlg::~CSearchResultsDlg(void)
{
	m_bBeingDestroyed = 1;
	
	gtk_signal_disconnect(GTK_OBJECT(m_pDialog), m_destroySig);
	
	gtk_widget_destroy(m_pDialog);
}
