/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <gtk/gtk.h>

#include "Vermie.h"
#include "VermTreeUtils.h"
#include "VermFrameWnd.h"
#include "VermTreeNode.h"
#include "VermTreeLeaf.h"
#include "VermProtocolMessage.h"
#include "VermSocket.h"
#include "ShareDlg.h"
#include "CreateProjectDlg.h"
#include "AfxMessageBox.h"

static void ListChildSelected(GtkWidget* list, GtkWidget*, CShareDlg* pDlg)
{
	if (list == GTK_WIDGET(pDlg->files))
	{
		if (pDlg->add)
			gtk_widget_set_sensitive(pDlg->add, 1);
	}
	else
	{
		if (pDlg->remove)
			gtk_widget_set_sensitive(pDlg->remove, 1);
	}
}

static void ListChildUnSelected(GtkWidget* list, GtkWidget*, CShareDlg* pDlg)
{
	if (list == GTK_WIDGET(pDlg->files))
	{
		if (pDlg->files && !GTK_LIST(pDlg->files)->selection)
			gtk_widget_set_sensitive(pDlg->add, 0);
	}
	else
	{
		if (pDlg->filesToAdd && !GTK_LIST(pDlg->filesToAdd)->selection)
			gtk_widget_set_sensitive(pDlg->remove, 0);
	}
}

static void AddClicked(GtkWidget*, CShareDlg* pDlg)
{
	GList* pFiles = GTK_LIST(pDlg->files)->selection;
	GList* pClearList = NULL;
	
	if (!pFiles)
		return;

	
	while (pFiles)
	{
		String* strNode = new String;

		GetPathFromTreeItem(pDlg->m_hCurSelItem, *strNode);
		
		GtkWidget *pItem = gtk_list_item_new_with_label(GetItemText((GtkWidget*)pFiles->data));
		gtk_widget_show(pItem);
		
		gtk_object_set_user_data(GTK_OBJECT(pItem), strNode);
		
		gtk_container_add(GTK_CONTAINER(pDlg->filesToAdd), pItem);

		pClearList = g_list_prepend(pClearList, pFiles->data);
		pFiles = pFiles->next;
	}

	pClearList = g_list_reverse(pClearList);
	
	gtk_list_remove_items(GTK_LIST(pDlg->files), pClearList);

	gtk_widget_set_sensitive(pDlg->ok, TRUE);

	GList* children = gtk_container_children(GTK_CONTAINER(pDlg->files));

	if (children)
		g_list_free(children);
	else
		gtk_widget_set_sensitive(pDlg->add, FALSE);

	gtk_widget_set_sensitive(pDlg->recursive, FALSE);
}

static void RemoveClicked(GtkWidget*, CShareDlg* pDlg)
{
	GList* pFiles = GTK_LIST(pDlg->filesToAdd)->selection;
	GList* pClearList = NULL;

	if (!pFiles)
		return;
	
	while (pFiles)
	{
		GtkWidget* pItem = gtk_list_item_new_with_label(GetItemText((GtkWidget*)pFiles->data));
		gtk_widget_show(pItem);
		
		gtk_container_add(GTK_CONTAINER(pDlg->files), pItem);

		pClearList = g_list_prepend(pClearList, pFiles->data);
		
		pFiles = pFiles->next;
	}

	pClearList = g_list_reverse(pClearList);
	
	gtk_list_remove_items(GTK_LIST(pDlg->filesToAdd), pClearList);

	GList* children = gtk_container_children(GTK_CONTAINER(pDlg->filesToAdd));

	if (children)
		g_list_free(children);
	else
	{
		gtk_widget_set_sensitive(pDlg->recursive, TRUE);
		gtk_widget_set_sensitive(pDlg->ok, FALSE);
	}
}

static void SelectionChanged(GtkWidget*, CShareDlg* pDlg)
{
	GList* sel = GTK_TREE_SELECTION(pDlg->projects);

	if (!sel)
		return;
	
	pDlg->m_hCurSelItem = GTK_WIDGET(sel->data);
	
	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(pDlg->m_hCurSelItem));

	String strNode;
	
	GetPathFromTreeItem(pDlg->m_hCurSelItem, strNode);

	if (pNode->m_bVisited)
	{
		pDlg->AddFiles(strNode);
	}
	else
	{
		pDlg->m_pParent->ProjectItemSelected(strNode);
	}
}

void DialogDestroyed(GtkWidget*, CShareDlg* pDlg)
{
	if (!pDlg->m_bBeingDestroyed)
		pDlg->m_pParent->ShareDlgGone();
}

static void KillDialog(GtkWidget*, CShareDlg* pDlg)
{
	pDlg->m_pParent->ShareDlgGone();
}

static void OkPressed(GtkWidget*, CShareDlg* pDlg)
{
	GList* children = gtk_container_children(GTK_CONTAINER(pDlg->filesToAdd));

	if (!children)
	{
		CCreateProjectDlg dlg("Share Project", MB_OK|MB_CANCEL);

		String strSourceProject;

		GetPathFromTreeItem(pDlg->m_hCurSelItem, strSourceProject);

		if (GTK_TOGGLE_BUTTON(pDlg->recursive)->active)
		{
			int lenSource = strlen(strSourceProject.chars());
			int lenShareProj = strlen(pDlg->m_strShareProject.chars());

			if (lenShareProj >= lenSource &&
				!strncmp(strSourceProject.chars(), pDlg->m_strShareProject.chars(), lenSource))
			{
				String s("You may not recursively share a project into a child project");
				AfxMessageBox(s);
				return;
			}
		}

		int pos = strSourceProject.ReverseFind('/');

		String strProjName = strSourceProject.after(pos);

		gtk_entry_set_text(GTK_ENTRY(dlg.project), strProjName.chars());

		if (dlg.DoModal() == IDOK)
		{
			CVermProtocolMessage* pMessage = new CVermProtocolMessage();

			pMessage->m_number = PM_SHARE_PROJECT;

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strSourceProject));
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SHARE_PROJECT, pDlg->m_strShareProject));
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

			String strProject = gtk_entry_get_text(GTK_ENTRY(dlg.project));
			
			// user wants to call it something else ...
			if (fcompare(strProject, strProjName))
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_NEW_PROJECT_NAME, strProject));

			if (GTK_TOGGLE_BUTTON(pDlg->branchAfter)->active)
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_BRANCH_AFTER_SHARE, "yes"));

			if (GTK_TOGGLE_BUTTON(pDlg->recursive)->active)
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_RECURSIVE, "yes"));

			pDlg->m_pParent->SetWaitState(1);
			
			pDlg->m_pParent->m_pSocket->SendMessage(pMessage);
		}

	}
	else
	{
		while (children)
		{
			String* strNode = (String*)gtk_object_get_user_data(GTK_OBJECT(children->data));
			char* file = GetItemText(GTK_WIDGET(children->data));
			
			CVermProtocolMessage* pMessage = new CVermProtocolMessage();
			
			pMessage->m_number = PM_SHARE_FILE;
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, *strNode));
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, file));
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SHARE_PROJECT,
														pDlg->m_strShareProject));
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
			
			if (GTK_TOGGLE_BUTTON(pDlg->branchAfter)->active)
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_BRANCH_AFTER_SHARE, "yes"));

			pDlg->m_pParent->SetWaitState(TRUE);
			
			pDlg->m_pParent->m_pSocket->SendMessage(pMessage);
			
			children = g_list_remove_link(children, children);
			
			delete strNode;
		}
	}
	
	pDlg->m_pParent->ShareDlgGone();
}

CShareDlg::CShareDlg(CVermFrameWnd* pParent, GtkWidget* pTree, String& strProj)
{
	GtkWidget *dialog_vbox1;
	GtkWidget *table1;
	GtkWidget *dialog_action_area1;
	GtkWidget* scrollWin1;
	GtkWidget* scrollWin2;
	GtkWidget* scrollWin3;
	GtkWidget* hbuttonbox1;
	GtkWidget* button6;

	m_bBeingDestroyed = 0;
	
	m_strShareProject = strProj;
	
	m_hCurSelItem = NULL;
	
	m_pParent = pParent;
	
	m_pDialog = gtk_dialog_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "m_pDialog", m_pDialog);

	char title[2048];
	sprintf(title, "Share with %s", strProj.chars());
	
	gtk_window_set_title (GTK_WINDOW (m_pDialog), title);
	gtk_window_set_policy (GTK_WINDOW (m_pDialog), TRUE, TRUE, FALSE);
	m_destroySig = gtk_signal_connect(GTK_OBJECT(m_pDialog), "destroy",
									  GTK_SIGNAL_FUNC(DialogDestroyed), this);
	
	dialog_vbox1 = GTK_DIALOG (m_pDialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	table1 = gtk_table_new (6, 3, FALSE);
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "table1", table1);
	gtk_widget_show (table1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), table1, TRUE, TRUE, 0);

	scrollWin3 = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin3),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	
	gtk_widget_set_usize(scrollWin3, 200, 200);
	gtk_widget_show(scrollWin3);
	gtk_table_attach (GTK_TABLE (table1), scrollWin3, 0, 1, 6, 8,
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);
	
	files = gtk_list_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "files", files);
	gtk_widget_show (files);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin3), files);
	gtk_list_set_selection_mode (GTK_LIST (files), GTK_SELECTION_EXTENDED);
	gtk_signal_connect(GTK_OBJECT(files), "select_child",
					   GtkSignalFunc(ListChildSelected), this);
	gtk_signal_connect(GTK_OBJECT(files), "unselect_child",
					   GtkSignalFunc(ListChildUnSelected), this);

	add = gtk_button_new_with_label ("Add");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "add", add);
	gtk_widget_show (add);
	gtk_table_attach (GTK_TABLE (table1), add, 1, 2, 6, 7,
					  (GtkAttachOptions) 0, (GtkAttachOptions) 0, 5, 5);

	gtk_widget_set_sensitive(add, FALSE);
	gtk_signal_connect(GTK_OBJECT(add), "clicked",
					   GtkSignalFunc(AddClicked), this);
	
	remove = gtk_button_new_with_label ("Remove");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "remove", remove);
	gtk_widget_show (remove);
	gtk_table_attach (GTK_TABLE (table1), remove, 1, 2, 7, 8,
					  (GtkAttachOptions) 0, (GtkAttachOptions) 0, 5, 5);
	gtk_signal_connect(GTK_OBJECT(remove), "clicked",
					   GtkSignalFunc(RemoveClicked), this);

	gtk_widget_set_sensitive(remove, FALSE);
	
	scrollWin2 = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin2),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	
	gtk_widget_set_usize(scrollWin2, 200, 200);
	gtk_widget_show(scrollWin2);
	
	gtk_table_attach (GTK_TABLE (table1), scrollWin2, 2, 3, 6, 8,
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);

	filesToAdd = gtk_list_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "filesToAdd", filesToAdd);
	gtk_widget_show (filesToAdd);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin2), filesToAdd);
	gtk_signal_connect(GTK_OBJECT(filesToAdd), "select_child",
					   GtkSignalFunc(ListChildSelected), this);
	gtk_signal_connect(GTK_OBJECT(filesToAdd), "unselect_child",
					   GtkSignalFunc(ListChildUnSelected), this);

	gtk_list_set_selection_mode (GTK_LIST (filesToAdd), GTK_SELECTION_EXTENDED);

	scrollWin1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollWin1),
								   GTK_POLICY_AUTOMATIC,
								   GTK_POLICY_AUTOMATIC);
	gtk_widget_show(scrollWin1);
	
	gtk_widget_set_usize(scrollWin1, 200, 200);
	gtk_table_attach (GTK_TABLE (table1), scrollWin1, 0, 3, 1, 3,
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);
	
	projects = gtk_tree_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "projects", projects);
	gtk_widget_show (projects);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollWin1), projects);

	recursive = gtk_check_button_new_with_label("Recursive");
	gtk_widget_show(recursive);
	gtk_table_attach(GTK_TABLE(table1), recursive, 2, 3, 4, 5, 
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);

	branchAfter = gtk_check_button_new_with_label("Branch after share");
	gtk_widget_show(branchAfter);
	gtk_table_attach(GTK_TABLE(table1), branchAfter, 2, 3, 5, 6, 
					  (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);
	
	CopyTree(pTree, projects);
	
	// Expand out the tree to the parent of this project ...
	GList* children = gtk_container_children(GTK_CONTAINER(projects));

	GtkWidget* pNode = GetTreeItemFromProject(GTK_WIDGET(children->data),strProj.chars());

	gtk_tree_item_expand(GTK_TREE_ITEM(children->data));
	
	g_list_free(children);

	while (pNode)
	{
		gtk_tree_item_expand(GTK_TREE_ITEM(pNode));

		pNode = GTK_TREE(pNode->parent)->tree_owner;
	}
	
	gtk_signal_connect (GTK_OBJECT(projects), "selection_changed",
						GTK_SIGNAL_FUNC(SelectionChanged), this);	

	dialog_action_area1 = GTK_DIALOG (m_pDialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "hbuttonbox1", hbuttonbox1);
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);
	
	ok = gtk_button_new_with_label ("OK");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "ok", ok);
	GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(ok);
	gtk_widget_show (ok);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), ok);

	gtk_signal_connect(GTK_OBJECT(ok), "clicked",
					   (GtkSignalFunc)OkPressed, this);
	
	button6 = gtk_button_new_with_label ("Cancel");
	gtk_object_set_data (GTK_OBJECT (m_pDialog), "button6", button6);
	gtk_widget_show (button6);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button6);

	gtk_signal_connect(GTK_OBJECT(button6), "clicked",
					   (GtkSignalFunc)KillDialog, this);
}

CShareDlg::~CShareDlg()
{
	m_bBeingDestroyed = 1;
	
	// Clean up all the project nodes
	CleanTree(projects);

	gtk_signal_disconnect(GTK_OBJECT(m_pDialog), m_destroySig);
	
	gtk_widget_destroy(m_pDialog);
}


void CShareDlg::AddFiles(String& strProject)
{
	// reset the file list

	gtk_list_clear_items(GTK_LIST(files), 0, -1);

	gtk_widget_set_sensitive(add, FALSE);

	GList* children = gtk_container_children(GTK_CONTAINER(projects));

	GtkWidget* pItem = GTK_WIDGET(children->data);

	g_list_free(children);

	CVermTreeNode* pNode = GetTreeNodeFromProject(pItem, strProject.chars());

	VermTreeListNode* pLeafs = pNode->m_pLeafs->m_pHead;

	if (pLeafs)
		gtk_widget_set_sensitive(add, TRUE);
	else
		gtk_widget_set_sensitive(add, FALSE);
		
	while (pLeafs)
	{
		GtkWidget* pItem = gtk_list_item_new_with_label(pLeafs->m_pLeaf->m_name.chars());
		gtk_widget_show(pItem);

		gtk_container_add(GTK_CONTAINER(files), pItem);
		
		pLeafs = pLeafs->m_pNext;
	}

}

void CShareDlg::SetFileList(CVermProtocolMessage* pMessageReceived,
							CVermProtocolMessage* pMessageSent)
{
	// determine which tree node this file list is for
	const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
	
	GList* children = gtk_container_children(GTK_CONTAINER(projects));

	GtkWidget* pItem = GTK_WIDGET(children->data);

	g_list_free(children);

	GtkWidget* hItem = GetTreeItemFromProject(pItem, pszProject);
	
	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));
	
	// parse the file list
	String str("");

	int bPinInfo = pMessageReceived->GetHeaderValue(HEADER_PIN_STATUS) != NULL;
	int bChecksums = pMessageSent->GetHeaderValue(HEADER_SEND_CHECKSUM) != NULL;
	
	ParseFileList(pNode, pMessageReceived, m_pParent->m_username.chars(), str, bChecksums, bPinInfo);
	
	// mark the node as visited
	pNode->m_bVisited = 1;
	
	// update the file list view
	String strProject = pszProject;
	
	if (hItem == m_hCurSelItem)
		AddFiles(strProject);
}

void CShareDlg::SetWaitState(int state)
{
	if (state == 1)
		gtk_widget_set_sensitive(projects, FALSE);
	else
		gtk_widget_set_sensitive(projects, TRUE);
}
