/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdio.h>
#include <string.h>

#include "Vermie.h"
#include "VermByteQueue.h"

VermByteList::VermByteList(char* bytes, int len, int allocLen)
{
	if (len > allocLen)
		allocLen = len;
	
	if (allocLen % 8)
		allocLen += 8 - (allocLen % 8);
	
	m_pBytes = new char[allocLen];
	memcpy(m_pBytes, bytes, len);

	m_nLen = len;
	m_nAllocLen = allocLen;
	
	m_pNext = NULL;
}

VermByteList::~VermByteList()
{
	if (m_pBytes)
		delete []m_pBytes;
}

void VermByteList::AddBytes(char* bytes, int len)
{
	// blindly assume our parent has checked to make sure we fit
	memcpy(m_pBytes + m_nLen, bytes, len);
	m_nLen += len;
}

VermByteQueue::VermByteQueue()
{
	m_pHead = NULL;
	m_pTail = NULL;
}

VermByteQueue::~VermByteQueue()
{
	if (m_pHead)
		delete m_pHead;
}

void VermByteQueue::AddBytes(char* bytes, int len)
{
	if (m_pHead == NULL)
	{
		m_pHead = new VermByteList(bytes, len);
		m_pTail = m_pHead;
	}
	else
	{
		// will it fit in our current buffer?
		if (m_pTail->m_nLen + len < m_pTail->m_nAllocLen)
		{
			m_pTail->AddBytes(bytes, len);
		}
		else
		{
			m_pTail->m_pNext = new VermByteList(bytes, len);
			m_pTail = m_pTail->m_pNext;
		}
	}
}

int VermByteQueue::GetBytes(char** bytes)
{
	if (m_pHead == NULL)
	{
		*bytes = NULL;
		return 0;
	}

	*bytes = m_pHead->m_pBytes;
	return m_pHead->m_nLen;
}

void VermByteQueue::RemoveBytes(int len)
{
	if (!m_pHead)
		return;

	// All the bytes we gave back got eaten - trash the head pointer
	// and advance to the next
	if (m_pHead->m_nLen == len)
	{
		VermByteList* pTmp = m_pHead;
		m_pHead = m_pHead->m_pNext;
		delete pTmp;

		if (!m_pHead)
			m_pTail = NULL;
		return;
	}

	// Only a portion of the head pointer bytes was eaten - delete them
	// off
	//
	// TODO
	// is this safe across all compilers?
	memmove(m_pHead->m_pBytes, m_pHead->m_pBytes + len, m_pHead->m_nLen - len);
	m_pHead->m_nLen -= len;
}

VermByteQueue& VermByteQueue::operator+=(const String& str)
{
	AddBytes((char*)((String)(str)).chars(), ((String)(str)).length());

	return *this;
}

VermByteQueue& VermByteQueue::operator+=(const VermByteQueue& byteQ)
{
	if (!m_pHead)
		m_pHead = byteQ.m_pHead;
	else
		m_pTail->m_pNext = byteQ.m_pHead;


	VermByteList *tmp = m_pTail->m_pNext;

	while (tmp)
	{
		m_pTail = tmp;
		tmp = tmp->m_pNext;
	}

	return *this;
}
