/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#ifndef BUILD_STANDARD

#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>

#include <time.h>

extern "C"
{
#include "md5.h"
}

#include "Vermie.h"

#include "VermCrypto.h"
#include "VermByteQueue.h"

#include "VermProtocolHeader.h"
#include "VermProtocolMessage.h"
#include "AfxMessageBox.h"

CVermCrypto::CVermCrypto()
{
	m_pKey = NULL;
	m_bCtxInitialized = 0;
	m_nKeyLength = 16;
}

CVermCrypto::~CVermCrypto()
{
	if (m_pKey)
		delete m_pKey;
}


CVermProtocolMessage* CVermCrypto::DecryptKey(char* digest, char* pFileData, int nDataSize, int keyLength)
{
	CVermProtocolMessage* pParsedFile = new CVermProtocolMessage;

	// create a temporary crypto object using the weak key

	CVermCrypto tmpCrypto;

	tmpCrypto.SetKey(digest);

	// the first 8 bytes from the key file is the IV

	char* pData = pFileData;

	char IV[8];

	int i;
	
	for (i = 0; i < 8; i++)
	{
		IV[i] = *pData;
		pData++;
	}

	// the next 8 bytes is the encrypted length of the file

	char size[CRYPTO_BLOCK_SIZE];
	char sizeBuffer[CRYPTO_BLOCK_SIZE];

	for (i = 0; i < 8; i++)
	{
		sizeBuffer[i] = *pData;
		pData++;
	}

	tmpCrypto.Decrypt(sizeBuffer, size, CRYPTO_BLOCK_SIZE, IV);

	i = 0;

	while ((i < CRYPTO_BLOCK_SIZE) && (size[i] != PAD_CHAR))
		i++;

	size[i] = '\0';

	// decrypt the rest of the key file data

	char* decryptedData = new char[nDataSize - 16];

	tmpCrypto.Decrypt(pData, decryptedData, nDataSize - 16, IV);

	// attempt to parse the decrypted key file

	String strData(decryptedData, nDataSize - 16);

	int nFound = 0;
	
	while (!strData.empty())
	{
		int nIndex = strData.index(':');

		if (nIndex < 0)
			break;
		
		String strName = strData.before(nIndex);

		strData = strData.after(nIndex);
		
		nIndex = strData.index('\r');

		String strValue = strData.before(nIndex);

		if (nIndex + 1 < (int)strData.length())
			strData = strData.after(nIndex + 1);
		else
			strData.del(0, strData.length());

		pParsedFile->AddHeader(new CVermProtocolHeader(strName, strValue));
		++nFound;
	}

	delete []decryptedData;

	if (nFound != 6)
	{
		delete pParsedFile;
		return NULL;
	}
	else
		return pParsedFile;
}

int CVermCrypto::GetUserKey(const char* user, const char* server, char** key)
{
	// open the user's key file

	String strServerName = server;

	strServerName.downcase();

	int nDotIndex = strServerName.index('.');

	if (nDotIndex > 0)
		strServerName = strServerName.before(nDotIndex);
	
	String strUserName = user;
	strUserName.downcase();
	
	String strKeyFile = Vermie::m_strPrefsDir + "/" + strServerName + ".key";
	
	int keyFile = open(strKeyFile.chars(), O_RDONLY);

	if (keyFile < 0)
	{
		String s("The client was unable to locate key file '");
		s += strKeyFile;
		s += "'";
		AfxMessageBox(s, MB_OK);
		return 0;
	}

	// read in the encrypted data in one big chunk

	int nDataSize = lseek(keyFile, 0, SEEK_END);

	lseek(keyFile, 0, SEEK_SET);

	char* pFileData = new char[nDataSize];

	read(keyFile, pFileData, nDataSize);

	close(keyFile);

	// reconstruct the weak key used for encrypting the key file

	String strWeakKey(strServerName);
	strWeakKey += strUserName;
	
	MD5_CTX mdContext;

	MD5Init(&mdContext);
	MD5Update(&mdContext, (unsigned char*)strWeakKey.chars(), strWeakKey.length());
	MD5Final(&mdContext);

	CVermProtocolMessage* pParsedFile = DecryptKey((char*)mdContext.digest, pFileData, nDataSize, 16);

	// make sure this key is of the correct format

	// if 128 bit decrypt fails, try again with 56
	if (pParsedFile == NULL)
	{
		pParsedFile = DecryptKey((char*)mdContext.digest, pFileData, nDataSize, 7);

		if (!pParsedFile)
		{
			String s("Unable to decrypt user key");
			AfxMessageBox(s);
			return FALSE;
		}
	}

	delete []pFileData;


	*key = new char[m_nKeyLength];
	
	const char* pKey = pParsedFile->GetHeaderValue(KEY_HEADER_KEY);

	if (pKey)
	{
		if (strlen(pKey) == 34)
			m_nKeyLength = 16;
		else if (strlen(pKey) == 16)
			m_nKeyLength = 7;
		else
			m_nKeyLength = 0;
	}

	const char* pFormat = pParsedFile->GetHeaderValue(KEY_HEADER_FORMAT);

	if ((!pFormat) || (atoi(pFormat) != KEY_FORMAT))
	{
		String s("Incompatible key format");
		AfxMessageBox(s, MB_OK);
		delete pParsedFile;

		return 0;
	}

	// convert the user key from a hex string to a byte array

	pKey += 2;

	for (int i = 0; i < m_nKeyLength; i++)
	{
		if (*pKey >= '0' && *pKey <= '9')
			(*key)[i] = 16 * (*pKey - '0');
		else
			(*key)[i] = 16 * ((*pKey - 'a') + 10);

		pKey++;

		if (*pKey >= '0' && *pKey <= '9')
			(*key)[i] += (*pKey - '0');
		else
			(*key)[i] += ((*pKey - 'a') + 10);

		pKey++;
	}

	// convert the hash from a hex string to a byte array

	const char* pHash = pParsedFile->GetHeaderValue(KEY_HEADER_HASH);

	pHash += 2;

	unsigned char* hash = new unsigned char[m_nKeyLength];

	for (int i = 0; i < m_nKeyLength; i++)
	{
		if (*pHash >= '0' && *pHash <= '9')
			hash[i] = 16 * (*pHash - '0');
		else
			hash[i] = 16 * ((*pHash - 'a') + 10);
		
		pHash++;

		if (*pHash >= '0' && *pHash <= '9')
			hash[i] += (*pHash - '0');
		else
			hash[i] += ((*pHash - 'a') + 10);

		pHash++;
	}

	delete pParsedFile;

	// verify the hash

	String strConcat(*key, m_nKeyLength);
	strConcat += strUserName;

	MD5_CTX mdCTX;

	MD5Init(&mdCTX);
	MD5Update(&mdCTX, (unsigned char*)strConcat.chars(), strConcat.length());
	MD5Final(&mdCTX);

	for (int i = 0; i < m_nKeyLength; i++)
	{
		if (hash[i] != mdCTX.digest[i])
		{
			String s("Bad key file");
			AfxMessageBox(s, MB_OK);

			delete []hash;
			return 0;
		}
	}

	delete []hash;
	return 1;
}

void CVermCrypto::GetRandomNumber(char *number, int nLength)
{
	if (!m_bCtxInitialized)
	{
		// save the seed times

		Vermie::m_timeStamps.seedClock = clock();
		Vermie::m_timeStamps.seedTime = time(NULL);

		// seed the RNG - TODO Make this more robust?

		for (int i = 0; i < RANDSIZ; ++i)
			m_CTX.randrsl[i] = 0;

		TimeStamps* pTimeStamps = &Vermie::m_timeStamps;

		m_CTX.randrsl[0] = pTimeStamps->startClock;
		m_CTX.randrsl[1] = pTimeStamps->startTime;
		m_CTX.randrsl[2] = pTimeStamps->challengeClock;
		m_CTX.randrsl[3] = pTimeStamps->challengeTime;
		m_CTX.randrsl[4] = pTimeStamps->seedClock;
		m_CTX.randrsl[5] = pTimeStamps->seedTime;

		// initialize the library

		randinit(&m_CTX, TRUE);

		m_bCtxInitialized = TRUE;
	}

	// create a random number nLength bytes long

	int nByte = 0;
	int nIterations = nLength / 4;

	for (int i = 0; i < nIterations; i++)
	{
		int randNum = irand(&m_CTX);

		number[nByte++] = (char)((randNum << 24) >> 24);
		number[nByte++] = (char)((randNum << 16) >> 24);
		number[nByte++] = (char)((randNum << 8) >> 24);
		number[nByte++] = (char)(randNum >> 24);
	}
}

int CVermCrypto::SetKey(char* key)
{
	if (m_pKey)
		delete m_pKey;

	m_pKey = new BF_KEY;

	BF_set_key(m_pKey, m_nKeyLength, (unsigned char*)key);

	return 1;
}

void CVermCrypto::Decrypt(char* in, char* out, int size, char* IV)
{
	BF_cbc_encrypt((unsigned char*)in, (unsigned char*)out, size,
				   m_pKey, (unsigned char*)IV, BF_DECRYPT);
}

VermByteQueue* CVermCrypto::Encrypt(String* pStrData, char* IV)
{
	VermByteQueue* pQueue = new VermByteQueue();
	
	int nLength = pStrData->length();

	// do we need to pad the data?

	int padding = 0;
	
	if (nLength < CRYPTO_BLOCK_SIZE)
	{
		padding = CRYPTO_BLOCK_SIZE - nLength;
	}
	else
	{
		int remainder = nLength % CRYPTO_BLOCK_SIZE;

		if (remainder > 0)
			padding = CRYPTO_BLOCK_SIZE - remainder;
	}

	if (padding != 0)
	{
		for (int i = 0; i < padding; i++)
			*pStrData += PAD_CHAR;

		nLength = pStrData->length();
	}

	// create a buffer to hold the encrypted result

	//char* out = new char[nLength];
	char* out = new char[nLength]; //pStrResult->chars();

	// encrypt the data

	BF_cbc_encrypt((unsigned char*)pStrData->chars(), (unsigned char*)out, nLength,
				   m_pKey, (unsigned char*)IV, BF_ENCRYPT);


//	pStrResult->ReleaseBuffer(nLength);

	pQueue->AddBytes(out, nLength);

	delete []out;

	return pQueue;
}

VermByteQueue* CVermCrypto::Encrypt(VermByteQueue* pQueue,  char* IV)
{
	int nLength = pQueue->m_pHead->m_nLen;

	char *pData;

	pQueue->GetBytes(&pData);
	
	int padding = 0;
	
	if (nLength < CRYPTO_BLOCK_SIZE)
	{
		padding = CRYPTO_BLOCK_SIZE - nLength;
	}
	else
	{
		int remainder = nLength % CRYPTO_BLOCK_SIZE;

		if (remainder > 0)
			padding = CRYPTO_BLOCK_SIZE - remainder;
	}

	if (padding != 0)
	{
		if (padding + pQueue->m_pHead->m_nLen > pQueue->m_pHead->m_nAllocLen)
			exit(0);
		
		for (int i = 0; i < padding; i++)
			pData[nLength++] += PAD_CHAR;
		pQueue->m_pHead->m_nLen = nLength;
	}

	// create a buffer to hold the encrypted result

	//char* out = new char[nLength];
	char* out = new char[nLength]; //pStrResult->chars();

	// encrypt the data

	BF_cbc_encrypt((unsigned char*)pData, (unsigned char*)out, nLength,
				   m_pKey, (unsigned char*)IV, BF_ENCRYPT);

	pQueue->RemoveBytes(nLength);
	
	pQueue->AddBytes(out, nLength);

	delete[] out;

	return pQueue;
}

#endif // !BUILD_STANDARD
