/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "Vermie.h"
#include "VermFrameWnd.h"
#include "VermFileView.h"
#include "VermTreeNode.h"
#include "VermTreeLeaf.h"
#include "Vermie.h"
#include "VermProjectView.h"
#include "VermTreeUtils.h"
#include "AfxMessageBox.h"
#include "ViewEditDlg.h"

static char* colHeadings[5] = {"File", "Locked By", "Local Time",
							   "Remote Time", "Status"};

static CVermFileView* s_pFileView;

#define FILE_STATUS_MISSING		"Missing"
#define FILE_STATUS_CURRENT		""
#define FILE_STATUS_OLD			"Old"
#define FILE_STATUS_EDITED		"Edited"
#define FILE_STATUS_RENEGADE	"Renegade"
#define FILE_STATUS_UNKNOWN		"Unknown"
#define FILE_STATUS_NEW			"More Recent"

static int FileButtonPress(GtkWidget*, GdkEventButton*, CVermFileView*);

static int SortCompare(GtkCList* list, gconstpointer p1, gconstpointer p2)
{
	GtkCListRow *row1 = (GtkCListRow *) p1;
	GtkCListRow *row2 = (GtkCListRow *) p2;

	char *t1 = GTK_CELL_TEXT(row1->cell[list->sort_column])->text;
	char *t2 = GTK_CELL_TEXT(row2->cell[list->sort_column])->text;

	switch (list->sort_column)
	{
		// File
	case 0:
		if (!t1 || !t2)
		{
			return 0;
		}
		
		if (!*t1)
		{
			if (*t2)
				return 1;
		}

		if (!*t2)
		{
			if (*t1)
				return -1;
		}
		
		return strcasecmp(t1, t2);
		break;

		// Locked By
	case 1:
		if (!t1 || !t2)
		{
			return 0;
		}		

		if (!*t1)
		{
			if (*t2)
			{
				return 1;
			}
		}

		if (!*t2)
		{
			if (*t1)
			{
				return -1;
			}
		}
		
		if ((!*t1 && !*t2) || !strcasecmp(t1, t2))
		{
			char* p1 = GTK_CELL_TEXT(row1->cell[0])->text;
			char* p2 = GTK_CELL_TEXT(row2->cell[0])->text;

			return strcasecmp(p1, p2);
		}
		else
		{
			return strcasecmp(t1, t2);
		}
		break;

		// Local/Remote Date
	case 2:
	case 3:
	{
		if (!t1 || !t2)
		{
			return 0;
		}
		
		if (!*t1)
		{
			if (*t2)
				return -1;
		}

		if (!*t2)
		{
			if (*t1)
				return 1;
		}

		if (!*t1 && !*t2)
		{
			char* p1 = GTK_CELL_TEXT(row1->cell[0])->text;
			char* p2 = GTK_CELL_TEXT(row2->cell[0])->text;

			return strcasecmp(p1, p2);
		}

		int m1, d1, y1, h1, min1;
		int m2, d2, y2, h2, min2;

		sscanf(t1, "%02d/%02d/%04d %02d:%02d", &m1, &d1, &y1, &h1, &min1);
		sscanf(t2, "%02d/%02d/%04d %02d:%02d", &m2, &d2, &y2, &h2, &min2);

		// check years
		if (y1 < y2)
			return -1;
		else if (y2 < y1)
			return 1;

		// years are same
		if (m1 < m2)
			return -1;
		else if (m2 < m1)
			return 1;

		// months are same
		if (d1 < d2)
			return -1;
		else if (d2 < d1)
			return 1;
		
		// days are same
		if (h1 < h2)
			return -1;
		else if (h2 < h1)
			return 1;

		// hours are same
		if (min1 < min2)
			return -1;
		else if (min2 < min1)
			return 1;

		// minutes are same
		char* p1 = GTK_CELL_TEXT(row1->cell[0])->text;
		char* p2 = GTK_CELL_TEXT(row2->cell[0])->text;
		
		return strcasecmp(p1, p2);

		break;
	}
	
	case 4:
		if (!t1 || !t2)
		{
			return 0;
		}

		if (!*t1)
			if (*t2)
				return -1;

		if (!*t2)
			if (*t1)
				return 1;

		if (!*t1 & !*t2 || !strcasecmp(t1, t2))
		{
			char* p1 = GTK_CELL_TEXT(row1->cell[0])->text;
			char* p2 = GTK_CELL_TEXT(row2->cell[0])->text;
			
			return strcasecmp(p1, p2);
		}

		return strcasecmp(t1, t2);
	}

	return 0;
}

static void ColumnClicked(GtkCList* clist, int column, gpointer)
{
	if (column == clist->sort_column)
    {
		if (clist->sort_type == GTK_SORT_ASCENDING)
			clist->sort_type = GTK_SORT_DESCENDING;
		else
			clist->sort_type = GTK_SORT_ASCENDING;
    }
	else
	{
		clist->sort_type = GTK_SORT_ASCENDING;
		gtk_clist_set_sort_column (clist, column);
		Vermie::m_nSortColumn = column;
	}

	gtk_clist_sort(clist);
}

static void MenuItemSelected(GtkWidget*, int which)
{
	switch(which)
	{
	case SOURCE_BRANCH:
		s_pFileView->m_pParent->OnSourceBranch();
		break;
	case FILE_VIEW:
		s_pFileView->m_pParent->OnFileView(FALSE);
		break;
		
	case FILE_EDIT:
		s_pFileView->m_pParent->OnFileEdit(FALSE);
		break;
		
	case FILE_DELETE:
		s_pFileView->m_pParent->OnFileDelete();
		break;

	case FILE_RENAME:
		s_pFileView->m_pParent->OnFileRename();
		break;

	case FILE_LABEL:
		s_pFileView->m_pParent->OnFileLabel();
		break;

	case SOURCE_GET_LATEST_VERSION:
		s_pFileView->m_pParent->OnSourceGetLatestVersion();
		break;

	case SOURCE_CHECK_IN:
		s_pFileView->m_pParent->OnSourceCheckIn();
		break;

	case SOURCE_CHECK_OUT:
		s_pFileView->m_pParent->OnSourceCheckOut();
		break;

	case SOURCE_UNDO_CHECK_OUT:
		s_pFileView->m_pParent->OnSourceUndoCheckout();
		break;

	case TOOLS_SHOW_DIFFERENCES:
		s_pFileView->m_pParent->OnToolsShowDifferences();
		break;

	case TOOLS_SHOW_HISTORY:
		s_pFileView->m_pParent->OnToolsShowHistory();
		break;
		
	default:
		break;
	}		
}

static int FileButtonPress(GtkWidget*, GdkEventButton* ev,
						   CVermFileView* pView)
{
	if (ev->button == 3)
	{
		GList* selection = GTK_CLIST(pView->m_pList)->selection;

		int bFilesCheckedOut = 0;
		int bFilesNotCheckedOut = 0;
		int bSharedFiles = 0;
		
		int numFiles = 0;
		
		if (!selection)
		{
			gtk_widget_set_sensitive(pView->m_mView, FALSE);
			gtk_widget_set_sensitive(pView->m_mEdit, FALSE);
			gtk_widget_set_sensitive(pView->m_mCheckIn, FALSE);
			gtk_widget_set_sensitive(pView->m_mGetLatestVersion, FALSE);
			gtk_widget_set_sensitive(pView->m_mShowHistory, FALSE);
			gtk_widget_set_sensitive(pView->m_mShowDifferences, FALSE);
			gtk_widget_set_sensitive(pView->m_mCheckOut, FALSE);
			gtk_widget_set_sensitive(pView->m_mUndoCheckout, FALSE);
			gtk_widget_set_sensitive(pView->m_mDelete, FALSE);
			gtk_widget_set_sensitive(pView->m_mRename, FALSE);
			gtk_widget_set_sensitive(pView->m_mLabel, FALSE);
			gtk_widget_set_sensitive(pView->m_mBranch, FALSE);
		}
		else
		{
			gtk_widget_set_sensitive(pView->m_mGetLatestVersion, TRUE);
			gtk_widget_set_sensitive(pView->m_mDelete, TRUE);
			gtk_widget_set_sensitive(pView->m_mLabel, TRUE);

			if (!selection->next)
			{
				CVermTreeLeaf* pLeaf = (CVermTreeLeaf*) gtk_clist_get_row_data(GTK_CLIST(pView->m_pList), (int) selection->data);

				if (!pLeaf->m_userLocked.empty())
					gtk_widget_set_sensitive(pView->m_mRename, FALSE);
				else
					gtk_widget_set_sensitive(pView->m_mRename, TRUE);
					
				gtk_widget_set_sensitive(pView->m_mView, TRUE);
				gtk_widget_set_sensitive(pView->m_mEdit, TRUE);
			}
			else
			{
				gtk_widget_set_sensitive(pView->m_mView, FALSE);
				gtk_widget_set_sensitive(pView->m_mEdit, FALSE);
			}
		}

		while(selection)
		{
			CVermTreeLeaf* pLeaf = (CVermTreeLeaf*) gtk_clist_get_row_data(GTK_CLIST(pView->m_pList), (int) selection->data);

			String strProject;
			GetPathFromTreeItem(pView->m_pParent->m_pProjectView->m_hCurSelItem, strProject);
			int nLockStatus = IsFileLocked(pLeaf, strProject, pView->m_pParent->m_username);

			if (nLockStatus > 0)
				bFilesCheckedOut = 1;
			else if (pLeaf->m_nPinnedVersion == -1)
				bFilesNotCheckedOut = 1;

			if (pLeaf && pLeaf->m_bShared)
				bSharedFiles = 1;
			
			selection = selection->next;
			++numFiles;
		}

		if (bSharedFiles)
		{
			gtk_widget_set_sensitive(pView->m_mBranch, Vermie::m_bUseVSS6Features);
		}
		else
		{
			gtk_widget_set_sensitive(pView->m_mBranch, FALSE);
		}
		
		if (bFilesCheckedOut)
		{
			gtk_widget_set_sensitive(pView->m_mCheckIn, TRUE);
			gtk_widget_set_sensitive(pView->m_mUndoCheckout, TRUE);
		}
		else
		{
			gtk_widget_set_sensitive(pView->m_mCheckIn, FALSE);
			gtk_widget_set_sensitive(pView->m_mUndoCheckout, FALSE);
		}

		if (bFilesNotCheckedOut)
			gtk_widget_set_sensitive(pView->m_mCheckOut, TRUE);
		else
			gtk_widget_set_sensitive(pView->m_mCheckOut, FALSE);

		if (numFiles == 1)
		{
			gtk_widget_set_sensitive(pView->m_mShowHistory, TRUE);
			gtk_widget_set_sensitive(pView->m_mShowDifferences, TRUE);
		}
		else
		{
			gtk_widget_set_sensitive(pView->m_mRename, FALSE);
			gtk_widget_set_sensitive(pView->m_mShowHistory, FALSE);
			gtk_widget_set_sensitive(pView->m_mShowDifferences, FALSE);
		}

		gtk_menu_popup(GTK_MENU(pView->m_popupMenu), NULL, NULL, NULL, NULL,
					   ev->button, ev->time);
	}
	pView->m_pParent->SetFocus(FOCUS_FILEVIEW);

	return 1;
}
		
void SelectClist(GtkWidget*, int, int, GdkEventButton* ev, CVermFileView* pView)
{
	// Capture the double-click and either view the file/edit the file/or raise the dialog
	if (ev && ev->type == GDK_2BUTTON_PRESS)
	{
		if (Vermie::m_strFileDoubleClick == "Ask")
		{
			CViewEditDlg dlg("View/Edit File");

			if (dlg.DoModal() == IDOK)
			{
				if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.view)))
					pView->m_pParent->OnFileView(TRUE);
				else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.edit)))
					pView->m_pParent->OnFileEdit(TRUE);
			}
		}
		else if (Vermie::m_strFileDoubleClick == "View File")
		{
			pView->m_pParent->OnFileView(FALSE);
		}
		else if (Vermie::m_strFileDoubleClick == "Edit File")
		{
			pView->m_pParent->OnFileEdit(FALSE);
		}
	}

	pView->m_pParent->SetFocus(FOCUS_FILEVIEW);
}

static void enter_callback(GtkWidget *widget, CVermFileView* pView)
{
	gchar *entry_text;
	entry_text = gtk_entry_get_text(GTK_ENTRY(widget));

	pView->SetWorkingDirectory(entry_text);
}

void CVermFileView::SetWorkingDirectory(char* newDir)
{
	String str = newDir;
	String strOld;

	if (m_pParent->m_pProjectView->m_hCurSelItem)
		strOld = GetWorkingDirectory(m_pParent->m_pProjectView->m_hCurSelItem);
	else
		strOld = "";
	

	if (str != strOld)
	{
		String s("Change working directory to\n");
		s += str;
		s += " ?";

		if (AfxMessageBox(s, MB_OK | MB_CANCEL) == IDOK)
			m_pParent->SetWorkingDirectory(str);
	}
}

CVermFileView::CVermFileView(CVermFrameWnd* pParent)
{
	s_pFileView = this;
	
	m_pParent = pParent;
	m_pNodeDisplayed = NULL;
	m_bTextOn = 0;
	
	m_pWidget = gtk_vbox_new(FALSE, 5);
	m_pScrolledWin = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_usize(m_pScrolledWin, Vermie::m_FViewWidth,
						 Vermie::m_FViewHeight);
	gtk_widget_show(m_pScrolledWin);

	m_pList = gtk_clist_new_with_titles(5, colHeadings);

	gtk_signal_connect(GTK_OBJECT(m_pList),
                          "select_row",
                          (GtkSignalFunc)SelectClist,
                          this);

	gtk_signal_connect(GTK_OBJECT(m_pList),
                          "unselect_row",
                          (GtkSignalFunc)SelectClist,
                          this);

//	gtk_clist_set_border(GTK_CLIST(m_pList), GTK_SHADOW_IN);

	gtk_clist_set_column_width(GTK_CLIST(m_pList), 0, Vermie::m_nColumnWidths[0]);
	gtk_clist_set_column_width(GTK_CLIST(m_pList), 1, Vermie::m_nColumnWidths[1]);
	gtk_clist_set_column_width(GTK_CLIST(m_pList), 2, Vermie::m_nColumnWidths[2]);
	gtk_clist_set_column_width(GTK_CLIST(m_pList), 3, Vermie::m_nColumnWidths[3]);
	gtk_clist_set_column_width(GTK_CLIST(m_pList), 4, Vermie::m_nColumnWidths[4]);

	gtk_clist_set_selection_mode(GTK_CLIST(m_pList), GTK_SELECTION_EXTENDED);
	gtk_clist_set_sort_column(GTK_CLIST(m_pList), Vermie::m_nSortColumn);
//	GTK_CLIST_AUTO_SORT(GTK_CLIST(m_pList));
	gtk_clist_set_sort_type(GTK_CLIST(m_pList), GTK_SORT_ASCENDING);
//	gtk_clist_set_auto_sort(GTK_CLIST(m_pList), 1);

	gtk_clist_set_compare_func(GTK_CLIST(m_pList),
							   (GtkCListCompareFunc)SortCompare);
	
	gtk_signal_connect(GTK_OBJECT(m_pList), "button_press_event",
					   (GtkSignalFunc)FileButtonPress, this);

	gtk_signal_connect(GTK_OBJECT(m_pList), "click_column",
					   (GtkSignalFunc) ColumnClicked, this);
	
	gtk_widget_set_events(m_pList, GDK_BUTTON_PRESS);
	
	gtk_widget_show(m_pList);
	
	gtk_container_add(GTK_CONTAINER(m_pScrolledWin), m_pList);
	
//	gtk_widget_set_usize(m_pList, 200, 500);

	GtkWidget* frame = gtk_frame_new("");
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

	GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), hbox);
	
	GtkWidget* lab = gtk_label_new("Working Directory:");
	gtk_widget_show(lab);
	
	m_pWorkingDirLabel = gtk_entry_new();
	
	gtk_widget_show(m_pWorkingDirLabel);
    gtk_signal_connect(GTK_OBJECT(m_pWorkingDirLabel), "activate",
                       GTK_SIGNAL_FUNC(enter_callback), this);


	m_pProgressLabel = gtk_label_new("Progress: ");
	gtk_misc_set_alignment(GTK_MISC(m_pProgressLabel), 1.0, 0.0);
	gtk_widget_show(m_pProgressLabel);

	GtkAdjustment* adj = (GtkAdjustment *) gtk_adjustment_new (0, 0, 100, 0, 0, 0);
	m_pProgressBar = gtk_progress_bar_new_with_adjustment(adj);
	gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(m_pProgressBar),
								   GTK_PROGRESS_CONTINUOUS);
	gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(m_pProgressBar),
									 GTK_PROGRESS_LEFT_TO_RIGHT);
	gtk_progress_set_show_text(GTK_PROGRESS(m_pProgressBar), 1);
	gtk_widget_show(m_pProgressBar);
	
	gtk_box_pack_start(GTK_BOX(hbox), lab, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), m_pWorkingDirLabel, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), m_pProgressLabel, FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), m_pProgressBar, FALSE, FALSE, 5);
	
	gtk_box_pack_start(GTK_BOX(m_pWidget), frame, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(m_pWidget), m_pScrolledWin, TRUE, TRUE, 0);

	gtk_widget_show(hbox);
	gtk_widget_show(frame);

    // The popup menu
	m_popupMenu = gtk_menu_new();
	
	m_mView = gtk_menu_item_new_with_label("View");
	gtk_widget_show(m_mView);
	gtk_menu_append(GTK_MENU(m_popupMenu), m_mView);
	gtk_signal_connect(GTK_OBJECT(m_mView), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_VIEW);
	
	m_mEdit = gtk_menu_item_new_with_label("Edit");
	gtk_widget_show(m_mEdit);
	gtk_menu_append(GTK_MENU(m_popupMenu), m_mEdit);
	gtk_signal_connect(GTK_OBJECT(m_mEdit), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_EDIT);
	
	GtkWidget* sep3 = gtk_menu_item_new();
	gtk_widget_show(sep3);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep3);

	m_mGetLatestVersion = gtk_menu_item_new_with_label("Get Latest Version");
	gtk_widget_show(m_mGetLatestVersion);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mGetLatestVersion);
	gtk_signal_connect(GTK_OBJECT(m_mGetLatestVersion), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_GET_LATEST_VERSION);

	m_mCheckOut = gtk_menu_item_new_with_label("Check Out");
	gtk_widget_show(m_mCheckOut);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mCheckOut);
	gtk_signal_connect(GTK_OBJECT(m_mCheckOut), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_CHECK_OUT);

	m_mCheckIn = gtk_menu_item_new_with_label("Check In...");
	gtk_widget_show(m_mCheckIn);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mCheckIn);
	gtk_signal_connect(GTK_OBJECT(m_mCheckIn), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_CHECK_IN);

	m_mUndoCheckout = gtk_menu_item_new_with_label("Undo Checkout");
	gtk_widget_show(m_mUndoCheckout);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mUndoCheckout);
	gtk_signal_connect(GTK_OBJECT(m_mUndoCheckout), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_UNDO_CHECK_OUT);

	m_mBranch = gtk_menu_item_new_with_label("Branch...");
	gtk_widget_show(m_mBranch);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mBranch);
	gtk_signal_connect(GTK_OBJECT(m_mBranch), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_BRANCH);

	GtkWidget* sep = gtk_menu_item_new();
	gtk_widget_show(sep);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep);

	m_mDelete = gtk_menu_item_new_with_label("Delete...");
	gtk_widget_show(m_mDelete);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mDelete);
	gtk_signal_connect(GTK_OBJECT(m_mDelete), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_DELETE);

	m_mRename = gtk_menu_item_new_with_label("Rename...");
	gtk_widget_show(m_mRename);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mRename);
	gtk_signal_connect(GTK_OBJECT(m_mRename), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_RENAME);

	m_mLabel = gtk_menu_item_new_with_label("Label...");
	gtk_widget_show(m_mLabel);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mLabel);
	gtk_signal_connect(GTK_OBJECT(m_mLabel), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_LABEL);

	GtkWidget* sep2 = gtk_menu_item_new();
	gtk_widget_show(sep2);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep2);

	m_mShowDifferences = gtk_menu_item_new_with_label("Show Differences...");
	gtk_widget_show(m_mShowDifferences);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mShowDifferences);
	gtk_signal_connect(GTK_OBJECT(m_mShowDifferences), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)TOOLS_SHOW_DIFFERENCES);

	m_mShowHistory = gtk_menu_item_new_with_label("Show History...");
	gtk_widget_show(m_mShowHistory);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_mShowHistory);
	gtk_signal_connect(GTK_OBJECT(m_mShowHistory), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)TOOLS_SHOW_HISTORY);
	
}

CVermFileView::~CVermFileView()
{

}

void CVermFileView::Reset(void)
{
	m_pNodeDisplayed = NULL;
	gtk_clist_clear(GTK_CLIST(m_pList));
}

char** CVermFileView::GetSelectedRows(void)
{
	GList* selection = GTK_CLIST(m_pList)->selection;
	int numSelected = 0;

	if (!selection)
		return NULL;
	
	while (selection)
	{
		++numSelected;
		selection = selection->next;
	}

	char** rows = new char*[numSelected + 1];

	selection = GTK_CLIST(m_pList)->selection;

	int i = 0;
	
	while (selection)
	{
		char* file = NULL;
		guint8 spacing;
		GdkPixmap* pix;
		GdkBitmap* bmp;
		
		gtk_clist_get_pixtext(GTK_CLIST(m_pList), (int)selection->data, 0, &file, &spacing, &pix, &bmp);

		if (!file)
			gtk_clist_get_text(GTK_CLIST(m_pList), (int)selection->data, 0, &file);
		
		rows[i] = new char[strlen(file) + 1];
		strcpy(rows[i], file);

		i++;
		selection = selection->next;
	}

	rows[i] = NULL;

	return rows;
}

void CVermFileView::ReselectRows(char** rows)
{
	int i = 0;

	while (rows[i])
	{
		for (int j = 0; j < GTK_CLIST(m_pList)->rows; j++)
		{
			char* file = NULL;
			guint8 spacing;
			GdkPixmap* pix;
			GdkBitmap* bmp;

			gtk_clist_get_pixtext(GTK_CLIST(m_pList), j, 0, &file, &spacing, &pix, &bmp);

			if (!file)
				gtk_clist_get_text(GTK_CLIST(m_pList), j, 0, &file);

			if (file && !strcmp(file, rows[i]))
			{
				gtk_clist_select_row(GTK_CLIST(m_pList), j, 0);
				break;
			}
		}
		
		delete rows[i];
		i++;
	}

	delete []rows;
}

void CVermFileView::SetFileList(CVermTreeNode* pNode, String& strProject,
								String& strUsername, String& strWorkingDir,
								CVermTreeLeaf* pLeaf)
{
	char **selectedRows = NULL;

	if (pLeaf)
	{
		if (pNode != m_pNodeDisplayed)
			return;

		int index = gtk_clist_find_row_from_data(GTK_CLIST(m_pList), pLeaf);
		
		int i = 0;
		if (index >= 0)
			SetFileInfo(strProject, strUsername, strWorkingDir,
						pLeaf, index);
		else
		{
			char *junk[5] = {"","","","",""};
			junk[0] = (char*)pLeaf->m_name.chars();

			i = gtk_clist_append(GTK_CLIST(m_pList), junk);
			gtk_clist_set_row_data(GTK_CLIST(m_pList), i, pLeaf);
			SetFileInfo(strProject, strUsername, strWorkingDir,
						pLeaf, i);
		}
	}
	else
	{
		// If this is the currently displyed node and we are refreshing
		// it, save off the currently selected rows and re-select them
		// when we are done
		if (pNode == m_pNodeDisplayed)
			selectedRows = GetSelectedRows();
			
		// delete all of the items currently in the file list view
		gtk_clist_clear(GTK_CLIST(m_pList));

		gtk_clist_freeze(GTK_CLIST(m_pList));
//		if (m_selItems)
//		{
//			delete m_selItems;
//			m_selItems = NULL;
//		}

		m_numSelItems = 0;

		// save our new displayed node

		m_pNodeDisplayed = pNode;

		// iterate through each file in this node

		CVermTreeLeaf* pLeaf;

		VermTreeListNode* pList;

		if (pNode->m_pLeafs)
			pList = pNode->m_pLeafs->m_pHead;
		else
			pList = NULL;

		int i = 0;
		
		char *junk[5] = {"","","","",""};
		while (pList)
		{
			// get the next leaf

			pLeaf = pList->m_pLeaf;

			
			junk[0] = (char*)pLeaf->m_name.chars();
			// insert the new item in the list

			i = gtk_clist_insert(GTK_CLIST(m_pList), 0, junk);
			gtk_clist_set_row_data(GTK_CLIST(m_pList), i, pLeaf);

			if (pLeaf->m_bShared)
				gtk_clist_set_pixtext(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars(), 3, Vermie::m_pShared, Vermie::m_pSharedMask);
			else if (pLeaf->m_nPinnedVersion > 0)
				gtk_clist_set_pixtext(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars(), 3, Vermie::m_pPin, Vermie::m_pPinMask);
									  
			SetFileInfo(strProject, strUsername, strWorkingDir,
						pLeaf, i);

			pList = pList->m_pNext;
		}
	}

	gtk_clist_sort(GTK_CLIST(m_pList));
	gtk_clist_thaw(GTK_CLIST(m_pList));

	if (selectedRows)
		ReselectRows(selectedRows);
}

void CVermFileView::UpdateFileInfo(CVermTreeNode* pNode, String& strProject,
								   String& strUsername, String& strWorkingDir)
{
	CVermTreeLeaf* pLeaf;

	VermTreeListNode* pList;

	if (pNode->m_pLeafs)
		pList = pNode->m_pLeafs->m_pHead;
	else
		pList = NULL;

	while (pList)
	{
		pLeaf = pList->m_pLeaf;

		int index = gtk_clist_find_row_from_data(GTK_CLIST(m_pList), pLeaf);

		if (index >= 0)
			SetFileInfo(strProject, strUsername, strWorkingDir,
						pLeaf, index);

        pList = pList->m_pNext;
	}

	gtk_clist_sort(GTK_CLIST(m_pList));
}

void CVermFileView::UpdatePinStatus(CVermTreeLeaf* pLeaf)
{
	int i = gtk_clist_find_row_from_data(GTK_CLIST(m_pList), pLeaf);

	if (pLeaf->m_nPinnedVersion == -1)
	{
		gtk_clist_set_text(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars());
	}
	else
	{
		gtk_clist_set_pixtext(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars(), 3, Vermie::m_pPin, Vermie::m_pPinMask);
	}
}

void CVermFileView::UpdateSharedStatus(CVermTreeLeaf* pLeaf)
{
	int i = gtk_clist_find_row_from_data(GTK_CLIST(m_pList), pLeaf);

	// leaf is now shared
	if (pLeaf->m_bShared)
		gtk_clist_set_pixtext(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars(), 3, Vermie::m_pShared, Vermie::m_pSharedMask);
	else if (pLeaf->m_nPinnedVersion > 0)
		gtk_clist_set_pixtext(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars(), 3, Vermie::m_pPin, Vermie::m_pPinMask);
	else
		gtk_clist_set_text(GTK_CLIST(m_pList), i, 0, pLeaf->m_name.chars());

	gtk_clist_set_text(GTK_CLIST(m_pList), i, 1, pLeaf->m_userLocked.chars());
}

void CVermFileView::SetFileInfo(String& strProject,
								String& strUsername, String& strWorkingDir,
								CVermTreeLeaf* pLeaf, int index)
{
	String strBuffer;

	// does the file exist locally?

	int bFileExists = 0;

	int localFile;

	String strFileName = strWorkingDir + "/" + pLeaf->m_name;

	char* fileName = NULL;

	if (!pLeaf->m_bShared || pLeaf->m_nPinnedVersion == -1)
		gtk_clist_get_text(GTK_CLIST(m_pList), index, 0, &fileName);
	else
	{
		guint8 spacing;
		GdkPixmap* pix;
		GdkBitmap* bmp;
		
		gtk_clist_get_pixtext(GTK_CLIST(m_pList), index, 0, &fileName, &spacing, &pix, &bmp);
	}

		if (pLeaf->m_bShared)
			gtk_clist_set_pixtext(GTK_CLIST(m_pList), index, 0, pLeaf->m_name.chars(), 3, Vermie::m_pShared, Vermie::m_pSharedMask);
		else if (pLeaf->m_nPinnedVersion > 0)
			gtk_clist_set_pixtext(GTK_CLIST(m_pList), index, 0, pLeaf->m_name.chars(), 3, Vermie::m_pPin, Vermie::m_pPinMask);
		else
			gtk_clist_set_text(GTK_CLIST(m_pList), index, 0, pLeaf->m_name.chars());
	
	if (!strWorkingDir.empty())
	{
		localFile = open(strFileName.chars(), O_RDONLY);
		bFileExists = (localFile >= 0);

		if (bFileExists)
			close(localFile);
	}

	// if the file exists, get status info and close it

	struct stat fileStatus;
	memset(&fileStatus, 0, sizeof(struct stat));

	if (bFileExists)
	{
		stat(strFileName.chars(), &fileStatus);
	}
			
	// set name of user who has lock

	String strUserLocked = pLeaf->m_userLocked;
	
	if (!strUserLocked.empty())
	{
		String strFirstLetter = strUserLocked.before(1);

		strFirstLetter.upcase();

		String after = strUserLocked.after(0).chars();
		strUserLocked = strFirstLetter + after;
	}

	char* lockedBy = NULL;
	gtk_clist_get_text(GTK_CLIST(m_pList), index, 1, &lockedBy);

	strBuffer = lockedBy;
	
	if (!lockedBy || fcompare(strBuffer,strUserLocked))
		gtk_clist_set_text(GTK_CLIST(m_pList), index, 1,
						   strUserLocked.chars());

	// set local mod date

	if (bFileExists)
	{
		char *text = NULL;

		struct tm* tm;
		tm = localtime(&fileStatus.st_mtime);

		char date[2048];

		sprintf(date, "%02d/%02d/%04d %02d:%02d",
				tm->tm_mon + 1, tm->tm_mday, tm->tm_year + 1900,
				tm->tm_hour, tm->tm_min);
		
		gtk_clist_get_text(GTK_CLIST(m_pList), index, 2, &text);

		if (!text || strcmp(text, date))
			gtk_clist_set_text(GTK_CLIST(m_pList), index, 2,
							   date);
	}

	// set date of revision
	char strRemoteDate[2048];
	
	struct tm* tm;
	tm = localtime(&pLeaf->m_pRemoteVersionDate);

	sprintf(strRemoteDate, "%02d/%02d/%04d %02d:%02d",
			tm->tm_mon + 1, tm->tm_mday, tm->tm_year + 1900,
			tm->tm_hour, tm->tm_min);

	char* rTime = NULL;

	gtk_clist_get_text(GTK_CLIST(m_pList), index, 3, &rTime);
	
	if (!rTime || strcmp(rTime, strRemoteDate))
		gtk_clist_set_text(GTK_CLIST(m_pList), index, 3,
						   strRemoteDate);

	// set the status label

	String strNewStatus = FILE_STATUS_CURRENT;

	char* status = NULL;
	gtk_clist_get_text(GTK_CLIST(m_pList), index, 4, &status);
	strBuffer = status;
	
	if (bFileExists)
	{
		if (pLeaf->m_nLocalVersionNumber < 0 || pLeaf->m_pLocalVersionDate < 0)
		{
			strNewStatus = FILE_STATUS_UNKNOWN;
		}
		else
		{
			if (pLeaf->m_nLocalVersionNumber < pLeaf->m_nRemoteVersionNumber)
			{
				strNewStatus = FILE_STATUS_OLD;
			}
			else if (pLeaf->m_nLocalVersionNumber > pLeaf->m_nRemoteVersionNumber)
			{
				strNewStatus = FILE_STATUS_NEW;
			}
			else
			{
				if (pLeaf->m_pLocalVersionDate < fileStatus.st_mtime)
				{
					int nLockStatus = IsFileLocked(pLeaf, strProject,
												   strUsername);
					
					if (nLockStatus > 0)
					{
						strNewStatus = FILE_STATUS_EDITED;
					}
					else
					{
						strNewStatus = FILE_STATUS_RENEGADE;
					}
				}
				else
				{
					strNewStatus = FILE_STATUS_CURRENT;
				}
			}
		}
	}
	else
	{
		strNewStatus = FILE_STATUS_MISSING;
		// reset local date incase there was something displayed before
		gtk_clist_set_text(GTK_CLIST(m_pList), index, 2, "");
	}

	if (fcompare(strBuffer, strNewStatus))
		gtk_clist_set_text(GTK_CLIST(m_pList), index, 4, strNewStatus.chars());
}

void CVermFileView::SetWorkingDirPane(String& str)
{
	gtk_entry_set_text(GTK_ENTRY(m_pWorkingDirLabel), str.chars());
}

void CVermFileView::ResetStatus(void)
{
	SetStatus(-1.0);
}

void CVermFileView::SetStatus(double dPercentage)
{
	if (dPercentage < 0.0)
	{
		if (m_bTextOn == 1)
		{
			gtk_progress_set_show_text(GTK_PROGRESS(m_pProgressBar), 0);
			m_bTextOn = 0;
		}
		dPercentage = 0.0;
	}
	else
	{
		if (m_bTextOn == 0)
		{
			gtk_progress_set_show_text(GTK_PROGRESS(m_pProgressBar), 1);
			m_bTextOn = 1;
		}
	}
	
	gtk_progress_set_value(GTK_PROGRESS(m_pProgressBar), (float)dPercentage * 100.0);
}

void CVermFileView::GetFileName(int nIndex, String& strFile)
{
	char *name = NULL;
	gtk_clist_get_text(GTK_CLIST(m_pList), nIndex, 0, &name);

	if (name)
	{
		strFile = name;
	}
	else
	{
		GdkPixmap *pix;
		GdkBitmap *bmp;
		guint8 spacing;

		gtk_clist_get_pixtext(GTK_CLIST(m_pList), nIndex, 0, &name, &spacing, &pix, &bmp);

		strFile = name;
	}
}

void CVermFileView::SetWaitState(int state)
{
	if (state == 0)
	{
//		gtk_widget_set_sensitive(m_pWidget, TRUE);
		gtk_widget_set_sensitive(m_pList, TRUE);
	}
	else if (state == 1)
	{
//		gtk_widget_set_sensitive(m_pWidget, FALSE);
		gtk_widget_set_sensitive(m_pList, FALSE);
	}		
}

void CVermFileView::GetColumnWidths(int *columns)
{
	GtkCList* list = GTK_CLIST(m_pList);

	columns[0] = list->column[0].width;
	columns[1] = list->column[1].width;
	columns[2] = list->column[2].width;
	columns[3] = list->column[3].width;
	columns[4] = list->column[4].width;
}

void CVermFileView::RemoveFile(CVermTreeLeaf* pLeaf)
{
	int i = gtk_clist_find_row_from_data(GTK_CLIST(m_pList), pLeaf);

	if (i >= 0)
		gtk_clist_remove(GTK_CLIST(m_pList), i);
}
