/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#ifndef VERMFRAMEWND_H
#define VERMFRAMEWND_H

#include <gtk/gtk.h>

#include <String.h>
#include "VermEdit.h"

typedef struct _FileDiff
{
	String strTmpDir;
	String strSourceTmpDir;
	
	String strFileOne;
	String strFileTwo;

	String strFileOneVersion;
	String strFileTwoVersion;

	short bProjectDiff;
	short bRecurse;
	String strProject;
	
	short bHaveFileOne;
	short bHaveFileTwo;

	short bOneIsTempFile;
	short bTwoIsTempFile;
} FileDiff;

class CVermProjectView;
class CVermFileView;
class CVermMessageView;
class CVermSocket;
class CVermCrypto;
class CVermProtocolMessage;
class CVermTreeNode;
class CVermTreeLeaf;
class CHistoryDlg;
class CShareDlg;
class CSearchResultsDlg;

#define FOCUS_PROJECTVIEW	1
#define FOCUS_FILEVIEW		2
#define FOCUS_MESSAGEAREA	3

#define FILE_CONNECT_TO_SERVER                    1
#define TOOLS_IMPORT_ENCRYPTION_KEY               2
#define FILE_EXIT                                 3
#define FILE_CREATE_PROJECT                       4
#define FILE_ADD_FILES                            5
#define FILE_SET_WORKING_FOLDER                   6
#define FILE_DELETE                               7
#define FILE_RENAME                               8
#define FILE_LABEL                                9
#define VIEW_REFRESH_FILE_LIST                   10
#define VIEW_REFRESH_PROJECT_TREE                11
#define SOURCE_GET_LATEST_VERSION                12
#define SOURCE_CHECK_OUT                         13
#define SOURCE_CHECK_IN                          14
#define SOURCE_UNDO_CHECK_OUT                    15
#define TOOLS_SHOW_HISTORY                       16
#define TOOLS_SHOW_DIFFERENCES                   17
#define TOOLS_OPTIONS                            18
#define HELP_ABOUT                               19
#define SOURCE_SHARE                             20
#define HELP_SUPPORT                             21
#define FILE_VIEW                                22
#define FILE_EDIT                                23
#define TOOLS_WEB_DEPLOY                         24
#define TOOLS_CLOAK_PROJECT                      25
#define TOOLS_STOP                               26
#define SOURCE_BRANCH                            27
#define VIEW_SEARCH                              28
#define TOOLS_CHANGE_PASSWORD					 29
#define VIEW_CLEAR_STATUS						 30

class CVermFrameWnd
{
 public:
	GtkWidget* m_pWidget;
	GtkWidget* m_pToolbar;
	GtkWidget* m_pMenubar;
	GtkWidget* m_pVPane;
	GtkWidget* m_pHPane;
	GtkItemFactory* m_pItemFactory;

	// Toolbar button widgets
	GtkWidget* m_pCreateProject;
	GtkWidget* m_pAddFiles;
	GtkWidget* m_pLabel;
	GtkWidget* m_pDelete;
	GtkWidget* m_pGetLatestVersion;
	GtkWidget* m_pCheckout;
	GtkWidget* m_pCheckin;
	GtkWidget* m_pUndoCheckout;
	GtkWidget* m_pShare;
	GtkWidget* m_pShowDifferences;
	GtkWidget* m_pShowHistory;
	GtkWidget* m_pSetWorkingFolder;
	GtkWidget* m_pRefreshFileList;
	GtkWidget* m_pRefreshProjectTree;
	GtkWidget* m_pStop;
	GtkWidget* m_pBranch;
	GtkWidget* m_pView;
	GtkWidget* m_pEdit;
	
	// Menu widgets
	GtkWidget* m_pMCreateProject;
	GtkWidget* m_pMAddFiles;
	GtkWidget* m_pMLabel;
	GtkWidget* m_pMDelete;
	GtkWidget* m_pMRename;
	GtkWidget* m_pMGetLatestVersion;
	GtkWidget* m_pMCheckout;
	GtkWidget* m_pMCheckin;
	GtkWidget* m_pMUndoCheckout;
	GtkWidget* m_pMShare;
	GtkWidget* m_pMBranch;
	GtkWidget* m_pMShowDifferences;
	GtkWidget* m_pMShowHistory;
	GtkWidget* m_pMSetWorkingFolder;
	GtkWidget* m_pMRefreshFileList;
	GtkWidget* m_pMRefreshProjectTree;
	GtkWidget* m_pMSearch;
	GtkWidget* m_pMWebDeploy;
	GtkWidget* m_pMCloakProject;
	GtkWidget* m_pMStop;
	GtkWidget* m_pMEdit;
	GtkWidget* m_pMView;
	GtkWidget* m_pMPassword;
	GtkWidget* m_pMClearStatus;
	
	String m_username;
	String m_password;
	String m_database;
	String m_databaseAlias;
	String m_server;

	// var used to save state for recursive checkins
	String   m_strComment;

	short    m_bKeepCheckedOut;
	short    m_bRemoveLocalCopy;
	short    m_bSilentLogin;
	
	int   m_port;
	CVermProjectView* m_pProjectView;
	CVermFileView* m_pFileView;
	CVermMessageView *m_pMessageView;
	CVermSocket* m_pSocket;

	int m_nWidth;
	int m_nHeight;
	int m_nCurrentCommand;
	
 private:
	String m_strLastFileForProjectDiff;
	String m_strLastProjectForProjectDiff;
	
	short m_bToolbarActive;
	
	int m_modState;
	int m_lockState;
	int m_writeState;

	int m_nWaitCount;
	int m_nChildWithFocus;
	short m_bDuringCreation;
	int m_nTimerID;
	int m_nKeepAliveTimerID;
	int m_bPrefsSaved;

	int m_destroySig;
	
	GdkCursor *m_WaitCursor;
	GdkCursor *m_NormalCursor;

	FileDiff* m_pFileDiff;
	FileEdit* m_pFileEdit;
	
	CHistoryDlg* m_pHistoryDlg;
	CShareDlg*   m_pShareDlg;
	CSearchResultsDlg* m_pSearchResultsDlg;
	
	CVermCrypto* m_pCrypto;

	
	short m_bConnected;

	
public:
	CVermFrameWnd();
	~CVermFrameWnd();

	void OnViewClearStatus(void);
	void OnToolsChangePassword(void);
	void OnActivate(void);
	void OnSourceBranch(void);
	void OnViewSearch(void);
	void OnSourceShare(void);
	void OnFileRename(void);
	void OnFileLabel(void);
	void OnViewRefreshProjectTree(void);
	void OnToolsStop(void);
	void OnToolsWebDeploy(void);
	void OnToolsCloakProject(void);
	void OnToolsOptions(void);
	void OnToolsShowDifferences(void);
	void OnToolsShowHistory(void);
	void OnSourceUndoCheckout(void);
	void OnSourceCheckIn(void);
	void OnViewRefreshFileList(void);
	void OnFileDelete(void);
	void OnSourceCheckOut(void);
	void OnSourceGetLatestVersion(void);
	void OnFileView(int);
	void OnFileEdit(int);
	void OnFileAddFiles(void);
	void OnFileCreateProject(void);
	void OnFileSetWorkingFolder(void);
	void OnFileConnectToServer(void);
	void OnToolsImportEncryptionKey(void);
	void OnFileExit(void);
	void MessageReceived(CVermProtocolMessage*, CVermProtocolMessage*, int);
	void SetWaitState(int);
	void AddMessage(String& str);
	void ConnectToServer(String& server, int port);
	void ProjectItemSelected(String& strSelectedItem);
	void UpdateFileView(CVermTreeNode* pNode, String& strWorkingDir,
						CVermTreeLeaf* pLeaf, int bUpdateWorkingDirPane = 0);
	void SetPercentDone(double dPercentage);
	void SetFocus(int which);
	void GetDiffFiles(FileDiff* pFileDiff, String& pszProject);
	void HistoryDlgGone(void);
	void ShareDlgGone(void);
	void SearchResultsDlgGone(void);
	void StartUp(void);
	void UpdateNodeDisplay(CVermTreeNode* pNode, String& strWorkingDir);
	void UpdateFileInfo(CVermProtocolMessage* pMessage);
	void ServerShutdown(String& strMessage);
	void SavePrefs(void);
	int IsSafeToSave(void);
	int GetFile(String& strProject, String& strFile, short bMakeWritable, int nVerNum = 0);
	void GetProjectByLabel(String& strProject, String& strLabel);
	void SetWorkingDirectory(String& strWorkingDir);
	int CheckOutFile(CVermTreeNode* pNode, String& strProject, String& strWorkingDir, String& strFile, int bEditFile);
	void GetEditFile(FileEdit* pFileEdit, const char* pszProject);
	int RequestChecksums(String& strProject);
	void ViewFile(String& strProject, String& strFile);
	void DiffFile(String& strWorkingDir, String& strProject, String& strFile);
	void CheckInFile(CVermTreeNode* pNode, String& strProject,
					 String& strWorkingDir,
					 String& strFile, String& strDlgComment,
					 int bRemoveLocalCopy, int bKeepCheckedOut, int bLocked = 0);
	void UndoCheckout(CVermTreeNode* pNode, String& strWorkingDir,
					  String& strProject, String& strFile, const char* pCookie, int bLocked = 0);
	void GetHistory(String& strProject, String& strFile);
	
private:
	void BranchFile(String& strProject, String& strFile, String& strWorkingDir, String& strComment);
	void BranchFilesForNode(String& strProject);
	void ProjectDiffFinished(void);
	void ResetToolbar(short);
	int SetWorkingFolder(void);
	int SetWorkingFolderForProject(String&);
	int FinishWorkingFolder(String&, String&);
	
	void AddAllFiles(String& strProject, String& strDir,
					 String& strComment, int recurse, short bCheckOutFiles);

	void CreateProjects(String& strProject, String& strDir,
						String& strComment, int recurse, short bLockFiles);

	void CreateProject(String& strProject, String& strComment, String& strCookie);
	char* ConvertServerName(const char* pServer);
	void CompressMessageBody(CVermProtocolMessage* pMessage);
	void UndoCheckoutForNode(CVermTreeNode* pNode, String& strWorkingDir, String& strProject, const char* pCookie);
	void ReceivedDiffFile(CVermProtocolMessage* pMessageReceived);
	void CleanupFileDiff(void);
	void GetProjectListings(GtkWidget* hItem, char *cookie);
	void CheckinFilesForNode(CVermTreeNode* pNode,
							 String& strWorkingDir,
							 String& strProject,
							 String& strComment,
							 int bRemoveLocalCopy,
							 int bKeepCheckedOut);
	int FetchEntireProject(String& strWorkingDir);
	void FetchNodeFiles(CVermTreeNode* pNode,
						String& strWorkingDir,
						const char* pszProject,
						int bLockFiles, int bDontOverwrite, int bMakeWritable);

	void GetDiffFilesForProject(CVermTreeNode* pNode, String& strDir, String& strProject);
	void CopyFile(String& strLocalFile, String& strTmpFile);

};

#endif // VERMFRAMEWND_H
