/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <time.h>
#include <signal.h>
#include <stdio.h>

#include "Vermie.h"
#include "VermSocket.h"
#include "VermFrameWnd.h"
#include "VermProjectView.h" // SERVER_DIRECTORY_NAME def

int Vermie::m_nWidth = 0;
int Vermie::m_nHeight = 0;

int Vermie::m_bPinAvailable = 0;

GtkStyle* Vermie::m_pCloakedStyle;
GtkStyle* Vermie::m_pWebDeployStyle;
GtkStyle* Vermie::m_pCloakedDeployStyle;
GtkStyle* Vermie::m_pNormalStyle;
	
GdkColor Vermie::m_CloakedColor = { 0, 0xffff, 0, 0 };
GdkColor Vermie::m_DeployColor = { 0, 0, 0xffff, 0};
GdkColor Vermie::m_CloakedDeployColor = { 0, 0, 0, 0xffff };

GdkPixmap* Vermie::m_pShared = NULL;
GdkBitmap* Vermie::m_pSharedMask = NULL;
GdkPixmap* Vermie::m_pPin = NULL;
GdkBitmap* Vermie::m_pPinMask = NULL;

short Vermie::m_bUseCompression = 1;
short Vermie::m_bCanUseCompression = 1;
short Vermie::m_bUpdateFileList = 0;
short Vermie::m_bSendKeepAlive = 0;
short Vermie::m_bRemoveLocalCopy = 0;
short Vermie::m_bDontFetchFileOnCheckout = 0;
short Vermie::m_bAlwaysKeepFilesCheckedOut = 0;
short Vermie::m_bRecurseOnProjects = 0;
short Vermie::m_bDontFetchFileOnUndoCheckout = 0;
short Vermie::m_bMakeFilesReadOnly = 1;
short Vermie::m_bUseHTTPProxy = 0;
short Vermie::m_bUseVSS6Features = 0;
short Vermie::m_bShowShareIcon = 1;
short Vermie::m_bUseChecksums = 0;
short Vermie::m_bDisplayPinStatus = 0;

short Vermie::m_bProjectGetLatestVersion = 1;
short Vermie::m_bProjectCheckout = 1;
short Vermie::m_bProjectUndoCheckout = 1;
short Vermie::m_bProjectDeploy = 1;

short Vermie::m_bFileGetLatestVersion = 1;
short Vermie::m_bFileCheckout = 1;
short Vermie::m_bFileUndoCheckout = 1;
short Vermie::m_bFileEditAndView = 1;

TimeStamps Vermie::m_timeStamps;

String Vermie::m_strServerList;
String Vermie::m_strPortList;
String Vermie::m_strLastPort;
String Vermie::m_strLastServer;
String Vermie::m_strUserName;
String Vermie::m_strProxyName;
String Vermie::m_strPrefsDir;
String Vermie::m_strDatabaseName;
String Vermie::m_strDiffProgram;
String Vermie::m_strMergeProgram;
String Vermie::m_strProjectDiffProgram;
String Vermie::m_strTimeToRefresh;
String Vermie::m_strTimeToKeepAlive;
String Vermie::m_strViewProgram;
String Vermie::m_strFileDoubleClick("Ask");
String Vermie::m_strFileEOL(UNIX_EOL);

int Vermie::m_ProxyPort = 0;
int Vermie::m_nSortColumn = 0;
int Vermie::m_nColumnWidths[5] = { 150, 100, 150, 150, 50 };
int Vermie::m_nLinesToDisplay = 200;
int Vermie::m_nServersToKeep = 5;

int Vermie::m_PViewWidth = 200;
int Vermie::m_PViewHeight = 300;
int Vermie::m_FViewWidth = 450;
int Vermie::m_FViewHeight = 300;
int Vermie::m_MViewWidth = 650;
int Vermie::m_MViewHeight = 100;


static void loadPrefs(void);
void ParseServerPrefs(String& file);
static int SplitBuf(char* buf, String& name, String& value);

int main(int argc, char **argv)
{
	signal(SIGPIPE, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);
	
	gtk_init (&argc, &argv);
          
	loadPrefs();
	
	CVermFrameWnd* pFrameWnd = new CVermFrameWnd();
	
	gtk_window_set_title (GTK_WINDOW (pFrameWnd->m_pWidget), "SourceOffSite");
	gtk_signal_connect (GTK_OBJECT (pFrameWnd->m_pWidget), "destroy",
						GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
	//	gtk_container_border_width (GTK_CONTAINER (pFrameWnd->m_pWidget), 10);
          
	gtk_widget_show (pFrameWnd->m_pWidget);

	Vermie::m_timeStamps.startClock = clock();
	Vermie::m_timeStamps.startTime = time(NULL);

	pFrameWnd->StartUp();
	
	gtk_main ();

	delete pFrameWnd;
	
	return 0;
}

static void loadPrefs(void)
{
	Vermie::m_strPrefsDir = getenv("SOSHOME");

	if (Vermie::m_strPrefsDir.empty())
	{
		Vermie::m_strPrefsDir = getenv("HOME");

		if (Vermie::m_strPrefsDir.empty())
		{
			return;
		}

		Vermie::m_strPrefsDir += "/.sos";
	}
	
	// check to make sure the dir exists ...
	DIR* d = opendir(Vermie::m_strPrefsDir.chars());

	if (!d)
	{
		mkdir(Vermie::m_strPrefsDir.chars(), 0700);
	}

	String strFile = Vermie::m_strPrefsDir + "/prefs";

	FILE *f = fopen(strFile.chars(), "r");

	if (!f)
	{
		return;
	}

	// load prefs ...
	char buf[1024];
	while (fgets(buf, 1024, f))
	{
		String name, value;

		if (!SplitBuf(buf, name, value))
		{
			if (!fcompare(name, "projectgetlatestversion"))
				Vermie::m_bProjectGetLatestVersion = atoi(value.chars());
			else if (!fcompare(name, "eol"))
				Vermie::m_strFileEOL = value;
			else if (!fcompare(name, "projectcheckout"))
				Vermie::m_bProjectCheckout = atoi(value.chars());
			else if (!fcompare(name, "projectundocheckout"))
				Vermie::m_bProjectUndoCheckout = atoi(value.chars());
			else if (!fcompare(name, "projectdeploy"))
				Vermie::m_bProjectDeploy = atoi(value.chars());
			else if (!fcompare(name, "filegetlatestversion"))
				Vermie::m_bFileGetLatestVersion = atoi(value.chars());
			else if (!fcompare(name, "filecheckout"))
				Vermie::m_bFileCheckout = atoi(value.chars());
			else if (!fcompare(name, "fileundocheckout"))
				Vermie::m_bFileUndoCheckout = atoi(value.chars());
			else if (!fcompare(name, "fileeditandview"))
				Vermie::m_bFileEditAndView = atoi(value.chars());
			else if (!fcompare(name, "usechecksums"))
			{
				Vermie::m_bUseChecksums = atoi(value.chars());
			}
			else if (!fcompare(name, "displaypinstatus"))
			{
				Vermie::m_bDisplayPinStatus = atoi(value.chars());
			}
			else if (!fcompare(name, "showshareicon"))
			{
				Vermie::m_bShowShareIcon = atoi(value.chars());
			}
			else if (!fcompare(name, "cloakedcolor"))
			{
				int r, g, b;

				if (3 == sscanf(value.chars(), "%i, %i, %i", &r, &g, &b))
				{
					Vermie::m_CloakedColor.red = r;
					Vermie::m_CloakedColor.green = g;
					Vermie::m_CloakedColor.blue = b;
				}
			}
			else if (!fcompare(name, "deploycolor"))
			{
				int r, g, b;

				if (3 == sscanf(value.chars(), "%i, %i, %i", &r, &g, &b))
				{
					Vermie::m_DeployColor.red = r;
					Vermie::m_DeployColor.green = g;
					Vermie::m_DeployColor.blue = b;
				}
			}
			else if (!fcompare(name, "cloakeddeploycolor"))
			{
				int r, g, b;

				if (3 == sscanf(value.chars(), "%i, %i, %i", &r, &g, &b))
				{
					Vermie::m_CloakedDeployColor.red = r;
					Vermie::m_CloakedDeployColor.green = g;
					Vermie::m_CloakedDeployColor.blue = b;
				}
			}
			if (!fcompare(name, "serverlist"))
			{
				Vermie::m_strServerList = value;

			}
			else if (!fcompare(name, "portlist"))
			{
				Vermie::m_strPortList = value;
			}
			else if (!fcompare(name, "lastport"))
			{
				Vermie::m_strLastPort = value;
			}
			else if (!fcompare(name, "lastserver"))
			{
				Vermie::m_strLastServer = value;

				if (!value.empty())
				{
					String serverFile = Vermie::m_strPrefsDir + "/";
					serverFile += SERVER_DIRECTORY_NAME;
					serverFile += "/";
					serverFile += value;
					serverFile += "/prefs";
					
					ParseServerPrefs(serverFile);
				}
			}
			else if (!fcompare(name, "diffprogram"))
			{
				Vermie::m_strDiffProgram = value;
			}
			else if (!fcompare(name, "mergeprogram"))
			{
				Vermie::m_strMergeProgram = value;
			}
			else if (!fcompare(name, "projectdiffprogram"))
			{
				Vermie::m_strProjectDiffProgram = value;
			}
			else if (!fcompare(name, "sortcolumn"))
			{
				Vermie::m_nSortColumn = atoi(value.chars());
				if (Vermie::m_nSortColumn < 0 || Vermie::m_nSortColumn > 4)
					Vermie::m_nSortColumn = 0;
			}
			else if (!fcompare(name, "proxyserver"))
			{
				Vermie::m_strProxyName = value;
			}
			else if (!fcompare(name, "proxyport"))
			{
				Vermie::m_ProxyPort = atoi(value.chars());
			}
			else if (!fcompare(name, "makefilesreadonly"))
			{
				Vermie::m_bMakeFilesReadOnly = atoi(value.chars());
			}
			else if (!fcompare(name, "refreshtime"))
			{
				Vermie::m_strTimeToRefresh = value;
			}
			else if (!fcompare(name, "keepalivetime"))
			{
				Vermie::m_strTimeToKeepAlive = value;
			}
			else if (!fcompare(name, "linestodisplay"))
			{
				Vermie::m_nLinesToDisplay = atoi(value.chars());
			}
			else if (!fcompare(name, "updatefilelist"))
			{
				Vermie::m_bUpdateFileList = atoi(value.chars());
			}
			else if (!fcompare(name, "sendkeepalive"))
			{
				Vermie::m_bSendKeepAlive = atoi(value.chars());
			}
			else if (!fcompare(name, "windowwidth"))
				Vermie::m_nWidth = atoi(value.chars());
			else if (!fcompare(name, "windowheight"))
				Vermie::m_nHeight = atoi(value.chars());
			else if (!fcompare(name, "column1width"))
				Vermie::m_nColumnWidths[0] = atoi(value.chars());
			else if (!fcompare(name, "column2width"))
				Vermie::m_nColumnWidths[1] = atoi(value.chars());
			else if (!fcompare(name, "column3width"))
				Vermie::m_nColumnWidths[2] = atoi(value.chars());
			else if (!fcompare(name, "column4width"))
				Vermie::m_nColumnWidths[3] = atoi(value.chars());
			else if (!fcompare(name, "column5width"))
				Vermie::m_nColumnWidths[4] = atoi(value.chars());
			else if (!fcompare(name, "pviewwidth"))
				Vermie::m_PViewWidth = atoi(value.chars());
			else if (!fcompare(name, "pviewheight"))
				Vermie::m_PViewHeight = atoi(value.chars());
			else if (!fcompare(name, "fviewwidth"))
				Vermie::m_FViewWidth = atoi(value.chars());
			else if (!fcompare(name, "fviewheight"))
				Vermie::m_FViewHeight = atoi(value.chars());
			else if (!fcompare(name, "mviewwidth"))
				Vermie::m_MViewWidth = atoi(value.chars());
			else if (!fcompare(name, "mviewheight"))
				Vermie::m_MViewHeight = atoi(value.chars());
			else if (!fcompare(name, "dontfetchoncheckout"))
				Vermie::m_bDontFetchFileOnCheckout = atoi(value.chars());
			else if (!fcompare(name, "alwayskeepcheckedout"))
				Vermie::m_bAlwaysKeepFilesCheckedOut = atoi(value.chars());
			else if (!fcompare(name, "recurseonprojects"))
				Vermie::m_bRecurseOnProjects = atoi(value.chars());
			else if (!fcompare(name, "removelocalcopy"))
				Vermie::m_bRemoveLocalCopy = atoi(value.chars());
			else if (!fcompare(name, "viewprogram"))
				Vermie::m_strViewProgram = value;
			else if (!fcompare(name, "usehttpproxy"))
			{
				Vermie::m_bUseHTTPProxy = atoi(value.chars());
			}				
			else if (!fcompare(name, "filedoubleclick"))
			{
				if (value == "Ask" || value == "View File" || value == "Edit File")
					Vermie::m_strFileDoubleClick = value;
				else
					Vermie::m_strFileDoubleClick = "Ask";
			}
		}
	}
}

static int SplitBuf(char* buf, String& name, String& value)
{
	name = buf;

	int nIndex = name.index(':');

	if (nIndex < 0)
		return -1;

	value = name.after(nIndex);
	if (value[value.length() - 1] == '\n')
		value = value.before((int)value.length() - 1);

	name = name.before(nIndex);

	return 0;
}

void ParseServerPrefs(String& file)
{
	FILE* f = fopen(file.chars(), "r");

	if (!f)
	{
		return;
	}

	char buf[2048];

	while (fgets(buf, 2048, f))
	{
		String name, value;
		
		if (!SplitBuf(buf, name, value))
		{
			if (!fcompare(name, "usecompression"))
				Vermie::m_bUseCompression = atoi(value.chars());
			else if (!fcompare(name, "username"))
				Vermie::m_strUserName = value;
			else if (!fcompare(name, "database"))
				Vermie::m_strDatabaseName = value;
		}
	}
}



