/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdio.h>

#include "Vermie.h"
#include "VermProtocolHeader.h"
#include "VermProtocolMessage.h"
#include "VermMessageStore.h"

CVermMessageStore::CVermMessageStore()
{
	m_pMessages = NULL;
	m_nCounter = 0;
}

void CVermMessageStore::Empty(void)
{
	MessageLink* pMessage = m_pMessages;

	while (pMessage)
	{
		MessageLink* tmp = pMessage;

		pMessage = pMessage->pNext;

		delete tmp;
	}

	m_pMessages = NULL;
}

CVermMessageStore::~CVermMessageStore()
{
	Empty();
}

void CVermMessageStore::AddMessage(CVermProtocolMessage *pMessage)
{
	if (m_pMessages == NULL)
	{
		m_pMessages = new MessageLink;
		m_pTailMessage = m_pMessages;
	}
	else
	{
		m_pTailMessage->pNext = new MessageLink;
		m_pTailMessage = m_pTailMessage->pNext;
	}

	m_pTailMessage->nMessageNum = m_nCounter++;
	m_pTailMessage->pNext = NULL;
	m_pTailMessage->pMessage = pMessage;

	char strNum[256];

	sprintf(strNum, "%d", m_pTailMessage->nMessageNum);
	
	pMessage->AddHeader(new CVermProtocolHeader("Message-ID", strNum));
}

CVermProtocolMessage* CVermMessageStore::FindMessage(int num)
{
	MessageLink *pMessage = m_pMessages;

	while (pMessage)
	{
		if (pMessage->nMessageNum == num)
			return pMessage->pMessage;

		pMessage = pMessage->pNext;
	}

	return NULL;
}

int CVermMessageStore::RemoveMessage(int num)
{
	MessageLink *pMessage = m_pMessages;
	MessageLink *pPrev = NULL;

	while (pMessage)
	{
		if (pMessage->nMessageNum == num)
		{
			if (pPrev == NULL)  // first node on the list
			{
				m_pMessages = m_pMessages->pNext;
			}
			else
			{
				pPrev->pNext = pMessage->pNext;

				if (pPrev->pNext == NULL)
					m_pTailMessage = pPrev;
			}

			delete pMessage->pMessage;

			delete pMessage;

			return 0;
		}

		pPrev = pMessage;
		pMessage = pMessage->pNext;
	}

	return -1;
}
