/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <gtk/gtk.h>

#include "Vermie.h"
#include "AfxMessageBox.h"
#include "VermFrameWnd.h"
#include "VermProjectView.h"
#include "VermUtils.h"
#include "VermTreeNode.h"
#include "VermTreeLeaf.h"
#include "VermProtocolMessage.h"
#include "VermProtocolHeader.h"
#include "VermTreeUtils.h"
#include "YesNoDlg.h"


// Yes, I'm ashamed
static CVermProjectView* s_pProject;

static void MenuItemSelected(GtkWidget*, int which)
{
//	m_pParent->SetFocus(FOCUS_PROJECTVIEW);
	
	switch(which)
	{
	case TOOLS_SHOW_DIFFERENCES:
		s_pProject->m_pParent->OnToolsShowDifferences();
		break;
		
	case TOOLS_CLOAK_PROJECT:
		s_pProject->m_pParent->OnToolsCloakProject();
		break;
		
	case TOOLS_SHOW_HISTORY:
		s_pProject->m_pParent->OnToolsShowHistory();
		break;
		
	case FILE_CREATE_PROJECT:
		s_pProject->m_pParent->OnFileCreateProject();
		break;

	case FILE_ADD_FILES:
		s_pProject->m_pParent->OnFileAddFiles();
		break;
		
	case FILE_SET_WORKING_FOLDER:
		s_pProject->m_pParent->OnFileSetWorkingFolder();
		break;

	case FILE_DELETE:
		s_pProject->m_pParent->OnFileDelete();
		break;

	case FILE_RENAME:
		s_pProject->m_pParent->OnFileRename();
		break;

	case FILE_LABEL:
		s_pProject->m_pParent->OnFileLabel();
		break;

	case VIEW_REFRESH_PROJECT_TREE:
		s_pProject->m_pParent->OnViewRefreshProjectTree();
		break;

	case SOURCE_GET_LATEST_VERSION:
		s_pProject->m_pParent->OnSourceGetLatestVersion();
		break;

	case SOURCE_CHECK_IN:
		s_pProject->m_pParent->OnSourceCheckIn();
		break;

	case SOURCE_CHECK_OUT:
		s_pProject->m_pParent->OnSourceCheckOut();
		break;

	case SOURCE_UNDO_CHECK_OUT:
		s_pProject->m_pParent->OnSourceUndoCheckout();
		break;

	case SOURCE_SHARE:
		s_pProject->m_pParent->OnSourceShare();
		break;
		
	default:
		break;
	}		
}


static void SelectionChanged(GtkWidget*, CVermProjectView* pView)
{
	GList* sel = GTK_TREE_SELECTION(pView->m_pTree);

	if (!sel)
		return;
	
	pView->m_hCurSelItem = GTK_WIDGET(sel->data);
	
	pView->OnSelChanged();
}

static void FocusChanged(GtkWidget*, GdkEventFocus*, CVermProjectView* pView)
{
	pView->m_pParent->SetFocus(FOCUS_PROJECTVIEW);
}

void CVermProjectView::OnFocus(void)
{
	m_pParent->SetFocus(FOCUS_PROJECTVIEW);
}

void CVermProjectView::OnSelChanged(void)
{
	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(m_hCurSelItem));

	
	if (pNode->m_bVisited)
	{
		String strWorkingDir = GetWorkingDirectory(m_hCurSelItem);
		m_pParent->UpdateFileView(pNode, strWorkingDir, NULL, TRUE);
		if (pNode->m_bCloaked)
		{
			gtk_label_set(GTK_LABEL(GTK_BIN(m_pCloak)->child), "UnCloak Project");
			gtk_label_set(GTK_LABEL(GTK_BIN(m_pParent->m_pMCloakProject)->child), "UnCloak Project");
		}
		else
		{
			gtk_label_set(GTK_LABEL(GTK_BIN(m_pCloak)->child), "Cloak Project");
			gtk_label_set(GTK_LABEL(GTK_BIN(m_pParent->m_pMCloakProject)->child), "Cloak Project");
		}
	}
	else
	{
		String strNode;
		GetPathSelectedObject(strNode);
		m_pParent->ProjectItemSelected(strNode);
	}
}


CVermProjectView::CVermProjectView(CVermFrameWnd* pParent)
{
	s_pProject = this;
	
	m_pParent = pParent;
	m_bWaitState = 0;
	m_hCurSelItem = NULL;
	
	m_pWidget = gtk_scrolled_window_new(NULL, NULL);

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_pWidget),
								 GTK_POLICY_AUTOMATIC,
								 GTK_POLICY_AUTOMATIC);

	gtk_widget_set_usize(m_pWidget, Vermie::m_PViewWidth,
						 Vermie::m_PViewHeight);

	m_pTree = gtk_tree_new();
	gtk_tree_set_selection_mode(GTK_TREE(m_pTree), GTK_SELECTION_BROWSE);
	
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(m_pWidget), m_pTree);
	
	gtk_widget_show(m_pTree);

//	gtk_signal_connect(GTK_OBJECT(m_pTree), "button_press_event",
//					   GTK_SIGNAL_FUNC(cb_focus), this);

//	gtk_widget_set_events(m_pTree, GDK_BUTTON_PRESS_MASK);
	
	gtk_signal_connect (GTK_OBJECT(m_pTree), "selection_changed",
						GTK_SIGNAL_FUNC(SelectionChanged), this);	

    // The popup menu
	m_popupMenu = gtk_menu_new();
	
	GtkWidget* menuitem = gtk_menu_item_new_with_label("Create Project ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_CREATE_PROJECT);

	menuitem = gtk_menu_item_new_with_label("Add Files ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_ADD_FILES);

	menuitem =gtk_menu_item_new_with_label("Set Working Folder ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_SET_WORKING_FOLDER);

	GtkWidget* sep = gtk_menu_item_new();
	gtk_widget_show(sep);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep);

	menuitem = gtk_menu_item_new_with_label("Delete ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_DELETE);

	menuitem = gtk_menu_item_new_with_label("Rename ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_RENAME);

	menuitem = gtk_menu_item_new_with_label("Label ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)FILE_LABEL);

	GtkWidget* sep2 = gtk_menu_item_new();
	gtk_widget_show(sep2);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep2);

	menuitem = gtk_menu_item_new_with_label("Get Latest Version ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_GET_LATEST_VERSION);

	menuitem = gtk_menu_item_new_with_label("Check Out ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_CHECK_OUT);

	menuitem = gtk_menu_item_new_with_label("Check In ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_CHECK_IN);

	menuitem = gtk_menu_item_new_with_label("Undo Checkout ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_UNDO_CHECK_OUT);

	GtkWidget* sep3 = gtk_menu_item_new();
	gtk_widget_show(sep3);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep3);

	menuitem = gtk_menu_item_new_with_label("Share ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)SOURCE_SHARE);

	menuitem = gtk_menu_item_new_with_label("Show History ...");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)TOOLS_SHOW_HISTORY);

	menuitem = gtk_menu_item_new_with_label("Show Differences");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)TOOLS_SHOW_DIFFERENCES);

	GtkWidget* sep5 = gtk_menu_item_new();
	gtk_widget_show(sep5);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep5);

	m_pCloak = gtk_menu_item_new_with_label("Cloak");
	gtk_widget_show(m_pCloak);
	gtk_menu_append(GTK_MENU (m_popupMenu), m_pCloak);
	gtk_signal_connect(GTK_OBJECT(m_pCloak), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)TOOLS_CLOAK_PROJECT);

	GtkWidget* sep4 = gtk_menu_item_new();
	gtk_widget_show(sep4);
	gtk_menu_append(GTK_MENU(m_popupMenu), sep4);

	menuitem = gtk_menu_item_new_with_label("Refresh Project Tree");
	gtk_widget_show(menuitem);
	gtk_menu_append(GTK_MENU (m_popupMenu), menuitem);
	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
					   GTK_SIGNAL_FUNC (MenuItemSelected),
					   (gpointer)VIEW_REFRESH_PROJECT_TREE);
}

CVermProjectView::~CVermProjectView()
{
}

void CVermProjectView::SetOpenProject(char* pProject)
{
	if (!pProject)
		return;

	GtkWidget* hItem = GetTreeItemFromProject(GetRootItem(), pProject);

	if (!hItem)
		return;

	m_hCurSelItem = hItem;
	
	gtk_tree_select_child(GTK_TREE(m_pTree), hItem);

	GtkTree* tree = NULL;

	if (hItem->parent)
		tree = GTK_TREE(hItem->parent);

	
	GtkTreeItem* item = NULL;

	if (tree && tree->tree_owner)
		item = GTK_TREE_ITEM(tree->tree_owner);

	while (item != NULL)
	{
		gtk_tree_item_expand(item);

		if (tree)
		{
			if (GTK_WIDGET(tree)->parent)
				tree = GTK_TREE(GTK_WIDGET(tree)->parent);
			else
				tree = NULL;

			if (tree && tree->tree_owner)
				item = GTK_TREE_ITEM(tree->tree_owner);
			else
				item = NULL;
		}
		else
			item = NULL;
	}
}

void CVermProjectView::SetWaitState(int bState)
{
	m_bWaitState = bState;

	if (m_bWaitState)
	{
		if (m_pWidget)
			gtk_widget_set_sensitive(m_pWidget, FALSE);
	}
	else
	{
		if (m_pWidget)
			gtk_widget_set_sensitive(m_pWidget, TRUE);
	}

}

void CVermProjectView::Reset()
{
	CleanTree(m_pTree);
	
	gtk_tree_clear_items(GTK_TREE(m_pTree), 0, g_list_length(GTK_TREE(m_pTree)->children));

	m_hCurSelItem = NULL;
}

void CVermProjectView::SetProjectTree(char* pMessageBody, int nBodySize)
{
	int pos = 0;

	GtkTree* pTree = GTK_TREE(m_pTree);

	GList* children = gtk_container_children(GTK_CONTAINER(m_pTree));
	
	if (children)
	{
		gtk_tree_remove_items(GTK_TREE(pTree), children);
		m_hCurSelItem = NULL;
		g_list_free(children);
	}

	ParseTree(m_pTree, pMessageBody, &pos, nBodySize - 1, m_popupMenu, this);

//	if (!treeCtrl.GetSelectedItem())
//	{
//		treeCtrl.Expand(treeCtrl.GetRootItem(), TVE_EXPAND);
//
//		treeCtrl.SelectItem(treeCtrl.GetRootItem());
//	}
}

void CVermProjectView::UpdateProjectTree(char* pBody, GtkWidget* hItem, int* pos, int maxPos)
{
	CVermTreeNode* pTreeNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

	pTreeNode->m_bChecked = TRUE;

	int bParseError;

	char* nodeName = ParseNameFromBuffer(pBody, pos, maxPos, bParseError);

	while (nodeName != NULL)
	{
		String strNode = nodeName;
		
		GtkWidget* hChild = GetTreeChildByName(hItem, strNode);

		if (!hChild)
		{
			GtkWidget* subTree = GTK_TREE_ITEM(hItem)->subtree;

			if (!subTree)
			{
				subTree = gtk_tree_new();
				gtk_tree_item_set_subtree(GTK_TREE_ITEM(hItem), subTree);
				gtk_widget_show(subTree);
			}
			
			hChild = gtk_tree_item_new_with_label(nodeName);
			gtk_signal_connect(GTK_OBJECT(hChild), "button_press_event",
							   GTK_SIGNAL_FUNC (ButtonPress), m_popupMenu);
			gtk_widget_set_events(hChild, GDK_FOCUS_CHANGE_MASK);
			gtk_signal_connect(GTK_OBJECT(hChild), "focus_in_event",
							   GTK_SIGNAL_FUNC(FocusChanged), this);

			String strNode = nodeName;
			InsertItemIntoTree(subTree, hChild, strNode);
			
			gtk_widget_show(hChild);

			gtk_widget_set_style(GTK_BIN(hChild)->child, Vermie::m_pNormalStyle);
			
			CVermTreeNode* pNewNode = new CVermTreeNode();

			pNewNode->m_bChecked = TRUE;

			gtk_object_set_user_data(GTK_OBJECT(hChild), pNewNode);
		}

		UpdateProjectTree(pBody, hChild, pos, maxPos);

		nodeName = ParseNameFromBuffer(pBody, pos, maxPos, bParseError);
	}
}

GtkWidget* CVermProjectView::GetRootItem(void)
{
	if (!m_pTree)
		return NULL;

	GList* children = gtk_container_children(GTK_CONTAINER(m_pTree));

	GtkWidget* pItem = GTK_WIDGET(children->data);

	g_list_free(children);

	return pItem;
}

void CVermProjectView::PruneTree(GtkWidget* hNode)
{
	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hNode));

	// if this node was not checked off, then it should be removed

	if (!pNode->m_bChecked)
	{
		gtk_container_remove(GTK_CONTAINER(GTK_WIDGET(hNode)->parent), hNode);
	}
	else
	{
		pNode->m_bChecked = FALSE;		// reset

		// iterate over all of the subprojects in this project

		if (GTK_TREE_ITEM(hNode)->subtree)
		{
			GList* children = gtk_container_children(GTK_CONTAINER(GTK_TREE_ITEM(hNode)->subtree));
		
			while (children)
			{
				PruneTree(GTK_WIDGET(GTK_TREE_ITEM(children->data)));
				children = g_list_remove_link(children, children);
			}
		}
	}
}

int CVermProjectView::GetPathSelectedObject(String& path)
{
	if (!m_hCurSelItem)
	{
		path = "";
		return 0;
	}

	return GetPathFromTreeItem(m_hCurSelItem, path);
}

int CVermProjectView::IsEmpty()
{
	GtkTree* root = GTK_TREE_ROOT_TREE(m_pTree);

	GList* children = gtk_container_children(GTK_CONTAINER(root));

	if (!children)
		return 1;

	g_list_free(children);
	return 0;
}

void CVermProjectView::SetFileList(CVermProtocolMessage* pMessageReceived,
								   CVermProtocolMessage* pMessageSent)
{
	// determine which tree node this file list is for

	const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
	const char* pszDeployDir = pMessageReceived->GetHeaderValue(HEADER_DEPLOY_DIR);
	const char* pszChecksum = pMessageReceived->GetHeaderValue(HEADER_SEND_CHECKSUM);
	
	GtkWidget* hItem = GetTreeItemFromProject(GetRootItem(), pszProject);

	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

	// get the fully qualified working directory for the node
	String strWorkingDir = GetWorkingDirectory(hItem);

	// parse the file list
	int bChecksum = 0;
	int bPinStatus = pMessageReceived->GetHeaderValue(HEADER_PIN_STATUS) != NULL;

	if (pszChecksum)
	{
		bChecksum = 1;
	}
	
	ParseFileList(pNode, pMessageReceived, m_pParent->m_username.chars(), strWorkingDir, bChecksum, bPinStatus);

	// mark the node as visited

	pNode->m_bVisited = 1;

	if (pszDeployDir)
	{
		pNode->m_bWebDeploy = 1;
		pNode->m_strDeployDir = pszDeployDir;

		if (pNode->m_bCloaked)
			gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pCloakedDeployStyle);
		else
			gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pWebDeployStyle);
	}
	else
	{
		pNode->m_bWebDeploy = 0;
		pNode->m_strDeployDir = "";

		if (pNode->m_bCloaked)
			gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pCloakedStyle);
		else
			gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pNormalStyle);
	}
	
	// update the file list view
	if (hItem == m_hCurSelItem)
		m_pParent->UpdateFileView(pNode, strWorkingDir, NULL, TRUE);
}


void CVermProjectView::SetWorkingDirectory(String& strDirectory)
{
	GtkWidget* hSelItem = m_hCurSelItem;

	if (hSelItem == NULL)
		return;

	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hSelItem));

	// If it's the same directory, nothing to do
	if (pNode->m_strWorkingDir == strDirectory)
		return;

	// working dir for this node is already set - chuck all the leaf info we have stored
	if (!pNode->m_strWorkingDir.empty())
	{
		pNode->DeleteLeafInformation();

		if (GTK_TREE_ITEM(hSelItem)->subtree)
		{
			DeleteAllNodeInformation(GTK_TREE_ITEM(hSelItem)->subtree);
		}
	}

	pNode->m_strWorkingDir = strDirectory;

	m_pParent->UpdateNodeDisplay(pNode, strDirectory);
}

void CVermProjectView::DeleteAllNodeInformation(GtkWidget* hItem)
{
	if (hItem == NULL)
		return;

	GList* children = gtk_container_children(GTK_CONTAINER(hItem));

	while (children)
	{
		GtkWidget* pChild = (GtkWidget*) children->data;
		
		CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(children->data));

		// sanity check
		if (!pNode)
			return;
		
		// if the working dir is empty, delete all leaf info, otherwise this branch of the tree is ok
		if (pNode->m_strWorkingDir.empty())
		{
			pNode->DeleteLeafInformation();
			
			GtkWidget* hChild = GTK_TREE_ITEM(pChild)->subtree;

			if (hChild)
				DeleteAllNodeInformation(hChild);
		}

		children = g_list_remove_link(children, children);
	}		
}
	
CVermTreeNode* CVermProjectView::GetNodeFromProject(const char* pszProject)
{
	GtkWidget* hItem = GetTreeItemFromProject(GetRootItem(), pszProject);

	return (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));
}

int CVermProjectView::SetFile(CVermProtocolMessage* pMessageReceived,
							  CVermProtocolMessage* pMessageSent,
							  int bAskUserToOverwrite, int& writeState, int bMakeWritable)
{
	const char* pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
	const char* pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
	const char* pszVersion = pMessageReceived->GetHeaderValue(HEADER_VERSION);
	const char* pszContentLen = pMessageReceived->GetHeaderValue(HEADER_CONTENT_LENGTH);
	const char* pszRemoteDate = pMessageReceived->GetHeaderValue(HEADER_DATE);
	const char* pszLockedBy = pMessageReceived->GetHeaderValue(HEADER_LOCKED_BY);
	const char* pszLockedProject = pMessageReceived->GetHeaderValue(HEADER_LOCKED_PROJECT);
	
	if (pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE))
		pszContentLen = pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE);

	String strProject = pszProject;

	if (strProject.index("$//") == 0)
	{
		String strProj = "$/";
		strProj += strProject.after(2);

		strProject = strProj;
	}
	
	// get the path of the file on disk
	
	GtkWidget* hItem = GetTreeItemFromProject(GetRootItem(), strProject.chars());

	String strWorkingDir;

	// project share might not have new sub directories created yet
	if (!hItem)
	{
		if (pMessageSent->m_number == PM_SHARE_PROJECT)
		{
			hItem = CreateProject(strProject);
			strWorkingDir = GetWorkingDirectory(hItem);
		}
		else if (pMessageSent->m_number == PM_GET_PROJECT && pMessageSent->GetHeaderValue(HEADER_LABEL) != NULL)
		{
			// this is a deleted project coming back to haunt us from a label
			BuildNewWorkingDirectory(strProject, strWorkingDir);
		}
	}
	else
		strWorkingDir = GetWorkingDirectory(hItem);
	
	
	String strFile = strWorkingDir + "/";
	strFile += pszFile;
	
	// create the file on disk
	
	if (!CreateDirectory(strWorkingDir))
	{
		String s("Unable to create working directory:");
		s += strWorkingDir;
		AfxMessageBox(s);
		return 0;
	}

	int bWriteFile = 1;
	
	// if it's writable, ask the user to confirm over-write
	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));
	
	int bFileExists = (stat(strFile.chars(), &sb) == 0);
	
	if (bFileExists && (sb.st_mode & S_IWUSR))
	{
		if (bAskUserToOverwrite)
		{
			if (writeState != IDYESTOALL && writeState != IDNOTOALL)
			{
			
				// TODO - check & see if we have it locked and display a different message?
				String s = strFile + " is writable, overwrite?";
				CYesNoDlg dlg(s, MB_YES|MB_NO|MB_CANCEL);
				writeState = dlg.DoModal();
			}

			if (writeState != IDYESTOALL && writeState != IDYES)
				bWriteFile = 0;
		}
	}

	if (bWriteFile)
	{
		// if it's read only - flip the bit so we can over-write it
	
		if (bFileExists && MakeFileWriteable(strFile) < 0)
		{
			String str("Unable to set file permissions: ");
			str += strFile;
						
			m_pParent->AddMessage(str);
		}
	
		// WARNING the following line will overwrite an existing file
	
		int newFile = open(strFile.chars(), O_WRONLY|O_CREAT|O_TRUNC, 0750);
	
		if (newFile < 0)
		{
			String s("Unable to open file:" + strFile);
			m_pParent->AddMessage(s);
			return 0;
		}
	
		write(newFile, pMessageReceived->m_body, atoi(pszContentLen));
	
		close(newFile);
	
		// get file status
		memset(&sb, 0, sizeof(struct stat));
		
		stat(strFile.chars(), &sb);
		
		// update the leaf information
		CVermTreeNode* pNode = NULL;
		short bIgnoreThisFile = FALSE;

		// no version info on deleted files
		if (!pszVersion)
			bIgnoreThisFile = TRUE;
		
		if (hItem)
			pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));
		else
			bIgnoreThisFile = TRUE;
		
		String strPSZFile = pszFile;

		CVermTreeLeaf* pLeaf = NULL;
		
		if (pNode)
			pLeaf = pNode->GetLeafByName(strPSZFile);
		
		if (pLeaf)
		{
			pLeaf->m_nLocalVersionNumber = atoi(pszVersion);
			
			pLeaf->m_pLocalVersionDate = sb.st_mtime;
		}
		else if (!bIgnoreThisFile)
		{
			pLeaf = new CVermTreeLeaf();
			
			pLeaf->m_name = pszFile;
			pLeaf->m_nRemoteVersionNumber = atoi(pszVersion);
			pLeaf->m_nLocalVersionNumber = atoi(pszVersion);
			pLeaf->m_pRemoteVersionDate = ParseServerDate((char *)pszRemoteDate);
			pLeaf->m_pLocalVersionDate = sb.st_mtime;

			if (pszLockedProject)
				pLeaf->m_projectLocked = pszLockedProject;

			if (pszLockedBy)
				pLeaf->m_userLocked = pszLockedBy;
			
			pNode->AddLeaf(pLeaf);
		}
		

		String strProject = pszProject;
		
		// if we have the file locked, don't make it read-only
		int nLockStatus = IsFileLocked(pLeaf, strProject, m_pParent->m_username);
		
		if (nLockStatus < 2 && !bMakeWritable)
		{
			if (MakeFileReadOnly(strFile) < 0)
			{
				String str("Unable to set file permissions: ");
				str += strFile;
						
				m_pParent->AddMessage(strFile);
			}
		}
		// update the file list view for this file

		if (!bIgnoreThisFile)
			m_pParent->UpdateFileView(pNode, strWorkingDir, pLeaf, 0);
	}

	return 1;
}


void CVermProjectView::AddNewProject(GtkWidget* hParentItem,
									 String& strNewProject)
{
	GtkWidget* newItem = gtk_tree_item_new_with_label((char*)strNewProject.chars());
	gtk_signal_connect(GTK_OBJECT(newItem), "button_press_event",
					   GTK_SIGNAL_FUNC (ButtonPress), m_popupMenu);
	GTK_WIDGET_SET_FLAGS(newItem, GTK_CAN_FOCUS);

	gtk_widget_set_events(newItem, GDK_FOCUS_CHANGE_MASK);
	gtk_signal_connect(GTK_OBJECT(newItem), "focus_in_event",
					   GTK_SIGNAL_FUNC(FocusChanged), this);

	gtk_widget_show(newItem);
	
	if (!GTK_TREE_ITEM(hParentItem)->subtree)
	{
		GtkWidget* pSubtree = gtk_tree_new();
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(hParentItem), pSubtree);
		gtk_widget_show(pSubtree);
	}
	
	gtk_tree_item_expand(GTK_TREE_ITEM(hParentItem));
	
	InsertItemIntoTree(GTK_TREE_ITEM(hParentItem)->subtree, newItem,
					   strNewProject);
	
	CVermTreeNode* pChildNode = new CVermTreeNode();
	
	gtk_object_set_user_data(GTK_OBJECT(newItem), pChildNode);
}

GtkWidget* CVermProjectView::CreateProject(String& strProject)
{
	int p = strProject.ReverseFind('/');

	if (p >= 0)
	{
		String strProjLeft = strProject.before(p);
		String strProjRight = strProject.after(p);

		if (strProjLeft == "$")
			strProjLeft = "$/";
		
		GtkWidget* hItem = GetTreeItemFromProject(GetRootItem(), strProject);
		
		if (!hItem)
		{
			hItem = CreateProject(strProjLeft);

			// at this point, our parent had better exist ...
			// add the child to this item
			if (hItem)
			{
				GtkWidget* hChildNode = gtk_tree_item_new_with_label(strProjRight.chars());
				gtk_widget_show(hChildNode);
                gtk_widget_set_style(GTK_BIN(hChildNode)->child, Vermie::m_pNormalStyle);

                gtk_signal_connect(GTK_OBJECT(hChildNode), "button_press_event",
                                                   GTK_SIGNAL_FUNC (ButtonPress), m_popupMenu);

                gtk_signal_connect(GTK_OBJECT(hChildNode), "focus_in_event",
                                                   GTK_SIGNAL_FUNC(FocusChanged), this);

				GtkWidget* subTree = GTK_TREE_ITEM(hItem)->subtree;

				if (!subTree)
				{
					subTree = gtk_tree_new();
					gtk_tree_item_set_subtree(GTK_TREE_ITEM(hItem), subTree);
					gtk_widget_show(subTree);
				}
				
				InsertItemIntoTree(subTree, hChildNode, strProjRight);

				CVermTreeNode* pChildNode = new CVermTreeNode();

				gtk_object_set_user_data(GTK_OBJECT(hChildNode), pChildNode);
				
				return hChildNode;
			}
		}
		else
			return hItem;
	}

	return NULL;
}

void CVermProjectView::BuildNewWorkingDirectory(String& strProject, String& strWorkingDir)
{
	String strNode;
	String strParent;

	int p = strProject.ReverseFind('/');

	strNode = strProject.after(p);
	strParent = strProject.before(p);

	GtkWidget* hItem = GetTreeItemFromProject(GetRootItem(), strParent);

	if (!hItem)
		BuildNewWorkingDirectory(strParent, strWorkingDir);
	else
		strWorkingDir = GetWorkingDirectory(GetRootItem(), strParent.chars());

	strWorkingDir += "/" + strNode;
}
