/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */


#ifndef VERMPROTOCOLMESSAGE_H
#define VERMPROTOCOLMESSAGE_H

#include "VermProtocolHeader.h"

// Client -> Server messages

#define PM_LOGIN						100
#define PM_GET_PROJECT_TREE				101
#define PM_GET_FILE_LIST				102
#define PM_CREATE_PROJECT				103
#define PM_ADD_FILE						104
#define PM_GET_FILE						105
#define PM_LOCK_FILE					106
#define PM_UNLOCK_FILE					107
#define PM_GET_FILE_HISTORY				108
#define PM_ADD_LABEL					109
#define PM_CHECKIN						110
#define PM_GET_PROJECT					111
#define PM_GET_FILE_INFO				112
#define PM_DELETE						113
#define PM_RENAME						114
#define PM_CHECKOUT_PROJECT				115
#define PM_SHARE_FILE                   116
#define PM_PROJECT_GET_HISTORY          117
#define PM_PIN_FILE						118
#define PM_UNPIN_FILE					119
#define PM_DEPLOY                       120
#define PM_BRANCH                       121
#define PM_SEARCH                       122
#define PM_NOOP                         123
#define PM_CHANGE_PASSWORD				124
#define PM_SHARE_PROJECT				125
#define PM_EXIT							199

// Server -> Client messages

#define PM_CHALLENGE					200
#define PM_PROJECT_TREE					201
#define PM_FILE_LIST					202
#define PM_FILE							203
#define PM_FILE_HISTORY					204
#define PM_FILE_INFO					205
#define PM_PROJECT_HISTORY              206
#define PM_SEARCH_RESULTS               207
#define PM_SECURE_CHALLENGE				250

// success status codes

#define PM_SUCCESS						300
#define PM_CHECKIN_COMPLETE				301
#define PM_FILE_UNLOCKED				302
#define PM_FILE_ADDED					303
#define PM_FILE_LOCKED					304

// failure status codes

#define PM_OPERATION_FAILED				400
#define PM_NO_SUCH_FILE_OR_PROJECT		401
#define PM_FILE_OR_PROJECT_NOT_LOCKED	402
#define PM_AUTHENTICATION_FAILED		403
#define PM_MISSING_HEADER				404
#define PM_INVALID_LABEL				405
#define PM_INVALID_VERSION				406
#define PM_UNRECOGNIZED_MESSAGE			407
#define PM_INVALD_PROTOCOL_VERSION		408
#define PM_ALREADY_EXISTS				409
#define PM_ALREADY_LOCKED				410
#define PM_UNAUTHORIZED_USER			411
#define PM_SERVER_EXPIRED				412
#define PM_KEY_NOT_FOUND				413
#define PM_KEY_EXPIRED					414
#define PM_INVALID_KEY					415
#define PM_BAD_SRCSAFE_INI				416
#define PM_MISSING_SRCSAFE_AUTO			417
#define PM_MISSING_CRYPTO				418
#define PM_CHECKIN_MERGE_FAILED			419
#define PM_FAILED_DATABASE_OPEN			420

class String;

class VermHeader
{
public:	
	CVermProtocolHeader* m_pHeader;
	VermHeader*  m_pNext;

	inline VermHeader() { m_pNext = NULL; m_pHeader = NULL; }
	inline ~VermHeader() { if (m_pHeader) delete m_pHeader; }
};

class VermHeaderList
{
public:
	VermHeaderList();
	~VermHeaderList();
	
	VermHeader* m_pHead;
	VermHeader* m_pTail;

	void AddTail(CVermProtocolHeader* pHeader);
};

	
class CVermProtocolMessage
{
public:
	int	m_number;

	VermHeaderList* m_pHeaders;

	char* m_body;
	int m_bodyIndex;
	int m_bodySize;

public:
	CVermProtocolMessage();
	virtual ~CVermProtocolMessage();
	
	virtual void AddHeader(CVermProtocolHeader* pHeader);
	const char* GetHeaderValue(const char* pszName);
	void AppendHeadersToString(String& str);
};

#endif /* VERMPROTOCOLMESSAGE_H */
