/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <stream.h>

#include "Vermie.h"

#include "VermFrameWnd.h"
#include "VermProtocolHeader.h"
#include "VermProtocolMessage.h"
#include "VermMessageStore.h"
#include "VermByteQueue.h"

#include "AfxMessageBox.h"

#include "VermSocket.h"

#include "zlib.h"

#define STATE_ERROR		-1
#define STATE_INIT		 0
#define STATE_NUMBER	 1
#define STATE_HEADER	 2
#define STATE_BODY		 3
#define STATE_DONE		 4

static CVermSocket* s_pSocket;

static void socket_read(gpointer data, gint source, GdkInputCondition c);
static void socket_write(gpointer data, gint source, GdkInputCondition c);

static void socket_read(gpointer data, gint, GdkInputCondition c)
{
	CVermSocket* pSocket = (CVermSocket*) data;

	if (c & GDK_INPUT_EXCEPTION)
	{
		String s("Error reading from socket - server closed connection?");
		pSocket->m_pFrameWnd->ServerShutdown(s);
		return;
	}
	
	if (!(c & GDK_INPUT_READ))
		return;
	
	if (pSocket->m_socketBufferSize < 1)
	{
		pSocket->ReceiveResponse();
	}
	else
	{
		pSocket->m_bCallReceive = 1;
	}
}

static void socket_write(gpointer data, gint, GdkInputCondition c)
{
	CVermSocket* pSocket = (CVermSocket*) data;

	if (c & GDK_INPUT_EXCEPTION)
	{
		String s("Error writing to socket - server closed connection?");
		pSocket->m_pFrameWnd->ServerShutdown(s);
		return;
	}

	if (!(c & GDK_INPUT_WRITE))
		return;

	if (!pSocket)
	{
		cout << "SOCKET IS NULL!" << endl;
		return;
	}
	
	if (pSocket->m_writeTag)
	{
		gdk_input_remove(pSocket->m_writeTag);
		pSocket->m_writeTag = 0;
	}
	
	pSocket->SendData();
}

OutgoingData::OutgoingData(int size, CVermProtocolMessage* pMessage, int which)
{ 
	m_nMessageSize = size; 

	m_pMessageIn = pMessage;
	
	m_pWhich = which;
	m_pNext = NULL; 

}

OutgoingData::~OutgoingData()
{
}

OutgoingList::OutgoingList(CVermFrameWnd* pParent) 
{ 
	m_pParent = pParent; 
	m_pHead = m_pTail = NULL; 
	m_nSentSoFar = 0; 
}

void OutgoingList::AddMessage(int messageSize, CVermProtocolMessage* pMessage,
							  int messageNum) 
{	
	if (m_pHead == NULL) 
	{ 
		m_pHead = new OutgoingData(messageSize, pMessage, messageNum);
		m_pTail = m_pHead;
	}
	else
	{
		m_pTail->m_pNext = new OutgoingData(messageSize, pMessage, messageNum);
		m_pTail = m_pTail->m_pNext;
	}
}

void OutgoingList::DeleteHead()
{
	OutgoingData* pTmp = m_pHead;

	m_pHead = m_pHead->m_pNext;

	delete pTmp;
}

void OutgoingList::SentData(int sentLen)
{
	if (m_pHead == NULL)
		return;

	if (m_nSentSoFar == 0)
	{
		m_pParent->SetPercentDone((double)-1.0);
		if (m_pHead->m_pWhich == PM_CHECKIN)
		{
			String s("Checking in file: ");
			s += m_pHead->m_pMessageIn->GetHeaderValue(HEADER_PROJECT);
			s += "/";
			s += m_pHead->m_pMessageIn->GetHeaderValue(HEADER_FILE);
			
			m_pParent->AddMessage(s);
		}
		else if (m_pHead->m_pWhich == PM_ADD_FILE)
		{
			String s("Adding file: ");
			String strFile = m_pHead->m_pMessageIn->GetHeaderValue(HEADER_PROJECT);
			strFile += "/";
			strFile += m_pHead->m_pMessageIn->GetHeaderValue(HEADER_FILE);

			if (strFile.index("$//") == 0)
			{
				String strTmp = "$/";

				strTmp += strFile.after(2);

				strFile = strTmp;
			}
			
			s += strFile;
			
			m_pParent->AddMessage(s);
		}
	}

	m_nSentSoFar += sentLen;

	while(m_pHead && m_nSentSoFar >= m_pHead->m_nMessageSize)
	{
		m_pParent->SetPercentDone((double)1.0);
 
		m_nSentSoFar -= m_pHead->m_nMessageSize;
		DeleteHead();

		if (m_pHead == NULL)
		{
			m_nSentSoFar = 0;
		}
		else
		{
			if (m_pHead->m_pWhich == PM_CHECKIN)
			{
				String s("Checking in file: ");
				s += m_pHead->m_pMessageIn->GetHeaderValue(HEADER_PROJECT);
				s += "/";
				s += m_pHead->m_pMessageIn->GetHeaderValue(HEADER_FILE);
				m_pParent->AddMessage(s);
			}
			else if (m_pHead->m_pWhich == PM_ADD_FILE)
			{
				String s("Adding file: ");
				String strFile = m_pHead->m_pMessageIn->GetHeaderValue(HEADER_PROJECT);
				strFile += "/";
				strFile += m_pHead->m_pMessageIn->GetHeaderValue(HEADER_FILE);

				if (strFile.index("$//") == 0)
				{
					String strTmp = "$/";
					strTmp += strFile.after(2);
					strFile = strTmp;
				}
			
				s += strFile;

				m_pParent->AddMessage(s);
			}
		}
	}

	if (m_pHead)
		m_pParent->SetPercentDone((double)m_nSentSoFar/(double)m_pHead->m_nMessageSize);
	else
		m_pParent->SetPercentDone((double)-1.0);

}

/////////////////////////////////////////////////////////////////////////////
// CVermSocket

CVermSocket::CVermSocket(CVermFrameWnd* pFrameWnd)
{
	s_pSocket = this;
	
	m_pFrameWnd = pFrameWnd;

	m_pMessageStore = new CVermMessageStore();

	m_pMessageIn = NULL;

	m_bCallReceive = 0;

	m_readState = STATE_INIT;

	m_socketBufferSize = 0;

	m_bytesProcessed = 0;

#ifndef BUILD_STANDARD
	m_pCrypto = NULL;
	m_decryptBuffer = NULL;
#endif
	
	m_pOutgoingList = new OutgoingList(pFrameWnd);
}

CVermSocket::~CVermSocket()
{
	if (m_pMessageIn)
		delete m_pMessageIn;

	if (m_pMessageStore)
		delete m_pMessageStore;

	if (m_pOutgoingList)
		delete m_pOutgoingList;
}

/////////////////////////////////////////////////////////////////////////////
// CVermSocket member functions

int CVermSocket::Connect(String& realHost, int realPort)
{
	String host = realHost;
	int port = realPort;
	
	m_numBytesInMessage = 0;

#ifndef BUILD_STANDARD
	if (m_decryptBuffer)
		delete m_decryptBuffer;
	
	m_decryptBuffer = NULL;
	m_numBytesToDecrypt = 0;
	m_numBytesDecrypted = 0;

	m_decryptHeaderBufferSize = 0;
	
	m_decryptBufferMaxSize = 0;
#endif	 // !BUILD_STANDARD

	// reset the host & port if we are using a proxy ...
	if (Vermie::m_bUseHTTPProxy)
	{
		host = Vermie::m_strProxyName;
		port = Vermie::m_ProxyPort;
	}
	
	if (port == 0)
		return SOCKET_CANT_FIND_HOST;
	
	m_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if (m_socket < 0)
	{
		// ERROR
		return SOCKET_CANT_CREATE;
	}

	int ret = fcntl(m_socket, F_SETFL, O_NONBLOCK);

	if (ret < 0)
	{
		// Can't make socket non-blocking
		return SOCKET_MUST_BLOCK;
	}

	struct sockaddr_in addr;

	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);

	struct hostent* pHost = gethostbyname(host.chars());

	if (!pHost)
	{
		return SOCKET_CANT_FIND_HOST;
	}
	
	memcpy(&addr.sin_addr, pHost->h_addr, pHost->h_length);

	if (!addr.sin_addr.s_addr)
	{
		return SOCKET_CANT_FIND_HOST;
	}

	ret = connect(m_socket, (struct sockaddr*)&addr, sizeof(addr));

	if (ret < 0)
	{
		if (errno == EINPROGRESS || errno == EAGAIN)
		{
			struct timeval timeout;
			int tries = 0;

			ret = 0;

			while (ret <= 0)
			{
				fd_set writefds;

				// lets try and not loop forever, eh?
				if (tries++ >= 100)
				{
					return SOCKET_CANT_CONNECT;
				}
			  
				timeout.tv_sec = 0;
				timeout.tv_usec = 100000;

				FD_ZERO(&writefds);
				FD_SET(m_socket, &writefds);

				ret = select(FD_SETSIZE, NULL, &writefds, NULL, &timeout);

				if (ret < 0 && errno != EALREADY)
					return SOCKET_CANT_CONNECT;
				else if (ret > 0)
				{
					int status = connect(m_socket, (struct sockaddr*)&addr,
										 sizeof(addr));

					if (status < 0 && errno == EISCONN)
					{
						status = 0;
					}

					if (status)
					{
						if (errno == EALREADY)
						{
							ret = 0;  // keep looping
						}
						else
							return SOCKET_CANT_CONNECT;
					}
					else
						break;  // we're done
				}
			}
		}
	}

	m_bytesProcessed = 0;
	m_pMessageIn = new CVermProtocolMessage();
	m_parseBufferSize = 0;
	m_socketBufferSize = 0;
	m_readState = STATE_NUMBER;
		
	m_readTag = gdk_input_add(m_socket, GDK_INPUT_READ, socket_read, (void*) this);
   
	if (Vermie::m_bUseHTTPProxy)
	{
		String s("CONNECT ");
		s += realHost;
		s += ":";
		char buf[256];
		sprintf(buf, "%i", realPort);
		s += buf;
		s += " HTTP/1.1\r\n\r\n";

		m_strDataOut += s;

		m_writeTag = gdk_input_add(m_socket, GDK_INPUT_WRITE, socket_write, (void*) this);
	}
	
	return SOCKET_OK;
}

#ifndef BUILD_STANDARD
void CVermSocket::SetSecure(CVermCrypto* crypto)
{
	m_pCrypto = crypto;
}
#endif

/*
** Returns TRUE if there should be real message data to parse; FALSE otherwise
*/

int CVermSocket::ReadData()
{
#ifndef BUILD_STANDARD
	if (m_pCrypto)
	{
		if (m_numBytesInMessage == 0)
		{
			char tmpBuffer[CRYPTO_HEADER_SIZE];

			int bytesReceived = read(m_socket, tmpBuffer, CRYPTO_HEADER_SIZE - m_decryptHeaderBufferSize);

			// copy the bytes to our block header buffer

			int i;
			for (i = 0; i < bytesReceived; i++)
				m_decryptHeaderBuffer[m_decryptHeaderBufferSize++] = tmpBuffer[i];

			// do we have all of the header bytes?

			if (m_decryptHeaderBufferSize == CRYPTO_HEADER_SIZE)
			{
				// the first CRYPTO_HEADER_IV_SIZE bytes is the IV for this message

				for (i = 0; i < CRYPTO_HEADER_IV_SIZE; i++)
					m_IV[i] = m_decryptHeaderBuffer[i];

				// the next CRYPTO_HEADER_LENGTH_SIZE bytes is the encrypted length of the message

				char size[CRYPTO_HEADER_LENGTH_SIZE];
				char sizeBuffer[CRYPTO_HEADER_LENGTH_SIZE];
				int j = 0;

				for (i = CRYPTO_HEADER_IV_SIZE; i < CRYPTO_HEADER_SIZE; i++)
					sizeBuffer[j++] = m_decryptHeaderBuffer[i];

				m_pCrypto->Decrypt(sizeBuffer, size, CRYPTO_HEADER_LENGTH_SIZE, m_IV);

				i = 0;

				while ((i < CRYPTO_HEADER_LENGTH_SIZE) && (size[i] != PAD_CHAR))
					i++;

				size[i] = '\0';

				m_numBytesInMessage = atoi(size);

				// determine how many total bytes will be arriving including the pad?

				m_numBytesToDecrypt = m_numBytesInMessage;

				if (m_numBytesToDecrypt < CRYPTO_BLOCK_SIZE)
				{
					m_numBytesToDecrypt = CRYPTO_BLOCK_SIZE;
				}
				else
				{
					int remainder = m_numBytesToDecrypt % CRYPTO_BLOCK_SIZE;

					if (remainder > 0)
						m_numBytesToDecrypt += CRYPTO_BLOCK_SIZE - remainder;
				}

				// prepare for reading the actual encrypted message

				m_decryptBufferSize = 0;

				m_decryptBufferMaxSize = min(m_numBytesToDecrypt, SOCKET_BUFFER_SIZE);

				m_decryptBuffer = new char[m_decryptBufferMaxSize];
			}

			return 0;
		}
		else
		{
			char tmpBuffer[SOCKET_BUFFER_SIZE];

			int bytesReceived = read(m_socket, tmpBuffer, m_decryptBufferMaxSize - m_decryptBufferSize);

			// why are we here if there was nothing to read?
			
			if (bytesReceived < 0)
			{
				return 0;
			}
			
			// copy the bytes to our decrypt buffer

			char *pBuffer = m_decryptBuffer + m_decryptBufferSize;

			memcpy(pBuffer, tmpBuffer, bytesReceived);

			m_decryptBufferSize += bytesReceived;

			// do we have an entire block to decrypt?

			if (m_decryptBufferSize != m_decryptBufferMaxSize)
				return 0;

			// decrypt the block into the socket buffer for parsing

			m_pCrypto->Decrypt(m_decryptBuffer, m_socketBuffer, m_decryptBufferMaxSize, m_IV);
			
			m_numBytesDecrypted += m_decryptBufferMaxSize;

			m_socketBufferSize = m_decryptBufferMaxSize;

			// free up the encryption buffer

			delete []m_decryptBuffer;

			m_decryptBuffer = NULL;

			m_decryptBufferMaxSize = 0;

			m_decryptBufferSize = 0;

			// have we decrypted an entire message?

			if (m_numBytesDecrypted == m_numBytesToDecrypt)
			{
				if (m_numBytesDecrypted > m_numBytesInMessage)
				{
					// don't count the padding bytes when we send the buffer to the parsing

					m_socketBufferSize -= m_numBytesDecrypted - m_numBytesInMessage;
				}

				// reset to prepare for reading the next message

				m_decryptHeaderBufferSize = 0;

				m_numBytesInMessage = 0;

				m_numBytesDecrypted = 0;
				
				m_numBytesToDecrypt = 0;
			}
			else
			{
				// prepare for reading another block

				m_decryptBufferMaxSize = min(m_numBytesToDecrypt - m_numBytesDecrypted, SOCKET_BUFFER_SIZE);

				m_decryptBuffer = new char[m_decryptBufferMaxSize];
			}
		}
	}
	else
#endif
	{
		m_socketBufferSize = read(m_socket, m_socketBuffer, SOCKET_BUFFER_SIZE);

		if (!m_bGotProxyHeader && Vermie::m_bUseHTTPProxy)
		{
			if (!strncmp(m_socketBuffer, "HTTP", 4))
			{
				String strBuffer(m_socketBuffer, m_socketBufferSize);

				String strRespCode = strBuffer.substr(9, 3);

				int pos = strBuffer.index("\r\n\r\n");

				if (strRespCode == "200")
				{
					if (pos > 0)
					{
						strBuffer = strBuffer.after(pos + 3);
						m_bytesProcessed = pos + 4;
						
						m_bGotProxyHeader = TRUE;
					}
				}
				else
				{
					// HTTP error of somesort ...

					// pos > 0 means there was some sort of data in the body of the message
					if (pos > 0)
					{
						strBuffer = strBuffer.after(pos + 3);

						if (strBuffer != "")
							strBuffer = "The HTTP Proxy returned:\n" + strBuffer;
						else
							strBuffer = "Error conneting to remote server via HTTP Proxy";
					}
					else
					{
						strBuffer = "Error conneting to remote server via HTTP Proxy";
					}
					
					m_pFrameWnd->ServerShutdown(strBuffer);
					return 0;
				}
			}
		}
	}

	return 1;
}

void CVermSocket::ReceiveResponse()
{
	// read a chunk of data

	if (!ReadData())
		return;	// processed encrypted block header

	while (m_bytesProcessed < m_socketBufferSize)
	{
		// process the message

		if (m_readState == STATE_BODY)
			ReadBody();
		else
			ParseMessage();

		if (m_readState == STATE_DONE)
		{
			// we now have a complete message

			// If it's compressed, de-compress it
			if (m_pMessageIn->GetHeaderValue(HEADER_USE_COMPRESSION) && m_pMessageIn->m_bodySize != 0)
			{
				unsigned long newLen = atoi(m_pMessageIn->GetHeaderValue(HEADER_ORIGINAL_SIZE));
				Byte *pNewBody = new Byte[newLen + 1];
				int err;
				
				err = uncompress(pNewBody, &newLen, (unsigned char *)m_pMessageIn->m_body, m_pMessageIn->m_bodySize);

				pNewBody[newLen] = 0;

				delete []m_pMessageIn->m_body;
				m_pMessageIn->m_body = (char *)pNewBody;
				m_pMessageIn->m_bodySize = newLen;
			}

			// what message is this in response to?

			CVermProtocolMessage* pMessageOut = NULL;

			const char* pResponseTo = m_pMessageIn->GetHeaderValue(HEADER_RESPONSE_TO);

			if (pResponseTo != NULL)
			{
				pMessageOut = m_pMessageStore->FindMessage(atoi(pResponseTo));

				//ASSERT(pMessageOut);
			}

			int bRemoveMessageFromStore = 0;

			if (pResponseTo != NULL)
			{
				if (pMessageOut)
				{
					switch (pMessageOut->m_number)
					{
					case PM_SHARE_PROJECT:
						if (m_pMessageIn->m_number == PM_FILE || m_pMessageIn->m_number == PM_FILE_INFO)
							break;

					case PM_GET_PROJECT:
						if (m_pMessageIn->m_number == PM_FILE)
							break;

						// fallthrough

					case PM_CHECKOUT_PROJECT:
						if (m_pMessageIn->m_number == PM_FILE || m_pMessageIn->m_number == PM_FILE_LOCKED)
							break;

						// fallthrough
					default:
						bRemoveMessageFromStore = 1;
						break;
					}
				}
			}

			// pass the message up to our parent

			m_pFrameWnd->MessageReceived(m_pMessageIn, pMessageOut, bRemoveMessageFromStore);

			// remove the message which generated this response if necessary

			if (bRemoveMessageFromStore)
				m_pMessageStore->RemoveMessage(atoi(pResponseTo));

			// allocate a new message in object

			if (m_pMessageIn)
				delete m_pMessageIn;

			m_pMessageIn = new CVermProtocolMessage();

			m_parseBufferSize = 0;

			m_readState = STATE_NUMBER;
		}

		if ((m_bytesProcessed == m_socketBufferSize) && m_bCallReceive)
		{
			m_socketBufferSize = 0;
			m_bytesProcessed = 0;

			int bGotData = ReadData();

			if (bGotData && (m_socketBufferSize <= 0))
			{
#ifndef BUILD_STANDARD
				if (!m_pCrypto)
#endif // !BUILD_STANDARD
				{
					// we're hosed - close the connection
					String s("Error receiving data from server - closing connection");
					m_pFrameWnd->ServerShutdown(s);
					return;
				}
			}

			m_bCallReceive = 0;
		}
	}

	// we have finished processing the buffer, so reset and return

	m_bytesProcessed = 0;
	m_socketBufferSize = 0;
}

void CVermSocket::ParseMessage()
{
	char c;
	
	while (m_bytesProcessed < m_socketBufferSize)
	{
		c = m_socketBuffer[m_bytesProcessed++];

		if (c == '\n')
		{
			if (m_readState == STATE_NUMBER)
			{
				m_parseBuffer[m_parseBufferSize - 1] = '\0';

				m_pMessageIn->m_number = atoi(m_parseBuffer);

				m_parseBufferSize = 0;

				m_readState = STATE_HEADER;
			}
			else	// STATE_HEADER
			{
				if (m_parseBufferSize == 1)
				{
					// we have reached the end of the headers, is there a body?

					const char* pszContent = m_pMessageIn->GetHeaderValue(HEADER_CONTENT_LENGTH);

					if (!pszContent || (atoi(pszContent) == 0))
						m_readState = STATE_DONE;
					else
						m_readState = STATE_BODY;

					// special case message so that we get a "Receiving file: blah blah" with
					// the progress meter
					if (m_pMessageIn->m_number == PM_FILE)
					{
						String s("Receiving file ");
						s += m_pMessageIn->GetHeaderValue(HEADER_PROJECT);
						s += "/";
						s += m_pMessageIn->GetHeaderValue(HEADER_FILE);
						m_pFrameWnd->AddMessage(s);
					}

					return;
				}

				m_parseBuffer[m_parseBufferSize - 1] = '\0';

				String strHeader = m_parseBuffer;

				int nColonIndex = strHeader.index(':');

				CVermProtocolHeader* pHeader = new CVermProtocolHeader();

				pHeader->m_name = strHeader.before(nColonIndex);

				pHeader->m_value = strHeader.after(nColonIndex);
				
				m_pMessageIn->AddHeader(pHeader);

				m_parseBufferSize = 0;
			}
		}
		else
		{
			if (m_parseBufferSize == PARSE_BUFFER_SIZE)
			{
				// This can't happen - here for debugging during devolopment only
//				ASSERT(0); 
			}

			if ((m_parseBufferSize > 10) && (m_readState == STATE_NUMBER))
			{
				// tell our parent to close us down ...
				String s("Error receiving data from server - closing connection");
				m_pFrameWnd->ServerShutdown(s);
			}

			// just cram the value in our buffer

			m_parseBuffer[m_parseBufferSize++] = c;
		}
	}
}

void CVermSocket::ReadBody()
{
//	ASSERT(m_bytesProcessed < m_socketBufferSize);

	if (!m_pMessageIn->m_body)
	{
		// allocate memory to hold the message body

		const char* pszContent = m_pMessageIn->GetHeaderValue(HEADER_CONTENT_LENGTH);

		m_pMessageIn->m_bodySize = atoi(pszContent);

		m_pMessageIn->m_body = new char[m_pMessageIn->m_bodySize];
		m_pMessageIn->m_bodyIndex = 0;
	}

	// copy the data from the input buffer to our message body

	char *pSrc = m_socketBuffer + m_bytesProcessed;

	int readLen = m_socketBufferSize - m_bytesProcessed;
	if ((m_pMessageIn->m_bodySize - m_pMessageIn->m_bodyIndex) < readLen)
		readLen = m_pMessageIn->m_bodySize - m_pMessageIn->m_bodyIndex;

//	ASSERT(readLen > 0);

//	ASSERT(readLen + m_pMessageIn->m_bodyIndex <= m_pMessageIn->m_bodySize);

	char *pDest = m_pMessageIn->m_body;
	pDest = pDest + (m_pMessageIn->m_bodyIndex);

	memcpy(pDest, pSrc, readLen);

	m_pMessageIn->m_bodyIndex += readLen;

	m_bytesProcessed += readLen;

//	ASSERT(m_bytesProcessed <= m_socketBufferSize);

	// have we read all of the body?

	if (m_pMessageIn->m_bodyIndex == m_pMessageIn->m_bodySize)
	{
		m_readState = STATE_DONE;
		m_pFrameWnd->SetPercentDone(-1.0);
		return;
	}

	m_pFrameWnd->SetPercentDone((double) m_pMessageIn->m_bodyIndex / (double) m_pMessageIn->m_bodySize);
}

int CVermSocket::SendData()
{
	if (m_socket < 0)
		return -1;
	
	if (!m_strDataOut.m_pHead)
		return -1;

	while (m_strDataOut.m_pHead)
	{
		char *pData;
		int nBytesSent = 0;
		int nActualBytesSent = 0;
		int nBytes = m_strDataOut.GetBytes(&pData);
		
		while (nBytesSent < nBytes)
		{
			int maxlen = SOCKET_BUFFER_SIZE;

			if ((nBytes - nBytesSent) < maxlen)
				maxlen = nBytes - nBytesSent;
			
			nActualBytesSent = write(m_socket, pData, maxlen);
			
			// did anything go wrong?
			if (nActualBytesSent < 0)
			{
				if (errno != EAGAIN)
				{
					String s("Error sending data to server - connection closed");
					m_pFrameWnd->ServerShutdown(s);
					return -1;
				}
				m_writeTag = gdk_input_add(m_socket, GDK_INPUT_WRITE,
										   socket_write,
										   (void*) this);
				
				return 0;
			}

			nBytesSent += nActualBytesSent;
			m_strDataOut.RemoveBytes(nActualBytesSent);
			m_strDataOut.GetBytes(&pData);
		
			m_pOutgoingList->SentData(nBytesSent);
		}
	}

	return 0;
}

int CVermSocket::SendMessage(CVermProtocolMessage* pMessage, int bStore)
{
	if (m_socket < 0)
		return -1;
	
	if (pMessage->m_number != PM_LOGIN)
		m_pFrameWnd->m_nCurrentCommand = pMessage->m_number;
	
	// add the new message to the message store

	// user wants us to compress ... let the server know

	if (Vermie::m_bUseCompression && Vermie::m_bCanUseCompression &&
				 pMessage->m_number != PM_LOGIN)
	{
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_USE_COMPRESSION,
													"yes"));
	}

	if (Vermie::m_strFileEOL == UNIX_EOL)
	{
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_UNIX_EOL_MARKER,"yes"));
	}
	else if (Vermie::m_strFileEOL == MAC_EOL)
	{
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_MAC_EOL_MARKER,"yes"));
	}
							
	if (bStore)
		m_pMessageStore->AddMessage(pMessage);

	// If the String isn't empty, the previous sent message was blocked
	// and is still waiting to be sent.  Just append the new message
	// onto the end of the buffer and this message will be sent along when
	// the other gets sent.

	// build the message

	int bIsEmpty = (m_strDataOut.m_pHead == NULL);
	
	String strMessage;

	char number[20];
	sprintf(number, "%i\r\n", pMessage->m_number);

	strMessage = number;
	
	pMessage->AppendHeadersToString(strMessage);

	strMessage += "\r\n";

	int messageLen = 0;
	
#ifndef BUILD_STANDARD
	if (m_pCrypto)
	{
		// add the encryption block header

		char IV[CRYPTO_HEADER_IV_SIZE];

		m_pCrypto->GetRandomNumber(IV, CRYPTO_HEADER_IV_SIZE);

		String strIV(IV, 8);

		m_strDataOut += strIV;
		
		messageLen += strIV.length();
		
		// force the length to be CRYPTO_HEADER_LENGTH_SIZE bytes

		char strL[16];
		
		sprintf(strL, "%i", strlen(strMessage.chars()));

		String strLength = strL;
		
		int nLength = strlen(strLength.chars());

		for (int k = nLength; k < CRYPTO_HEADER_LENGTH_SIZE; k++)
			strLength += PAD_CHAR;
		
		VermByteQueue* pResult = m_pCrypto->Encrypt(&strLength, IV);

		m_strDataOut += *pResult;

		messageLen += pResult->m_pHead->m_nLen;
		
		pResult->m_pHead = NULL;
		delete pResult;
		
		// encrypt the message

		pResult = m_pCrypto->Encrypt(&strMessage, IV);

		m_strDataOut += *pResult;
		messageLen += pResult->m_pHead->m_nLen;
		
		pResult->m_pHead = NULL;
		delete pResult;
	}
	else
#endif
	{
		m_strDataOut += strMessage;
		messageLen += strMessage.length();
	}

	// send the message body

	if (pMessage->m_body)
	{
#ifndef BUILD_STANDARD
		if (m_pCrypto)
		{
			VermByteQueue* pStrBody = new VermByteQueue();
			pStrBody->AddBytes(pMessage->m_body, pMessage->m_bodySize);
			
			// now that we have made a copy of the message body, free the
			// original to save memory
			
			delete [] pMessage->m_body;
			pMessage->m_body = NULL;
			// add the encryption block header

			char IV[CRYPTO_HEADER_IV_SIZE];

			m_pCrypto->GetRandomNumber(IV, CRYPTO_HEADER_IV_SIZE);

			String strIV (IV, CRYPTO_HEADER_IV_SIZE);
			
			m_strDataOut += strIV;
			messageLen += strIV.length();
			
			// force the length to be CRYPTO_HEADER_LENGTH_SIZE bytes

			char strL[16];
			sprintf(strL, "%i", pMessage->m_bodySize);

			String strLength(strL);
			
			int nLength = strLength.length();

			for (int k = nLength; k < CRYPTO_HEADER_LENGTH_SIZE; k++)
				strLength += PAD_CHAR;

			VermByteQueue* pResult = m_pCrypto->Encrypt(&strLength, IV);

			m_strDataOut += *pResult;
			messageLen += pResult->m_pHead->m_nLen;			

			pResult->m_pHead = NULL;
			delete pResult;
			
			// encrypt the message

			pResult = m_pCrypto->Encrypt(pStrBody, IV);

			// free the body now to save memory

			m_strDataOut += *pResult;
			messageLen += pResult->m_pHead->m_nLen;			

			pResult->m_pHead = NULL;
			delete pResult;
		}
		else
#endif
		{
			m_strDataOut.AddBytes(pMessage->m_body, pMessage->m_bodySize);
			messageLen += pMessage->m_bodySize;

			delete []pMessage->m_body;
			pMessage->m_body = NULL;
		}
	}

	// reset the pointer to the outgoing buffer

	m_pOutgoingList->AddMessage(messageLen,
								pMessage,
								pMessage->m_number);

	if (bIsEmpty)
		return SendData();

	return 0;
}

void CVermSocket::Close(void)
{
	if (m_socket)
	{
		close(m_socket);
		shutdown(m_socket, 2);
		m_socket = -1;

		if (m_writeTag)
		{
			gdk_input_remove(m_writeTag);
			m_writeTag = 0;
		}

		gdk_input_remove(m_readTag);

		m_bCallReceive = 0;
	}

	m_bGotProxyHeader = FALSE;
	m_pMessageStore->Empty();
}

