/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#if !defined(AFX_VERMSOCKET_H__25BA4DD4_C7F4_11D1_A276_00A0C95709EC__INCLUDED_)
#define AFX_VERMSOCKET_H__25BA4DD4_C7F4_11D1_A276_00A0C95709EC__INCLUDED_

// VermSocket.h : header file
//

#ifndef BUILD_STANDARD
#include "VermCrypto.h"
#endif // !BUILD_STANDARD

#include "VermByteQueue.h"
#include <String.h>
#include "zlib.h"

#define PARSE_BUFFER_SIZE	512
#define SOCKET_BUFFER_SIZE	2048

// Error codes
#define SOCKET_OK               0
#define SOCKET_CANT_CREATE     -1
#define SOCKET_MUST_BLOCK      -2
#define SOCKET_CANT_FIND_HOST  -3
#define SOCKET_CANT_CONNECT    -4

class CVermFrameWnd;
class CVermMessageStore;
class CVermProtocolMessage;
class VermByteQueue;

class OutgoingData
{
public:
	int                   m_nMessageSize;
	CVermProtocolMessage* m_pMessageIn;
	int			          m_pWhich;
	OutgoingData*         m_pNext;

	OutgoingData(int size, CVermProtocolMessage*, int);
	~OutgoingData();
};

class OutgoingList
{
public:
	OutgoingData* m_pHead;
	OutgoingData* m_pTail;
	int m_nSentSoFar;
	CVermFrameWnd* m_pParent;

	OutgoingList(CVermFrameWnd* pParent);

	void AddMessage(int messageSize, CVermProtocolMessage*, int);
	void DeleteHead();
	void SentData(int sentLen);
};


/////////////////////////////////////////////////////////////////////////////
// CVermSocket command target

class CVermSocket
{
// Attributes
public:
	int m_writeTag;
	int m_readTag;
	short m_bCallReceive;
	short m_bGotProxyHeader;
	int m_socketBufferSize;
	
	CVermFrameWnd*		m_pFrameWnd;
	
private:
    int                 m_socket;
	OutgoingList*       m_pOutgoingList;
	CVermMessageStore*	m_pMessageStore;
	CVermProtocolMessage*	m_pMessageIn;

	
	// used for parsing messages

	int m_readState;

	char m_socketBuffer[SOCKET_BUFFER_SIZE];

	char m_parseBuffer[PARSE_BUFFER_SIZE];
	int m_parseBufferSize;

	int m_bytesProcessed;

	// used for decrypting messages

#ifndef BUILD_STANDARD
	CVermCrypto* m_pCrypto;

	char m_IV[CRYPTO_HEADER_IV_SIZE];

	char* m_decryptBuffer;
	int m_decryptBufferSize;
	int m_decryptBufferMaxSize;

	char m_decryptHeaderBuffer[CRYPTO_HEADER_SIZE];
	int m_decryptHeaderBufferSize;

	int m_numBytesToDecrypt;
	int m_numBytesDecrypted;
#endif
	
	int m_numBytesInMessage;

	// used for sending messages

	VermByteQueue m_strDataOut;
	char* m_pDataOut;
	int m_nBytesSent;

// Operations
public:
	CVermSocket(CVermFrameWnd *pFrameWnd);
	virtual ~CVermSocket();
	int Connect(String& host, int port);
	void Close(void);
	void ReceiveResponse();
	int SendData();
	
// Overrides
public:

// Implementation
private:

	void ReadBody();
	void ParseMessage();

	int ReadData();

public:
	int SendMessage(CVermProtocolMessage* pMessage, int bStore = 1);

#ifndef BUILD_STANDARD
	void SetSecure(CVermCrypto* crypto);
#endif
};

#endif // !defined(AFX_VERMSOCKET_H__25BA4DD4_C7F4_11D1_A276_00A0C95709EC__INCLUDED_)
