/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include "Vermie.h"
#include "VermTreeLeaf.h"
#include "VermTreeNode.h"

#include <String.h>

CVermTreeLeafList::CVermTreeLeafList()
{
	m_pHead = NULL;
	m_pTail = NULL;
}

CVermTreeLeafList::~CVermTreeLeafList()
{
	VermTreeListNode *pNode = m_pHead;

	while (pNode)
	{
		VermTreeListNode* pTmp = pNode;
		
		delete pNode->m_pLeaf;
		pNode = pNode->m_pNext;

		delete pTmp;
	}

}

void CVermTreeLeafList::AddTail(CVermTreeLeaf* pLeaf)
{
	if (!m_pHead)
	{
		m_pHead = new VermTreeListNode;
		m_pTail = m_pHead;
		m_pHead->m_pNext = NULL;
	}
	else
	{
		m_pTail->m_pNext = new VermTreeListNode;
		m_pTail = m_pTail->m_pNext;
		m_pTail->m_pNext = NULL;
	}

	m_pTail->m_pLeaf = pLeaf;
}

CVermTreeNode::CVermTreeNode()
{
	m_bVisited = 0;
	m_bChecked = 0;
	m_bWebDeploy = 0;
	m_strDeployDir = "";
	m_bCloaked = 0;
	
	m_pLeafs = new CVermTreeLeafList;
}

CVermTreeNode::CVermTreeNode(CVermTreeNode* pTreeNode)
{
	m_bVisited = pTreeNode->m_bVisited;
	m_bChecked = pTreeNode->m_bChecked;
	m_pLeafs = new CVermTreeLeafList;
	m_bWebDeploy = pTreeNode->m_bWebDeploy;
	m_strDeployDir = pTreeNode->m_strDeployDir;
	m_bCloaked = pTreeNode->m_bCloaked;
	
	m_strWorkingDir = pTreeNode->m_strWorkingDir;

	VermTreeListNode* pList = pTreeNode->m_pLeafs->m_pHead;

	while (pList)
	{
		CVermTreeLeaf* pLeaf = new CVermTreeLeaf(pList->m_pLeaf);
		AddLeaf(pLeaf);
		pList = pList->m_pNext;
	}
}

CVermTreeNode::~CVermTreeNode()
{
	if (m_pLeafs)
		delete m_pLeafs;
}

void CVermTreeNode::AddLeaf(CVermTreeLeaf* pLeaf)
{
	if (!m_pLeafs)
		return;

	m_pLeafs->AddTail(pLeaf);
}

CVermTreeLeaf* CVermTreeNode::GetLeafByName(String& pName)
{
	if (!m_pLeafs)
		return NULL;

	VermTreeListNode* pTmp = m_pLeafs->m_pHead;

	while (pTmp)
	{
		if (pName == pTmp->m_pLeaf->m_name)
			return pTmp->m_pLeaf;

		pTmp = pTmp->m_pNext;
	}
	
	return NULL;
}

void CVermTreeNode::DeleteLeaf(CVermTreeLeaf* pLeaf)
{
	if (!m_pLeafs)
		return;

	VermTreeListNode* pTmp = m_pLeafs->m_pHead;
	VermTreeListNode* pPrev = NULL;

	while (pTmp)
	{
		if (pTmp->m_pLeaf == pLeaf)
		{
			// head needs to be deleted
			if (!pPrev)
			{
				pTmp = m_pLeafs->m_pHead;
				m_pLeafs->m_pHead = m_pLeafs->m_pHead->m_pNext;
				delete pTmp;
				return;
			}
			pPrev->m_pNext = pTmp->m_pNext;
			delete pTmp;
			return;
		}

		pPrev = pTmp;
		pTmp = pTmp->m_pNext;
	}
}

void CVermTreeNode::DeleteMissingLeaves(void)
{
	if (!m_pLeafs)
		return;
	
	VermTreeListNode* pTmp = m_pLeafs->m_pHead;
	VermTreeListNode* pPrev = NULL;
	
	while (pTmp)
	{
		if (!pTmp->m_pLeaf->m_bChecked)
		{
			if (!pPrev)
			{
				pPrev = pTmp;
				m_pLeafs->m_pHead = m_pLeafs->m_pHead->m_pNext;

				pTmp = m_pLeafs->m_pHead;

				delete pPrev;

				pPrev = NULL;
			}
			else
			{
				pPrev->m_pNext = pTmp->m_pNext;
				delete pTmp;
				pTmp = pPrev->m_pNext;
			}
		}
		else
		{
			pTmp->m_pLeaf->m_bChecked = 0;
			pPrev = pTmp;
			pTmp = pTmp->m_pNext;
		}
	}
}

// So far, this function is only called by CProjectView::SetWorkingDirectory()
// to erase any local version information when the working directory
// is changed
//
//  Loop over all the leaves in this node, and remove local version info
void CVermTreeNode::DeleteLeafInformation(void)
{
	if (!m_pLeafs)
		return;
	
	VermTreeListNode* pTmp = m_pLeafs->m_pHead;

	while (pTmp)
	{
		pTmp->m_pLeaf->m_nLocalVersionNumber = -1;
		pTmp->m_pLeaf->m_pLocalVersionDate = 0;

		pTmp = pTmp->m_pNext;
	}
}
