/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdio.h>
#include <fcntl.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include "Vermie.h"
#include "VermTreeNode.h"
#include "VermTreeLeaf.h"
#include "VermFrameWnd.h"
#include "VermProjectView.h"
#include "YesNoDlg.h"
#include "VermUtils.h"
#include "VermTreeUtils.h"
#include "AfxMessageBox.h"
#include "VermProtocolMessage.h"
#include "zlib.h"

#define HEADER "$$$Version 3.0$$$"
#define FILE_EXTENSION	".sos"
#define SERVER_DIRECTORY_NAME "servers"
#define SOS_FILE_NAME "database"

static void FocusChanged(GtkWidget*, GdkEventFocus*, CVermProjectView* pView)
{
	if (!pView)
		return;
	
	pView->m_pParent->SetFocus(FOCUS_PROJECTVIEW);
}

char* GetNextString(char** pBody, int nBodySize, int* nCharsRead, int* bError)
{
	int i = 0;

	while (((*pBody)[i] != '\r') && ((*nCharsRead + i + 1) < nBodySize))
		i++;

	if ((*pBody)[i] != '\r')
	{
		*bError = 1;
		return NULL;
	}

	(*pBody)[i] = '\0';

	char* ret = *pBody;
	
	*pBody = *pBody + (i + 2);

	*nCharsRead += (i + 2);

	return ret;
}

CVermTreeLeaf* ParseFileInfo(char** pMessageBody, int nBodySize, int* nCharsRead, const char*, int bPinStatus)
{
	int bError = 0;
	
	if ((!pMessageBody) || (!*pMessageBody) || (*nCharsRead >= nBodySize))
		return NULL;


	// create a new tree leaf

	CVermTreeLeaf* pLeaf = new CVermTreeLeaf();

	// parse out the filename

	pLeaf->m_name = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		delete pLeaf;
		return NULL;
	}

	// parse out the version number

	char* verNum = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);
	
	if (bError)
	{
		delete pLeaf;
		return NULL;
	}

	pLeaf->m_nRemoteVersionNumber = atoi(verNum);

	if (bPinStatus)
	{
		char* strVer = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

		pLeaf->m_nPinnedVersion = atoi(strVer);

		if (bError)
		{
			delete pLeaf;
			return NULL;
		}
	}

	// parse out the remote date

	pLeaf->m_pRemoteVersionDate = ParseServerDate(GetNextString(pMessageBody, nBodySize, nCharsRead, &bError));

	if (bError)
	{
		delete pLeaf;
		return NULL;
	}

	// parse out the user holding a lock, if any

	pLeaf->m_userLocked = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		delete pLeaf;
		return NULL;
	}

	pLeaf->m_projectLocked = GetNextString(pMessageBody, nBodySize, nCharsRead, &bError);

	if (bError)
	{
		delete pLeaf;
		return NULL;
	}

	// return the new leaf

	return pLeaf;
}

void ParseFileList(CVermTreeNode *pNode, CVermProtocolMessage* pMessage, const char* pszUsername,
				   String& strWorkingDir, int bChecksum, int bPinStatus)
{
	int nCharsRead = 0;

	char *pBody = new char[pMessage->m_bodySize];
	int nBodySize = pMessage->m_bodySize;
	memcpy(pBody, pMessage->m_body, nBodySize);
	char *pParseBody = pBody;
	
	while (nCharsRead < nBodySize)
	{
		CVermTreeLeaf* pNewLeaf = ParseFileInfo(&pParseBody, nBodySize,
												&nCharsRead, pszUsername, bPinStatus);

		unsigned long nChecksum = 0;
		unsigned long nSerChecksum = 0;
		long modTime = 0L;
		
		if (bChecksum && pNewLeaf)
		{
			int bError = 0;
			char* checksum = GetNextString(&pParseBody, nBodySize, &nCharsRead, &bError);

			if (bError)
			{
				delete pNewLeaf;
				break;
			}

			nSerChecksum = strtoul(checksum, (char**) NULL, 10);

			String strFile = strWorkingDir + "/";
			strFile += pNewLeaf->m_name;

			struct stat st;
			memset(&st, 0, sizeof(struct stat));
				   
			if (!stat(strFile.chars(), &st))
				modTime = st.st_mtime;
			
			FILE* f = fopen(strFile.chars(), "r");

			if (f)
			{
				char body[2048];
				nChecksum = 0L;
				int nBytesRead = 0;
				fseek(f, 0, SEEK_END);                                          
				int bodySize = ftell(f);                                        
				fseek(f, 0, SEEK_SET);                                          

				while (nBytesRead < bodySize)
				{
					int nRead = fread(body, sizeof(char), 2048, f); 
                       
					if (nRead == 0)
						break;

					nChecksum = crc32(nChecksum, (Bytef*)body, nRead);

					nBytesRead += nRead;
				}
				
				fclose(f);                                                              
			}
		}

		if (Vermie::m_bUseVSS6Features && Vermie::m_bShowShareIcon)
		{
			int bError = 0;
			char* share = GetNextString(&pParseBody, nBodySize, &nCharsRead, &bError);

			if (bError)
			{
				if (pNewLeaf != NULL)
					delete pNewLeaf;
				break;
			}

			if (atoi(share) > 1)
				pNewLeaf->m_bShared = 1;
		}
		
		if (!pNewLeaf)
			break;

		// does the leaf already exist in the list?

		CVermTreeLeaf* pLeaf = pNode->GetLeafByName(pNewLeaf->m_name);

		if (pLeaf)
		{
			// just update the leaf's info

			pLeaf->m_nRemoteVersionNumber = pNewLeaf->m_nRemoteVersionNumber;

			pLeaf->m_pRemoteVersionDate = pNewLeaf->m_pRemoteVersionDate;

			pLeaf->m_userLocked = pNewLeaf->m_userLocked;

			pLeaf->m_projectLocked = pNewLeaf->m_projectLocked;

			pLeaf->m_nPinnedVersion = pNewLeaf->m_nPinnedVersion;
			
			pLeaf->m_bChecked = 1;

			pLeaf->m_bShared = pNewLeaf->m_bShared;
			
			if (bChecksum && (nChecksum == nSerChecksum))
			{
				pLeaf->m_nLocalVersionNumber = pNewLeaf->m_nRemoteVersionNumber;
				pLeaf->m_pLocalVersionDate = modTime;	
			}

			delete pNewLeaf;
		}
		else
		{
			pNewLeaf->m_bChecked = 1;

			if (bChecksum && (nChecksum == nSerChecksum))
			{
				pNewLeaf->m_nLocalVersionNumber = pNewLeaf->m_nRemoteVersionNumber;
				pNewLeaf->m_pLocalVersionDate = modTime;
			}

			pNode->AddLeaf(pNewLeaf);
		}
	}

	delete []pBody;
	
	// check to see if any of our leaves have been deleted from SS and
	// remove them
	pNode->DeleteMissingLeaves();
}

void ParseTree(GtkWidget* pTreeCtrl, char* buf, int* pos, int maxPos,
			   GtkWidget* pMenu, CVermProjectView* pFocusView)
{
	int bParseError;
	int numChildren = 0;
	
	char* nodeName = ParseNameFromBuffer(buf, pos, maxPos, bParseError);

	while (nodeName != NULL)
	{
		GtkWidget* hItem = gtk_tree_item_new_with_label(nodeName);
		gtk_widget_show(hItem);

		gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pNormalStyle);

		gtk_signal_connect(GTK_OBJECT(hItem), "button_press_event",
						   GTK_SIGNAL_FUNC (ButtonPress), pMenu);

		gtk_signal_connect(GTK_OBJECT(hItem), "focus_in_event",
						   GTK_SIGNAL_FUNC(FocusChanged), pFocusView);

		String strNode = nodeName;
		
		InsertItemIntoTree(pTreeCtrl, hItem, strNode);
		
		GtkWidget* pSubtree = gtk_tree_new();
		gtk_widget_show(pSubtree);
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(hItem), pSubtree);

		CVermTreeNode* pTreeNode = new CVermTreeNode();

		gtk_object_set_user_data(GTK_OBJECT(hItem), pTreeNode);

		ParseTree(pSubtree, buf, pos, maxPos, pMenu, pFocusView);

		nodeName = ParseNameFromBuffer(buf, pos, maxPos, bParseError);

		++numChildren;
	}

	if (numChildren == 0)
		gtk_tree_item_remove_subtree(GTK_TREE_ITEM(GTK_TREE(pTreeCtrl)->tree_owner));
}

void CopyTree(GtkWidget* pSrcTree, GtkWidget* destTree)
{
	CopyTreeNode(pSrcTree, destTree);
}

char* GetItemText(GtkWidget* pItem)
{
	char *name = NULL;
	GtkLabel* l;
	
	l = GTK_LABEL(GTK_BIN(pItem)->child);
	gtk_label_get(l, &name);

	return name;
}
	
void CopyTreeNode(GtkWidget* pSrcTree, GtkWidget* pDestTree)
{
	GList* children = NULL;

	children = gtk_container_children(GTK_CONTAINER(pSrcTree));

	while (children)
	{
		CVermTreeNode* pTreeNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(children->data));

		CVermTreeNode* pNewNode = new CVermTreeNode(pTreeNode);

		char* nodeName = GetItemText(GTK_WIDGET(children->data));

		GtkWidget* hItem = gtk_tree_item_new_with_label(nodeName);
		gtk_widget_show(hItem);

		gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pNormalStyle);
		
		gtk_object_set_user_data(GTK_OBJECT(hItem), pNewNode);
		
		String strNode = nodeName;
		
		InsertItemIntoTree(pDestTree, hItem, strNode);
		
		if (GTK_TREE_ITEM(children->data)->subtree)
		{
			GtkWidget* pSubtree = gtk_tree_new();
			gtk_widget_show(pSubtree);
			
			gtk_tree_item_set_subtree(GTK_TREE_ITEM(hItem), pSubtree);
			
			CopyTreeNode(GTK_TREE_ITEM(children->data)->subtree, pSubtree);
		}

		children = g_list_remove_link(children, children);
	}
}

int ReadTreeNode(GtkWidget* pSrcTree, char* pBuffer, int* nFilePos,int maxPos,
				 GtkWidget* pMenu, CVermProjectView* pFocusView)
{
	int bParseError;

	// read all child nodes for this node

	char* pszNodeName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);
	int numChildren = 0;
	
	while (pszNodeName != NULL)
	{
		String strNodeName = pszNodeName;
		
		short bWebProject = 0;
		short bCloaked = 0;
		
		if (strNodeName.index("$$$") == 0)
		{
			bWebProject = 1;

			strNodeName.del(0, 3);
		}

		if (strNodeName.index("$$") == 0)
		{
			bCloaked = 1;

			strNodeName.del(0, 2);
		}
		
		GtkWidget* hChildNode = gtk_tree_item_new_with_label(strNodeName.chars());
		gtk_widget_show(hChildNode);


		gtk_signal_connect(GTK_OBJECT(hChildNode), "button_press_event",
						   GTK_SIGNAL_FUNC (ButtonPress), pMenu);
		gtk_signal_connect(GTK_OBJECT(hChildNode), "focus_in_event",
						   GTK_SIGNAL_FUNC(FocusChanged), pFocusView);

		
		InsertItemIntoTree(pSrcTree, hChildNode, strNodeName);
		
		CVermTreeNode* pChildNode = new CVermTreeNode();

		pChildNode->m_bWebDeploy = bWebProject;

		pChildNode->m_bCloaked = bCloaked;

		if (pChildNode->m_bCloaked)
		{
			if (pChildNode->m_bWebDeploy)
				gtk_widget_set_style(GTK_BIN(hChildNode)->child, Vermie::m_pCloakedDeployStyle);
			else
				gtk_widget_set_style(GTK_BIN(hChildNode)->child, Vermie::m_pCloakedStyle);
		}
		else
		{
			if (pChildNode->m_bWebDeploy)
				gtk_widget_set_style(GTK_BIN(hChildNode)->child, Vermie::m_pWebDeployStyle);
			else
				gtk_widget_set_style(GTK_BIN(hChildNode)->child, Vermie::m_pNormalStyle);
		}
			
		gtk_object_set_user_data(GTK_OBJECT(hChildNode), pChildNode);
		GtkWidget* pSubtree = gtk_tree_new();
		gtk_widget_show(pSubtree);
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(hChildNode), pSubtree);
		
		// read this project's working directory

		char* pszWorkingDir = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);

		if (pszWorkingDir)
			pChildNode->m_strWorkingDir = pszWorkingDir;
		else if (bParseError)
			return FALSE;

		// recurse this child

		if (!ReadTreeNode(pSubtree, pBuffer, nFilePos, maxPos, pMenu,
						  pFocusView))
		{
			return 0;
		}

		++numChildren;
		// get the next child

		pszNodeName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);
	}

	GtkTreeItem* pItem = GTK_TREE_ITEM(GTK_TREE(pSrcTree)->tree_owner);

	if (!numChildren)
		gtk_tree_item_remove_subtree(pItem);
		
	if (bParseError)
		return 0;

	// now read any files for this node

	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(pItem));

	char* pszFileName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);

	while (pszFileName != NULL)
	{
		CVermTreeLeaf* pLeaf = new CVermTreeLeaf();

		pLeaf->m_name = pszFileName;

		// parse out the version number

		char* pszVerNum = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);

		if (pszVerNum)
			pLeaf->m_nLocalVersionNumber = atoi(pszVerNum);
		else if (bParseError)
			return 0;

		// parse out the fetch date

		char* pszLocalDate = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);

		if (pszLocalDate)
			pLeaf->m_pLocalVersionDate = (time_t)atoi(pszLocalDate);
		else if (bParseError)
			return 0;

		// parse out the unused filed

		 ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);

		// add the leaf to the node

		pNode->m_pLeafs->AddTail(pLeaf);

		// read the next leaf

		pszFileName = ParseNameFromBuffer(pBuffer, nFilePos, maxPos, bParseError);
	}

	if (bParseError)
		return 0;

	return 1;
}

void WriteTreeNode(GtkWidget* pSrcTree, FILE* file)
{
	// add the name of this node

	String strName = GetItemText(pSrcTree);
	strName += "\r\n";

	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(pSrcTree));

	if (pNode->m_bWebDeploy)
		strName = "$$$" + strName;

	if (pNode->m_bCloaked)
		strName = "$$" + strName;
	
	fprintf(file, strName.chars());

	// add its working directory

	String strWorkingDir = pNode->m_strWorkingDir + "\r\n";

	fprintf(file, strWorkingDir.chars());

	// iterate over all of the subprojects in this project

	GtkTree* tree = NULL;

	if (GTK_TREE_ITEM(pSrcTree)->subtree)
		tree = GTK_TREE(GTK_TREE_ITEM(pSrcTree)->subtree);
	
	if (tree)
	{
		GList* children = gtk_container_children(GTK_CONTAINER(tree));
		
		while (children)
		{
			GtkWidget* hChild = GTK_WIDGET(children->data);
			
			WriteTreeNode(hChild, file);
			children = g_list_remove_link (children, children);
		}
	}
	
	// no more subprojects

	fprintf(file, "\r\n");

	// iterate over all of the files in this project

	CVermTreeLeaf* pLeaf;
	VermTreeListNode* pLeafNode = pNode->m_pLeafs->m_pHead;
	
	while (pLeafNode)
	{
		pLeaf = pLeafNode->m_pLeaf;
		
		// add the name of this leaf
		
		String strTmp = pLeaf->m_name;
		strTmp += "\r\n";
		
		fwrite(strTmp.chars(), sizeof(char), strTmp.length(), file);
		
		// add its local version number
		
		if (pLeaf->m_nLocalVersionNumber != -1)
		{
			char c[256];
			sprintf(c, "%i", pLeaf->m_nLocalVersionNumber);
			
			fwrite(c, sizeof(char), strlen(c), file);
		}
		
		fwrite("\r\n", sizeof(char), 2, file);
		
		// add its fetch date
		
		if (pLeaf->m_nLocalVersionNumber != -1)
		{
			char c[256];
			sprintf(c, "%ld", pLeaf->m_pLocalVersionDate);
			fwrite(c, sizeof(char), strlen(c), file);
		}
		
		fwrite("\r\n", sizeof(char), 2, file);
		
		// This field is no longer used - just write 0

		fwrite("0\r\n", sizeof(char), 3, file);
		
		pLeafNode = pLeafNode->m_pNext;
	}
	
	// no more files
	fwrite("\r\n", sizeof(char), 2, file);
}

void SaveTreeToFile(GtkWidget* pSrcTree, String& strServer,
					String& strDatabase, String& strCurrentProject)
{
	FILE* file;
	
	String strFileName;
	int bOldFileLocation;

	if (!FindSOSFileName(strServer, strDatabase, strFileName,
						 bOldFileLocation, 1))
	{
		// THIS SHOULD NEVER HAPPEN
		return;
	}

	String strRev = reverse(strFileName);
	int index = strRev.index('/');

	String strPrefFile = strFileName.before((int)strFileName.length() - index);

	strPrefFile += "prefs";
	
	file = fopen(strPrefFile.chars(), "w");
	
	if (!file)
	{
		String s("Unable to save server preferences information");
		AfxMessageBox(s);
		return;
	}

	fwrite("usecompression:", sizeof(char), strlen("usecompression:"), file);
	if (Vermie::m_bUseCompression)
		fwrite("1\n", sizeof(char), 2, file);
	else
		fwrite("0\n", sizeof(char), 2, file);

	if (!Vermie::m_strUserName.empty())
	{
		fwrite("username:", sizeof(char), strlen("username:"), file);
		fwrite(Vermie::m_strUserName.chars(), sizeof(char),
			   Vermie::m_strUserName.length(), file);
		fwrite("\n", sizeof(char), 1, file);
	}

	if (!Vermie::m_strDatabaseName.empty())
	{
		fprintf(file, "database:%s\n", Vermie::m_strDatabaseName.chars());
	}
	
	fclose(file);
	
	file = fopen(strFileName.chars(), "w");
	
	if (!file)
	{
		String s("Unable to save database information");

		AfxMessageBox(s);

		return;
	}
	// next write out the current project setting

	
	// header
	fprintf(file, HEADER);
	fprintf(file, "\r\n");

	if (strCurrentProject.empty())
		strCurrentProject = "$/";
	
	strCurrentProject += "\r\n";

	fwrite(strCurrentProject.chars(), sizeof(char),
		   strCurrentProject.length(), file);

	// write out the project tree

	GtkTree* pRoot = GTK_TREE_ROOT_TREE(pSrcTree);

	if (pRoot != NULL)
	{
		GList* children = gtk_container_children(GTK_CONTAINER(pSrcTree));
		
		if (children)
		{
			WriteTreeNode(GTK_WIDGET(children->data), file);
			g_list_free(children);
		}
	}

	// flush to disk and close the file

	fflush(file);
	fclose(file);
}

// Create a new entry in the index file for this database

void CreateDatabaseEntryInIndex(String& strIndexFile, String& strDatabase,
								String& strSOSFile, int num)
{
	FILE* fp = fopen(strIndexFile.chars(), "a");
	
	fprintf(fp, "%s\r\n", strDatabase.chars());
	fprintf(fp, "%s%d%s\r\n", SOS_FILE_NAME, num + 1, FILE_EXTENSION);
	char s[2048];
	sprintf(s, "%s%d%s", SOS_FILE_NAME, num + 1, FILE_EXTENSION);
	strSOSFile = s;
	fclose(fp);
}

// Go through the index file looking for the specified database:file match

int LookupSOSFileByDatabase(String& strIndexFile, String& strDatabase,
							String& strSOSFile, int bCreateEntry)
{
	int file = open(strIndexFile.chars(), O_RDONLY);

	if (file < 0)
	{
		if (bCreateEntry)
		{
			CreateDatabaseEntryInIndex(strIndexFile, strDatabase, strSOSFile, 0);
			return 1;
		}

		return 0;
	}

	int nFileSize = lseek(file, 0, SEEK_END);
	lseek(file, 0, SEEK_SET);

	char* pBuffer = new char[nFileSize];

	read(file, pBuffer, nFileSize);
	close(file);

	int nBufPos = 0;
	int bParseError;

	char* pDatabase = ParseNameFromBuffer(pBuffer, &nBufPos, nFileSize - 1, bParseError);
	char* pFile = NULL;

	while (pDatabase)
	{
		pFile = ParseNameFromBuffer(pBuffer, &nBufPos, nFileSize - 1,
									bParseError);

		if (strDatabase == pDatabase)
		{
			strSOSFile = pFile;	
			delete []pBuffer;
			return 1;
		}

		pDatabase = ParseNameFromBuffer(pBuffer, &nBufPos, nFileSize - 1, bParseError);
	}

	// No entry found - check to see if we should create a new Entry

	if (bCreateEntry)
	{
		// last file stored in index
		if (pFile)
		{
			while (*pFile && (*pFile < '0' || *pFile > '9'))
				++pFile;

			int num = atoi(pFile);

			// grab it's number and pass that along to the creation func

			CreateDatabaseEntryInIndex(strIndexFile, strDatabase, strSOSFile, num);
		}
		else // no entries in the file
		{
			CreateDatabaseEntryInIndex(strIndexFile, strDatabase, strSOSFile, 0);
		}

		delete []pBuffer;
		return 1;
	}

	delete []pBuffer;
	return 0;
}

// Try and find the correct SOS file for this server/database combo

int FindSOSFileName(String& strServerName, String& strDatabase,
		    String& strSOSFile, int& bOldFileLocation,
		    int bCreateEntry /* = FALSE */)
{
//	int nDotIndex = strServerName.index('.');

//	if (nDotIndex > 0)
//		strServerName = strServerName.before(nDotIndex);

	String strServerDirectory = Vermie::m_strPrefsDir;

	strServerDirectory += "/";
	strServerDirectory += SERVER_DIRECTORY_NAME;

	struct stat st;
	memset(&st, 0, sizeof(struct stat));

	if (stat(strServerDirectory.chars(), &st))
		if (mkdir(strServerDirectory.chars(), 0770))
			return 0;
			
	strServerDirectory += "/" + strServerName;
	
	if (stat(strServerDirectory.chars(), &st))
		if (mkdir(strServerDirectory.chars(), 0770))
			return 0;

	strServerDirectory += "/";

	String strIndexFileName = strServerDirectory + "index.sos";

	String strRealSOSFile;

	if (!LookupSOSFileByDatabase(strIndexFileName, strDatabase, strRealSOSFile, bCreateEntry))
	{
		// 1.1 and before just dumped .sos files in the exepath dir - look there and if we find one, use it
		String s = Vermie::m_strPrefsDir;

		s += "/";
		s += strServerName + FILE_EXTENSION;

		if (stat(s.chars(), &st))
			return 0;
		else
		{
			bOldFileLocation = 1;
			strSOSFile = s;
			return 1;
		}
	}

	// if we are saving the sos file, check to see if there's a <=1.1 sos file laying around
	// if there is, delete it
	if (bCreateEntry)
	{
		String s = Vermie::m_strPrefsDir;

		s += strServerName + FILE_EXTENSION;

		if (!stat(s.chars(), &st))
		{
			unlink(s.chars());
		}
	}

	strSOSFile = strServerDirectory + strRealSOSFile;

	return 1;
}

char* LoadTreeFromFile(GtkWidget* pSrcTree, String& strServer,
					   String& strDatabase, GtkWidget* pMenu,
					   CVermProjectView* pFocusView)
{
	int file;
	
	String strServerName = strServer;

	String strFileName;
	
	int bOldFileLocation = 0;

	if (!FindSOSFileName(strServer, strDatabase, strFileName, bOldFileLocation))
		return NULL;

	file = open(strFileName.chars(), O_RDONLY);
	
	if (file < 0)
		return NULL;

	// read the entire file into a big buffer

	int nFileSize = lseek(file, 0, SEEK_END);

	lseek(file, 0, SEEK_SET);

	char* pBuffer = new char[nFileSize];

	read(file, pBuffer, nFileSize);

	close(file);

	int nFilePos = 0;

	int bParseError;

	// first parse out the database name

	String errorParsing("Error parsing ");
	errorParsing += strFileName;
	
	// next parse out the current project setting

	char* pszCurProject = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, bParseError);

	if (pszCurProject && !strncmp(pszCurProject, HEADER, strlen(HEADER)))
	{
		pszCurProject = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, bParseError);
	}

	if (!pszCurProject)
	{
		AfxMessageBox(errorParsing);
		delete []pBuffer;
		return NULL;
	}

	char* pszProject = new char[strlen(pszCurProject) + 1];

	if (!pszProject)
	{
		delete []pBuffer;
		return NULL;
	}

	strcpy(pszProject, pszCurProject);

	// read the project tree starting with the root

	GtkTree* pTree = GTK_TREE(pSrcTree);

	GList* children = gtk_container_children(GTK_CONTAINER(pTree));
	if (children)
	{
		gtk_tree_remove_items(GTK_TREE(pTree), children);
		g_list_free(children);
	}

	char* pszRootNode = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, bParseError);

	if (!pszRootNode)
	{
		AfxMessageBox(errorParsing);
		delete []pBuffer;
		delete []pszProject;
		return NULL;
	}

	GtkWidget* root = gtk_tree_new();
	gtk_widget_show(root);
	GtkWidget* item = gtk_tree_item_new_with_label(pszRootNode);
	gtk_widget_show(item);
	
	gtk_signal_connect(GTK_OBJECT(item), "button_press_event",
					   GTK_SIGNAL_FUNC (ButtonPress), pMenu);
	gtk_signal_connect(GTK_OBJECT(item), "focus_in_event",
					   GTK_SIGNAL_FUNC(FocusChanged), pFocusView);
	GTK_WIDGET_SET_FLAGS(item, GTK_CAN_FOCUS);

	gtk_widget_set_events(item, GDK_FOCUS_CHANGE_MASK);
//	gtk_signal_connect(GTK_OBJECT(item), "focus_in_event",
//					   GTK_SIGNAL_FUNC(FocusChanged), this);
	gtk_tree_append(GTK_TREE(pSrcTree), item);
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(item), root);
	
//	HTREEITEM hRootNode = treeCtrl.InsertItem(pszRootNode, 0, 1, TVI_ROOT);

	CVermTreeNode* pRootNode = new CVermTreeNode();

	gtk_object_set_user_data(GTK_OBJECT(item), pRootNode);
//	treeCtrl.SetItemData(hRootNode, (DWORD)pRootNode);

	char* pszWorkingDir = ParseNameFromBuffer(pBuffer, &nFilePos, nFileSize - 1, bParseError);

	if (pszWorkingDir)
		pRootNode->m_strWorkingDir = pszWorkingDir;
	else if (bParseError)
	{
		AfxMessageBox(errorParsing);
		delete []pBuffer;
		delete []pszProject;
		return NULL;
	}

	if (!ReadTreeNode(root, pBuffer, &nFilePos, nFileSize - 1, pMenu,
					  pFocusView))
	{
		AfxMessageBox(errorParsing);
		delete []pBuffer;
		delete []pszProject;
		return NULL;
	}

	if (pRootNode->m_bCloaked)
	{
		gtk_widget_set_style(GTK_BIN(item)->child, Vermie::m_pCloakedStyle);
	}
	else
	{
		gtk_widget_set_style(GTK_BIN(item)->child, Vermie::m_pNormalStyle);
	}
	
	delete []pBuffer;

	return pszProject;
}

int GetPathFromTreeItem(GtkWidget* hItem, String& path)
{
	if (hItem == NULL)
		return 0;

	GtkTree* tree = GTK_TREE(hItem->parent);
	GtkTreeItem* item = GTK_TREE_ITEM(hItem);

	while (item != NULL)
	{
		String str = GetItemText(GTK_WIDGET(item));
		
		if (GTK_WIDGET(item) == hItem)
			path = str;
		else
		{
			if (tree && tree->tree_owner)
				path = str + "/" + path;
			else
				path = str + path;
		}

		if (tree && (tree->tree_owner != NULL))
		{
			item = GTK_TREE_ITEM(tree->tree_owner);
			if (GTK_WIDGET(tree)->parent)
				tree = GTK_TREE(GTK_WIDGET(tree)->parent);
			else
				tree = NULL;
		}
		else
			item = NULL;
	}
	return 1;
}

GtkWidget* GetTreeChildByName(GtkWidget* hParent, String& strChild)
{
	if (!GTK_TREE_ITEM(hParent)->subtree)
		return NULL;
	
	GList* children;
	children = gtk_container_children(GTK_CONTAINER (GTK_TREE_ITEM(hParent)->subtree));
	
	if (!children)
		return NULL;

	while (strChild != GetItemText(GTK_WIDGET(children->data)))
	{
		children = g_list_remove_link(children, children);

		if (children == NULL)
			return NULL;
	}

	return GTK_WIDGET(children->data);
}

GtkWidget* GetTreeItemFromProject(GtkWidget* pTreeCtrl, String& pszProject)
{
  return GetTreeItemFromProject(pTreeCtrl, pszProject.chars());
}

GtkWidget* GetTreeItemFromProject(GtkWidget* pTreeCtrl, const char* pszProject)
{
	// make a copy of the project for parsing

	String strProjectPath(pszProject);

	// parse the project

	GtkWidget* hItem = pTreeCtrl;
	
	int nIndex = strProjectPath.index('/');

	String strProject = strProjectPath.before(nIndex + 1);

	if (strProject != GetItemText(hItem))
		return NULL;

	strProjectPath = strProjectPath.after(nIndex);

	nIndex = strProjectPath.index('/');

	if (nIndex == -1)
	{
		if (strProjectPath.empty())  // root node
			return hItem;
		else
			return GetTreeChildByName(hItem, strProjectPath);
	}

	while (nIndex >= 0)
	{
		// parse out the next node

		String strProject = strProjectPath.before(nIndex);

		hItem = GetTreeChildByName(hItem, strProject);

		if (!hItem)
			return NULL;

		strProjectPath = strProjectPath.after(nIndex);

		nIndex = strProjectPath.index('/');
	}

	return GetTreeChildByName(hItem, strProjectPath);
}

CVermTreeNode* GetTreeNodeFromProject(GtkWidget* pTreeCtrl,
									  const char* pszProject)
{
	GtkWidget* hItem = GetTreeItemFromProject(pTreeCtrl, pszProject);

	if (hItem)
		return (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));
	else
		return NULL;
}

String GetWorkingDirectory(GtkWidget* pTreeCtrl, const char* pszProject)
{
	GtkWidget* hItem = GetTreeItemFromProject(pTreeCtrl, pszProject);

	if (hItem)
		return GetWorkingDirectory(hItem);
	else
		return "";
}

String GetWorkingDirectory(GtkWidget* hItem)
{
	if (!hItem)
		return "";

	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

//	if (!pNode->m_strTmpWorkingDir.IsEmpty())
//		return pNode->m_strTmpWorkingDir;

	if (!pNode->m_strWorkingDir.empty())
		return pNode->m_strWorkingDir;

	int bFoundDir = FALSE;

	String strWorkingDir = GetItemText(hItem);

	// root node will have no parent
	if (!hItem->parent)
		return "";
	
	GtkTree* tree = GTK_TREE(hItem->parent);

	GtkTreeItem* item = NULL; 

	if (tree->tree_owner)
		item = GTK_TREE_ITEM(tree->tree_owner);
	
	while (item != NULL)
	{
		CVermTreeNode* pParentNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(item));

		String strTmpPath;

		if (pParentNode->m_strWorkingDir.empty())
		{
			strTmpPath = GetItemText(GTK_WIDGET(item));

			if (strTmpPath[strTmpPath.length() - 1] != '/')
				strTmpPath += "/";

			strWorkingDir = strTmpPath + strWorkingDir;
		}
		else
		{
//			if (pParentNode->m_strTmpWorkingDir.IsEmpty())
				strTmpPath = pParentNode->m_strWorkingDir;
//			else
//				strTmpPath = pParentNode->m_strTmpWorkingDir;

			if (strTmpPath[strTmpPath.length() - 1] != '/')
				strTmpPath += "/";

			strWorkingDir = strTmpPath + strWorkingDir;

			bFoundDir = TRUE;

			break;
		}

		if (tree)
		{
			if (GTK_WIDGET(tree)->parent)
				tree = GTK_TREE(GTK_WIDGET(tree)->parent);
			else
				tree = NULL;

			if (tree && tree->tree_owner)
				item = GTK_TREE_ITEM(tree->tree_owner);
			else
				item = NULL;
		}
		else
			item = NULL;
	}

	if (bFoundDir)
		return strWorkingDir;
	else
		return "";	// empty string
}

int IsFileModified(CVermTreeNode* pNode, const char* pszFileName)
{
	String strDir, strFile;

	ExtractChildFromParent(pszFileName, strDir, strFile, '\\');

	return IsFileModified(pNode, strFile, strDir);
}

int IsFileModified(CVermTreeNode* pNode, String& strFile, String& strDir)
{
	CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

	if (!pLeaf)
		return FILE_MISSING;

	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));

	String strFullFile = strDir + "/";
	strFullFile += strFile;
	
	if (stat(strFullFile.chars(), &sb))
		return FILE_MISSING;

	if (pLeaf->m_pLocalVersionDate >= 0
		&& (sb.st_mtime > pLeaf->m_pLocalVersionDate))
	{
		return FILE_MODIFIED;
	}

	if (pLeaf->m_nRemoteVersionNumber > pLeaf->m_nLocalVersionNumber)
		return FILE_OUT_OF_SYNC;

	return FILE_NOT_MODIFIED;
}

/*
** IsFileLocked returns 0 if the file is not locked by the user, 1 if the file is
** locked by the user in some project, 2 if the file is locked by the user in this
** specific project
*/

int IsFileLocked(CVermTreeNode* pNode, String& strProject, String& strFile,
				 String& strUsername)
{
	if (!pNode)
		return 0;

	CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

	return IsFileLocked(pLeaf, strProject, strUsername);
}

int IsFileLocked(CVermTreeLeaf* pLeaf, String& strProject, String& strUsername)
{
	if (!pLeaf)
		return 0;

	if (fcompare(pLeaf->m_userLocked, strUsername) != 0)
		return 0;

	if (fcompare(pLeaf->m_projectLocked, strProject) == 0)
		return 2;
	else
		return 1;
}

int ShouldFileBeFetched(CVermTreeLeaf* pLeaf, String& strProject,
						String& strUsername, struct stat& sb, int& modState,
						int& lockState, int& writeState, int& bOverride, int nVerNum)
{
	bOverride = 0;
	
	if (pLeaf)
	{
		// If we have the file locked, ask if they want to replace it.
		// If they do wish to replace it, treat it like a normal file ...
		int nLockStatus = IsFileLocked(pLeaf, strProject, strUsername);

		if (nLockStatus == 2)
		{
			if (lockState != IDYESTOALL && lockState != IDNOTOALL)
			{
				String s = pLeaf->m_name + " is currently locked by you.  Replace it?";
				CYesNoDlg dlg(s);
				lockState = dlg.DoModal();
			}

			if (lockState == IDNO || lockState == IDNOTOALL || lockState == IDCANCEL)
			{
				bOverride = 1;
				return 0;
			}
			else
				return 1;
		}

		// File is writeable - confirm over-write
		if (sb.st_mode & S_IWUSR)
		{
			if (writeState != IDYESTOALL && writeState != IDNOTOALL)
			{
				String s = pLeaf->m_name + " is writable - replace it?";
				CYesNoDlg dlg(s);
				writeState = dlg.DoModal();
			}

			if (writeState == IDNO || writeState == IDCANCEL || writeState == IDNOTOALL)
			{
				bOverride = 1;
				return 0;
			}
			else
				return 1;
		}

		// Unknown
		if (pLeaf->m_nLocalVersionNumber < 0)
			return 1;

		// The user is doing a get through the history dialog - just do it
		if (nVerNum > 0)
			return TRUE;

		// we currently have an older version ...
		if (pLeaf->m_nLocalVersionNumber < pLeaf->m_nRemoteVersionNumber)
			return 1;

		if (pLeaf->m_pLocalVersionDate < sb.st_mtime)
		{
			// File has been edited.  If it's not locked by us, ask the user to make sure they
			// want to lose changes
			if (modState != IDYESTOALL && modState != IDNOTOALL)
			{
				String s = pLeaf->m_name + " may have been modified, lose changes?";
				CYesNoDlg dlg(s);
				modState = dlg.DoModal();
			}

			if (modState == IDNO || modState == IDCANCEL || modState == IDNOTOALL)
			{
				bOverride = 1;
				return 0;
			}

			return 1;
		}
	}
	else
		return 1;
	
	return 0;
}

gint ButtonPress(GtkWidget*, GdkEventButton* event, GtkWidget* pMenu)
{
	if (!pMenu)
		return TRUE;
	
	// popup
	if (event->button == 3)
	{
		gtk_menu_popup(GTK_MENU(pMenu), NULL, NULL, NULL, NULL,
					   event->button, event->time);

		return TRUE;
	}

	return TRUE;
}

void CleanTree(GtkWidget* pTree)
{
	GList* children = gtk_container_children(GTK_CONTAINER(pTree));

	while (children)
	{
		GtkWidget* pChild = GTK_WIDGET(children->data);

		CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(pChild));

		delete pNode;

		if (GTK_TREE_ITEM(pChild)->subtree)
			CleanTree(GTK_TREE_ITEM(pChild)->subtree);
		
		children = g_list_remove_link(children, children);
	}
}

void InsertItemIntoTree(GtkWidget* pTree, GtkWidget* hItem, String& strName)
{
	GList* children = gtk_container_children(GTK_CONTAINER(pTree));
	
	int pos = 0;
	
	while (children)
	{
		String strItem = GetItemText(GTK_WIDGET(children->data));
		
		if (fcompare(strItem, strName) > 0)
			break;
		
		children = g_list_remove_link(children, children);
		++pos;
	}
	
	gtk_tree_insert(GTK_TREE(pTree), hItem, pos);
}

void ResetTree(GtkWidget* pTreeCtrl)
{
	GList* children = gtk_container_children(GTK_CONTAINER(pTreeCtrl));
	
	while (children)
	{
		GtkWidget* pChild = (GtkWidget*) children->data;
		
		CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(pChild));

		pNode->m_bVisited = 0;
		
		if (GTK_TREE_ITEM(pChild)->subtree)
			ResetTree(GTK_TREE_ITEM(pChild)->subtree);

		children = g_list_remove_link(children, children);
	}
}

void RefreshTreeColors(GtkWidget* pTreeCtrl)
{
	GList* children = gtk_container_children(GTK_CONTAINER(pTreeCtrl));
	
	while (children)
	{
		GtkWidget* pChild = (GtkWidget*) children->data;
		
		CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(pChild));

		if (pNode->m_bCloaked)
		{
			if (pNode->m_bWebDeploy)
				gtk_widget_set_style(GTK_BIN(pChild)->child, Vermie::m_pCloakedDeployStyle);
			else
				gtk_widget_set_style(GTK_BIN(pChild)->child, Vermie::m_pCloakedStyle);
		}
		else if (pNode->m_bWebDeploy)
		{
			gtk_widget_set_style(GTK_BIN(pChild)->child, Vermie::m_pWebDeployStyle);
		}
		
		if (GTK_TREE_ITEM(pChild)->subtree)
			RefreshTreeColors(GTK_TREE_ITEM(pChild)->subtree);

		children = g_list_remove_link(children, children);
	}
}
	
