/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#ifndef VERMTREEUTILS_H
#define VERMTREEUTILS_H

#include <stdio.h>

#define FILE_NOT_MODIFIED		0
#define FILE_MISSING			1
#define FILE_MODIFIED			2
#define FILE_OUT_OF_SYNC		3

class CVermTreeLeaf;
class CVermTreeNode;
class CVermProjectView;
class CVermProtocolMessage;

void RefreshTreeColors(GtkWidget* pSrcTree);
CVermTreeLeaf* ParseFileInfo(char** pMessageBody, int nBodySize,
							 int* nCharsRead, const char* pszUsername, int bPinStatus);
void ParseFileList(CVermTreeNode *pNode, CVermProtocolMessage* pMessage,
				   const char* pszUsername, String& strWorkingDir, int bChecksum= 0, int bPinStatus = 0);
void ParseTree(GtkWidget* pTreeCtrl, char* buf, int* pos, int maxPos,
			   GtkWidget* pMenu, CVermProjectView* pFocusView);
void CopyTree(GtkWidget* pSrcTree, GtkWidget* destTree);
void CopyTreeNode(GtkWidget* pSrcTree, GtkWidget* pDestTree);
void WriteTreeNode(GtkWidget* pSrcTree, FILE* file);
int ReadTreeNode(GtkWidget* pSrcTree, char* pBuffer, int* nFilePos,
				 int maxPos, GtkWidget* pMenu, CVermProjectView* pFocusView);
void SaveTreeToFile(GtkWidget* pSrcTree, String& strServer,
					String& strDatabase, String& strCurrentProject);
char* LoadTreeFromFile(GtkWidget* pSrcTree, String& strServer,
					   String& strDatabase, GtkWidget* pMenu,
					   CVermProjectView* pFocusView);
int GetPathFromTreeItem(GtkWidget* hItem, String& path);
GtkWidget* GetTreeChildByName(GtkWidget* hParent, String& strChild);
GtkWidget* GetTreeItemFromProject(GtkWidget* pTreeCtrl, String& pszProject);
GtkWidget* GetTreeItemFromProject(GtkWidget* pTreeCtrl, const char* pszProject);
CVermTreeNode* GetTreeNodeFromProject(GtkWidget* pTreeCtrl, const char* pszProject);
String GetWorkingDirectory(GtkWidget* pTreeCtrl, const char* pszProject);
String GetWorkingDirectory(GtkWidget* hItem);
int IsFileLocked(CVermTreeNode* pNode, String& strProject, String& strFile, String& strUsername);
int IsFileLocked(CVermTreeLeaf* pLeaf, String& strProject, String& strUsername);
int IsFileModified(CVermTreeNode* pNode, const char* pszFileName);
int IsFileModified(CVermTreeNode* pNode, String& strFile, String& strDir);
int ShouldFileBeFetched(CVermTreeLeaf* pLeaf, String& strProject,
			String& strUsername, struct stat& sb, int& modState, int& lockState, int& writeState,
			int& bOverride, int nVerNum = 0);

void ResetTree(GtkWidget* pTreeCtrl);

/* SOS File Functions */
int LookupSOSFileByDatabase(String& strIndexFile, String& strDatabase,
							String& strSOSFile, int bCreateEntry = 0);
int FindSOSFileName(String& strServer, String& strDatabase, String& strSOSFile,
					int& bOlfFileLocation,int bCreateEntry = 0);
void CreateDatabaseEntryInIndex(String& strIndexFile, String& strDatabase,
								String& strSOSFile, int num);
char* GetItemText(GtkWidget* pItem);
gint ButtonPress(GtkWidget* w, GdkEventButton* event, GtkWidget* pMenu);
void CleanTree(GtkWidget* pTree);
void InsertItemIntoTree(GtkWidget* pTree, GtkWidget* hItem, String& strName);

#endif /* VERMTREEUTILS_H */
