/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>

#include "Vermie.h"
#include "VermUtils.h"

char* ParseNameFromBuffer(char* buf, int *startPos, int maxPos, int &bParseError)
{
	bParseError = 0;

	int nInitPos = *startPos;
	int nSize = 0;
	
	while (buf[*startPos] != '\r' || buf[(*startPos) + 1] != '\n')
	{
		nSize++;
		(*startPos)++;

		if (*startPos > maxPos)
		{
			bParseError = 1;
			return NULL;
		}
	}

	buf[(*startPos)++] = '\0';

	(*startPos)++;	// jump past the \n

	if (nSize == 0)
		return NULL;
	else
		return (buf + nInitPos);
}

time_t ParseServerDate(char* pDate)
{
	if (!pDate)
		return 0;

	char* pTmp = pDate;

	// parse the month

	pTmp[2] = '\0';

	int nMonth = atoi(pTmp) - 1;

	pTmp += 3;

	// parse the day

	pTmp[2] = '\0';

	int nDay = atoi(pTmp);

	pTmp += 3;

	// parse the year

	pTmp[4] = '\0';

	int nYear = atoi(pTmp);

	pTmp += 5;

	// parse the hour

	pTmp[2] = '\0';

	int nHour = atoi(pTmp);

	pTmp += 3;

	// parse the minute

	int nMinute = atoi(pTmp);

	// create the new CTime object

	struct tm tm;
	memset(&tm, 0, sizeof(struct tm));
	tm.tm_min = nMinute;
	tm.tm_hour = nHour;

	if (nYear > 1900)
		nYear -= 1900;
	
	tm.tm_year = nYear;
	tm.tm_mday = nDay;
	tm.tm_mon = nMonth;
	
	return mktime(&tm);
}

void ExtractChildFromParent(const char* pszFullPath, String& strParent,
							String& strChild, char cSep)
{
	String strFullPath = pszFullPath;

	String strRev = reverse(strFullPath);

	int nIndex = strRev.index(cSep);

	if (nIndex == -1)
	{
		strParent = strFullPath;
		return;
	}

	if (nIndex == (int)strFullPath.length() - 1)
	{
		strFullPath = strFullPath.before((int)strFullPath.length() - 1);

		String strRev2 = reverse(strFullPath);
		nIndex = strRev2.index(cSep);

		if (nIndex == -1)
		{
			strParent = strFullPath;
			return;
		}
	}

	strChild = strFullPath.after((int)strFullPath.length() - (nIndex + 1));

	strParent = strFullPath.before((int)strFullPath.length() - nIndex);
}

const char* ConvertServerName(const char* pServer)
{
	if (!pServer)
		return NULL;

	int i = 0;

	char* pNewServer = new char[13];

	pNewServer[0] = 0;

	int numFound = 0;
	int pos = 0;

	while (pServer[i])
	{
		if (pServer[i] >= '0' && pServer[i] <= '9')
		{
			++numFound;
		}
		else if (pServer[i] == '.')
		{
			// bogus ip numbers ...
			if (numFound > 3)
			{
				delete []pNewServer;
				return (char*)pServer;
			}

			int tmp = numFound;
			while (tmp < 3)
			{
				pNewServer[pos++] = '0';
				++tmp;
			}

			tmp = numFound;

			while (tmp > 0)
			{
				pNewServer[pos++] = pServer[i - tmp];
				tmp--;
			}

			numFound = 0;
		}
		else
		{
			delete []pNewServer;
			return (char *)pServer;
		}

		++i;
	}

	if (pos < 12)
	{
		// bogus ip numbers ...
		if (numFound > 3)
		{
			delete []pNewServer;
			return (char*)pServer;
		}

		int tmp = numFound;
		while (tmp < 3)
		{
			pNewServer[pos++] = '0';
			++tmp;
		}

		tmp = numFound;

		while (tmp > 0)
		{
			pNewServer[pos++] = pServer[i - tmp];
			tmp--;
		}
	}

	pNewServer[12] = 0;

	return pNewServer;
}


int MakeFileReadOnly(String& strFile)
{
	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));

	stat(strFile.chars(), &sb);

	if (sb.st_mode & S_IWUSR)
		sb.st_mode ^= S_IWUSR;
	if (sb.st_mode & S_IWGRP)
		sb.st_mode ^= S_IWGRP;
	if (sb.st_mode & S_IWOTH)
		sb.st_mode ^= S_IWOTH;
	
	if (chmod(strFile.chars(), sb.st_mode))
	{
		return -1;
	}

	return 0;
}

int MakeFileWriteable(String& strFile)
{
	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));

	if (stat(strFile.chars(), &sb))
		return -1;

	if (!(sb.st_mode & S_IWUSR))
	{
		sb.st_mode |= S_IWUSR;
	
		if (chmod(strFile.chars(), sb.st_mode))
		{
			return -1;
		}
	}

	return 0;
}

int CreateDirectory(String& strDirectory)
{
    if (strDirectory.empty())
		return 0;

	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));

	// check to make sure there is a directory for us ...

	int ret = stat(strDirectory.chars(), &sb);

	if (ret || !(S_ISDIR(sb.st_mode)))
	{
		int len = strDirectory.length();

		int i;
		for (i = len - 1; i >= 0; i--)
		{
			if (strDirectory[i] == '/')
				break;
		}

		if (i == -1)
			return 0;
		
		String parentDir = strDirectory.before(i);

		if (!CreateDirectory(parentDir))
			return 0;
	}
	else
		return 1;

	if (mkdir(strDirectory.chars(), 0750))
		return 0;

	return 1;
}

void TrashDirectory(String& strTmpDir)
{
	DIR* d = opendir(strTmpDir.chars());
	struct dirent* dent;
	
	if (d)
	{
		while ((dent = readdir(d)))
		{
			if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
				continue;
			
			struct stat sb;
			memset(&sb, 0, sizeof(struct stat));

			String strFile = strTmpDir;
			strFile += "/";
			strFile += dent->d_name;
			
			stat(strFile.chars(), &sb);

			if (S_ISDIR(sb.st_mode))
			{
				TrashDirectory(strFile);
				rmdir(strFile.chars());
			}
			else
			{
				unlink(strFile.chars());
			}
		}
		closedir(d);
		rmdir(strTmpDir.chars());
	}
}
