/*      Copyright 1998-2002 SourceGear Corporation.
        All Rights Reserved. */

#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stream.h>

#include "zlib.h"

#include "md5.h"

#include "Vermie.h"
#include "VermByteQueue.h"
#include "VermFrameWnd.h"
#include "VermProjectView.h"
#include "VermFileView.h"
#include "VermMessageView.h"
#include "VermSocket.h"
#include "VermProtocolMessage.h"
#include "VermProtocolHeader.h"
#include "VermUtils.h"
#include "VermTreeNode.h"
#include "VermTreeLeaf.h"
#include "VermTreeUtils.h"
#include "VermUtils.h"

#include "AfxMessageBox.h"
#include "ServerDlg.h"
#include "LoginDlg.h"
#include "CreateProjectDlg.h"
#include "AddFilesDlg.h"
#include "YesNoDlg.h"
#include "DeployDlg.h"
#include "DeleteDlg.h"
#include "CheckinDlg.h"
#include "AboutDlg.h"
#include "HistoryDlg.h"
#include "CheckoutDlg.h"
#include "OptionsDlg.h"
#include "RenameDlg.h"
#include "DirectoryDlg.h"
#include "CommentDlg.h"
#include "ShareDlg.h"
#include "ProjectHistoryDlg.h"
#include "SupportDlg.h"
#include "SearchDlg.h"
#include "SearchResultsDlg.h"
#include "GetLatestDlg.h"
#include "UndoCheckoutDlg.h"
#include "ViewEditDlg.h"
#include "PasswordDlg.h"

#include "res/AddFiles.xpm"
#include "res/Checkin.xpm"
#include "res/Checkout.xpm"
#include "res/CreateProject.xpm"
#include "res/Delete.xpm"
#include "res/GetLatestVersion.xpm"
#include "res/Label.xpm"
#include "res/RefreshFileList.xpm"
#include "res/RefreshProjectTree.xpm"
#include "res/ShowDifferences.xpm"
#include "res/ShowHistory.xpm"
#include "res/UndoCheckout.xpm"
#include "res/WorkingFolder.xpm"
#include "res/Share.xpm"
#include "res/icon.xpm"
#include "res/Stop.xpm"
#include "res/shared.xpm"
#include "res/Edit.xpm"
#include "res/View.xpm"
#include "res/Branch.xpm"
#include "res/Pin.xpm"

#define PROTOCOL_VERSION                "2.0"

#define NEW_PROJECT				"New Project"
#define UPDATE_PROJECT			"Update Project"

// client cookie definitions
#define CHECKOUT_FILES			        "Checkout Files"
#define FETCH_FILES				        "Fetch Files"
#define FETCH_FILES_MAKE_WRITABLE		"Fetch Files Make Writable"
#define CHECKIN_FILES					"Checkin Files"
#define CHECKOUT_FILES_DONT_OVERWRITE 	"Checking Files Dont Overwrite"
#define UNDO_CHECKOUT_REPLACE			"Undo Checkout Replace"
#define UNDO_CHECKOUT_LEAVE				"Undo Checkout Leave"
#define UNDO_CHECKOUT_DELETE			"Undo Checkout Delete"
#define REMOVE_LOCAL_COPY				"Remove Local Copy"
#define EDIT_FILE               		"Edit File"
#define PROJECT_DIFF					"Project Diff"
#define MAKE_READONLY					"Make Readonly"
#define MAKE_WRITABLE					"Make Writable"
#define BRANCH_FILES					"Branch Files"

// refresh timer
#define FILE_LIST_UPDATE_TIMER	1

// static functions
static int ConfigureEvent(GtkWidget*, GdkEventConfigure* e, CVermFrameWnd* pWnd);

//static int IsFileModified(CVermTreeNode* pNode, String& strFile, String& strWorkingDir);
//static int IsFileLocked(CVermTreeNode* pNode, String& strFile, String& strUsername);
static void Destroy(GtkWidget* w, CVermFrameWnd* pFrame);

#ifndef BUILD_STANDARD
static void file_cancel_sel(GtkWidget *w, GtkFileSelection *f);
static void ImportKeyFileOkSel(GtkWidget *w,GtkFileSelection *f);
#endif

static int RefreshTimer(CVermFrameWnd*);
static int KeepAliveTimer(CVermFrameWnd*);
static void menu_cb2(void*, int, GtkWidget*);
static void menu_cb(GtkWidget *, gpointer data);

extern void ParseServerPrefs(String&);

// Oh, this is such a heinous hack
static CVermFrameWnd* s_pFrameWnd;


void CVermFrameWnd::ResetToolbar(short bOn)
{
	m_bToolbarActive = bOn;
	
	gtk_widget_set_sensitive(m_pCreateProject, bOn);
	gtk_widget_set_sensitive(m_pAddFiles, bOn);
	gtk_widget_set_sensitive(m_pLabel, bOn);
	gtk_widget_set_sensitive(m_pDelete, bOn);
	gtk_widget_set_sensitive(m_pGetLatestVersion, bOn);
	gtk_widget_set_sensitive(m_pCheckout, bOn);
	gtk_widget_set_sensitive(m_pCheckin, bOn);
	gtk_widget_set_sensitive(m_pUndoCheckout, bOn);
	gtk_widget_set_sensitive(m_pShare, bOn);
	gtk_widget_set_sensitive(m_pShowDifferences, bOn);
	gtk_widget_set_sensitive(m_pShowHistory, bOn);
	gtk_widget_set_sensitive(m_pSetWorkingFolder, bOn);
	gtk_widget_set_sensitive(m_pRefreshFileList, bOn);
	gtk_widget_set_sensitive(m_pRefreshProjectTree, bOn);
	
	gtk_widget_set_sensitive(m_pMCreateProject, bOn);
	gtk_widget_set_sensitive(m_pMAddFiles, bOn);
	gtk_widget_set_sensitive(m_pMLabel, bOn);
	gtk_widget_set_sensitive(m_pMDelete, bOn);
	gtk_widget_set_sensitive(m_pMRename, bOn);
	gtk_widget_set_sensitive(m_pMGetLatestVersion, bOn);
	gtk_widget_set_sensitive(m_pMCheckout, bOn);
	gtk_widget_set_sensitive(m_pMCheckin, bOn);
	gtk_widget_set_sensitive(m_pMUndoCheckout, bOn);
	gtk_widget_set_sensitive(m_pMShare, bOn);
	gtk_widget_set_sensitive(m_pMBranch, bOn && Vermie::m_bUseVSS6Features);
	gtk_widget_set_sensitive(m_pMShowDifferences, bOn);
	gtk_widget_set_sensitive(m_pMShowHistory, bOn);
	gtk_widget_set_sensitive(m_pMSetWorkingFolder, bOn);
	gtk_widget_set_sensitive(m_pMRefreshFileList, bOn);
	gtk_widget_set_sensitive(m_pMRefreshProjectTree, bOn);
	gtk_widget_set_sensitive(m_pMSearch, bOn);
	gtk_widget_set_sensitive(m_pMView, bOn);
	gtk_widget_set_sensitive(m_pMEdit, bOn);
	gtk_widget_set_sensitive(m_pMCloakProject, bOn);
	gtk_widget_set_sensitive(m_pMPassword, bOn);
	
	SetFocus(m_nChildWithFocus);
}

static int EnterEvent(GtkWidget*, GdkEvent*, CVermFrameWnd* pParent)
{
	pParent->OnActivate();

	return TRUE;
}

static int ConfigureEvent(GtkWidget*, GdkEventConfigure* e,
						  CVermFrameWnd* pWnd)
{
	pWnd->m_nWidth = e->width;
	pWnd->m_nHeight = e->height;

	return TRUE;
}

static int RefreshTimer(CVermFrameWnd* pWnd)
{
	pWnd->m_nCurrentCommand = VIEW_REFRESH_FILE_LIST;
	
	pWnd->OnViewRefreshFileList();
	return TRUE;
}

static int KeepAliveTimer(CVermFrameWnd* pWnd)
{
	CVermProtocolMessage* pMessage = new CVermProtocolMessage();
	pMessage->m_number = PM_NOOP;

	if (pWnd->m_pSocket->SendMessage(pMessage))
	{
		delete pMessage;
		return FALSE;
	}
	
	return TRUE;
}

static void menu_cb2(void* e, int which, GtkWidget* w)
{
	menu_cb(w, (gpointer)which);
}

static void menu_cb(GtkWidget *w, gpointer data)
{
	int which = (int) data;

	switch (which)
	{
	case VIEW_CLEAR_STATUS:
		s_pFrameWnd->OnViewClearStatus();
		break;
	case TOOLS_CHANGE_PASSWORD:
		s_pFrameWnd->OnToolsChangePassword();
		break;
	case SOURCE_BRANCH:
		s_pFrameWnd->OnSourceBranch();
		break;
	case VIEW_SEARCH:
		s_pFrameWnd->OnViewSearch();
		break;
	case TOOLS_STOP:
		// return here so that the currentcommand var doesn't get reset to stop
		s_pFrameWnd->OnToolsStop();
		return;
	case TOOLS_CLOAK_PROJECT:
		s_pFrameWnd->OnToolsCloakProject();
		break;
	case TOOLS_WEB_DEPLOY:
		s_pFrameWnd->OnToolsWebDeploy();
		break;
	case SOURCE_SHARE:
		s_pFrameWnd->OnSourceShare();
		break;
	case FILE_VIEW:
		s_pFrameWnd->OnFileView(FALSE);
		break;
	case FILE_EDIT:
		s_pFrameWnd->OnFileEdit(FALSE);
		break;
	case FILE_RENAME:
		s_pFrameWnd->OnFileRename();
		break;
	case FILE_LABEL:
		s_pFrameWnd->OnFileLabel();
		break;
	case TOOLS_OPTIONS:
		s_pFrameWnd->OnToolsOptions();
		break;
	case TOOLS_SHOW_DIFFERENCES:
		s_pFrameWnd->OnToolsShowDifferences();
		break;
	case TOOLS_SHOW_HISTORY:
		s_pFrameWnd->OnToolsShowHistory();
		break;
	case HELP_ABOUT:
	{
		CAboutDlg dlg;
		dlg.DoModal();
		break;
	}
	case HELP_SUPPORT:
	{
		CSupportDlg dlg;
		dlg.DoModal();
		break;
	}
	case SOURCE_UNDO_CHECK_OUT:
		s_pFrameWnd->OnSourceUndoCheckout();
		break;
	case SOURCE_CHECK_IN:
		s_pFrameWnd->OnSourceCheckIn();
		break;
	case VIEW_REFRESH_PROJECT_TREE:
		s_pFrameWnd->OnViewRefreshProjectTree();
		break;
	case VIEW_REFRESH_FILE_LIST:
		s_pFrameWnd->OnViewRefreshFileList();
		break;
	case SOURCE_CHECK_OUT:
		s_pFrameWnd->OnSourceCheckOut();
		break;
	case FILE_CONNECT_TO_SERVER:
		s_pFrameWnd->OnFileConnectToServer();
		break;
#ifndef BUILD_STANDARD
	case TOOLS_IMPORT_ENCRYPTION_KEY:
		s_pFrameWnd->OnToolsImportEncryptionKey();
		break;
#endif
	case FILE_EXIT:
		s_pFrameWnd->OnFileExit();
		break;
	case FILE_SET_WORKING_FOLDER:
		s_pFrameWnd->OnFileSetWorkingFolder();
		break;
	case FILE_CREATE_PROJECT:
		s_pFrameWnd->OnFileCreateProject();
		break;
	case FILE_ADD_FILES:
		s_pFrameWnd->OnFileAddFiles();
		break;
	case SOURCE_GET_LATEST_VERSION:
		s_pFrameWnd->OnSourceGetLatestVersion();
		break;
	case FILE_DELETE:
		s_pFrameWnd->OnFileDelete();
	default:
		break;
	}

//	s_pFrameWnd->m_nCurrentCommand = which;
}


#ifndef BUILD_STANDARD
static GtkItemFactoryEntry menuItems[47] =
#else
static GtkItemFactoryEntry menuItems[45] =
#endif // !BUILD_STANDARD
{
	{"/File", NULL, NULL, 0, "<Branch>"},
	{"/File/Connect To Server ...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_CONNECT_TO_SERVER, NULL},
	{"/File/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/File/Create Project...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_CREATE_PROJECT, NULL},
	{"/File/Add Files...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_ADD_FILES, NULL},
	{"/File/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/File/Set Working Folder...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_SET_WORKING_FOLDER, NULL},
	{"/File/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/File/View", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_VIEW, NULL},
	{"/File/Edit", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_EDIT, NULL},
	{"/File/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/File/Delete...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_DELETE, NULL},
	{"/File/Rename...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_RENAME, NULL},
	{"/File/Label...", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_LABEL, NULL},
	{"/File/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/File/Exit", NULL, (GtkItemFactoryCallback)menu_cb2, FILE_EXIT, NULL},
	{"/View", NULL, NULL, 0, "<Branch>"},
	{"/View/Refresh File List", NULL, (GtkItemFactoryCallback)menu_cb2, VIEW_REFRESH_FILE_LIST, NULL},
	{"/View/Refresh Project Tree", NULL, (GtkItemFactoryCallback)menu_cb2, VIEW_REFRESH_PROJECT_TREE, NULL},
	{"/View/Clear Status Window", NULL, (GtkItemFactoryCallback)menu_cb2, VIEW_CLEAR_STATUS, NULL},
	{"/View/Search...", NULL, (GtkItemFactoryCallback)menu_cb2, VIEW_SEARCH, NULL},
	{"/Source", NULL, NULL, 0, "<Branch>"},
	{"/Source/Get Latest Version", NULL, (GtkItemFactoryCallback)menu_cb2, SOURCE_GET_LATEST_VERSION, NULL},
	{"/Source/Check Out", NULL, (GtkItemFactoryCallback)menu_cb2, SOURCE_CHECK_OUT, NULL},
	{"/Source/Check In", NULL, (GtkItemFactoryCallback)menu_cb2, SOURCE_CHECK_IN, NULL},
	{"/Source/Undo Check Out", NULL, (GtkItemFactoryCallback)menu_cb2, SOURCE_UNDO_CHECK_OUT, NULL},
	{"/Source/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/Source/Share", NULL, (GtkItemFactoryCallback)menu_cb2, SOURCE_SHARE, NULL},
	{"/Source/Branch", NULL, (GtkItemFactoryCallback)menu_cb2, SOURCE_BRANCH, NULL},
	{"/Tools", NULL, NULL, 0, "<Branch>"},
	{"/Tools/Stop Current Operation", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_STOP, NULL},
	{"/Tools/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/Tools/Change Password", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_CHANGE_PASSWORD, NULL},
	{"/Tools/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/Tools/Show History", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_SHOW_HISTORY, NULL},
	{"/Tools/Show Differences", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_SHOW_DIFFERENCES, NULL},
	{"/Tools/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/Tools/Web Deploy", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_WEB_DEPLOY, NULL},
	{"/Tools/<separator>", NULL, NULL, 0, "<Separator>"},
	{"/Tools/Cloak Project", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_CLOAK_PROJECT, NULL},
	{"/Tools/<separator>", NULL, NULL, 0, "<Separator>"},
#ifndef BUILD_STANDARD
	{"/Tools/Import Crypto Key", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_IMPORT_ENCRYPTION_KEY, NULL},
	{"/Tools/<separator>", NULL, NULL, 0, "<Separator>"},
#endif
	{"/Tools/Options...", NULL, (GtkItemFactoryCallback)menu_cb2, TOOLS_OPTIONS, NULL},
	{"/Help", NULL, NULL, 0, "<LastBranch>"},
	{"/Help/Support...", NULL, (GtkItemFactoryCallback)menu_cb2, HELP_SUPPORT, NULL},
	{"/Help/About SourceOffSite...", NULL, (GtkItemFactoryCallback)menu_cb2, HELP_ABOUT, NULL},
};

static void Destroy(GtkWidget*, CVermFrameWnd* pFrame)
{
	// if we have no username, this user hasn't yet logged on - don't
	// overwrite the current vrm file

	if (pFrame->IsSafeToSave())
	{
		String curProject;

		pFrame->m_pProjectView->GetPathSelectedObject(curProject);
		
		SaveTreeToFile(pFrame->m_pProjectView->m_pTree, pFrame->m_server,
					   pFrame->m_database, curProject);
	}
	
	CleanTree(pFrame->m_pProjectView->m_pTree);

	pFrame->SavePrefs();
}

void CVermFrameWnd::StartUp(void)
{
	if (!Vermie::m_strLastServer.empty() && !Vermie::m_strLastPort.empty())
	{
		ConnectToServer(Vermie::m_strLastServer,  atoi(Vermie::m_strLastPort.chars()));
	}
	else
	{
		OnFileConnectToServer();
	}
}

CVermFrameWnd::CVermFrameWnd()
{
	GtkWidget* vbox;

	m_bToolbarActive = 1;
	
	m_bSilentLogin = 0;
	m_bPrefsSaved = 0;
	
	m_nWaitCount = 0;
	m_bDuringCreation = 0;

	m_pFileDiff = NULL;
	m_pFileEdit = NULL;

	m_pShareDlg = NULL;
	m_pHistoryDlg = NULL;
	m_pSearchResultsDlg = NULL;
	
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	m_nWidth = Vermie::m_nWidth;
	m_nHeight = Vermie::m_nHeight;
	
	m_pSocket = NULL;
	m_pCrypto = NULL;

	m_bConnected = 0;

	m_port = -1;

	if (m_nWidth <= 0)
		m_nWidth = 600;
	if (m_nHeight <= 0)
		m_nHeight = 480;

	m_pWidget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_policy(GTK_WINDOW(m_pWidget), TRUE, TRUE, TRUE);
	
	gtk_widget_set_usize(m_pWidget, m_nWidth, m_nHeight);
	
	gtk_signal_connect(GTK_OBJECT(m_pWidget), "enter_notify_event",
					   (GtkSignalFunc)EnterEvent, this);
	
//	gtk_signal_connect(GTK_OBJECT(m_pWidget), "expose_event",
//					   (GtkSignalFunc)ExposeEvent, this);

	//	gtk_container_border_width(GTK_CONTAINER(m_pWidget), 1);

	m_destroySig = gtk_signal_connect(GTK_OBJECT(m_pWidget), "destroy",
					  GTK_SIGNAL_FUNC(Destroy), this);
	
	gtk_signal_connect(GTK_OBJECT(m_pWidget), "configure_event",
			   GTK_SIGNAL_FUNC(ConfigureEvent), this);

	gtk_widget_realize(m_pWidget);
	
	
	Vermie::m_pCloakedStyle = gtk_style_copy(m_pWidget->style);
	Vermie::m_pNormalStyle = gtk_style_copy(m_pWidget->style);
	Vermie::m_pWebDeployStyle = gtk_style_copy(m_pWidget->style);
	Vermie::m_pCloakedDeployStyle = gtk_style_copy(m_pWidget->style);

	for (int i = 0; i < 5; i++)
	{
		Vermie::m_pCloakedStyle->base[i] = Vermie::m_CloakedColor;
		Vermie::m_pWebDeployStyle->base[i] = Vermie::m_DeployColor;
		Vermie::m_pCloakedDeployStyle->base[i] = Vermie::m_CloakedDeployColor;

		Vermie::m_pCloakedStyle->bg[i] = Vermie::m_CloakedColor;
		Vermie::m_pWebDeployStyle->bg[i] = Vermie::m_DeployColor;
		Vermie::m_pCloakedDeployStyle->bg[i] = Vermie::m_CloakedDeployColor;

		Vermie::m_pCloakedStyle->fg[i] = Vermie::m_CloakedColor;
		Vermie::m_pWebDeployStyle->fg[i] = Vermie::m_DeployColor;
		Vermie::m_pCloakedDeployStyle->fg[i] = Vermie::m_CloakedDeployColor;

	}

	GdkPixmap* i;
	GtkWidget* iw;
	GdkBitmap* m;
	
	Vermie::m_pShared = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &Vermie::m_pSharedMask,
									 &m_pWidget->style->bg[GTK_STATE_NORMAL],
									 sharedXPM);
														   
	Vermie::m_pPin = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &Vermie::m_pPinMask,
									 &m_pWidget->style->bg[GTK_STATE_NORMAL],
									 pinXPM);
														   
	i = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &m,
									 &m_pWidget->style->bg[GTK_STATE_NORMAL],
									 iconXPM);

	iw = gtk_pixmap_new(i, m);

	gdk_window_set_icon(m_pWidget->window, NULL, i, m);
	
	vbox = gtk_vbox_new(FALSE, 1);
	gtk_container_border_width(GTK_CONTAINER(vbox), 0);
	gtk_container_add(GTK_CONTAINER(m_pWidget), vbox);
	gtk_widget_show(vbox);

	// Create the mnue bar, and pointers to all the menu items
	
	int nItems = sizeof(menuItems) / sizeof(menuItems[0]);
	GtkAccelGroup*  accelGroup;

	accelGroup = gtk_accel_group_new();
	m_pItemFactory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",
										 accelGroup);
	
	gtk_item_factory_create_items(m_pItemFactory, nItems, menuItems, NULL);
	gtk_accel_group_attach(accelGroup, GTK_OBJECT(m_pWidget));
	
	m_pMCreateProject  = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[3].path);
	m_pMAddFiles  = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[4].path);
	m_pMSetWorkingFolder = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[6].path);
	m_pMView = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[8].path);
	m_pMEdit = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[9].path);
	m_pMDelete = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[11].path);
	m_pMRename = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[12].path);
	m_pMLabel  = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[13].path);
	m_pMRefreshFileList = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[17].path);
	m_pMRefreshProjectTree = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[18].path);
	m_pMClearStatus = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[19].path);
	m_pMSearch =  gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[20].path);
	m_pMGetLatestVersion = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[22].path);
	m_pMCheckout = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[23].path);
	m_pMCheckin = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[24].path);
	m_pMUndoCheckout = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[25].path);
	m_pMShare = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[27].path);
	m_pMBranch = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[28].path);
	m_pMStop = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[30].path);
	m_pMPassword = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[32].path);
	m_pMShowHistory = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[34].path);
	m_pMShowDifferences = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[35].path);
	m_pMWebDeploy = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[37].path);
	m_pMCloakProject = gtk_item_factory_get_widget(GTK_ITEM_FACTORY(m_pItemFactory), menuItems[39].path);
	
	m_pMenubar = gtk_item_factory_get_widget(m_pItemFactory, "<main>");

	gtk_box_pack_start(GTK_BOX(vbox), m_pMenubar, FALSE, TRUE, 0);

	gtk_widget_show(m_pMenubar);

	m_pToolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
								 GTK_TOOLBAR_ICONS);

	GdkPixmap* icon;
	GtkWidget* iconw;
	GdkBitmap* mask;
	
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   CreateProjectXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pCreateProject = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Creates a new project",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)FILE_CREATE_PROJECT);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   AddFiles);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pAddFiles = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Add files to project",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)FILE_ADD_FILES);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   Label);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pLabel = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Label",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)FILE_LABEL);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   Delete);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pDelete = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Delete",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)FILE_DELETE);
											   
	gtk_toolbar_append_space(GTK_TOOLBAR(m_pToolbar));
							 
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   GetLatestVersion);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pGetLatestVersion = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Get latest version",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)SOURCE_GET_LATEST_VERSION);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   Checkout);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pCheckout = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Checkout",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)SOURCE_CHECK_OUT);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   Checkin);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pCheckin = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Checkin",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)SOURCE_CHECK_IN);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   UndoCheckoutXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pUndoCheckout = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Undo checkout",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)SOURCE_UNDO_CHECK_OUT);
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   ShareXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pShare = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
									   NULL,
									   "Share",
									   "Private",
									   iconw,
									   (GtkSignalFunc) menu_cb,
									   (gpointer)SOURCE_SHARE);
	
	
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   BranchXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pBranch = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
									   NULL,
									   "Branch",
									   "Private",
									   iconw,
									   (GtkSignalFunc) menu_cb,
									   (gpointer)SOURCE_BRANCH);
	
	gtk_toolbar_append_space(GTK_TOOLBAR(m_pToolbar));

	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   ViewXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pView = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
									  NULL,
									  "View File",
									  "Private",
									  iconw,
									  (GtkSignalFunc) menu_cb,
									  (gpointer)FILE_VIEW);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   EditXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pEdit = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
									  NULL,
									  "Edit File",
									  "Private",
									  iconw,
									  (GtkSignalFunc) menu_cb,
									  (gpointer)FILE_EDIT);
	
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   ShowDifferences);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pShowDifferences = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Show differences",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)TOOLS_SHOW_DIFFERENCES);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   ShowHistory);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pShowHistory = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Show history",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)TOOLS_SHOW_HISTORY);
											   
	gtk_toolbar_append_space(GTK_TOOLBAR(m_pToolbar));

	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   WorkingFolder);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pSetWorkingFolder = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Set working folder",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)FILE_SET_WORKING_FOLDER);
											   
	gtk_toolbar_append_space(GTK_TOOLBAR(m_pToolbar));

	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   RefreshFileList);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pRefreshFileList = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Refresh file list",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)VIEW_REFRESH_FILE_LIST);
											   
	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   RefreshProjectTree);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pRefreshProjectTree = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
											   NULL,
											   "Refresh project tree",
											   "Private",
											   iconw,
											   (GtkSignalFunc) menu_cb,
											   (gpointer)VIEW_REFRESH_PROJECT_TREE);
											   
	gtk_toolbar_append_space(GTK_TOOLBAR(m_pToolbar));
	gtk_toolbar_append_space(GTK_TOOLBAR(m_pToolbar));

	icon = gdk_pixmap_create_from_xpm_d(m_pWidget->window, &mask,
									   &m_pWidget->style->bg[GTK_STATE_NORMAL],
									   StopXPM);

	iconw = gtk_pixmap_new(icon, mask);
	
	m_pStop = gtk_toolbar_append_item(GTK_TOOLBAR(m_pToolbar),
									  NULL,
									  "Stop current action",
									  "Private",
									  iconw,
									  (GtkSignalFunc) menu_cb,
									  (gpointer)TOOLS_STOP);
											   
	gtk_widget_show(m_pToolbar);
	gtk_box_pack_start(GTK_BOX(vbox), m_pToolbar, FALSE, TRUE, 0);
	
	m_pVPane = gtk_vpaned_new();

	gtk_box_pack_start(GTK_BOX(vbox), m_pVPane, TRUE, TRUE, 1);
	
	gtk_widget_show(m_pVPane);

	m_pHPane = gtk_hpaned_new();
	
	gtk_widget_show(m_pHPane);

//	gtk_paned_add1(GTK_PANED(m_pVPane), m_pHPane);
	gtk_paned_pack1(GTK_PANED(m_pVPane), m_pHPane, TRUE, TRUE);
	
	m_pMessageView = new CVermMessageView(this);

//	gtk_paned_add2(GTK_PANED(m_pVPane), m_pMessageView->m_pWidget);
	gtk_paned_pack2(GTK_PANED(m_pVPane), m_pMessageView->m_pWidget, TRUE, TRUE);
	
	gtk_widget_show(m_pMessageView->m_pWidget);

	m_pProjectView = new CVermProjectView(this);

//	gtk_paned_add1(GTK_PANED(m_pHPane), m_pProjectView->m_pWidget);
	gtk_paned_pack1(GTK_PANED(m_pHPane), m_pProjectView->m_pWidget, TRUE, TRUE);
	
	gtk_widget_show(m_pProjectView->m_pWidget);

	m_pFileView = new CVermFileView(this);
//	gtk_paned_add2(GTK_PANED(m_pHPane), m_pFileView->m_pWidget);
	gtk_paned_pack2(GTK_PANED(m_pHPane), m_pFileView->m_pWidget, TRUE, TRUE);
	
	gtk_widget_show(m_pFileView->m_pWidget);

	s_pFrameWnd = this;

	m_WaitCursor = gdk_cursor_new(GDK_WATCH);
	m_NormalCursor = gdk_cursor_new(GDK_TOP_LEFT_ARROW);

	SetFocus(FOCUS_PROJECTVIEW);
}

void CVermFrameWnd::OnActivate()
{
	String strProject;

	GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);

	if (!strProject.empty())
	{
		GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject.chars());

		String strWorkingDir = GetWorkingDirectory(hItem);

		CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

		if (pNode->m_bVisited)
			m_pFileView->UpdateFileInfo(pNode, strProject, m_username,
										strWorkingDir);
	}
}

void CVermFrameWnd::OnViewClearStatus(void)
{
	int n = m_pMessageView->m_nLinesToDisplay;

	m_pMessageView->SetLinesToDisplay(0);
	m_pMessageView->SetLinesToDisplay(n);
}

void CVermFrameWnd::OnSourceBranch(void)
{
	m_modState = 0;
	m_writeState = 0;
	m_lockState = 0;
	
	String strProject;

	GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);

	if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		// get the indices of the files selected for checkin

		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		while (selection)
		{
			int nIndex = (int) selection->data;
			
			CVermTreeLeaf* pLeaf = (CVermTreeLeaf*) gtk_clist_get_row_data(GTK_CLIST(m_pFileView->m_pList), nIndex);

			if (pLeaf && pLeaf->m_bShared)
			{
				CVermProtocolMessage* pMessage = new CVermProtocolMessage();
				pMessage->m_number = PM_BRANCH;

				pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pLeaf->m_name));

				if (!m_pSocket->SendMessage(pMessage))
					SetWaitState(1);
				else
				{
					delete pMessage;
					return;
				}
			}

			selection = selection->next;
		}
	}
	else
	{
		String strTitle = "Branch Project ";
		strTitle += strProject;

		CCommentDlg dlg(strTitle.chars(), TRUE);

		if (dlg.DoModal() == IDOK)
		{
			m_strComment = gtk_editable_get_chars(GTK_EDITABLE(dlg.comment), 0, -1);

			CVermProtocolMessage* pMessage = new CVermProtocolMessage();
			pMessage->m_number = PM_GET_FILE_LIST;
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, BRANCH_FILES));
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_SHARE_INFO, "yes"));

			String s("Updating file list: ");
			s += strProject;

			AddMessage(s);
			
			if (m_pSocket->SendMessage(pMessage))
				SetWaitState(1);
			else
			{
				delete pMessage;
				return;
			}

			GtkWidget* hChildItem = GTK_TREE_ITEM(m_pProjectView->m_hCurSelItem)->subtree;
		
			BranchFilesForNode(strProject);
			
			if (GTK_TOGGLE_BUTTON(dlg.recurse)->active && hChildItem)
			{
				GetProjectListings(hChildItem, BRANCH_FILES);
			}
		}
	}
}

void CVermFrameWnd::OnViewSearch(void)
{
	if (m_pSearchResultsDlg)
		return;
	
	CSearchDlg dlg("Search");

	gtk_entry_set_text(GTK_ENTRY(dlg.searchString), m_username.chars());

	if (dlg.DoModal() == IDOK)
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_SEARCH;

		String strProj("");

		if (dlg.bSearchAllProjects)
			strProj = "$/";
		else
			m_pProjectView->GetPathSelectedObject(strProj);

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProj));

		if (dlg.bSearchCurProjRecursively || dlg.bSearchAllProjects)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_RECURSIVE, "yes"));

		if (dlg.bSearchString)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEARCH_STRING, gtk_entry_get_text(GTK_ENTRY(dlg.searchString))));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
}

void CVermFrameWnd::OnToolsStop(void)
{
	// kill the conenction, then silently re-connect
	String s("");
	ServerShutdown(s);

	// create a new socket

	m_pSocket = new CVermSocket(this);

	// connect to the server

	SetWaitState(1);

	if (m_pSocket->Connect(m_server, m_port) != SOCKET_OK)
	{
		SetWaitState(0);

		String s("Unable to comminicate with server - please re-connect");
		AddMessage(s);
		
		delete m_pSocket;
		m_pSocket = NULL;
		
		ResetToolbar(FALSE);

		return;
	}

	m_bSilentLogin = 1;
}

void CVermFrameWnd::OnToolsCloakProject(void)
{
	String strProject;

	if (m_pProjectView->GetPathSelectedObject(strProject))
	{
		GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

		CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

		if (pNode)
		{
			pNode->m_bCloaked = !pNode->m_bCloaked;

			if (pNode->m_bCloaked)
			{
				gtk_label_set(GTK_LABEL(GTK_BIN(m_pMCloakProject)->child), "UnCloak Project");
				gtk_label_set(GTK_LABEL(GTK_BIN(m_pProjectView->m_pCloak)->child), "UnCloak Project");
//				gtk_pixmap_set(GTK_PIXMAP(GTK_TREE_ITEM(hItem)->plus_pix_widget), Vermie::m_pCloakedTreePlus, Vermie::m_pTreePlusMask);
//				gtk_pixmap_set(GTK_PIXMAP(GTK_TREE_ITEM(hItem)->minus_pix_widget), Vermie::m_pCloakedTreeMinus, Vermie::m_pTreeMinusMask);
				if (pNode->m_bWebDeploy)
					gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pCloakedDeployStyle);
				else
					gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pCloakedStyle);
			}
			else
			{
				gtk_label_set(GTK_LABEL(GTK_BIN(m_pMCloakProject)->child), "Cloak Project");
				gtk_label_set(GTK_LABEL(GTK_BIN(m_pProjectView->m_pCloak)->child), "Cloak Project");
//				gtk_pixmap_set(GTK_PIXMAP(GTK_TREE_ITEM(hItem)->plus_pix_widget), Vermie::m_pTreePlus, Vermie::m_pTreePlusMask);
//				gtk_pixmap_set(GTK_PIXMAP(GTK_TREE_ITEM(hItem)->minus_pix_widget), Vermie::m_pTreeMinus, Vermie::m_pTreeMinusMask);
				if (pNode->m_bWebDeploy)
					gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pWebDeployStyle);
				else
					gtk_widget_set_style(GTK_BIN(hItem)->child, Vermie::m_pNormalStyle);
			}
		}
	}

}

void CVermFrameWnd::OnToolsWebDeploy(void)
{
	String strProject;                                                     

	m_pProjectView->GetPathSelectedObject(strProject);

	if (strProject.empty())
		return;
	
	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);
	
	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

	int bMakeWritable = TRUE;

	if (m_nChildWithFocus == FOCUS_FILEVIEW || Vermie::m_bProjectDeploy)
	{
		CDeployDlg deployDlg("Web Deploy", pNode->m_strDeployDir, m_nChildWithFocus == FOCUS_FILEVIEW? TRUE : FALSE);

		GTK_TOGGLE_BUTTON(deployDlg.checkbutton)->active = 1;
		
		if (deployDlg.DoModal() == IDCANCEL)
			return;

		if (!GTK_TOGGLE_BUTTON(deployDlg.checkbutton)->active)
			bMakeWritable = FALSE;
	}
	
	if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;
		
		String strFile;

		while (selection)
		{
			m_pFileView->GetFileName((int)selection->data, strFile);

			CVermProtocolMessage* pMessage = new CVermProtocolMessage();

			pMessage->m_number = PM_DEPLOY;

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
			
			String strFile;

			m_pFileView->GetFileName((int)selection->data, strFile);

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

			if (!bMakeWritable)
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_READONLY, "Yes"));

			m_pSocket->SendMessage(pMessage);

			selection = selection->next;
		}
	}
	else
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_DEPLOY;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

		if (!bMakeWritable)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_READONLY, "Yes"));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
}

void CVermFrameWnd::OnSourceShare(void)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	if (m_pShareDlg)
		return;

	String strProj;

	GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProj);
	
	String strDir = GetWorkingDirectory(m_pProjectView->GetRootItem(), strProj.chars());

	if (strDir.empty())
	{
		if (SetWorkingFolder())
			return;

		strDir = GetWorkingDirectory(m_pProjectView->GetRootItem(), strProj.chars());

		if (strDir.empty())
			return;
	}
	
	m_pShareDlg = new CShareDlg(this, m_pProjectView->m_pTree, strProj);

	gtk_widget_show(m_pShareDlg->m_pDialog);
}

void CVermFrameWnd::OnFileView(int bAsked)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;

	if (!bAsked && Vermie::m_bFileEditAndView)
	{
		CViewEditDlg dlg("View/Edit File");
		
		if (fcompare(Vermie::m_strFileDoubleClick, "Ask"))
		{
			if (!fcompare(Vermie::m_strFileDoubleClick, "View File"))
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.view), 1);
			else
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.edit), 1);
		}
		
		if (dlg.DoModal() == IDOK)
		{
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.view)))
			{
				OnFileView(TRUE);
				return;
			}
			else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.edit)))
			{
				OnFileEdit(TRUE);
				return;
			}
		}
		else
			return;
	}
	
	// get the selected project

	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	// get the selected file

	GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

	String strFile;
		
	if (selection->next)
		return;
	
	int index = (int) selection->data;
			
	m_pFileView->GetFileName(index, strFile);

	ViewFile(strProject, strFile);
}

void CVermFrameWnd::ViewFile(String& strProject, String& strFile)
{
	// get the temp directory

	String strTmpDir = tempnam(NULL, NULL);

	// create a unique directory in the temp directory

	if (!CreateDirectory(strTmpDir))
	{
		String s("Unable to create temp directory for editor");
		AddMessage(s);
		return;
	}

	// create the edit struct

	FileEdit* pFileEdit = new FileEdit;

	strTmpDir += "/";
	
	pFileEdit->strTmpDir = strTmpDir;

	pFileEdit->strFile = strTmpDir + strFile;

	pFileEdit->bHaveFile = FALSE;

	pFileEdit->strEditProgram = Vermie::m_strViewProgram; 

	// send the messages

	GetEditFile(pFileEdit, strProject.chars());
}

void CVermFrameWnd::OnFileEdit(int bAsked)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;

	if (!bAsked && Vermie::m_bFileEditAndView)
	{
		CViewEditDlg dlg("View/Edit File");
		
		if (fcompare(Vermie::m_strFileDoubleClick, "Ask"))
		{
			if (!fcompare(Vermie::m_strFileDoubleClick, "View File"))
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.view), 1);
			else
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.edit), 1);
		}

		if (dlg.DoModal() == IDOK)
		{
			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.view)))
			{
				OnFileView(TRUE);
				return;
			}
			else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dlg.edit)))
			{
				OnFileEdit(TRUE);
				return;
			}
		}
		else
			return;
	}

	// load the selected project

	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

	// get the working directory

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (strWorkingDir.empty())
	{
		String s("No working directory set");
		AfxMessageBox(s);

		if (SetWorkingFolder())
			return;

		strWorkingDir = GetWorkingDirectory(hItem);

		if (strWorkingDir.empty())
			return;
	}

	GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

	String strFile;
		
	if (selection->next)
		return;
	
	int index = (int) selection->data;
			
	m_pFileView->GetFileName(index, strFile);

	// if we already have the file locked, then just launch the editor;
	// otherwise, first perform the checkout operation

	int nLockStatus = IsFileLocked(pNode, strProject, strFile, m_username);

	if (nLockStatus < 2)
		CheckOutFile(pNode, strProject, strWorkingDir, strFile, TRUE);
	else
	{
		String strFileName = strWorkingDir + "/";
		strFileName += strFile;

		VermEditFile(strFileName);
	}							
}

void CVermFrameWnd::OnFileRename(void) 
{
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		CRenameDlg renameDlg;

		String strOldName;
		String strRev = reverse(strProject);
		int nIndex = strRev.index('/');
		
		strOldName = strProject.after((int)strProject.length() - nIndex - 1);

		if (renameDlg.DoModal() == IDCANCEL)
			return;

		String newName = gtk_entry_get_text(GTK_ENTRY(renameDlg.entry1));

		if (newName == strOldName)
			return;

		if (newName.empty())
			return;
		
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_RENAME;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_NAME, newName));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
	else if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;
		
		if (!selection || selection->next)
			return;

		String strFile;

		m_pFileView->GetFileName((int) selection->data, strFile);

		CRenameDlg renameDlg;

		String strOldName = strFile;

		if (renameDlg.DoModal() == IDCANCEL)
			return;

		String newName = gtk_entry_get_text(GTK_ENTRY(renameDlg.entry1));

		if (newName == strOldName)
			return;

		if (newName.empty())
			return;
		
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_RENAME;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_NAME, newName));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
}

CVermFrameWnd::~CVermFrameWnd()
{
	if (m_pSocket)
	{
		m_pSocket->Close();
		delete m_pSocket;
	}

#ifndef BUILD_STANDARD
	if (m_pCrypto)
		delete m_pCrypto;
#endif // !BUILD_STANDARD

	if (m_pFileView)
		delete m_pFileView;

	if (m_pProjectView)
		delete m_pProjectView;

	if (m_pMessageView)
		delete m_pMessageView;
}

void CVermFrameWnd::OnFileLabel() 
{
	CCreateProjectDlg labelDlg("Label", MB_OK | MB_CANCEL);

	gtk_label_set(GTK_LABEL(labelDlg.label1), "Label:");
	
	if (labelDlg.DoModal() == IDCANCEL)
		return;

	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	String strLabel = gtk_entry_get_text(GTK_ENTRY(labelDlg.project));
	String strComment = gtk_editable_get_chars(GTK_EDITABLE(labelDlg.comment),
											   0, -1);
	
	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_ADD_LABEL;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_LABEL, strLabel));

		if (!strComment.empty())
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strComment));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
	else if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;
		
		String strFile;

		while (selection)
		{
			m_pFileView->GetFileName((int)selection->data, strFile);

			CVermProtocolMessage* pMessage = new CVermProtocolMessage();

			pMessage->m_number = PM_ADD_LABEL;

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_LABEL, strLabel));

			if (!strComment.empty())
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strComment));


			if (!m_pSocket->SendMessage(pMessage))
				SetWaitState(TRUE);
			else
			{
				delete pMessage;
				return;
			}

			selection = selection->next;
		}
	}
}

void CVermFrameWnd::OnViewRefreshProjectTree(void)
{
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);
	
	if (strProject.empty())
		return;
	
	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_GET_PROJECT_TREE;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, UPDATE_PROJECT));


	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
		delete pMessage;
}

void CVermFrameWnd::OnToolsOptions(void)
{
	COptionsDlg dlg;

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.useChecksums), Vermie::m_bUseChecksums);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.displayShareIcon), Vermie::m_bShowShareIcon);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.compression), Vermie::m_bUseCompression);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.useHTTPProxy), Vermie::m_bUseHTTPProxy);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.makeFilesReadOnly), Vermie::m_bMakeFilesReadOnly);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.displayPinStatus), Vermie::m_bDisplayPinStatus);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.fileGetLatest), Vermie::m_bFileGetLatestVersion);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.fileCheckout), Vermie::m_bFileCheckout);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.fileUndoCheckout), Vermie::m_bFileUndoCheckout);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.fileEditAndView), Vermie::m_bFileEditAndView);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.projectGetLatest), Vermie::m_bProjectGetLatestVersion);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.projectCheckout), Vermie::m_bProjectCheckout);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.projectUndoCheckout), Vermie::m_bProjectUndoCheckout);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.projectDeploy), Vermie::m_bProjectDeploy);

	gtk_entry_set_text(GTK_ENTRY(dlg.diffProgram), Vermie::m_strDiffProgram.chars());
	gtk_entry_set_text(GTK_ENTRY(dlg.projectDiffProgram), Vermie::m_strProjectDiffProgram.chars());
	gtk_entry_set_text(GTK_ENTRY(dlg.proxyServer), Vermie::m_strProxyName.chars());
	gtk_entry_set_text(GTK_ENTRY(dlg.mergeProgram), Vermie::m_strMergeProgram.chars());

	char buf[256];
	sprintf(buf, "%i", Vermie::m_ProxyPort);
	
	gtk_entry_set_text(GTK_ENTRY(dlg.proxyPort), buf);
	
	int bUpdateFileList = Vermie::m_bUpdateFileList;
	int bSendKeepAlive = Vermie::m_bSendKeepAlive;
	
	gtk_entry_set_text(GTK_ENTRY(dlg.timeToRefresh), Vermie::m_strTimeToRefresh.chars());
	gtk_entry_set_text(GTK_ENTRY(dlg.timeToKeepAlive), Vermie::m_strTimeToKeepAlive.chars());
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dlg.fileDClickCombo)->entry), Vermie::m_strFileDoubleClick.chars());
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(dlg.fileEOLCombo)->entry), Vermie::m_strFileEOL.chars());
	gtk_entry_set_text(GTK_ENTRY(dlg.editProgram), Vermie::m_strViewProgram.chars());
	
	if (bUpdateFileList)
	{
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.refresh), 1);
	}
	else
	{
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.refresh), 0);
		gtk_widget_set_sensitive(dlg.timeToRefresh, 0);
	}

	if (bSendKeepAlive)
	{
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.keepAlive), 1);
	}
	else
	{
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.keepAlive), 0);
		gtk_widget_set_sensitive(dlg.timeToKeepAlive, 0);
	}

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.recurseProjects),
								Vermie::m_bRecurseOnProjects);
	
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.keepFilesCheckedOut),
								Vermie::m_bAlwaysKeepFilesCheckedOut);

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.dontReplaceLocalCopy),
								Vermie::m_bDontFetchFileOnCheckout);

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.dontReplaceAfterUndo),
								Vermie::m_bDontFetchFileOnUndoCheckout);

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.removeLocalCopy),
								Vermie::m_bRemoveLocalCopy);

	String strTimeToRefresh = Vermie::m_strTimeToRefresh;
	String strTimeToKeepAlive = Vermie::m_strTimeToKeepAlive;
	
	if (dlg.DoModal() == IDOK)
	{
		Vermie::m_bUseCompression = GTK_TOGGLE_BUTTON(dlg.compression)->active;
		Vermie::m_bUseHTTPProxy = GTK_TOGGLE_BUTTON(dlg.useHTTPProxy)->active;
		Vermie::m_strDiffProgram = gtk_entry_get_text(GTK_ENTRY(dlg.diffProgram));
		Vermie::m_strMergeProgram = gtk_entry_get_text(GTK_ENTRY(dlg.mergeProgram));
		Vermie::m_strProjectDiffProgram = gtk_entry_get_text(GTK_ENTRY(dlg.projectDiffProgram));
		Vermie::m_strProxyName = gtk_entry_get_text(GTK_ENTRY(dlg.proxyServer));
		Vermie::m_ProxyPort = atoi(gtk_entry_get_text(GTK_ENTRY(dlg.proxyPort)));
		Vermie::m_bRemoveLocalCopy = GTK_TOGGLE_BUTTON(dlg.removeLocalCopy)->active;
		Vermie::m_bRecurseOnProjects = GTK_TOGGLE_BUTTON(dlg.recurseProjects)->active;
		Vermie::m_bAlwaysKeepFilesCheckedOut = GTK_TOGGLE_BUTTON(dlg.keepFilesCheckedOut)->active;
		Vermie::m_bDontFetchFileOnCheckout = GTK_TOGGLE_BUTTON(dlg.dontReplaceLocalCopy)->active;
		Vermie::m_bMakeFilesReadOnly = GTK_TOGGLE_BUTTON(dlg.makeFilesReadOnly)->active;
		Vermie::m_bDontFetchFileOnUndoCheckout = GTK_TOGGLE_BUTTON(dlg.dontReplaceAfterUndo)->active;
		Vermie::m_bShowShareIcon = GTK_TOGGLE_BUTTON(dlg.displayShareIcon)->active;
		Vermie::m_bUseChecksums = GTK_TOGGLE_BUTTON(dlg.useChecksums)->active;
		Vermie::m_bDisplayPinStatus = GTK_TOGGLE_BUTTON(dlg.displayPinStatus)->active;
		
		Vermie::m_bFileGetLatestVersion = GTK_TOGGLE_BUTTON(dlg.fileGetLatest)->active;
		Vermie::m_bFileCheckout = GTK_TOGGLE_BUTTON(dlg.fileCheckout)->active;
		Vermie::m_bFileUndoCheckout = GTK_TOGGLE_BUTTON(dlg.fileUndoCheckout)->active;
		Vermie::m_bFileEditAndView = GTK_TOGGLE_BUTTON(dlg.fileEditAndView)->active;
		Vermie::m_bProjectGetLatestVersion = GTK_TOGGLE_BUTTON(dlg.projectGetLatest)->active;
		Vermie::m_bProjectCheckout = GTK_TOGGLE_BUTTON(dlg.projectCheckout)->active;
		Vermie::m_bProjectUndoCheckout = GTK_TOGGLE_BUTTON(dlg.projectUndoCheckout)->active;
		Vermie::m_bProjectDeploy = GTK_TOGGLE_BUTTON(dlg.projectDeploy)->active;

		Vermie::m_strFileDoubleClick = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dlg.fileDClickCombo)->entry));
		Vermie::m_strFileEOL = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dlg.fileEOLCombo)->entry));
		Vermie::m_strViewProgram = gtk_entry_get_text(GTK_ENTRY(dlg.editProgram));
													  
		if (dlg.colorChanged)
		{
			Vermie::m_pCloakedStyle = gtk_style_copy(dlg.pCloakedStyle);
			Vermie::m_pWebDeployStyle = gtk_style_copy(dlg.pDeployStyle);
			Vermie::m_pCloakedDeployStyle = gtk_style_copy(dlg.pBothStyle);

			RefreshTreeColors(m_pProjectView->m_pTree);
		}
		
		if (Vermie::m_strFileDoubleClick != "Ask" && Vermie::m_strFileDoubleClick != "View File" && Vermie::m_strFileDoubleClick != "Edit File")
			Vermie::m_strFileDoubleClick = "Ask";
		
		int nNewBUpdate = GTK_TOGGLE_BUTTON(dlg.refresh)->active;
		int nNewBKeepAlive = GTK_TOGGLE_BUTTON(dlg.keepAlive)->active;

		Vermie::m_nLinesToDisplay = atoi(gtk_entry_get_text(GTK_ENTRY(dlg.statusWindowLines)));

		m_pMessageView->SetLinesToDisplay(Vermie::m_nLinesToDisplay);
		
		// user changed setting
		if (nNewBUpdate != bUpdateFileList)
		{
			Vermie::m_bUpdateFileList = nNewBUpdate;

			// was running - kill timer
			if (bUpdateFileList)
			{
				gtk_timeout_remove(m_nTimerID);
				m_nTimerID = 0;
			}
			else
			{
				Vermie::m_strTimeToRefresh = gtk_entry_get_text(GTK_ENTRY(dlg.timeToRefresh));

				int timeout = atoi(Vermie::m_strTimeToRefresh.chars());

				if (timeout)
				{
					m_nTimerID = gtk_timeout_add(timeout * 60 * 1000,
												 (GtkFunction)RefreshTimer, this);
				}
				else
				{
					Vermie::m_bUpdateFileList = 0;
				}
			}
		}
		else if (nNewBUpdate && fcompare(strTimeToRefresh, Vermie::m_strTimeToRefresh))
		{
			// they changed the time on us
			gtk_timeout_remove(m_nTimerID);

			int timeout = atoi(Vermie::m_strTimeToRefresh.chars());

			m_nTimerID = gtk_timeout_add(timeout * 60 * 1000,
										 (GtkFunction)RefreshTimer, this);
			
		}
		
		// user changed setting
		if (nNewBKeepAlive != bSendKeepAlive)
		{
			Vermie::m_bSendKeepAlive = nNewBKeepAlive;

			// was running - kill timer
			if (bSendKeepAlive)
			{
				gtk_timeout_remove(m_nKeepAliveTimerID);
				m_nKeepAliveTimerID = 0;
			}
			else
			{
				Vermie::m_strTimeToKeepAlive = gtk_entry_get_text(GTK_ENTRY(dlg.timeToKeepAlive));

				int timeout = atoi(Vermie::m_strTimeToKeepAlive.chars());

				if (timeout)
				{
					m_nKeepAliveTimerID = gtk_timeout_add(timeout * 60 * 1000,
														  (GtkFunction)KeepAliveTimer, this);
				}
				else
				{
					Vermie::m_bSendKeepAlive = 0;
				}
			}
		}
		else if (nNewBKeepAlive && fcompare(strTimeToKeepAlive, Vermie::m_strTimeToKeepAlive))
		{
			// they changed the time on us
			gtk_timeout_remove(m_nKeepAliveTimerID);

			int timeout = atoi(Vermie::m_strTimeToKeepAlive.chars());

			m_nKeepAliveTimerID = gtk_timeout_add(timeout * 60 * 1000,
												  (GtkFunction)KeepAliveTimer, this);
			
		}
	}
}

void CVermFrameWnd::OnToolsShowDifferences(void)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (strWorkingDir.empty())
	{
		String s("No working directory set");
		AfxMessageBox(s);

		if (SetWorkingFolder())
			return;

		strWorkingDir = GetWorkingDirectory(hItem);

		if (strWorkingDir.empty())
			return;
	}

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		if (Vermie::m_strProjectDiffProgram == "")
		{
			String s("You have not set a project diff program in the options dialog yet");
			AfxMessageBox(s);
			return;
		}
		
		int bRecurse = Vermie::m_bRecurseOnProjects;
		
		CCheckoutDlg dlg("Project Diff", TRUE, FALSE);
		
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.recursive),
									 bRecurse);
		
		if (dlg.DoModal() == IDCANCEL)
			return;
		
		bRecurse = GTK_TOGGLE_BUTTON(dlg.recursive)->active;

		// get the temp directory

		String strTmpDir = tempnam(NULL, NULL);

		if (strTmpDir[strTmpDir.length() - 1] != '/')
			strTmpDir += "/";
		
		// create a unique directory in the temp directory

		if (mkdir(strTmpDir.chars(), 0700))
		{
			String s("Unable to create temp directory for diff");
			AddMessage(s);
			return;
		}

		// create the diff struct

		m_pFileDiff = new FileDiff;

		m_pFileDiff->strTmpDir = strTmpDir;
		strTmpDir = tempnam(NULL, NULL);

		if (strTmpDir[strTmpDir.length() - 1] != '/')
			strTmpDir += "/";
		
		// create a unique directory in the temp directory

		if (mkdir(strTmpDir.chars(), 0700))
		{
			String s("Unable to create temp directory for diff");
			AddMessage(s);
			return;
		}

		m_pFileDiff->strSourceTmpDir = strTmpDir;
		
		m_pFileDiff->bHaveFileOne = 0;
		m_pFileDiff->bOneIsTempFile = 0;
		
		m_pFileDiff->bTwoIsTempFile = 0;
		m_pFileDiff->bHaveFileTwo = 0;

		m_pFileDiff->strProject = strProject;

		m_pFileDiff->bProjectDiff = 1;
		m_pFileDiff->bRecurse = bRecurse;

		CVermTreeNode* pNode = GetTreeNodeFromProject(m_pProjectView->GetRootItem(), strProject.chars());
		
		GtkWidget* hChildItem = GTK_TREE_ITEM(hItem)->subtree;
		
		String s("Updating file list: ");
		s += strProject;

		AddMessage(s);
		
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();
		pMessage->m_number = PM_GET_FILE_LIST;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, PROJECT_DIFF));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
		
		if (Vermie::m_bShowShareIcon && Vermie::m_bUseVSS6Features)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_SHARE_INFO, ""));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
		
		if (!pNode->m_bCloaked)
		{
			if (bRecurse && hChildItem)
			{
				GetProjectListings(hChildItem, PROJECT_DIFF);
			}
		}
	}
	else if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		if (Vermie::m_strDiffProgram == "")
		{
			String s("You have not set a file diff program in the options dialog yet");
			AfxMessageBox(s);
			return;
		}

		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		if (!selection || selection->next)
			return;

		String strFile;

		m_pFileView->GetFileName((int)selection->data, strFile);

		DiffFile(strWorkingDir, strProject, strFile);
	}
}

void CVermFrameWnd::DiffFile(String& strWorkingDir, String& strProject, String& strFile)
{
	// make sure the file exists locally
	if (strWorkingDir.empty())
	{
		String s("No working directory set");
		AfxMessageBox(s);
		return;
	}

	String strFileName = strWorkingDir + "/" + strFile;

	int file = open(strFileName.chars(), O_RDONLY);

	if (file < 0)
	{
		String s("Unable to open file: ");
		s += strFileName;
		AddMessage(s);
		return;
	}

	close(file);

		// get the temp directory

	String strTmpDir = tempnam(NULL, NULL);

	if (strTmpDir[strTmpDir.length() - 1] != '/')
		strTmpDir += "/";
		
		// create a unique directory in the temp directory

	if (mkdir(strTmpDir.chars(), 0700))
	{
		String s("Unable to create temp directory for diff");
		AddMessage(s);
		return;
	}

	// create the diff struct

	FileDiff* pFileDiff = new FileDiff;

	pFileDiff->bProjectDiff = 0;
		
	pFileDiff->strTmpDir = strTmpDir;

	pFileDiff->strFileOne = strTmpDir + strFile;

	pFileDiff->bHaveFileOne = 0;
	pFileDiff->bOneIsTempFile = 1;
		
	pFileDiff->strFileTwo = strFileName;

	pFileDiff->bTwoIsTempFile = 0;
	pFileDiff->bHaveFileTwo = 1;

	// send the messages

	GetDiffFiles(pFileDiff, strProject);
}

void CVermFrameWnd::OnToolsShowHistory(void)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	if (m_pHistoryDlg)
		return;
	
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		CProjectHistoryDlg dlg;

		if (dlg.DoModal() == IDCANCEL)
			return;

		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_PROJECT_GET_HISTORY;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_ACTION, "ues"));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_LABEL_COMMENT, "yes"));

		if (GTK_TOGGLE_BUTTON(dlg.recurse)->active)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_RECURSIVE, "yes"));
		if (!GTK_TOGGLE_BUTTON(dlg.includeFiles)->active)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_IGNORE_FILES, "yes"));
		String strFrom = gtk_editable_get_chars(GTK_EDITABLE(dlg.from), 0, -1);

		if (!strFrom.empty())
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FROM_DATE, strFrom));

		String strTo = gtk_editable_get_chars(GTK_EDITABLE(dlg.to), 0, -1);

		if (!strTo.empty())
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_TO_DATE, strTo));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
		// We currently do not support project history
	}
	else if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		GList* selected = GTK_CLIST(m_pFileView->m_pList)->selection;
		
		if (selected->next != NULL)
			return;

		String strFile;

		m_pFileView->GetFileName((int)selected->data, strFile);

		GetHistory(strProject, strFile);
	}

}

void CVermFrameWnd::GetHistory(String& strProject, String& strFile)
{
	// create the message

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_GET_FILE_HISTORY;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_LABEL_COMMENT, "yes"));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_ACTION, "ues"));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}
}
	
void CVermFrameWnd::OnSourceUndoCheckout(void) 
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	String strProject;

	// load the selected project and its working directory

	m_pProjectView->GetPathSelectedObject(strProject);

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);

	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		int bRecurse = Vermie::m_bRecurseOnProjects;
		char* localCopy = "Replace";

		if (Vermie::m_bDontFetchFileOnUndoCheckout)
			localCopy = "Leave";
		
		if (Vermie::m_bProjectUndoCheckout)
		{
			CUndoCheckoutDlg dlg("Undo Checkout", localCopy, TRUE);

			if (bRecurse)
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.recursive), 1);
			
			if (dlg.DoModal() == IDCANCEL)
				return;

			bRecurse = GTK_TOGGLE_BUTTON(dlg.recursive)->active;
			localCopy = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dlg.localCopy)->entry));
		}
		
		char* pCookie = "Replace";

		if (Vermie::m_bDontFetchFileOnUndoCheckout)
			pCookie = "Leave";
		
		if (!strcmp(localCopy, "Replace"))
			pCookie = UNDO_CHECKOUT_REPLACE;
		else if (!strcmp(localCopy, "Leave"))
			pCookie = UNDO_CHECKOUT_LEAVE;
		else if (!strcmp(localCopy, "Delete"))
			pCookie = UNDO_CHECKOUT_DELETE;
		
		GtkWidget* hChildItem = GTK_TREE_ITEM(hItem)->subtree;
		
		String strNode;
		GetPathFromTreeItem(hItem, strNode);
		
		UndoCheckoutForNode(pNode, strWorkingDir, strNode, pCookie);
			
		if (bRecurse && hChildItem)
		{
			GetProjectListings(hChildItem, pCookie);
		}
	}
	else
	{
		// get the list of files selected for undo checkout
		String strFile;

		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		char* localCopy = "Replace";

		if (Vermie::m_bDontFetchFileOnUndoCheckout)
			localCopy = "Leave";

		CUndoCheckoutDlg dlg("Undo Checkout", localCopy, FALSE);

		if (Vermie::m_bFileUndoCheckout)
		{
			if (dlg.DoModal() != IDOK)
				return;
			
			localCopy = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dlg.localCopy)->entry));
		}

		char* pCookie = "Replace";

		if (Vermie::m_bDontFetchFileOnUndoCheckout)
			pCookie = "Leave";
		
		if (!strcmp(localCopy, "Replace"))
			pCookie = UNDO_CHECKOUT_REPLACE;
		else if (!strcmp(localCopy, "Leave"))
			pCookie = UNDO_CHECKOUT_LEAVE;
		else if (!strcmp(localCopy, "Delete"))
			pCookie = UNDO_CHECKOUT_DELETE;
		
		
		while (selection)
		{
			int nIndex = (int) selection->data;
			
			m_pFileView->GetFileName(nIndex, strFile);

			if (!IsFileLocked(pNode, strProject, strFile, m_username))
			{
				selection = selection->next;
				continue;
			}

			UndoCheckout(pNode, strWorkingDir, strProject, strFile, pCookie);

			if (m_modState == IDCANCEL)
			{
				return;
			}

			selection = selection->next;
		}
	}
}

void CVermFrameWnd::OnSourceCheckIn(void)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	// get the project

	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	// get the working directory

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (strWorkingDir.empty())
	{
		if (SetWorkingFolder() < 0)
			return;

		strWorkingDir = GetWorkingDirectory(hItem);		
	}

	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		int bRecurse = Vermie::m_bRecurseOnProjects;
		int bRemoveLocalCopy = Vermie::m_bRemoveLocalCopy;
		int bKeepCheckedOut = Vermie::m_bAlwaysKeepFilesCheckedOut;
		
		CCheckinDlg dlg;
		
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.removeLocalCopy),
									bRemoveLocalCopy);
		
		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.keepCheckedOut),
									bKeepCheckedOut);

		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.recursive), bRecurse);
									
		if (dlg.DoModal() == IDCANCEL)
			return;
		
		bRecurse = GTK_TOGGLE_BUTTON(dlg.recursive)->active;
		m_bRemoveLocalCopy = GTK_TOGGLE_BUTTON(dlg.removeLocalCopy)->active;
		m_bKeepCheckedOut = GTK_TOGGLE_BUTTON(dlg.keepCheckedOut)->active;
		m_strComment = gtk_editable_get_chars(GTK_EDITABLE(dlg.comment),
											  0, -1);
					
		GtkWidget* hChildItem = GTK_TREE_ITEM(hItem)->subtree;
		
		String strNode;
		GetPathFromTreeItem(hItem, strNode);
		
		CheckinFilesForNode(pNode, strWorkingDir, strNode, m_strComment,
							m_bRemoveLocalCopy, m_bKeepCheckedOut);
		
		if (bRecurse && hChildItem)
		{
			GetProjectListings(hChildItem, CHECKIN_FILES);
		}
	}
	else
	{
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		int bDiff = 0;

		if (selection && !selection->next)
			bDiff = 1;
		
		CCheckinDlg dlg(this, bDiff);

		gtk_widget_set_sensitive(dlg.recursive, FALSE);
		
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.keepCheckedOut),
									 Vermie::m_bAlwaysKeepFilesCheckedOut);

		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.removeLocalCopy),
									 Vermie::m_bRemoveLocalCopy);
		
		if (dlg.DoModal() == IDCANCEL)
			return;

		// get the indices of the files selected for checkin

		String strFile;

		String strComment = gtk_editable_get_chars(GTK_EDITABLE(dlg.comment),
												   0, -1);
		int bKeepCheckedOut = GTK_TOGGLE_BUTTON(dlg.keepCheckedOut)->active;
		int bRemoveLocalCopy = GTK_TOGGLE_BUTTON(dlg.removeLocalCopy)->active;
		
		while (selection)
		{
			int nIndex = (int) selection->data;
			
			m_pFileView->GetFileName(nIndex, strFile);

			CheckInFile(pNode, strProject, strWorkingDir, strFile,
						strComment, bRemoveLocalCopy, bKeepCheckedOut);

			selection = selection->next;
		}
	}
}

void CVermFrameWnd::OnViewRefreshFileList(void)
{
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);
	
	if (strProject.empty())
		return;

	ProjectItemSelected(strProject);
}

void CVermFrameWnd::OnFileDelete(void)
{
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		CDeleteDlg deleteDlg("Delete Project");

		gtk_entry_set_text(GTK_ENTRY(deleteDlg.deleteItems), strProject.chars());
		if (deleteDlg.DoModal() == IDCANCEL)
			return;

		int bDestroy = GTK_TOGGLE_BUTTON(deleteDlg.destroyButton)->active;

		if (bDestroy)
		{
			String s("Destroy cannot be undone; information will be lost permanently!\nContinue Anyway?");
			if (AfxMessageBox(s, MB_YES|MB_NO) == IDNO)
				return;
		}
		else
		{
			String s("Delete all specified items?");
			if (AfxMessageBox(s, MB_YES|MB_NO) == IDNO)
				return;
		}

		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_DELETE;

		String strItem = gtk_entry_get_text(GTK_ENTRY(deleteDlg.deleteItems));

		if (!GetTreeItemFromProject(m_pProjectView->GetRootItem(), strItem))
		{
			String s("Invalid Project");
			AfxMessageBox(s);
			return;
		}
		
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strItem));

		if (bDestroy)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PERMANENT, "Yes"));
		else
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PERMANENT, "No"));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(TRUE);
		else
		{
			delete pMessage;
			return;
		}

	}
	else if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		// load all of the selected files into one string to display in the dialog

		String strFile;
		String strItems;
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		while (selection)
		{
			int nIndex = (int) selection->data;
			
			m_pFileView->GetFileName(nIndex, strFile);

			strItems += "\"" + strFile + "\"";

			if (selection->next)
				strItems += " ";

			selection = selection->next;
		}

		// raise the dialog

		CDeleteDlg deleteDlg("Delete Files");

		gtk_entry_set_text(GTK_ENTRY(deleteDlg.deleteItems), strItems.chars());

		if (deleteDlg.DoModal() == IDCANCEL)
			return;

		int bDestroy = 0;

		if (GTK_TOGGLE_BUTTON(deleteDlg.destroyButton)->active)
			bDestroy = 1;
		
		if (bDestroy)
		{
			String s("Destroy cannot be undone; information will be lost permanently!\nContinue Anyway?");
			if (AfxMessageBox(s, MB_YES|MB_NO) == IDNO)
				return;
		}
		else
		{
			String s("Delete all specified items?");
			if (AfxMessageBox(s, MB_YES|MB_NO) == IDNO)
				return;
		}

		// delete all of the specified files

		strItems = gtk_entry_get_text(GTK_ENTRY(deleteDlg.deleteItems));

		while (1)
		{
			int nFirstIndex = strItems.index('\"');  // "

			if (nFirstIndex < 0)
				break;

			strItems = strItems.after(nFirstIndex);

			int nSecIndex = strItems.index('\"'); // "

			if (nSecIndex < 0)
				break;

			strFile = strItems.before(nSecIndex);

			strItems = strItems.after(nSecIndex);

			CVermProtocolMessage* pMessage = new CVermProtocolMessage();

			pMessage->m_number = PM_DELETE;

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

			if (bDestroy)
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_PERMANENT, "Yes"));
			else
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_PERMANENT, "No"));

			if (!m_pSocket->SendMessage(pMessage))
				SetWaitState(1);
			else
			{
				delete pMessage;
				return;
			}
		}
	}
	
}

void CVermFrameWnd::OnSourceCheckOut(void)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	String strProject;

	// load the selected project and its working directory

	m_pProjectView->GetPathSelectedObject(strProject);

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);

	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (strWorkingDir.empty())
	{
		if (SetWorkingFolder() < 0)
			return;

		strWorkingDir = GetWorkingDirectory(hItem);
	}

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		int bRecurse = Vermie::m_bRecurseOnProjects;
		int bDontOverwrite = Vermie::m_bDontFetchFileOnCheckout;
		
		if (Vermie::m_bProjectCheckout)
		{
			CCheckoutDlg dlg("Check out files", TRUE);
			gtk_widget_show(dlg.dontOverwrite);
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.dontOverwrite), bDontOverwrite);
			
			if (dlg.DoModal() == IDCANCEL)
				return;

			bRecurse = GTK_TOGGLE_BUTTON(dlg.recursive)->active;
			bDontOverwrite = GTK_TOGGLE_BUTTON(dlg.dontOverwrite)->active;
		}
			
		GtkWidget* hChildItem = GTK_TREE_ITEM(hItem)->subtree;
		
//			pNode->m_strTmpWorkingDir = dlg.m_strDirectory;

		String cookie = CHECKOUT_FILES;

		if (bDontOverwrite)
			cookie = CHECKOUT_FILES_DONT_OVERWRITE;
		
		String strNode;
		GetPathFromTreeItem(hItem, strNode);
		
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();
		pMessage->m_number = PM_GET_FILE_LIST;
			
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strNode));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, cookie));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

		if (Vermie::m_bShowShareIcon && Vermie::m_bUseVSS6Features)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_SHARE_INFO, ""));
			
		if (Vermie::m_bDisplayPinStatus)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_PIN_STATUS, ""));

		String strMessage = "Updating file list: ";
		strMessage += strNode;
			
		AddMessage(strMessage);
			
		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(TRUE);
		else
		{
			delete pMessage;
			return;
		}
			
		if (bRecurse && hChildItem)
		{
			GetProjectListings(hChildItem, cookie.chars());
		}
	}
	else
	{
		// get the list of files selected for checkout

		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		if (!selection)
			return;

		String strFile;

		int bDontOverwrite = Vermie::m_bDontFetchFileOnCheckout;
		
		if (Vermie::m_bFileCheckout)
		{
			CCheckoutDlg dlg("Check out files", FALSE);
			gtk_widget_show(dlg.dontOverwrite);
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dlg.dontOverwrite), bDontOverwrite);
			
			if (dlg.DoModal() == IDCANCEL)
				return;

			bDontOverwrite = GTK_TOGGLE_BUTTON(dlg.dontOverwrite)->active;
		}

		while(selection)
		{
			int nIndex = (int) selection->data;
			
			m_pFileView->GetFileName(nIndex, strFile);

			int r;
			
			if ((r = IsFileModified(pNode, strFile, strWorkingDir)) != FILE_NOT_MODIFIED && !bDontOverwrite)
			{			
				// if it's missing, just fetch it, but if it's been modified, ask ...
				if (r == FILE_MODIFIED && (m_modState != IDYESTOALL && m_modState != IDNOTOALL))
				{
					String s = strFile + " may have been modified, lose changes?";
					CYesNoDlg dlg(s);
					m_modState = dlg.DoModal();
				}

				// bail out if they hit cancel
				if (m_modState == IDCANCEL)
				{
					return;
				}

				if (m_modState == IDYES || m_modState == IDYESTOALL)
				{
					CVermProtocolMessage* pMessage = new CVermProtocolMessage();

					pMessage->m_number = PM_GET_FILE;
					
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
					
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
					
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

					if (!m_pSocket->SendMessage(pMessage))
						SetWaitState(1);
					else
					{
						delete pMessage;
						return;
					}
				}
			}

			CVermProtocolMessage* pMessage = new CVermProtocolMessage();

			pMessage->m_number = PM_LOCK_FILE;

			if (Vermie::m_bDontFetchFileOnCheckout)
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, CHECKOUT_FILES_DONT_OVERWRITE));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

			if (!m_strComment.empty())
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, m_strComment));
			
			if (!m_pSocket->SendMessage(pMessage))
				SetWaitState(1);
			else
			{
				delete pMessage;
				return;
			}

			selection = selection->next;
		}
	}
}

void CVermFrameWnd::OnSourceGetLatestVersion(void)
{
	m_modState = IDYES;
	m_lockState = IDYES;
	m_writeState = IDYES;
	m_strComment = "";
	m_bKeepCheckedOut = 0;
	m_bRemoveLocalCopy = 0;
	
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	if (m_nChildWithFocus == FOCUS_PROJECTVIEW)
	{
		GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);

		String strWorkingDir = GetWorkingDirectory(hItem);

		if (strWorkingDir.empty())
		{
			if (SetWorkingFolder() < 0)
				return;

			strWorkingDir = GetWorkingDirectory(hItem);
		}

		int bRecurse = Vermie::m_bRecurseOnProjects;
		int bWritable = !Vermie::m_bMakeFilesReadOnly;
		
		if (Vermie::m_bProjectGetLatestVersion)
		{
			CGetLatestDlg dlg("Get Latest Version", TRUE);

			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.recursive),
										 bRecurse);
			
			if (!Vermie::m_bMakeFilesReadOnly)
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dlg.writable), 1);
										 
			if (dlg.DoModal() == IDCANCEL)
				return;

			bRecurse = GTK_TOGGLE_BUTTON(dlg.recursive)->active;
			bWritable = GTK_TOGGLE_BUTTON(dlg.writable)->active;
		}
			
		GtkWidget* hChildItem = GTK_TREE_ITEM(hItem)->subtree;
		
		String strNode;
		GetPathFromTreeItem(hItem, strNode);

		char* pCookie = FETCH_FILES;

		if (bWritable)
			pCookie = FETCH_FILES_MAKE_WRITABLE;
			
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();
		pMessage->m_number = PM_GET_FILE_LIST;
		
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strNode));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, pCookie));
		
		if (Vermie::m_bShowShareIcon)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_SHARE_INFO, ""));
		
		if (Vermie::m_bDisplayPinStatus)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_PIN_STATUS, ""));
		
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
		
		String strMessage = "Updating file list: ";
		strMessage += strNode;
		
		AddMessage(strMessage);
		
		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
		
		if (bRecurse && hChildItem)
		{
			GetProjectListings(hChildItem, pCookie);
		}
	}
	else if (m_nChildWithFocus == FOCUS_FILEVIEW)
	{
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		String strFile;

		int bWritable = !Vermie::m_bMakeFilesReadOnly;
		
		if (Vermie::m_bFileGetLatestVersion)
		{
			CGetLatestDlg dlg("Get Latest Version", FALSE);

			if (dlg.DoModal() == IDOK)
			{
				bWritable = GTK_TOGGLE_BUTTON(dlg.writable)->active;
			}
			else
				return;
		}
		
		while (selection)
		{
			int index = (int) selection->data;
			
			m_pFileView->GetFileName(index, strFile);

			if (!GetFile(strProject, strFile, bWritable))
			{
				return;
			}

			if (m_modState != IDNOTOALL && m_modState != IDYESTOALL)
				m_modState = IDYES;

			if (m_lockState != IDNOTOALL && m_lockState != IDYESTOALL)
				m_lockState = IDYES;

			if (m_writeState != IDNOTOALL && m_writeState != IDYESTOALL)
				m_writeState = IDYES;
			
			selection = selection->next;
		}
	}
}

void CVermFrameWnd::CreateProjects(String& strProject, String& strDir,
								   String& strComment, int recurse, short bLockFiles)
{
	DIR* dir = opendir(strDir.chars());

	if (!dir)
		return;

	struct dirent* dent;

	while ((dent = readdir(dir)))
	{
		if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
			continue;

		struct stat sb;
		memset(&sb, 0, sizeof(struct stat));

		String strNewFile = strDir + "/";
		strNewFile += dent->d_name;
		
		if (stat(strNewFile.chars(), &sb))
			continue;

		if (S_ISDIR(sb.st_mode) && recurse)
		{
			// create a project for the new dir ...
			String strNewProj = strProject;

			strNewProj += "/";
			
			strNewProj += dent->d_name;

			if (!GetTreeItemFromProject(m_pProjectView->GetRootItem(),strNewProj))
			{
				String strCookie = strNewFile;
				strCookie += "|";

				strCookie += strComment;
				
				if (bLockFiles)
					strCookie += "|y";
				else
					strCookie += "|n";
					
				CreateProject(strNewProj, strComment, strCookie);
			}
			
			CreateProjects(strNewProj, strNewFile, strComment, recurse, bLockFiles);
			continue;
		}
	}
}

void CVermFrameWnd::AddAllFiles(String& strProject, String& strDir,
								String& strComment, int recurse, short bLockFiles)
{
	DIR* dir = opendir(strDir.chars());

	if (!dir)
		return;

	struct dirent* dent;

	while ((dent = readdir(dir)))
	{
		if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
			continue;

		struct stat sb;
		memset(&sb, 0, sizeof(struct stat));

		String strNewFile = strDir + "/";
		strNewFile += dent->d_name;
		
		if (stat(strNewFile.chars(), &sb))
			continue;

		if (S_ISDIR(sb.st_mode) && recurse)
		{
			// create a project for the new dir ...
			String strNewProj = strProject;

			strNewProj += "/";
			
			strNewProj += dent->d_name;
			
			if (!GetTreeItemFromProject(m_pProjectView->GetRootItem(),strNewProj))
			{
				String strCookie = strNewFile;
				strCookie += "|";

				strCookie += strComment;
				
				if (bLockFiles)
					strCookie += "|y";
				else
					strCookie += "|n";
					

				CreateProject(strNewProj, strComment, strCookie);
			}
			
			AddAllFiles(strNewProj, strNewFile, strComment, recurse, bLockFiles);
			continue;
		}

		FILE* f = fopen(strNewFile.chars(), "r");

		if (!f)
		{
			String s("Unable to open file: ");
			s += strNewFile;
			
			AddMessage(s);
			continue;
		}

        CVermProtocolMessage* pMessage = new CVermProtocolMessage();

        // read the entire file into the message body
		fseek(f, 0, SEEK_END);
        pMessage->m_bodySize = ftell(f); 
		fseek(f, 0, SEEK_SET);

        pMessage->m_body = new char[pMessage->m_bodySize];

		fread(pMessage->m_body, sizeof(char), pMessage->m_bodySize, f);

        fclose(f);

        pMessage->m_number = PM_ADD_FILE;

        pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

        pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, dent->d_name));

        char szContentLength[64];

        if (Vermie::m_bCanUseCompression && Vermie::m_bUseCompression)
        {
                sprintf(szContentLength, "%i", pMessage->m_bodySize);

                pMessage->AddHeader(new CVermProtocolHeader(HEADER_ORIGINAL_SIZE, szContentLength));

                CompressMessageBody(pMessage);
        }

        sprintf(szContentLength, "%i", pMessage->m_bodySize);

        pMessage->AddHeader(new CVermProtocolHeader(HEADER_CONTENT_LENGTH, szContentLength));

        if (!strComment.empty())
                pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strComment));

		// If the user wants use to lock the file, lock it when the server responds that the file was added
		if (bLockFiles)
		{
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, CHECKOUT_FILES));
		}
		
        if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(TRUE);
		else
		{
			delete pMessage;
			return;
		}
		
	}
}

void CVermFrameWnd::CreateProject(String& strProject, String& strComment, String& strCookie)
{
	CVermProtocolMessage* pMessage = new CVermProtocolMessage();
	
	pMessage->m_number = PM_CREATE_PROJECT;
	
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	
	if (!strCookie.empty())
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, strCookie));
	
	if (!strComment.empty())
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strComment));
	
	String strMessage = "Creating project ";
	strMessage += strProject;

	AddMessage(strMessage);
	
	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(TRUE);
	else
	{
		delete pMessage;
		return;
	}
}

void CVermFrameWnd::OnFileAddFiles(void)
{
	// get the project
	String strProject;

	m_pProjectView->GetPathSelectedObject(strProject);

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);

	// raise the add file dialog
	String workingDir = GetWorkingDirectory(hItem);
	
	CAddFilesDlg dlg((CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem)));
	
	dlg.SetOpenDirectory(workingDir);
	
	if (dlg.DoModal() == IDOK)
	{
		short bCheckOutFiles = GTK_TOGGLE_BUTTON(dlg.keepCheckedOut)->active;
		
		if (!Vermie::m_bDontFetchFileOnCheckout && bCheckOutFiles)
		{
			String strWorkingFolder = GetWorkingDirectory(hItem);
			
			if (strWorkingFolder == "")
			{
				String s("You do not have a working folder set - please set one");
				AfxMessageBox(s);
				
				if (SetWorkingFolderForProject(strProject))
					return;
			}
		}
		
		GList* children = gtk_container_children(GTK_CONTAINER(dlg.filesToAdd));
		
		CCommentDlg comdlg("Add Files", children == NULL);

		String strCom = "";
		
		if (comdlg.DoModal() == IDOK)
		{
			strCom = gtk_editable_get_chars(GTK_EDITABLE(comdlg.comment), 0,
											-1);
		}
		else
		{
			return;
		}
		
		if (!children)
		{
			String strProject;
			String strDir = dlg.GetDirectory(dlg.m_pNodeSelected);
			GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);

			const char* dir = strDir.chars();
			char* p = strrchr(dir, '/');

			if (p == dir)
			{
				String strError = "You cannot add files from the root directory";
				AfxMessageBox(strError, MB_OK);
				return;
			}

			String strNewProject = strProject + p;

			int recurse = GTK_TOGGLE_BUTTON(comdlg.recurse)->active;
			
			String strCookie = dir;
			strCookie += "|";

			strCookie += strCom;
			
			if (bCheckOutFiles)
				strCookie += "|y";
			else
				strCookie += "|n";
					
			CreateProject(strNewProject, strCom, strCookie);
			
		    CreateProjects(strNewProject, strDir, strCom, recurse, bCheckOutFiles);
			
			return;
		}
		
		while (children)
		{
			String strFile = dlg.GetDirectory(dlg.m_pNodeSelected);

			strFile += "/";

			strFile += GetItemText((GtkWidget*)children->data);
			
			int file = open(strFile.chars(), O_RDONLY);
			
			if (file < 0)
			{
				String s("Unable to open file:");
				s += strFile;				
				AddMessage(s);
				break;
			}

			// create the message

			CVermProtocolMessage* pMessage = new CVermProtocolMessage();

			// read the entire file into the message body

			pMessage->m_bodySize = lseek(file, 0, SEEK_END);

			lseek(file, 0, SEEK_SET);
			
			pMessage->m_body = new char[pMessage->m_bodySize];

			read(file, pMessage->m_body, pMessage->m_bodySize);

			close(file);

			// send the message

			pMessage->m_number = PM_ADD_FILE;

			if (!strCom.empty())
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG,
															strCom));
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));

			String sFile = GetItemText((GtkWidget*)children->data);

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, sFile));

			char szContentLength[64];

			if (Vermie::m_bCanUseCompression && Vermie::m_bUseCompression)
			{
				sprintf(szContentLength, "%i", pMessage->m_bodySize);

				pMessage->AddHeader(new CVermProtocolHeader(HEADER_ORIGINAL_SIZE, szContentLength));

				CompressMessageBody(pMessage);
			}

			sprintf(szContentLength, "%i", pMessage->m_bodySize);

			pMessage->AddHeader(new CVermProtocolHeader(HEADER_CONTENT_LENGTH, szContentLength));

			// If the user wants use to lock the file, lock it when the server responds that the file was added
			if (bCheckOutFiles)
			{
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, CHECKOUT_FILES));
			}

			if (!m_pSocket->SendMessage(pMessage))
				SetWaitState(1);
			else
			{
				delete pMessage;
				return;
			}

			children = g_list_remove_link(children, children);
		}
	}
	
}

void CVermFrameWnd::OnFileCreateProject(void)
{
	// get the current project
	
	String strProject;
	
	m_pProjectView->GetPathSelectedObject(strProject);
	
	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);
	
	// raise the create project dialog
	
	CCreateProjectDlg dlg("Create Project", MB_OK | MB_CANCEL);
	
	if (dlg.DoModal() == IDCANCEL)
		return;
	
	// does the project already exist?
	String m_strProject = gtk_entry_get_text(GTK_ENTRY(dlg.project));
	
	if (GTK_TREE_ITEM(hItem)->subtree)
	{
		GList* children = gtk_container_children(GTK_CONTAINER(GTK_TREE_ITEM(hItem)->subtree));
		
		while (children)
		{
			if (m_strProject == GetItemText(GTK_WIDGET(children->data)))
			{
				String msg("Project \"");
				msg += m_strProject + "\" alread exists.";
				AfxMessageBox(msg);
				return;
			}

			children = g_list_remove_link(children, children);
		}
	}
	else
	{
		GtkWidget* subtree = gtk_tree_new();
		gtk_tree_item_set_subtree(GTK_TREE_ITEM(hItem), subtree);
	}

	// create and send the message
	
	CVermProtocolMessage* pMessage = new CVermProtocolMessage();
	
	pMessage->m_number = PM_CREATE_PROJECT;
	
	String strNewProject = strProject + "/" + m_strProject;
	
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strNewProject));
	
	String strComment = gtk_editable_get_chars(GTK_EDITABLE(dlg.comment), 0, -1);
	
	if (!strComment.empty())
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strComment.chars()));
	
	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}
}

#ifndef BUILD_STANDARD
static void file_cancel_sel(GtkWidget *, GtkFileSelection *f)
{
	gtk_widget_destroy(GTK_WIDGET(f));
}
#endif // !BUILD_STANDARD

void CVermFrameWnd::OnFileSetWorkingFolder(void)
{
	SetWorkingFolder();
}

int CVermFrameWnd::SetWorkingFolder(void)
{
	if (!m_pProjectView->m_hCurSelItem)
		return -1;

	String strProj;

	m_pProjectView->GetPathSelectedObject(strProj);
	
	return SetWorkingFolderForProject(strProj);
}

int CVermFrameWnd::SetWorkingFolderForProject(String& strProj)
{
	CDirectoryDlg dlg;

	if (dlg.DoModal() == IDOK)
	{
		String dir = gtk_entry_get_text(GTK_ENTRY(dlg.directory));

		return FinishWorkingFolder(strProj, dir);
	}

	return -1;
}

void CVermFrameWnd::SetWorkingDirectory(String& strWorkingDir)
{
	String strProj;
	
	m_pProjectView->GetPathSelectedObject(strProj);
	
	FinishWorkingFolder(strProj, strWorkingDir);
}

int CVermFrameWnd::FinishWorkingFolder(String& strProj, String& dir)
{
	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));
	
	if (!dir.empty())
	{
		if (stat(dir.chars(), &sb))
		{
			String s("Directory doesn't exist - create it?");
			if (AfxMessageBox(s, MB_OK | MB_CANCEL) == IDOK)
			{
				if (!CreateDirectory(dir))
				{
					String s("Unable to create directory: ");
					s += dir;
					AfxMessageBox(s);
					return -1;
				}
				m_pProjectView->SetWorkingDirectory(dir);
				m_pFileView->SetWorkingDirPane(dir);
				return 0;
			}
			else
				return -1;
		}
		
		if (!S_ISDIR(sb.st_mode))
		{
			String s = dir + " is not a directory.";
			AfxMessageBox(s);
			return -1;
		}
	}
	
	String strCurProj;

	m_pProjectView->GetPathSelectedObject(strCurProj);
	
	if (strCurProj == strProj)
	{
		m_pProjectView->SetWorkingDirectory(dir);
		m_pFileView->SetWorkingDirPane(dir);
	}

	return 0;
}

void CVermFrameWnd::OnFileConnectToServer(void)
{
	CServerDlg dlg("Connect To Server", MB_OK | MB_CANCEL);

	if (dlg.DoModal() == IDOK)
	{
		String server = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dlg.server)->entry));

		if (server.empty())
		{
			String s("Please enter a server name");
			AfxMessageBox(s);
			return;
		}

		String port = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dlg.port)->entry));

		if (port.empty())
		{
			String s("Please enter a port number");
			AfxMessageBox(s);
			return;
		}

		if (Vermie::m_strServerList.empty())
			Vermie::m_strServerList = server;
		else
		{
			if (Vermie::m_strServerList.index(server.chars()) < 0)
			{
				int nServers = 1;
				int len = Vermie::m_strServerList.length();

				// check to make sure our list isn't full
				for (int i = 0; i < len; i++)
				{
					if (Vermie::m_strServerList[i] == ';')
						++nServers;
				}

				if (nServers >= Vermie::m_nServersToKeep)
				{
					int pos = Vermie::m_strServerList.ReverseFind(';');
					Vermie::m_strServerList = Vermie::m_strServerList.before(pos);
				}
				
				String str = server + ";";
				str += Vermie::m_strServerList;
			
				Vermie::m_strServerList = str;
			}
		}		

		if (Vermie::m_strPortList.empty())
			Vermie::m_strPortList = port;
		else
		{
			if (Vermie::m_strPortList.index(port.chars()) < 0)
			{
				int nPorts = 1;
				int len = Vermie::m_strPortList.length();

				// check to make sure our list isn't full
				for (int i = 0; i < len; i++)
				{
					if (Vermie::m_strPortList[i] == ';')
						++nPorts;
				}

				if (nPorts >= Vermie::m_nServersToKeep)
				{
					int pos = Vermie::m_strPortList.ReverseFind(';');
					Vermie::m_strPortList = Vermie::m_strPortList.before(pos);
				}
				
				String str = port + ";";
				str += Vermie::m_strPortList;
			
				Vermie::m_strPortList = str;
			}
		}
		
		String strServerPrefs = Vermie::m_strPrefsDir;
		strServerPrefs += "/";
		strServerPrefs += SERVER_DIRECTORY_NAME;
		strServerPrefs += "/";
		strServerPrefs += server;
		strServerPrefs += "/prefs";

		// save off the current tree
		if (IsSafeToSave())
		{
			String curProject;

			m_pProjectView->GetPathSelectedObject(curProject);
		
			SaveTreeToFile(m_pProjectView->m_pTree, m_server, m_database,
						   curProject);
		}
		
		ParseServerPrefs(strServerPrefs);
		
		ConnectToServer(server, atoi(port.chars()));
	}
	else
	{
		// no server connection - turn off the toolbar
		if (!m_pSocket)
			ResetToolbar(FALSE);
	}
}

void CVermFrameWnd::OnFileExit(void)
{
	// if we have no username, this user hasn't yet logged on - don't
	// overwrite the current vrm file

	if (IsSafeToSave())
	{
		String curProject;

		m_pProjectView->GetPathSelectedObject(curProject);
		
		SaveTreeToFile(m_pProjectView->m_pTree, m_server, m_database,
					   curProject);
	}

	// persist the current size and position of the window

	SavePrefs();
	
	CleanTree(m_pProjectView->m_pTree);

	// don't catch the destory when we cause it ...
	gtk_signal_disconnect(GTK_OBJECT(m_pWidget), m_destroySig);
	
	gtk_widget_destroy(m_pWidget);
}

int CVermFrameWnd::IsSafeToSave(void)
{
	if (m_server.empty() && m_username.empty())
		return 0;

	if (m_pProjectView->IsEmpty())
		return 0;

	return 1;
}

void CVermFrameWnd::MessageReceived(CVermProtocolMessage* pMessageReceived,
									CVermProtocolMessage* pMessageSent,
									int bLastMessage)
{
	if (!pMessageSent)
	{
		// The only unsolicited message we should receive is a CHALLENGE
		
		if ((pMessageReceived->m_number != PM_CHALLENGE) &&
			(pMessageReceived->m_number != PM_SECURE_CHALLENGE) &&
			(pMessageReceived->m_number != PM_SERVER_EXPIRED))
		{
			String s("Response for no message sent!");
			AfxMessageBox(s);
			return;
		}
	}

	switch (pMessageReceived->m_number)
	{
	case PM_SEARCH_RESULTS:
		if (bLastMessage)
			SetWaitState(0);

		m_pSearchResultsDlg = new CSearchResultsDlg("Search Results", pMessageReceived, this);
		gtk_widget_show(m_pSearchResultsDlg->m_pDialog);
		
		break;

	case PM_PROJECT_HISTORY:
		m_pHistoryDlg = new CHistoryDlg(this, pMessageReceived, 0);
		gtk_widget_show(m_pHistoryDlg->m_pDialog);
		if (bLastMessage)
			SetWaitState(0);
		break;
		
	case PM_FILE_ADDED:
		{
			const char* pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
			const char* pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
			const char* pszCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);
			const char* pszContentLength = pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE);

			if (!pszContentLength)
				pszContentLength = pMessageReceived->GetHeaderValue(HEADER_CONTENT_LENGTH);
			
			// get updated file status

			// TODO what about files added from non-working directories?

			String strProject = pszProject;

			if (strProject.index("$//") == 0)
			{
				String strProj = "$/";
				strProj +=  strProject.after(2);
				strProject = strProj;
			}
			
			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
													  strProject);

			int bFileExists = 0;
			
			String strWorkingDir = GetWorkingDirectory(hItem);
			String strFile = strWorkingDir;
			String strFileName = strProject;
			strFileName += "/";
			strFileName += pszFile;

			if (!strWorkingDir.empty())
			{
				if (CreateDirectory(strWorkingDir))
				{
					MakeFileWriteable(strFileName);
					
					int f = open(strFileName.chars(), O_CREAT | O_WRONLY);

					if (f > -1)
					{
						write(f, pMessageReceived->m_body, pMessageReceived->m_bodySize);
						close(f);
					}
				}
			}
			
			struct stat sb;
			memset(&sb, 0, sizeof(struct stat));
				
			if (strFile != "")
			{
				strFile += "/";
				strFile += pszFile;
				
				bFileExists = (stat(strFile.chars(), &sb) == 0);
			}
			
			// make the file read-only

			if (bFileExists && Vermie::m_bMakeFilesReadOnly)
				if (MakeFileReadOnly(strFile) < 0)
				{
					String str("Unable to set file permissions: ");
					str += strFile;

					AddMessage(strFile);
				}

			// create a new tree leaf

			CVermTreeLeaf* pLeaf = new CVermTreeLeaf();

			pLeaf->m_name = pszFile;

			const char* pszRemoteDate = pMessageReceived->GetHeaderValue(HEADER_DATE);

			if (pszRemoteDate)
			{
				pLeaf->m_nRemoteVersionNumber = 1;

				char pszDate[64];
				strcpy(pszDate, pszRemoteDate);
				pLeaf->m_pRemoteVersionDate = ParseServerDate(pszDate);
			}
			else
			{
				pLeaf->m_nRemoteVersionNumber = -1;
				pLeaf->m_pRemoteVersionDate = 0;
			}

			if (bFileExists)
			{
				pLeaf->m_nLocalVersionNumber = 1;
				pLeaf->m_pLocalVersionDate = sb.st_mtime;
			}
			else
			{
				pLeaf->m_nLocalVersionNumber = -1;
				pLeaf->m_pLocalVersionDate = -1;
			}

			const char* pszLockedBy = pMessageReceived->GetHeaderValue(HEADER_LOCKED_BY);

			if (pszLockedBy)
				pLeaf->m_userLocked = pszLockedBy;
			else
				pLeaf->m_userLocked = "";
			
			const char* pszLockedProject = pMessageReceived->GetHeaderValue(HEADER_LOCKED_PROJECT);

			if (pszLockedProject)
				pLeaf->m_projectLocked = pszLockedProject;
			else
				pLeaf->m_projectLocked = "";

			// add the leaf to the node

			CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

			if (!pNode)
				return;
			
			pNode->AddLeaf(pLeaf);

			UpdateFileView(pNode, strWorkingDir, pLeaf, 0);

			String s = "Added file: ";
			s += strFileName;

			AddMessage(s);
			
			if (bLastMessage)
				SetWaitState(0);

			short bDontOverwrite = pszCookie && !strcmp(pszCookie, CHECKOUT_FILES_DONT_OVERWRITE);

			// User wants us to lock the file
			if (pszCookie && (!strcmp(pszCookie, CHECKOUT_FILES) || bDontOverwrite))
			{
				String strSSFile = pszFile;

				// If the file doesn't exist, and the user wants us to lock the file, fetch it then lock it
				if (!bDontOverwrite && !bFileExists)
				{
					CVermProtocolMessage* pFileMessage = new CVermProtocolMessage();

					pFileMessage->m_number = PM_GET_FILE;

					pFileMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
					pFileMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strSSFile));
					pFileMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

					if (!m_pSocket->SendMessage(pFileMessage))
						SetWaitState(1);
					else
					{
						delete pFileMessage;
						return;
					}
				}

				CVermProtocolMessage* pLockMessage = new CVermProtocolMessage();

				pLockMessage->m_number = PM_LOCK_FILE;

				pLockMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
				pLockMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strSSFile));

				if (bDontOverwrite)
				pLockMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, CHECKOUT_FILES_DONT_OVERWRITE));
				
				if (!m_pSocket->SendMessage(pLockMessage))
					SetWaitState(1);
				else
				{
					delete pLockMessage;
					return;
				}
			}
			break;
		}
	case PM_AUTHENTICATION_FAILED:
		{
			String s("Invalid password - login failed");
			AfxMessageBox(s, MB_OK);

			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_UNAUTHORIZED_USER:
		{
			String s("You are not authorized to use this server");
			AfxMessageBox(s, MB_OK);

			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_database.del(0, m_database.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_SERVER_EXPIRED:
		{
			String s("The server has expired - connection refused");
			AfxMessageBox(s, MB_OK);
			
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

#ifndef BUILD_STANDARD
	case PM_KEY_NOT_FOUND:
		{
			String s("The server was unable to locate an encryption key");
			AfxMessageBox(s, MB_OK);
			
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_KEY_EXPIRED:
		{
			String s("Your encryption key has expired");
			AfxMessageBox(s, MB_OK);
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_INVALID_KEY:
		{
			String s("Invalid encryption key");
			AfxMessageBox(s, MB_OK);

			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}
	case PM_MISSING_CRYPTO:
		{
			String s("The server was unable to locate the encryption component");
			AfxMessageBox(s, MB_OK);
			
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

#endif // !BUILD_STANDARD
		
	case PM_BAD_SRCSAFE_INI:
		{
			String s("The server was unable to load the SourceSafe initialization file");
			AfxMessageBox(s, MB_OK);
			
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_FAILED_DATABASE_OPEN:
		{
			String s("The server was unable to connect to the SourceSafe database.\n\nPlease verify that SourceSafe was properly installed on the server machine.");
			AfxMessageBox(s, MB_OK);
			
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_MISSING_SRCSAFE_AUTO:
		{
			String s("The server was unable to locate the SourceSafe automation component");
			AfxMessageBox(s, MB_OK);
			
			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}


	case PM_INVALD_PROTOCOL_VERSION:
		{
			String s("The client you are using has an invalid protocol version for this server");
			AfxMessageBox(s, MB_OK);

			SetWaitState(0);

			m_username.del(0, m_username.length());
			m_database.del(0, m_database.length());
			m_databaseAlias.del(0, m_databaseAlias.length());
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();

			m_bConnected = 0;

			ResetToolbar(FALSE);
			
			break;
		}

	case PM_CHECKIN_MERGE_FAILED:
		{
			const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
			const char* pszFile = pMessageSent->GetHeaderValue(HEADER_FILE);
			String s("Checkin failed: ");
			s += pszProject;
			s += "/";
			s += pszFile;
			
			AddMessage(s);

			s = "Automatic merge failed.  Conflicts must be resolved before checkin can be completed";
			AfxMessageBox(s, MB_OK);

			// if we have a merge program set, launch it with the file sent back from the server
			if (Vermie::m_strMergeProgram != "" && pMessageReceived->m_body)
			{
				// dump the file into a temp file
				String strTmpDir = tempnam(NULL, NULL);

				if (!CreateDirectory(strTmpDir))
				{
					String s("Unable to create temporary directory for file merge");
					AfxMessageBox(s);
					SetWaitState(0);
					return;
				}

				String strFile = strTmpDir + "/";
				strFile += pszFile;

				int file = open(strFile.chars(), O_RDWR | O_CREAT, 0664);

				if (file < 0)
				{
					String s("Unable to create temporary file for file merge");
					AfxMessageBox(s);
					SetWaitState(0);
					return;
				}

				if (write(file, pMessageReceived->m_body, pMessageReceived->m_bodySize) != pMessageReceived->m_bodySize)
				{
					String s("Unable to create temporary file for file merge");
					AfxMessageBox(s);
					SetWaitState(0);
					return;
				}

				close(file);

				String workingDir = GetWorkingDirectory(m_pProjectView->GetRootItem(), pszProject);

				// sanity check - how could this be empty?
				if (workingDir.empty())
				{
					SetWaitState(0);
					return;
				}

				String strLocalFile = workingDir + "/";
				strLocalFile += pszFile;

				char buf[2048];
				sprintf(buf, Vermie::m_strMergeProgram.chars(), strLocalFile.chars(), strFile.chars());
						
				system(buf);

				String s("Re-checkin merged changes?");

				if (AfxMessageBox(s, MB_OK | MB_CANCEL) == IDOK)
				{
					int file = open(strLocalFile.chars(), O_RDONLY);
					
					CVermProtocolMessage* pMessage = new CVermProtocolMessage();

					// read the entire file into the message body

					pMessage->m_bodySize = lseek(file, 0, SEEK_END);
					lseek(file, 0, SEEK_SET);

					pMessage->m_body = new char[pMessage->m_bodySize];

					read(file, pMessage->m_body, pMessage->m_bodySize);

					close(file);

					// send the message

					pMessage->m_number = PM_CHECKIN;

					pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pszFile));
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

					char szContentLength[64];

					if (Vermie::m_bUseCompression && Vermie::m_bCanUseCompression)
					{
						sprintf(szContentLength, "%i", pMessage->m_bodySize);

						pMessage->AddHeader(new CVermProtocolHeader(HEADER_ORIGINAL_SIZE, szContentLength));
						CompressMessageBody(pMessage);
					}

					sprintf(szContentLength, "%i", pMessage->m_bodySize);

					pMessage->AddHeader(new CVermProtocolHeader(HEADER_CONTENT_LENGTH, szContentLength));

					pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, pMessageSent->GetHeaderValue(HEADER_LOG)));

					if (Vermie::m_bRemoveLocalCopy)
						pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, REMOVE_LOCAL_COPY));

					if (Vermie::m_bAlwaysKeepFilesCheckedOut)
						pMessage->AddHeader(new CVermProtocolHeader(HEADER_KEEP_CHECKED_OUT, "yes"));

					if (!m_pSocket->SendMessage(pMessage))
						SetWaitState(1);
					else
					{
						delete pMessage;
						return;
					}
					// check-in the file
				}
			}
			
			SetWaitState(0);
			break;
		}

	case PM_OPERATION_FAILED:
		{
			if (m_pFileDiff && m_pFileDiff->bProjectDiff)
			{
				const char* pszFile = pMessageSent->GetHeaderValue(HEADER_FILE);
				const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
				
				if (m_strLastFileForProjectDiff == pszFile && m_strLastProjectForProjectDiff == pszProject)
					ProjectDiffFinished();
			}
				
			const char* pszContentLength = pMessageReceived->GetHeaderValue(HEADER_CONTENT_LENGTH);
			
			if (pMessageSent->m_number == PM_CHECKOUT_PROJECT)
			{
				String s("Unable to lock file: ");

				const char* pszStr = pMessageReceived->GetHeaderValue(HEADER_PROJECT);

				if (pszStr)
				{
					s += pszStr;
					s += "/";
				}
				pszStr = pMessageReceived->GetHeaderValue(HEADER_FILE);
				if (pszStr)
					s += pszStr;
				
				AddMessage(s);
			}
			else if (pszContentLength)
			{
				String str(pMessageReceived->m_body,
						   atoi(pszContentLength));

				AfxMessageBox(str);
			}
			else
			{
				String s("Operation failed");
				AddMessage(s);
			}
			
			SetWaitState(0);
			break;
		}

	case PM_MISSING_HEADER:
		{
			String s("Error in message format - missing header");
			AddMessage(s);
			SetWaitState(0);
			break;
		}

	case PM_UNRECOGNIZED_MESSAGE:
		{
			// HACK HACK HACK
			// The file size for crypto has been bumped up from 8 bits to 16 bits
			// The 1.0 server can't handle this and throws this error back on our PM_LOGIN
			// message.  Check for it here, and inform the user that they are using an invalid
			// protocol to connect to the server instead of this bogus message

			
			if (pMessageSent->m_number == PM_LOGIN)
			{
				String s("The client you are using has an invalid protocol version for this server");
				AfxMessageBox(s, MB_OK);
			}
			else
			{
				String s("The server received an unrecognized message");
				AddMessage(s);
			}

			SetWaitState(0);
			break;
		}

	case PM_NO_SUCH_FILE_OR_PROJECT:
		{
			String nsfop("No such file or project: ");
			const char *pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
			const char *pszFile = pMessageSent->GetHeaderValue(HEADER_FILE);

			nsfop += pszProject;

			if (pszFile)
			{
				nsfop += "/";
				nsfop += pszFile;
			}

			m_pFileView->Reset();
			
			AddMessage(nsfop);
			SetWaitState(0);

			break;
		}

	case PM_FILE_OR_PROJECT_NOT_LOCKED:
		{
			String fnl("File not locked: ");
			fnl += pMessageSent->GetHeaderValue(HEADER_PROJECT);
			fnl += "/";
			fnl += pMessageSent->GetHeaderValue(HEADER_FILE);

			AddMessage(fnl);
			SetWaitState(0);

			break;
		}

	case PM_INVALID_LABEL:
		{
			String il("Invalid Label: ");

			il += pMessageSent->GetHeaderValue(HEADER_LABEL);

			AddMessage(il);

			SetWaitState(0);

			break;
		}

	case PM_INVALID_VERSION:
		{
			String iv("Invalid file version: ");
			iv += pMessageSent->GetHeaderValue(HEADER_VERSION);

			AddMessage(iv);
			SetWaitState(0);

			break;
		}

	case PM_ALREADY_EXISTS:
		{
			String ae("Unable to add File or Project: ");

			ae += pMessageSent->GetHeaderValue(HEADER_PROJECT);

			ae += "/";

			if (pMessageSent->GetHeaderValue(HEADER_FILE))
				ae += pMessageSent->GetHeaderValue(HEADER_FILE);

			AddMessage(ae);

			SetWaitState(0);

			break;
		}

	case PM_ALREADY_LOCKED:
		{
			String al("Unable to lock file: ");

			al += pMessageSent->GetHeaderValue(HEADER_PROJECT);
			al += "/";
			al += pMessageSent->GetHeaderValue(HEADER_FILE);

			AddMessage(al);

			SetWaitState(0);

			break;
		}

	case PM_FILE_INFO:
		{
			UpdateFileInfo(pMessageReceived);
			if (bLastMessage)
				SetWaitState(FALSE);
			break;
		}

	case PM_FILE_HISTORY:
		{
			// TODO perhaps the server should return the project with a file history?

			const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
			const char* pszFile = pMessageSent->GetHeaderValue(HEADER_FILE);
			
			pMessageReceived->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));

			// launch the history dialog as a modeless dialog

			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), pszProject);

			CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

			String strFile = pszFile;
			
			CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

			int nPinnedVersion = 0;
			
			if (pLeaf)
			{
				nPinnedVersion = pLeaf->m_nPinnedVersion;

				String strProject = pszProject;
				
				if (!fcompare(pLeaf->m_projectLocked, strProject))
					nPinnedVersion = 0;	// this file cannot be pinned
			}

			m_pHistoryDlg = new CHistoryDlg(this, pMessageReceived, nPinnedVersion, Vermie::m_bPinAvailable);

			gtk_widget_show(m_pHistoryDlg->m_pDialog);

			SetWaitState(0);

			break;
		}
		
	case PM_FILE_UNLOCKED:
		{
			const char* pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
			const char* pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);

			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
													  pszProject);

			// make the file read-only again

			String strFile = GetWorkingDirectory(hItem) + "/";
			strFile += pszFile;

			if (Vermie::m_bMakeFilesReadOnly && MakeFileReadOnly(strFile) < 0)
			{
				String str("Unable to set file permissions: ");
				str += strFile;
				
				AddMessage(str);
			}

			// update the leaf info

			CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

			String strPSZFile = pszFile;
			
			CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strPSZFile);

			if (pLeaf)
			{
				const char* pszLockedBy = pMessageReceived->GetHeaderValue(HEADER_LOCKED_BY);

				if (pszLockedBy)
					pLeaf->m_userLocked = pszLockedBy;
				else
					pLeaf->m_userLocked = "";
				
				const char* pszLockedProject = pMessageReceived->GetHeaderValue(HEADER_LOCKED_PROJECT);

				if (pszLockedProject)
					pLeaf->m_projectLocked = pszLockedProject;
				else
					pLeaf->m_projectLocked = "";

				// make the file read-only again
				
				String strProject = pszProject;
				
				int nLockStatus = IsFileLocked(pLeaf, strProject, m_username);
				
				if (nLockStatus < 2 && Vermie::m_bMakeFilesReadOnly)
					if (MakeFileReadOnly(strFile) < 0)
					{
						String str("Unable to set file permissions: ");
						str += strFile;
						
						AddMessage(strFile);
					}
			}

			String strWorkingDir = GetWorkingDirectory(hItem); 
			UpdateFileView(pNode, strWorkingDir, pLeaf);

			if (bLastMessage)
				SetWaitState(0);

			String strMessage;
			strMessage = strMessage + "File ";
			strMessage += pszProject;
			strMessage += "/";
			strMessage += pszFile;
			strMessage += " unlocked";
			
			AddMessage(strMessage);

			break;
		}

	case PM_CHECKIN_COMPLETE:
		{
			const char* pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
			const char* pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
			const char* pszContentLen = pMessageReceived->GetHeaderValue(HEADER_CONTENT_LENGTH);

			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
													  pszProject);

			String strFileName = pszProject;
			strFileName += "/";
			strFileName += pszFile;
			
			// get any client cookies

			const char *pCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);

			int bRemoveLocalCopy = 0;

			if (pCookie && !strcmp(pCookie, REMOVE_LOCAL_COPY))
				bRemoveLocalCopy = 1;

			String strFile = GetWorkingDirectory(hItem);
			strFile += "/";
			strFile += pszFile;

			// get updated file status
			struct stat sb;
			memset(&sb, 0, sizeof(struct stat));

			int bFileExists = (stat(strFile.chars(), & sb) == 0);

			if (pszContentLen)
			{
				if (bFileExists)
					if (MakeFileWriteable(strFile) < 0)
					{
						String str("Unable to set file permissions: ");
						str += strFile;
						
						AddMessage(strFile);
					}

				if (pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE))
					pszContentLen = pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE);

				int newFile = open(strFile.chars(), O_CREAT | O_WRONLY, 0600);

				if (newFile >= 0)
				{
					write(newFile, pMessageReceived->m_body, atoi(pszContentLen));

					close(newFile);
					bFileExists = 1;
				}
			}


			if (bFileExists)
			{
				// remove the local copy if needed

				if (bRemoveLocalCopy)
				{
					if (unlink(strFile.chars()))
					{
						String s("Unable to remove file: ");
						s += strFile;
						AfxMessageBox(s);

						bFileExists = 0;
					}
				}
			}

			// update the leaf info

			CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

			String strPSZFile = pszFile;
			CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strPSZFile);

			if (pLeaf)
			{

				const char* pszLockedBy = pMessageReceived->GetHeaderValue(HEADER_LOCKED_BY);
				const char* pszLockedProject = pMessageReceived->GetHeaderValue(HEADER_LOCKED_PROJECT);

				if (pszLockedBy)
					pLeaf->m_userLocked = pszLockedBy;
				else
					pLeaf->m_userLocked = "";

				if (pszLockedProject)
					pLeaf->m_projectLocked = pszLockedProject;
				else
					pLeaf->m_projectLocked = "";
				
				const char* pszRemoteVer = pMessageReceived->GetHeaderValue(HEADER_VERSION);

				int nRemoteVer = atoi(pszRemoteVer);

				// The version we just checked in isn't 1 more than the version
				// we have on disk - someone checked in a version behind our back
				// fetch the latest copy
				if ((pLeaf->m_nLocalVersionNumber < nRemoteVer - 1) && !bRemoveLocalCopy)
				{
					CVermProtocolMessage* pMessage = new CVermProtocolMessage();

					pMessage->m_number = PM_GET_FILE;
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pszFile));

					if (!m_pSocket->SendMessage(pMessage))
						SetWaitState(1);
					else
					{
						delete pMessage;
						return;
					}
				}

				pLeaf->m_nRemoteVersionNumber = nRemoteVer;

				const char* pszRemoteDate = pMessageReceived->GetHeaderValue(HEADER_DATE);

				if (pszRemoteDate)
				{
					char pszDate[64];
					strcpy(pszDate, pszRemoteDate);
					pLeaf->m_pRemoteVersionDate = ParseServerDate(pszDate);
				}

				if (bFileExists)
				{
					if (pszRemoteVer)
						pLeaf->m_nLocalVersionNumber = nRemoteVer;

					pLeaf->m_pLocalVersionDate = sb.st_mtime;
				}
				else
				{
					pLeaf->m_nLocalVersionNumber = -1;

					pLeaf->m_pLocalVersionDate = -1;
				}
				// make the file read-only again
				String strProject = pszProject;
				int nLockStatus = IsFileLocked(pLeaf, strProject, m_username);

				if (nLockStatus < 2 && bFileExists && Vermie::m_bMakeFilesReadOnly)
					if (MakeFileReadOnly(strFile) < 0)
					{
						String str("Unable to set file permissions: ");
						str += strFile;
						
						AddMessage(strFile);
					}
			}

			String strWorkingDir = GetWorkingDirectory(hItem); 
			UpdateFileView(pNode, strWorkingDir, pLeaf);

			String s = "Checkin complete: ";
			s += strFileName;

			AddMessage(s);
			// CVermSocket is now handling this message
			//AddMessage(CString("Completed checkin of ") + pszFile);

			if (bLastMessage)
				SetWaitState(0);

			break;
		}

	case PM_FILE_LOCKED:
		{
			// file is locked:make sure we have the latest version locally ...
			const char* pszRemoteVersion = pMessageReceived->GetHeaderValue(HEADER_VERSION);
			const char* pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
			const char* pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
			const char* pszRemoteDate = pMessageReceived->GetHeaderValue(HEADER_DATE);
			const char* pCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);

			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
													  pszProject);
			String strFile = GetWorkingDirectory(hItem);
			strFile += "/";
			strFile += pszFile;

			struct stat sb;
			memset(&sb, 0, sizeof(struct stat));

			if (stat(strFile.chars(), &sb) >= 0 && MakeFileWriteable(strFile) < 0)
			{
				String str("Unable to set file permissions: ");
				str += strFile;
				
				AddMessage(str);
			}

			CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

			String strPSZFile = pszFile;
			CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strPSZFile);

			pLeaf->m_userLocked = Vermie::m_strUserName;
			pLeaf->m_projectLocked = pszProject;
			pLeaf->m_nRemoteVersionNumber = atoi(pszRemoteVersion);
			pLeaf->m_pRemoteVersionDate = ParseServerDate((char*)pszRemoteDate);

			String strWorkingDir = GetWorkingDirectory(hItem); 
			UpdateFileView(pNode, strWorkingDir, pLeaf);

			if (bLastMessage)
				SetWaitState(0);

			String strMessage;
			String strFileName = pszProject;
			strFileName += "/";
			strFileName += pszFile;
			strMessage = strMessage + "File " + strFileName + " locked";
			AddMessage(strMessage);

			// local copy is older - fetch a new copy
			if ((!pCookie || strcmp(pCookie, CHECKOUT_FILES_DONT_OVERWRITE)) && pLeaf->m_nLocalVersionNumber < pLeaf->m_nRemoteVersionNumber)
			{
				CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();

				pNewMessage->m_number = PM_GET_FILE;

				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pszFile));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

				String s = strFileName + " is out of date - fetching latest copy";
				AddMessage(s);
				
				if (!m_pSocket->SendMessage(pNewMessage))
					SetWaitState(1);
				else
				{
					delete pNewMessage;
					return;
				}
			}

			if (pCookie && !strcmp(pCookie, EDIT_FILE))
				VermEditFile(strFile);

			break;

		}
		
	case PM_FILE:
		{
			if (m_pFileDiff)
			{
				ReceivedDiffFile(pMessageReceived);

				if (m_pFileDiff && m_pFileDiff->bProjectDiff)
				{
					const char* pszFile = pMessageSent->GetHeaderValue(HEADER_FILE);
					const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);

					if (m_strLastFileForProjectDiff == pszFile && m_strLastProjectForProjectDiff == pszProject)
						ProjectDiffFinished();
				}

				SetWaitState(0);
			}
			else if (m_pFileEdit)                                   
			{                                                       
				VermViewFile(pMessageReceived, m_pFileEdit);    
				
                // delete the file diff struct member           
				
				delete m_pFileEdit;                             
				
				m_pFileEdit = NULL;                             

				SetWaitState(0);
			}
			else
			{
				int bMakeWritable = !Vermie::m_bMakeFilesReadOnly;
				const char* pCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);

				if (pMessageSent->m_number == PM_GET_PROJECT)
					pCookie = pMessageSent->GetHeaderValue(HEADER_CLIENT_COOKIE);
			
				if (pCookie)
				{
					if (!strcmp(pCookie, MAKE_WRITABLE) || !strcmp(pCookie, FETCH_FILES_MAKE_WRITABLE))
						bMakeWritable = 1;
					else if (!strcmp(pCookie, MAKE_READONLY) || !strcmp(pCookie, FETCH_FILES))
						bMakeWritable = 0;
				}
				
				m_pProjectView->SetFile(pMessageReceived, pMessageSent, !bLastMessage, m_writeState, bMakeWritable);
				
				if (pMessageSent->m_number == PM_SHARE_FILE || pMessageSent->m_number == PM_SHARE_PROJECT)
				{
					const char* pszFile = pMessageSent->GetHeaderValue(HEADER_FILE);
					const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
					const char* pszShareProject = pMessageSent->GetHeaderValue(HEADER_SHARE_PROJECT);

					// project share didn't have a file header in the outgoing message, but the server will fill
					// it in for us
					if (pMessageSent->m_number == PM_SHARE_PROJECT)
					{
						pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
						pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
					}

					GtkWidget* pTree = m_pProjectView->GetRootItem();
					CVermTreeNode* pNode = GetTreeNodeFromProject(pTree, pszProject);
					CVermTreeNode* pShareNode = GetTreeNodeFromProject(pTree, pszShareProject);

					// if we branched after we shared, the files aren't shared any more
					if (Vermie::m_bUseVSS6Features && !pMessageSent->GetHeaderValue(HEADER_BRANCH_AFTER_SHARE))
					{
						String strFile = pszFile;
						
						CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

						if (pLeaf)
							pLeaf->m_bShared = TRUE;

						pLeaf = pShareNode->GetLeafByName(strFile);

						if (pLeaf)
							pLeaf->m_bShared = TRUE;
					}

					String strShareDir = GetWorkingDirectory(pTree, pszShareProject);

					String strProject = pszProject;
					
					m_pFileView->UpdateFileInfo(pShareNode, strProject, m_username, strShareDir);

					String strMessage = "File ";
					strMessage += pszProject;
					strMessage += "/";
					strMessage += pszFile;
					strMessage += " shared to ";
					strMessage += pszShareProject;

					AddMessage(strMessage);

				}

				if (m_writeState == IDNO)
					m_writeState = IDYES;	// reset

				if (bLastMessage)
					SetWaitState(0);
			}

			break;
		}
	case PM_FILE_LIST:
		{
			if (m_pShareDlg)
				m_pShareDlg->SetFileList(pMessageReceived, pMessageSent);
			
			m_pProjectView->SetFileList(pMessageReceived, pMessageSent);

			const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);

			const char* pszCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);

			String str("Received file list: ");
			str += pszProject;
			
			AddMessage(str);

			if (pszCookie)
			{
				GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), pszProject);
				int bCheckoutFiles = (!strcmp(pszCookie, CHECKOUT_FILES) || !strcmp(pszCookie, CHECKOUT_FILES_DONT_OVERWRITE));
				int bFetchFiles = bCheckoutFiles || !strcmp(pszCookie, FETCH_FILES) || !strcmp(pszCookie, FETCH_FILES_MAKE_WRITABLE) ;
				int bMakeWritable = !strcmp(pszCookie, FETCH_FILES_MAKE_WRITABLE);

				CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

				String strDir = GetWorkingDirectory(hItem); 
				String strProject = pszProject;

				if (bFetchFiles)
				{
					FetchNodeFiles(pNode, strDir, pszProject, bCheckoutFiles, !strcmp(pszCookie, CHECKOUT_FILES_DONT_OVERWRITE), bMakeWritable);
				}
				else if (!strncmp(pszCookie, "Undo Checkout", 13))
				{
					UndoCheckoutForNode(pNode, strDir, strProject, pszCookie);
				}
				else if (!strcmp(pszCookie, CHECKIN_FILES))
				{
					CheckinFilesForNode(pNode, strDir, strProject, m_strComment,
										m_bRemoveLocalCopy, m_bKeepCheckedOut);
				}					
				else if (!strcmp(pszCookie, PROJECT_DIFF))
				{
					GetDiffFilesForProject(pNode, strDir, strProject);
				}
				else if (!strcmp(pszCookie, BRANCH_FILES))
				{
					BranchFilesForNode(strProject);
				}
			}
			
			if (bLastMessage)
			{
				SetWaitState(0);
			}

			break;
		}

	case PM_PROJECT_TREE:
		{
			const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
			const char* pszCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);

			String strCookie(pszCookie);

			if (strCookie == NEW_PROJECT)
			{
				if (bLastMessage)
					SetWaitState(0);

				m_pProjectView->SetProjectTree(pMessageReceived->m_body, pMessageReceived->m_bodySize);
				String s("Received project tree: ");
				s += pszProject;
				AddMessage(s);
				m_pProjectView->SetOpenProject("$/");
			}
			else	// UPDATE_PROJECT
			{
				if (bLastMessage)
					SetWaitState(0);

				int pos = 0;
				int bError;

				GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), pszProject);

				ParseNameFromBuffer(pMessageReceived->m_body, &pos, pMessageReceived->m_bodySize - 1, bError);

				m_pProjectView->UpdateProjectTree(pMessageReceived->m_body, hItem, &pos, pMessageReceived->m_bodySize - 1);

				m_pProjectView->PruneTree(hItem);
				String s("Refreshed project tree: ");
				s += pszProject;
				AddMessage(s);
			}

			break;
		}
#ifndef BUILD_STANDARD
	case PM_SECURE_CHALLENGE:
#endif // !BUILD_STANDARD
	case PM_CHALLENGE:
	{
		SetWaitState(0);
		
		// save the challenge times
		
		Vermie::m_timeStamps.challengeClock = clock();
		Vermie::m_timeStamps.challengeTime = time(NULL);
		

		// make sure we can speak the server's protocol version
		
		const char* pszProtocolVersion = pMessageReceived->GetHeaderValue(HEADER_PROTOCOL_VERSION);
		
		if ((!pszProtocolVersion))
		{
			SetWaitState(0);
			String s("The server you are connecting to has an invalid protocol version for this client");
			AfxMessageBox(s, MB_OK);
			
			m_server.del(0, m_server.length());
			m_port = -1;

			m_pSocket->Close();
			
			m_bConnected = FALSE;
			
			ResetToolbar(FALSE);
			
			return;
		}
		else
		{
			double verNum = atof(pszProtocolVersion);
			
			// If the server is post 1.0, it supports compression
			if (verNum > 1.0)
			{
				Vermie::m_bCanUseCompression = 1;
			}
			else
			{
				Vermie::m_bCanUseCompression = 0;
			}
		}

		const char* pszDaysLeft = pMessageReceived->GetHeaderValue(HEADER_DAYS_LEFT);

		if (pszDaysLeft)
		{
			char c[2048];
			sprintf(c, "This evaluation copy of SourceOffSite will expire in less than %d day(s).\n\nContact sales@sourceoffsite.com for purchasing information.", atoi(pszDaysLeft));
			
			String strExpiresMessage = c;

			AfxMessageBox(strExpiresMessage, MB_OK);
		}


		// extract the database from the message body

		char* pBody = pMessageReceived->m_body;
		int len = pMessageReceived->m_bodySize;
		
#ifndef BUILD_STANDARD
		if (pMessageReceived->m_number == PM_SECURE_CHALLENGE)
		{
			const char* pszKeyLen = pMessageReceived->GetHeaderValue(HEADER_KEY_LENGTH);

			if (pszKeyLen)
			{
				int keyLen = atoi(pszKeyLen);
				pBody += keyLen; 
				len -= keyLen;

				String s;

				if (keyLen == 16)
					s = "Server sending 128 bit challenge.";
				else
					s = "Server sending 56 bit challenge.";

				AddMessage(s);
			}
			else
			{
				String s("This 3.0 SourceOffSite client cannot\ncommunicate with non 3.0 servers.");

				AfxMessageBox(s);
				m_server.del(0, -1);
				m_port = -1;

				ResetToolbar(FALSE);
				
				m_pSocket->Close();

				m_bConnected = FALSE;

				return;
			}
		}
#endif // !BUILD_STANDARD
		
		// ignore the trailing \r\n
		String strDatabases(pBody, len - 2);
		const char* pszAliases = pMessageReceived->GetHeaderValue(HEADER_DATABASE_ALIASES);
		String strAliases;

		if (pszAliases)
			strAliases = pszAliases;
		
		int nDBs = 0;
		
		for (int i = 0; i < len; i++)
			if (strDatabases[i] == '\r')
				++nDBs;
		
		Databases* db = new Databases[nDBs];

		int nextIndex = strDatabases.index('\r');
		int aliasIndex = strAliases.index(',');
		
		int num = 0;

		while (nextIndex >= 0)
		{
			db[num].strDatabase = strDatabases.before(nextIndex);
			
			if (aliasIndex < 0)
			{
				if (!strAliases.empty())
				{
					db[num].strAlias = strAliases;
					strAliases.del(0, -1);
				}
				else
				{
					db[num].strAlias = db[num].strDatabase;
				}
			}
			else
			{
				db[num].strAlias = strAliases.before(aliasIndex);
				strAliases = strAliases.after(aliasIndex);

				aliasIndex = strAliases.index(',');
			}
			
			if (nextIndex + 1 < (int)strDatabases.length())
			{
				strDatabases = strDatabases.after(nextIndex + 1);

				nextIndex = strDatabases.index('\r');
			}
			else
				nextIndex = -1;

			++num;
		}

		// prompt the user to login to the server

		String strServPort;
		char strsrv[256];
		sprintf(strsrv, "%s:%i", m_server.chars(), m_port);

		strServPort = strsrv;
		
#ifndef BUILD_STANDARD
		int bShowSecure = ((pMessageReceived->m_number == PM_SECURE_CHALLENGE) ? 1 : 0);
#endif // !BUILD_STANDARD
		String title("Login to ");
		title += strServPort;

#ifndef BUILD_STANDARD
		if (bShowSecure)
			title += " (secure)";
#endif // !BUILD_STANDARD
		
		if (!m_bSilentLogin)
		{
			CLoginDlg loginDlg(title.chars(), db, num, &Vermie::m_strUserName);
			
			if (loginDlg.DoModal() == IDCANCEL)
			{
				SetWaitState(0);
				
				m_server.del(0, m_server.length());
				m_port = -1;
				
				m_pSocket->Close();
				
				ResetToolbar(FALSE);
				
				m_bConnected = 0;
				
				ResetToolbar(FALSE);

				delete []db;
				
				return;
			}

			// save the info in the dialog
			
			m_username = gtk_entry_get_text(GTK_ENTRY(loginDlg.username));
			m_password = gtk_entry_get_text(GTK_ENTRY(loginDlg.password));

			String s = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(loginDlg.database)->entry));

			for (int i = 0; i < num; i++)
			{
				if (s == db[i].strAlias)
				{
					m_database = db[i].strDatabase;
					m_databaseAlias = db[i].strAlias;
				}
			}
		}

		delete []db;
		
		// send the login message to the server

#ifndef BUILD_STANDARD
		if (pMessageReceived->m_number == PM_SECURE_CHALLENGE)
		{
			if (m_pCrypto)
				delete m_pCrypto;

			// create a new Crypto object

			m_pCrypto = new CVermCrypto;

			// load the user key

			char* userKey = NULL;

			if (!m_pCrypto->GetUserKey(m_username.chars(), ::ConvertServerName(m_server.chars()), &userKey))
			{
				SetWaitState(0);

				m_server.del(0, m_server.length());
				m_port = -1;

				m_pSocket->Close();

				m_bConnected = 0;

				ResetToolbar(FALSE);
			
				return;
			}

			String s;

			if (m_pCrypto->m_nKeyLength == 16)
				s = "Using 128 bit encryption to communicate with server.";
			else
				s = "Using 56 bit encryption to communicate with server.";

			AddMessage(s);
			
			// set the key to be the user key for the Login message

			m_pCrypto->SetKey(userKey);

			delete []userKey;

			// create a session key

			char *sessionKey = new char[m_pCrypto->m_nKeyLength];

			m_pCrypto->GetRandomNumber(sessionKey, m_pCrypto->m_nKeyLength);

			// Send back a limited LOGIN message in the clear

			CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();

			pNewMessage->m_number = PM_LOGIN;

			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_USERNAME, m_username.chars()));
			char buf[256];
			sprintf(buf, "%i", m_pCrypto->m_nKeyLength);
			
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_KEY_LENGTH, buf));
			
			if (m_pSocket->SendMessage(pNewMessage))
			{
				delete pNewMessage;
				return;
			}

			// turn on encryption at the socket

			m_pSocket->SetSecure(m_pCrypto);

			// now send the encrypted LOGIN message

			CVermProtocolMessage encryptedMessage;

			encryptedMessage.m_number = PM_LOGIN;

			encryptedMessage.m_bodySize = m_pCrypto->m_nKeyLength * 2;
			encryptedMessage.m_body = new char[encryptedMessage.m_bodySize];

			int i;
			for (i = 0; i < m_pCrypto->m_nKeyLength; i++)
				encryptedMessage.m_body[i] = pMessageReceived->m_body[i];

			for (i = 0; i < m_pCrypto->m_nKeyLength; i++)
				encryptedMessage.m_body[i + m_pCrypto->m_nKeyLength] = sessionKey[i];

			char szContentLength[64];
			sprintf(szContentLength, "%i", encryptedMessage.m_bodySize);

			encryptedMessage.AddHeader(new CVermProtocolHeader(HEADER_PROTOCOL_VERSION, PROTOCOL_VERSION));
			encryptedMessage.AddHeader(new CVermProtocolHeader(HEADER_CONTENT_LENGTH, szContentLength));
			encryptedMessage.AddHeader(new CVermProtocolHeader(HEADER_DATABASE, m_database.chars()));
			encryptedMessage.AddHeader(new CVermProtocolHeader(HEADER_PASSWORD, m_password.chars()));

			if (m_pSocket->SendMessage(&encryptedMessage, 0))
				return;

			// now set the key to be the session key

			m_pCrypto->SetKey(sessionKey);

			delete []sessionKey;
			
			// disable encryption until we have received the
			// login response

			m_pSocket->SetSecure(NULL);
		}
		else
#endif // !BUILD_STANDARD
		{
#ifndef BUILD_STANDARD
			if (m_pCrypto)
			{
				m_pSocket->SetSecure(NULL);

				delete m_pCrypto;

				m_pCrypto = NULL;
			}
#endif // !BUILD_STANDARD
			
			CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();

			pNewMessage->m_number = PM_LOGIN;

			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROTOCOL_VERSION, PROTOCOL_VERSION));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_DATABASE, m_database.chars()));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_USERNAME, m_username.chars()));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PASSWORD, m_password.chars()));

			if (m_pSocket->SendMessage(pNewMessage))
			{
				delete pNewMessage;
				return;
			}
		}
		
		SetWaitState(1);
		String strMessage;
		strMessage = strMessage + "Logging in...";
		AddMessage(strMessage);
	}
		break;

	case PM_SUCCESS:
		switch (pMessageSent->m_number)
		{

		case PM_SHARE_PROJECT:
			SetWaitState(0);
			break;
		case PM_CHANGE_PASSWORD:
		{
			String s("Your password has been changed.");

			AddMessage(s);
			
			m_password = pMessageSent->GetHeaderValue(HEADER_PASSWORD);

			SetWaitState(0);
			
			break;
		}
		
		case PM_PIN_FILE:
		{
			String strProject(pMessageSent->GetHeaderValue(HEADER_PROJECT));
			String strFile(pMessageSent->GetHeaderValue(HEADER_FILE));
			String strVersion(pMessageSent->GetHeaderValue(HEADER_VERSION));
			const char* pszDate = pMessageReceived->GetHeaderValue(HEADER_DATE);
						   
			String strMessage = "Pinned " + strProject + "/" + strFile + " to version " + strVersion;
			
			AddMessage(strMessage);
			
			if (m_pHistoryDlg)
				m_pHistoryDlg->PinUpdate(atoi(strVersion.chars()));
			
			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject.chars());
			
			String strWorkingDir = GetWorkingDirectory(hItem);
			
			CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));
			
			CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);
			
			if (Vermie::m_bDisplayPinStatus)
				pLeaf->m_nPinnedVersion = atoi(strVersion.chars());
			
			pLeaf->m_nRemoteVersionNumber = atoi(strVersion.chars());

			if (pszDate)
			{
				pLeaf->m_pRemoteVersionDate = ParseServerDate((char*)pszDate);
			}
			
			m_pFileView->UpdatePinStatus(pLeaf);
			
			if (bLastMessage)
				SetWaitState(FALSE);
			
			// does the file need to be refetched?
			
			if ((pLeaf->m_nLocalVersionNumber != pLeaf->m_nRemoteVersionNumber) && !strWorkingDir.empty())
			{
				int nModifiedStatus = IsFileModified(pNode, strFile, strWorkingDir);
				
				if (nModifiedStatus == FILE_MODIFIED)
				{
					String s = strProject +"/" + strFile + " may have been modified, lose changes?";
					if (AfxMessageBox(s, MB_YES | MB_NO) == IDNO)
						return;
				}
				
				// fetch the file
				
				CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();
				
				pNewMessage->m_number = PM_GET_FILE;
				
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
				
				if (!m_pSocket->SendMessage(pNewMessage))
					SetWaitState(1);
				else
				{
					delete pNewMessage;
					return;
				}
			}
			
			break;
		}
		
		case PM_UNPIN_FILE:
		{
			String strProject(pMessageSent->GetHeaderValue(HEADER_PROJECT));
			String strFile(pMessageSent->GetHeaderValue(HEADER_FILE));
			String strVersion(pMessageReceived->GetHeaderValue(HEADER_VERSION));

			const char* pszDate = pMessageReceived->GetHeaderValue(HEADER_DATE);
			
			String strMessage = "Unpinned file: " + strProject + "/" + strFile;
			
			AddMessage(strMessage);
			
			if (m_pHistoryDlg)
				m_pHistoryDlg->PinUpdate(-1);
			
			GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject.chars());
			
			String strWorkingDir = GetWorkingDirectory(hItem);
			
			CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hItem));
			
			CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);
			
			pLeaf->m_nPinnedVersion = -1;
			
			pLeaf->m_nRemoteVersionNumber = atoi(strVersion.chars());

			if (pszDate)
				pLeaf->m_pRemoteVersionDate = ParseServerDate((char*) pszDate);
			
			m_pFileView->UpdatePinStatus(pLeaf);
			
			if (bLastMessage)
				SetWaitState(0);
			
			// does the file need to be refetched?
			
			if ((pLeaf->m_nLocalVersionNumber != pLeaf->m_nRemoteVersionNumber) && !strWorkingDir.empty())
			{
				int nModifiedStatus = IsFileModified(pNode, strFile, strWorkingDir);
				
				if (nModifiedStatus == FILE_MODIFIED)
				{
					String s = strProject + "/" + strFile + " may have been modified, lose changes?";
					
					if (AfxMessageBox(s, MB_YES | MB_NO) == IDNO)
						return;
				}
				
				// fetch the file
				
				CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();
				
				pNewMessage->m_number = PM_GET_FILE;
				
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
				
				if (!m_pSocket->SendMessage(pNewMessage))
					SetWaitState(1);
				else
				{
					delete pNewMessage;
					return;
				}
			}
			
			break;
		}
		
		case PM_NOOP:
			break;

		case PM_BRANCH:
		{
			String strProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
			String strFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
			const char* pVersion = pMessageReceived->GetHeaderValue(HEADER_VERSION);
			
			String str = strProject + "/" + strFile;
			str += " branched";
			
			AddMessage(str);
			
			CVermTreeNode* pNode = GetTreeNodeFromProject(m_pProjectView->GetRootItem(), strProject.chars());
			
			if (pNode)
			{
				CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);
				
				if (pLeaf)
				{
					// Since we brnached, we're no longer shared, and the remote version
					// number has changed
					pLeaf->m_bShared = FALSE;

					if (!fcompare(pLeaf->m_userLocked, m_username) && fcompare(pLeaf->m_projectLocked, strProject))
						pLeaf->m_userLocked = "";
					
					if (pVersion)
					{
						pLeaf->m_nRemoteVersionNumber = atoi(pVersion);
						++pLeaf->m_nLocalVersionNumber;
					}
					
					m_pFileView->UpdateSharedStatus(pLeaf);
				}
			}
			
			if (bLastMessage)
				SetWaitState(FALSE);
		}
		break;

		case PM_LOGIN:
			{
				// we have successfully connected to a new database
				
				if (pMessageReceived->GetHeaderValue(HEADER_SUPPORT_PIN_STATUS))
					Vermie::m_bPinAvailable = 1;
				else
					Vermie::m_bPinAvailable = 0;
					
				if (pMessageReceived->GetHeaderValue(HEADER_VSS6_BUILD))
					Vermie::m_bUseVSS6Features = 1;
				else
					Vermie::m_bUseVSS6Features = 0;
				
				// re-enable encryption if necessary
#ifndef BUILD_STANDARD
				if (m_pCrypto)
					m_pSocket->SetSecure(m_pCrypto);
#endif // !BUILD_STANDARD
				
				// save the new info

				if (!m_bSilentLogin)
				{
					String strMessage("Login successful");
					
					AddMessage(strMessage);

					m_bConnected = 1;
					
					Vermie::m_strUserName = m_username;
					Vermie::m_strDatabaseName = m_database;
					
					// set the window title to reflect the server we are attached to
					
					char port[256];
					sprintf(port, "%i", m_port);
					
					String strTitle = "SourceOffSite - " + m_server + ":" + port;
					
#ifndef BUILD_STANDARD
					if (m_pCrypto)
						strTitle += " (secure)";
#endif // !BUILD_STANDARD
					
					strTitle += " -- ";
					strTitle += m_username;
					strTitle += ", ";
					strTitle += m_databaseAlias;
					
					gtk_window_set_title(GTK_WINDOW(m_pWidget), strTitle.chars());

					// attempt to read the project tree from persisted data
					
					char* pszCurProject = LoadTreeFromFile(m_pProjectView->m_pTree,
														   m_server, m_database,
														   m_pProjectView->m_popupMenu,
														   m_pProjectView);
				
					if (pszCurProject)
					{
						if (bLastMessage)
							SetWaitState(0);
						
						m_pProjectView->SetOpenProject(pszCurProject);
						
						delete []pszCurProject;
					}
					else
					{
						CVermProtocolMessage* pTreeMessage = new CVermProtocolMessage;
						
						pTreeMessage->m_number = PM_GET_PROJECT_TREE;
						
						pTreeMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, "$/"));
						
						pTreeMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, NEW_PROJECT));
						
						if (m_pSocket->SendMessage(pTreeMessage))
						{
							delete pTreeMessage;
							return;
						}
						
						String strNextMessage("Downloading project tree - this may be a lengthy process - please wait...");
						
						AddMessage(strNextMessage);
					}
				}
				else
				{
					String s("Re-connect to server successful.");
					AddMessage(s);

					// clear up any wait state from a previous operation
					m_nWaitCount = 0;		// reset wait count
					SetWaitState(FALSE);
					m_bConnected = 1;

					// reset all the visited flags to false;
					
					ResetTree(m_pProjectView->m_pTree);
					
					// force a file-list refresh if we didn't just interrupt a file-list refresh
					if (m_nCurrentCommand != PM_GET_FILE_LIST)
					{
						String s("");
						m_pProjectView->GetPathSelectedObject(s);
						
						ProjectItemSelected(s);
					}
				}
				
				m_bSilentLogin = 0;

				break;
			}

		case PM_CREATE_PROJECT:
			{
				const char* pszProject = pMessageSent->GetHeaderValue(HEADER_PROJECT);
				const char* pszCookie = pMessageReceived->GetHeaderValue(HEADER_CLIENT_COOKIE);
				
				String strProject(pszProject);

				String strReverse;
				strReverse = reverse(strProject);

				int nIndex = strProject.length() - strReverse.index('/') - 1;
				
				String strNewProject = strProject.after(nIndex);
				String strParentProject = strProject.before(nIndex);

				if (strParentProject.index("$//") == 0)
				{
					String strProj = "$/";
					strProj += strParentProject.after(2);
					strParentProject = strProj;
				}
				
				GtkWidget* hParentItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strParentProject.chars());

				m_pProjectView->AddNewProject(hParentItem, strNewProject);

				if (strProject.index("$//") == 0)
				{
					String strProj = "$/";
					strProj += strProject.after(2);
					strProject = strProj;
				}
				
				GtkWidget* pItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

				CVermTreeNode* pNode = new CVermTreeNode();

				gtk_object_set_user_data(GTK_OBJECT(pItem), pNode);
				
				if (bLastMessage)
					SetWaitState(0);

				String strMessage = "Created project " + strProject;
				AddMessage(strMessage);

				if (pszCookie)
				{
					String strCookie = pszCookie;
					String strDir;
					String strComment;
					short bLockFiles = 0;

					int p = strCookie.index('|');

					if (p > 0)
					{
						strDir = strCookie.before(p);
						strCookie = strCookie.after(p);

						p = strCookie.index('|');

						if (p > 0)
						{
							strComment = strCookie.before(p);
							strCookie = strCookie.after(p);
						}

						if (strCookie[0] == 'y')
							bLockFiles = 1;

						
						AddAllFiles(strProject, strDir, strComment, 0, bLockFiles);
					}
				}
				break;
			}

		case PM_DEPLOY:
		{
			String strProject(pMessageSent->GetHeaderValue(HEADER_PROJECT));

			String s = strProject + " deployed";
			AddMessage(s);

			if (bLastMessage)
				SetWaitState(0);
			break;
		}

		case PM_DELETE:
			{
				String strProject(pMessageSent->GetHeaderValue(HEADER_PROJECT));

				GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

				CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

				if (pMessageSent->GetHeaderValue(HEADER_FILE))
				{
					String strFile(pMessageSent->GetHeaderValue(HEADER_FILE));

					CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

					m_pFileView->RemoveFile(pLeaf);
					
					pNode->DeleteLeaf(pLeaf);

					String strDir = GetWorkingDirectory(hItem);

					String strMessage = strFile + " deleted";
					AddMessage(strMessage);

					if (bLastMessage)
						SetWaitState(0);
				}
				else
				{
					// This item just went away - give our parent the selection
					GtkWidget* parItem = GTK_TREE(hItem->parent)->tree_owner;
					
					CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

					delete pNode;

					if (GTK_TREE_ITEM(hItem)->subtree)
						CleanTree(GTK_TREE_ITEM(hItem)->subtree);
					
					gtk_container_remove(GTK_CONTAINER(hItem->parent), hItem);

					if (bLastMessage)
						SetWaitState(0);
					
					gtk_tree_select_child(GTK_TREE(m_pProjectView->m_pTree),
										  parItem);

					String strMessage = strProject + " deleted";
					AddMessage(strMessage);
					// clear the file view
					m_pFileView->Reset();
				}

				break;
			}

		case PM_CHECKOUT_PROJECT:
		case PM_GET_PROJECT:
			{
				if (bLastMessage)
					SetWaitState(0);

				break;
			}


		case PM_ADD_LABEL:
			{
				String strLabel(pMessageSent->GetHeaderValue(HEADER_LABEL));
				String strProject(pMessageSent->GetHeaderValue(HEADER_PROJECT));

				if (pMessageSent->GetHeaderValue(HEADER_FILE))
				{
					String strFile(pMessageSent->GetHeaderValue(HEADER_FILE));

					// adding a label to a file increments the file's version number

					GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

					CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

					CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

					if (pLeaf && (pLeaf->m_nLocalVersionNumber > 0))
						pLeaf->m_nLocalVersionNumber++;

					String strMessage = strProject + "/" + strFile + " labeled \"" + strLabel + "\"";
					AddMessage(strMessage);
				}
				else
				{
					String strMessage = strProject + " labeled \"" + strLabel + "\"";
					AddMessage(strMessage);
				}

				if (bLastMessage)
					SetWaitState(0);

				break;
			}

		case PM_RENAME:
			{
				String strNewName(pMessageSent->GetHeaderValue(HEADER_NAME));

				String strProject(pMessageSent->GetHeaderValue(HEADER_PROJECT));

				GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

				CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

				if (pMessageSent->GetHeaderValue(HEADER_FILE))
				{
					String strFile(pMessageSent->GetHeaderValue(HEADER_FILE));

					CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

					pLeaf->m_name = strNewName;

					String strDir = GetWorkingDirectory(hItem);
					UpdateFileView(pNode, strDir, pLeaf);

					String strMessage = strFile + " renamed " + strNewName;
					AddMessage(strMessage);
				}
				else
				{
					gtk_label_set(GTK_LABEL(GTK_BIN(hItem)->child),
								  strNewName.chars());

					String dir = GetWorkingDirectory(m_pProjectView->m_hCurSelItem);

					m_pFileView->SetWorkingDirPane(dir);
					
					String strMessage = strProject + " renamed " + strNewName;
					AddMessage(strMessage);
				}

				if (bLastMessage)
					SetWaitState(0);

				break;
			}
		}

		break;

	default:
		char strMessage[2048];
		
#ifdef BUILD_STANDARD
		if (pMessageReceived->m_number == 250)
		{
			String s("This client does not support encryption\nYou are trying to conenct to an encrypted server port");
			ServerShutdown(s);
			return;
		}
#endif // BUILD_STANDARD
		
		sprintf(strMessage, "Unrecognized Message: %i", pMessageReceived->m_number);

		String s = strMessage;
		AddMessage(s);
		break;
	}

}

void CVermFrameWnd::SetWaitState(int state)
{
	if (state == 1)
		++m_nWaitCount;
	else if (state == 0)
		--m_nWaitCount;

	if (m_nWaitCount < 0)
		m_nWaitCount = 0;

	if (m_nWaitCount == 1)
	{
		ResetToolbar(FALSE);
		gtk_widget_set_sensitive(m_pStop, TRUE);
		gtk_widget_set_sensitive(m_pMStop, TRUE);
		
		gtk_widget_set_sensitive(m_pMCreateProject, FALSE);
		gtk_widget_set_sensitive(m_pMAddFiles, FALSE);
		gtk_widget_set_sensitive(m_pMLabel, FALSE);
		gtk_widget_set_sensitive(m_pMDelete, FALSE);
		gtk_widget_set_sensitive(m_pMRename, FALSE);
		gtk_widget_set_sensitive(m_pMGetLatestVersion, FALSE);
		gtk_widget_set_sensitive(m_pMCheckout, FALSE);
		gtk_widget_set_sensitive(m_pMCheckin, FALSE);
		gtk_widget_set_sensitive(m_pMUndoCheckout, FALSE);
		gtk_widget_set_sensitive(m_pMShare, FALSE);
		gtk_widget_set_sensitive(m_pMBranch, FALSE);
		gtk_widget_set_sensitive(m_pMShowDifferences, FALSE);
		gtk_widget_set_sensitive(m_pMShowHistory, FALSE);
		gtk_widget_set_sensitive(m_pMSetWorkingFolder, FALSE);
		gtk_widget_set_sensitive(m_pMRefreshFileList, FALSE);
		gtk_widget_set_sensitive(m_pMRefreshProjectTree, FALSE);
		gtk_widget_set_sensitive(m_pMSearch, FALSE);
		gtk_widget_set_sensitive(m_pMView, FALSE);
		gtk_widget_set_sensitive(m_pMEdit, FALSE);
		gtk_widget_set_sensitive(m_pMWebDeploy, FALSE);
		gtk_widget_set_sensitive(m_pMCloakProject, FALSE);
		
		if (m_pProjectView->m_pTree->window)
			gdk_window_set_cursor(m_pProjectView->m_pTree->window, m_WaitCursor);
		if (m_pFileView->m_pList->window)
			gdk_window_set_cursor(m_pFileView->m_pList->window, m_WaitCursor);
			
		if (m_pShareDlg && m_pShareDlg->projects->window)
			gdk_window_set_cursor(m_pShareDlg->projects->window, m_WaitCursor);

		if (m_pShareDlg)
			m_pShareDlg->SetWaitState(1);

		m_pProjectView->SetWaitState(1);
		m_pFileView->SetWaitState(1);
	}
	else if (m_nWaitCount == 0)
	{
		ResetToolbar(TRUE);
		gtk_widget_set_sensitive(m_pStop, FALSE);
		gtk_widget_set_sensitive(m_pMStop, FALSE);

		gtk_widget_set_sensitive(m_pMCreateProject, TRUE);
		gtk_widget_set_sensitive(m_pMAddFiles, TRUE);
		gtk_widget_set_sensitive(m_pMLabel, TRUE);
		gtk_widget_set_sensitive(m_pMDelete, TRUE);
		gtk_widget_set_sensitive(m_pMRename, TRUE);
		gtk_widget_set_sensitive(m_pMGetLatestVersion, TRUE);
		gtk_widget_set_sensitive(m_pMCheckout, TRUE);
		gtk_widget_set_sensitive(m_pMCheckin, TRUE);
		gtk_widget_set_sensitive(m_pMUndoCheckout, TRUE);
		gtk_widget_set_sensitive(m_pMShare, TRUE);
		gtk_widget_set_sensitive(m_pMBranch, Vermie::m_bUseVSS6Features);
		gtk_widget_set_sensitive(m_pMShowDifferences, TRUE);
		gtk_widget_set_sensitive(m_pMShowHistory, TRUE);
		gtk_widget_set_sensitive(m_pMSetWorkingFolder, TRUE);
		gtk_widget_set_sensitive(m_pMRefreshFileList, TRUE);
		gtk_widget_set_sensitive(m_pMSearch, TRUE);
		gtk_widget_set_sensitive(m_pMRefreshProjectTree, TRUE);
		gtk_widget_set_sensitive(m_pMView, TRUE);
		gtk_widget_set_sensitive(m_pMEdit, TRUE);
		gtk_widget_set_sensitive(m_pMWebDeploy, TRUE);
		gtk_widget_set_sensitive(m_pMCloakProject, TRUE);

		if (m_pProjectView->m_pTree->window)
			gdk_window_set_cursor(m_pProjectView->m_pTree->window, m_NormalCursor);
		if (m_pFileView->m_pList->window)
			gdk_window_set_cursor(m_pFileView->m_pList->window, m_NormalCursor);
		if (m_pShareDlg && m_pShareDlg->projects->window)
			gdk_window_set_cursor(m_pShareDlg->projects->window, m_NormalCursor);
		m_pProjectView->SetWaitState(0);
		m_pFileView->SetWaitState(0);

		if (m_pShareDlg)
			m_pShareDlg->SetWaitState(0);
		
		SetFocus(m_nChildWithFocus);
	}		
}
	
void CVermFrameWnd::ConnectToServer(String& strServer, int nPort)
{
	// terminate any current connection

	if (m_pSocket)
	{
		m_pSocket->Close();

		delete m_pSocket;
		m_pSocket = NULL;

		m_bConnected = 0;
	}

	// reset the views

	m_pProjectView->Reset();
	m_pFileView->Reset();

        // create a new socket

	m_pSocket = new CVermSocket(this);

	// connect to the server

	String strConnectMessage("Connecting to ");
	
	strConnectMessage += strServer;

	char buf[100];
	sprintf(buf, "%i", nPort);

	Vermie::m_strLastServer = strServer;
	Vermie::m_strLastPort = buf;
	
	strConnectMessage += ":";
	strConnectMessage += buf;

	if (Vermie::m_bUseHTTPProxy)
	{
		strConnectMessage += " via HTTP Proxy ";
		strConnectMessage += Vermie::m_strProxyName;
		strConnectMessage += ":";

		sprintf(buf, "%i", Vermie::m_ProxyPort);
		strConnectMessage += buf;
	}

	AddMessage(strConnectMessage);

	SetWaitState(1);

	if (m_pSocket->Connect(strServer, nPort) != SOCKET_OK)
	{
		SetWaitState(0);

		String s("Unable to connect to server");
		AddMessage(s);
		
		delete m_pSocket;
		m_pSocket = NULL;
		
		ResetToolbar(FALSE);

		return;
	}
	
	m_server = strServer;
	m_port = nPort;
}

#ifndef BUILD_STANDARD
void CVermFrameWnd::OnToolsImportEncryptionKey(void)
{
	GtkWidget* fileDlg;

	fileDlg = gtk_file_selection_new("Select Key File");

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileDlg)->ok_button),
					   "clicked", (GtkSignalFunc) ImportKeyFileOkSel, fileDlg);

	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fileDlg)->cancel_button),
					   "clicked", (GtkSignalFunc) file_cancel_sel, fileDlg);

	gtk_widget_show(fileDlg);
	gtk_main();
	gtk_main_quit();
	return;
}

static void ImportKeyFileOkSel(GtkWidget *, GtkFileSelection *f)
{
    //TODO TODO TODO - don't hardcode this :-)
	FILE* pFile = NULL;

	String file = gtk_file_selection_get_filename(f);
	
	pFile = fopen(file.chars(), "r");

	gtk_widget_destroy(GTK_WIDGET(f));
	
	if (!pFile)
	{
		String s("Unable to open file");
		AfxMessageBox(s, MB_OK);
		return;
	}

	// read in the file and parse it into a fake protocol message

	CVermProtocolMessage* pParsedFile = new CVermProtocolMessage;

	char szLine[1024];

	while (fgets(szLine, 1024, pFile) != NULL)
	{
		szLine[strlen(szLine) - 1] = '\0';

		if (szLine[strlen(szLine) - 1] == '\r')
			szLine[strlen(szLine) - 1] = 0;
		
		char* pValue = szLine;

		while (*pValue && *pValue != ':')
			pValue++;

		if (!*pValue)
		{
			fclose(pFile);
			String s("Error reading key file");
			AfxMessageBox(s, MB_OK);

			delete pParsedFile;

			return;
		}
		
		*pValue = '\0';

		pParsedFile->AddHeader(new CVermProtocolHeader(szLine, ++pValue));
	}

	fclose(pFile);

	const char* pServer = pParsedFile->GetHeaderValue(KEY_HEADER_SERVER);
	const char* pUsername = pParsedFile->GetHeaderValue(KEY_HEADER_USERNAME);

	if (!pServer || !pUsername)
	{
		String s("Error reading key file");
		AfxMessageBox(s, MB_OK);

		delete pParsedFile;
		return;
	}

	String strServerName = pServer;
	strServerName.downcase();

	int nDotIndex = strServerName.index('.');

	if (nDotIndex > 0)
		strServerName = strServerName.before(nDotIndex);

	String strUserName = pUsername;
	strUserName.downcase();

	// create a weak key to protect the user's key on disk

	String strWeakKey(strServerName);

	strWeakKey += strUserName;

	MD5_CTX mdContext;

	MD5Init(&mdContext);
	MD5Update(&mdContext, (unsigned char*)(strWeakKey.chars()), strWeakKey.length());
	MD5Final(&mdContext);

        // create a temporary crypto object using the weak key

	CVermCrypto tmpCrypto;

	tmpCrypto.SetKey((char*)mdContext.digest);

        // convert the protocol message into a CString object for encrypting

	String strContents;

	pParsedFile->AppendHeadersToString(strContents);
	
	delete pParsedFile;
	
	// create the key file

	String strKeyFile = Vermie::m_strPrefsDir;

	strKeyFile += "/";
	strKeyFile += pServer;
	strKeyFile += ".key";

	FILE* keyFile = fopen(strKeyFile.chars(), "w+");

	if (keyFile < 0)
	{
		String s("Error creating key file: ");
		s += strKeyFile.chars();

		AfxMessageBox(s);
		delete pParsedFile;
		
		return;
	}
	
	// write the encryption block header

	char IV[8];

	tmpCrypto.GetRandomNumber(IV, 8);

	fwrite(IV, sizeof(char), 8, keyFile);

	String strLength;
	char str[256];
	sprintf(str, "%i", strContents.length());
	
	strLength = str;

	VermByteQueue* pResult = tmpCrypto.Encrypt(&strLength, IV);

	fwrite(pResult->m_pHead->m_pBytes, sizeof(char),
		   pResult->m_pHead->m_nAllocLen, keyFile);

	delete pResult;

	// write the encrypted contents

	pResult = tmpCrypto.Encrypt(&strContents, IV);

	fwrite(pResult->m_pHead->m_pBytes, sizeof(char),
		   pResult->m_pHead->m_nAllocLen, keyFile);

	delete pResult;

	// close the file

	fflush(keyFile);
	fclose(keyFile);

	String s("Encryption key successfully imported");
	AfxMessageBox(s, MB_OK);
}
#endif // !BUILD_STANDARD

void CVermFrameWnd::AddMessage(String& str)
{
	m_pMessageView->AddMessage(str);
//	cout << str << endl;
}

void CVermFrameWnd::ProjectItemSelected(String& strSelectedItem)
{
	if (!m_bConnected)
		return;

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_GET_FILE_LIST;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strSelectedItem.chars()));

	if (Vermie::m_bShowShareIcon)
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_SHARE_INFO, ""));

	if (Vermie::m_bDisplayPinStatus)
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_PIN_STATUS, ""));

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

	if (RequestChecksums(strSelectedItem))
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKSUM, "yes"));

	String strMessage = "Updating file list: ";
	strMessage += strSelectedItem;
	

	// Toggle the Cloak/UnCloak menu label depending on the node ...
	CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(m_pProjectView->m_hCurSelItem));
	
	if (pNode->m_bCloaked)
	{
		gtk_label_set(GTK_LABEL(GTK_BIN(m_pMCloakProject)->child), "UnCloak Project");
		gtk_label_set(GTK_LABEL(GTK_BIN(m_pProjectView->m_pCloak)->child), "UnCloak Project");
	}
	else
	{
		gtk_label_set(GTK_LABEL(GTK_BIN(m_pMCloakProject)->child), "Cloak Project");
		gtk_label_set(GTK_LABEL(GTK_BIN(m_pProjectView->m_pCloak)->child), "Cloak Project");
	}
	
	AddMessage(strMessage);

	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}
}

void CVermFrameWnd::UpdateFileView(CVermTreeNode* pNode, String& strWorkingDir,
								   CVermTreeLeaf* pLeaf,
								   int bUpdateWorkingDirPane)
{
	String strProject;

	SetFocus(FOCUS_PROJECTVIEW);

	GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);
	
	m_pFileView->SetFileList(pNode, strProject, m_username, strWorkingDir,
							 pLeaf);

	if (bUpdateWorkingDirPane)
	{
		m_pFileView->SetWorkingDirPane(strWorkingDir);
	}
}

void CVermFrameWnd::SetPercentDone(double dPercentage)
{
	if (dPercentage == (double) -1.0)
		m_pFileView->ResetStatus();
	else
	{
		if (dPercentage < 0.0)
			dPercentage = 0.0;

		if (dPercentage > 1.0)
			dPercentage = 1.0;

		m_pFileView->SetStatus(dPercentage);
	}
}

void CVermFrameWnd::SetFocus(int which)
{
	m_nChildWithFocus = which;

	// if the toolbar is inactive, no point in being here
	if (!m_bToolbarActive)
		return;

	gtk_widget_set_sensitive(m_pMPassword, Vermie::m_bUseVSS6Features);
	
	// Set the tool bar 
	if (which == FOCUS_PROJECTVIEW)
	{
		gtk_widget_set_sensitive(m_pShowDifferences, TRUE);
		gtk_widget_set_sensitive(m_pShare, TRUE);
		gtk_widget_set_sensitive(m_pMBranch, FALSE);
		gtk_widget_set_sensitive(m_pShowHistory, TRUE);
		gtk_widget_set_sensitive(m_pCheckout, TRUE);
		gtk_widget_set_sensitive(m_pUndoCheckout, TRUE);
		gtk_widget_set_sensitive(m_pCheckin, TRUE);
		gtk_widget_set_sensitive(m_pGetLatestVersion, TRUE);

		gtk_widget_set_sensitive(m_pView, FALSE);
		gtk_widget_set_sensitive(m_pEdit, FALSE);
		gtk_widget_set_sensitive(m_pBranch, FALSE);
		

		gtk_widget_set_sensitive(m_pMShowDifferences, TRUE);
		gtk_widget_set_sensitive(m_pMShare, TRUE);
		gtk_widget_set_sensitive(m_pMShowHistory, TRUE);
		gtk_widget_set_sensitive(m_pMGetLatestVersion, TRUE);
		gtk_widget_set_sensitive(m_pMCheckout, TRUE);
		gtk_widget_set_sensitive(m_pMCheckout, TRUE);
		gtk_widget_set_sensitive(m_pMUndoCheckout, TRUE);

		if (GTK_CLIST(m_pFileView->m_pList)->selection)
		{
			gtk_widget_set_sensitive(m_pMView, TRUE);
			gtk_widget_set_sensitive(m_pMEdit, TRUE);
		}
		else
		{
			gtk_widget_set_sensitive(m_pMView, FALSE);
			gtk_widget_set_sensitive(m_pMEdit, FALSE);
		}
		
		if (m_pProjectView->m_hCurSelItem)
		{
			CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(m_pProjectView->m_hCurSelItem));
			
			if (pNode->m_bWebDeploy)
				gtk_widget_set_sensitive(m_pMWebDeploy, TRUE);
			else
				gtk_widget_set_sensitive(m_pMWebDeploy, FALSE);

			gtk_widget_set_sensitive(m_pMCloakProject, TRUE);
			
		}
		else
		{
			gtk_widget_set_sensitive(m_pMWebDeploy, FALSE);
		}
			
	}
	else
	{
		GList* selection = GTK_CLIST(m_pFileView->m_pList)->selection;

		if (selection)
		{
			int bFilesLocked = 0;
			int bFilesNotLocked = 0;
			int bFilesShared = 0;
			
			gtk_widget_set_sensitive(m_pView, TRUE);
			gtk_widget_set_sensitive(m_pEdit, TRUE);
			gtk_widget_set_sensitive(m_pBranch, Vermie::m_bUseVSS6Features);
			gtk_widget_set_sensitive(m_pGetLatestVersion, TRUE);
			gtk_widget_set_sensitive(m_pShare, TRUE);
			
			gtk_widget_set_sensitive(m_pMGetLatestVersion, TRUE);
			gtk_widget_set_sensitive(m_pMShare, TRUE);

			if (!selection->next)
			{
				gtk_widget_set_sensitive(m_pMView, TRUE);
				gtk_widget_set_sensitive(m_pMEdit, TRUE);
				gtk_widget_set_sensitive(m_pShowHistory, TRUE);

				String strProject;
				m_pProjectView->GetPathSelectedObject(strProject);

				String strDir = GetWorkingDirectory(m_pProjectView->GetRootItem(), strProject.chars());

				String strFile;
				m_pFileView->GetFileName((int)selection->data, strFile);

				GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);
				
				CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(m_pProjectView->m_hCurSelItem));
				CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

				if (!pLeaf->m_userLocked.empty())
					gtk_widget_set_sensitive(m_pMRename, FALSE);
				else
					gtk_widget_set_sensitive(m_pMRename, TRUE);
					
				strDir += "/";
				strFile = strDir + strFile;
				
				struct stat sb;
				memset(&sb, 0, sizeof(struct stat));

				if (!stat(strFile.chars(), &sb))
				{
					gtk_widget_set_sensitive(m_pMShowDifferences, TRUE);
					gtk_widget_set_sensitive(m_pShowDifferences, TRUE);
				}
				else
				{
					gtk_widget_set_sensitive(m_pMShowDifferences, FALSE);
					gtk_widget_set_sensitive(m_pShowDifferences, FALSE);
				}

				gtk_widget_set_sensitive(m_pMShowHistory, TRUE);
				gtk_widget_set_sensitive(m_pView, TRUE);
				gtk_widget_set_sensitive(m_pEdit, TRUE);
			}
			else
			{
				gtk_widget_set_sensitive(m_pMView, FALSE);
				gtk_widget_set_sensitive(m_pMEdit, FALSE);
				gtk_widget_set_sensitive(m_pView, FALSE);
				gtk_widget_set_sensitive(m_pEdit, FALSE);
				gtk_widget_set_sensitive(m_pShowDifferences, FALSE);
				gtk_widget_set_sensitive(m_pShowHistory, FALSE);

				gtk_widget_set_sensitive(m_pMShowDifferences, FALSE);
				gtk_widget_set_sensitive(m_pMShowHistory, FALSE);
				gtk_widget_set_sensitive(m_pMView, FALSE);
				gtk_widget_set_sensitive(m_pMEdit, FALSE);
				gtk_widget_set_sensitive(m_pMRename, FALSE);
			}

			while (selection)
			{
				CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(m_pProjectView->m_hCurSelItem));

				String strFile;
				m_pFileView->GetFileName((int)selection->data, strFile);
				String strProject;
				GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);
				
				CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

				if (pLeaf && pLeaf->m_bShared)
					bFilesShared = 1;
				   
				int nLockStatus = IsFileLocked(pLeaf, strProject, m_username);

				if (nLockStatus > 0)
				{
					if (!bFilesLocked)
					{
						bFilesLocked = 1;
						gtk_widget_set_sensitive(m_pCheckin, TRUE);
						gtk_widget_set_sensitive(m_pUndoCheckout, TRUE);

						gtk_widget_set_sensitive(m_pMCheckin, TRUE);
						gtk_widget_set_sensitive(m_pMUndoCheckout, TRUE);
					}
				}
				else if (!bFilesNotLocked && pLeaf->m_nPinnedVersion == -1)
				{
					bFilesNotLocked = 1;
					gtk_widget_set_sensitive(m_pCheckout, TRUE);

					gtk_widget_set_sensitive(m_pMCheckout, TRUE);
				}
					
				selection = selection->next;
			}

			if (!bFilesLocked)
			{
				gtk_widget_set_sensitive(m_pCheckin, FALSE);
				gtk_widget_set_sensitive(m_pUndoCheckout, FALSE);

				gtk_widget_set_sensitive(m_pMCheckin, FALSE);
				gtk_widget_set_sensitive(m_pMUndoCheckout, FALSE);
			}

			if (!bFilesNotLocked)
			{
				gtk_widget_set_sensitive(m_pCheckout, FALSE);

				gtk_widget_set_sensitive(m_pMCheckout, FALSE);
			}

			if (bFilesShared)
			{
				gtk_widget_set_sensitive(m_pMBranch, Vermie::m_bUseVSS6Features);
				gtk_widget_set_sensitive(m_pBranch, Vermie::m_bUseVSS6Features);
			}
			else
			{
				gtk_widget_set_sensitive(m_pMBranch, FALSE);
				gtk_widget_set_sensitive(m_pBranch, FALSE);
			}
		}
		else
		{
			gtk_widget_set_sensitive(m_pShowHistory, FALSE);
			gtk_widget_set_sensitive(m_pShowDifferences, FALSE);
			gtk_widget_set_sensitive(m_pShare, TRUE);
			gtk_widget_set_sensitive(m_pCheckout, FALSE);
			gtk_widget_set_sensitive(m_pCheckin, FALSE);
			gtk_widget_set_sensitive(m_pUndoCheckout, FALSE);
			gtk_widget_set_sensitive(m_pGetLatestVersion, FALSE);
			gtk_widget_set_sensitive(m_pView, FALSE);
			gtk_widget_set_sensitive(m_pEdit, FALSE);
			gtk_widget_set_sensitive(m_pBranch, FALSE);

			gtk_widget_set_sensitive(m_pMShowHistory, FALSE);
			gtk_widget_set_sensitive(m_pMShare, TRUE);
			gtk_widget_set_sensitive(m_pMBranch, FALSE);
			gtk_widget_set_sensitive(m_pMShowDifferences, FALSE);
			gtk_widget_set_sensitive(m_pMCheckout, FALSE);
			gtk_widget_set_sensitive(m_pMCheckin, FALSE);
			gtk_widget_set_sensitive(m_pMUndoCheckout, FALSE);
			gtk_widget_set_sensitive(m_pMGetLatestVersion, FALSE);
		}
	}
}


int CVermFrameWnd::GetFile(String& strProject, String& strFile, short bMakeWritable, int nVerNum)
{
	// Verify that a working directory has been set

	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject.chars());

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (strWorkingDir.empty())
	{
		if (SetWorkingFolder() < 0)
		{
			return 0;
		}
		strWorkingDir = GetWorkingDirectory(hItem);		
	}

	String strFullFile = strWorkingDir + "/" + strFile;

	// check to see if the file exists - and if it is currently writeable

	struct stat sb;
	memset(&sb, 0, sizeof(struct stat));
	
	int bFileExists = 0;

	bFileExists = (stat(strFullFile.chars(), &sb) == 0);
	
	// get a pointer to the tree node for this project

	CVermTreeNode* pNode = m_pProjectView->GetNodeFromProject(strProject.chars());

//	ASSERT(pNode);

	int bFetchFile = 0;

	// determine if the file actually needs to be fetched

	int bOverride;
		
	if (!bFileExists)
	{
		bFetchFile = 1;
	}
	else
	{
		CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

		bFetchFile = ShouldFileBeFetched(pLeaf, strProject, m_username,
										 sb, m_modState, m_lockState, m_writeState, bOverride, nVerNum);

		if ((m_modState == IDCANCEL) || (m_lockState == IDCANCEL) || (m_writeState == IDCANCEL))
			return 0;
	}

	// create and send the message to the server
	// if bFetchFile is set to TRUE, just immediately grab the file from the
	// server otherwise, send off a FILE_INFO request for this file because we
	// currently think it's up-to-date.

	if (bFetchFile)
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();
		
		pMessage->m_number = PM_GET_FILE;
		
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT,
													strProject));
			
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

		if (bMakeWritable)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, MAKE_WRITABLE));	
		else
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, MAKE_READONLY));
				
		if (nVerNum > 0)
		{
			char strVersion[256];
			sprintf(strVersion, "%i", nVerNum);
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_VERSION,
														strVersion));
		}
		
		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return 0;
		}
	}
	else if (!bOverride)
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();
		
		pMessage->m_number = PM_GET_FILE_INFO;
		
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
		
		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return 0;
		}
	}

	return 1;
}

void CVermFrameWnd::GetProjectByLabel(String& strProject, String& strLabel)
{
	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(),
											  strProject);

	String strWorkingDir = GetWorkingDirectory(hItem);

	if (strWorkingDir.empty())
	{
		String s("No working directory set");
		AfxMessageBox(s);
		return;
	}

	int bRecurseProject = Vermie::m_bRecurseOnProjects;

	CCheckoutDlg dlg("Get Project By Label", TRUE);

	if (dlg.DoModal() == IDCANCEL)
		return;

	bRecurseProject = GTK_TOGGLE_BUTTON(dlg.recursive)->active;

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_GET_PROJECT;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_LABEL, strLabel));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

	if (bRecurseProject)
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_RECURSIVE, "yes"));

	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(TRUE);
	else
	{
		delete pMessage;
		return;
	}
}



void CVermFrameWnd::CheckInFile(CVermTreeNode* pNode, String& strProject,
								String& strWorkingDir,
								String& strFile, String& strDlgComment,
								int bRemoveLocalCopy, int bKeepCheckedOut, int bLocked /* = 0 */)
{
	String strFileName = strWorkingDir + "/" + strFile;

	int file = open(strFileName.chars(), O_RDONLY);
	
	if (file < 0)
	{
		String s("Unable to open file: ");
		s += strFileName;
		AddMessage(s);
		return;
	}

	int nLockStatus = 2;

	if (!bLocked)
		nLockStatus = IsFileLocked(pNode, strProject, strFile, m_username);
	
	if (nLockStatus == 0)
		return;
	
	if (nLockStatus == 1)
	{
		String strWarning;
		
		CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);
		
		if (pLeaf->m_projectLocked.empty())
			strWarning = "This file was checked out to a different project.";
		else
			strWarning = "This file was checked out to project " + pLeaf->m_projectLocked;
		
		strWarning += "\n\nProceed with checkin from project " + strProject + " ?";
		
		if (AfxMessageBox(strWarning, MB_YES | MB_NO) == IDNO)
			return;
	}

	// create the message

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	// read the entire file into the message body

	pMessage->m_bodySize = lseek(file, 0, SEEK_END);
	lseek(file, 0, SEEK_SET);

	pMessage->m_body = new char[pMessage->m_bodySize];

	read(file, pMessage->m_body, pMessage->m_bodySize);

	close(file);

	// send the message

	pMessage->m_number = PM_CHECKIN;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

	// if the merge fails, and we have a merge utility, send the file back in the error message
	if (Vermie::m_strMergeProgram != "")
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SFOMF, "yes"));
	
	char szContentLength[64];

	if (Vermie::m_bUseCompression && Vermie::m_bCanUseCompression)
	{
		sprintf(szContentLength, "%i", pMessage->m_bodySize);

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_ORIGINAL_SIZE, szContentLength));
		CompressMessageBody(pMessage);
	}

	sprintf(szContentLength, "%i", pMessage->m_bodySize);

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_CONTENT_LENGTH, szContentLength));

	// strip any newline characters out of the comment

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strDlgComment));

	if (bRemoveLocalCopy)
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, REMOVE_LOCAL_COPY));

	if (bKeepCheckedOut)
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_KEEP_CHECKED_OUT, "yes"));

	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}
}


void CVermFrameWnd::CompressMessageBody(CVermProtocolMessage* pMessage)
{
	unsigned long newLen = (long)((double)pMessage->m_bodySize * 1.1) + 12;

	unsigned char* pNewBody = new Byte[newLen];

	compress(pNewBody, &newLen, (unsigned char *)pMessage->m_body,
			 (unsigned long)pMessage->m_bodySize);

	delete []pMessage->m_body;

	pMessage->m_body = (char *)pNewBody;
	pMessage->m_bodySize = (int) newLen;
}

void CVermFrameWnd::UndoCheckout(CVermTreeNode* pNode, String& strWorkingDir,
								 String& strProject, String& strFile, const char* pCookie, int bLocked /* = 0 */)
{
	int nLockStatus = 2;

	if (!bLocked)
		nLockStatus = IsFileLocked(pNode, strProject, strFile, m_username);

	if (nLockStatus == 0)
		return; // file is not locked
	
	if (nLockStatus == 1)
	{
		String strWarning;
		
		CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);
		
		if (pLeaf->m_projectLocked.empty())
			strWarning = "This file was checked out to a different project.";
		else
			strWarning = "This file was checked out to project " + pLeaf->m_projectLocked;
		
		
		strWarning += "\n\nProceed with undo checkout from project " + strProject + " ?";
		
		if (AfxMessageBox(strWarning, MB_YES | MB_NO) == IDNO)
			return;
	}

	int nModifiedState = IsFileModified(pNode, strFile, strWorkingDir);
	
	if (nModifiedState == FILE_MODIFIED && strcmp(pCookie, UNDO_CHECKOUT_LEAVE))
	{
		if (m_modState != IDNOTOALL && m_modState != IDYESTOALL)
		{
			String s = strFile + " may have been modified, lose changes?";
			CYesNoDlg dlg(s);
			m_modState = dlg.DoModal();
		}

		// bail out if they hit cancel
		if (m_modState == IDCANCEL)
		{
			return;
		}
	}

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_UNLOCK_FILE;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
	
	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}

	if (!strcmp(pCookie, UNDO_CHECKOUT_DELETE))
	{
		String file = strWorkingDir + "/";
		file += strFile;

		if (unlink(file.chars()))
		{
			String s("Unable to remove file: ");
			s += file;
			
			AddMessage(s);
		}
		else
		{
			String s("Deleted file: ");
			s += file;

			AddMessage(s);
		}
		
		return;
	}
	
	// don't get a new copy if they said no or there's no workding dir ...
	if (strWorkingDir.empty() || nModifiedState != FILE_MODIFIED || m_modState == IDNO ||
		m_modState == IDNOTOALL || !strcmp(pCookie, UNDO_CHECKOUT_LEAVE));
	{
		return;
	}

	if (!strcmp(pCookie, UNDO_CHECKOUT_REPLACE))
	{
		CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();

		pNewMessage->m_number = PM_GET_FILE;
		
		pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
		pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));
		pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
		
		if (!m_pSocket->SendMessage(pNewMessage))
			SetWaitState(1);
		else
		{
			delete pNewMessage;
			return;
		}
	}
}

void CVermFrameWnd::UndoCheckoutForNode(CVermTreeNode* pNode,
										String& strWorkingDir,
										String& strProject, const char* pCookie)
{
	if (!pNode || !pNode->m_pLeafs)
		return;
	
	VermTreeListNode* pTmp = pNode->m_pLeafs->m_pHead;
	
	while (pTmp)
	{
		CVermTreeLeaf* pLeaf = pTmp->m_pLeaf;

		int nLockStatus = IsFileLocked(pLeaf, strProject, m_username);

		if (nLockStatus > 0)
		{
			UndoCheckout(pNode, strWorkingDir, strProject, pLeaf->m_name, pCookie);

			if (m_modState == IDCANCEL)
				return;
		}

		pTmp = pTmp->m_pNext;
	}
}

void CVermFrameWnd::GetDiffFiles(FileDiff* pFileDiff, String& pszProject)
{
	if (!pFileDiff)
		return;

	m_pFileDiff = pFileDiff;

	CVermProtocolMessage* pMessage;

	if (pFileDiff->bProjectDiff)
	{
		pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_GET_PROJECT;
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

		if (pFileDiff->bRecurse)
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_RECURSIVE,"yes"));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
			delete pMessage;

		return;
	}
	
	if (!pFileDiff->bHaveFileOne)
	{
		// create the message

		pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_GET_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));

		String strRev = reverse(pFileDiff->strFileOne);

		int nSlashIndex = strRev.index('/');
		
		String strFile = pFileDiff->strFileOne.after((int)strRev.length() - nSlashIndex - 1);

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

		if (!pFileDiff->strFileOneVersion.empty())
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_VERSION, pFileDiff->strFileOneVersion));

		if (m_pSocket->SendMessage(pMessage))
		{
			delete pMessage;
			return;
		}
		else
			SetWaitState(1);
	}

	if (!pFileDiff->bHaveFileTwo)
	{
		// create the message

		pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_GET_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));

		String strRev = reverse(pFileDiff->strFileTwo);

		int nSlashIndex = strRev.index('/');

		String strFile = pFileDiff->strFileTwo.after((int)strRev.length() - nSlashIndex - 1);

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

		if (!pFileDiff->strFileTwoVersion.empty())
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_VERSION, pFileDiff->strFileTwoVersion));

		if (m_pSocket->SendMessage(pMessage))
		{
			delete pMessage;
			return;
		}
		else
			SetWaitState(1);
	}
}

void CVermFrameWnd::ReceivedDiffFile(CVermProtocolMessage* pMessageReceived)
{
	const char* pszFile = pMessageReceived->GetHeaderValue(HEADER_FILE);
	const char* pszVersion = pMessageReceived->GetHeaderValue(HEADER_VERSION);
	const char* pszContentLen = pMessageReceived->GetHeaderValue(HEADER_CONTENT_LENGTH);
	const char* pszProject = pMessageReceived->GetHeaderValue(HEADER_PROJECT);
	
	if (pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE))
		pszContentLen = pMessageReceived->GetHeaderValue(HEADER_ORIGINAL_SIZE);

	// put the fetched file in the temp directory

	String strFile = m_pFileDiff->strTmpDir;
	strFile += pszFile;

	// which file did we get?

	if (m_pFileDiff->bProjectDiff)
	{
		// work some magic to get the direcotory correct for recursive diffs

		String strProject = pszProject;

		String dir("");

		if (strProject.length() > m_pFileDiff->strProject.length())
			dir = strProject.after(m_pFileDiff->strProject.length());

		// create the VSS source dir
		String ndir = m_pFileDiff->strTmpDir + dir;
		
		CreateDirectory(ndir);
		
		strFile = ndir;
		
		// Now create the local copy directory and dump the local file into it ...
		ndir = m_pFileDiff->strSourceTmpDir + dir;
		
		CreateDirectory(ndir);
		
		String strWorkingDir = GetWorkingDirectory(m_pProjectView->GetRootItem(), pszProject);
		
		if (strWorkingDir != "")
		{
			String strLocalFile = strWorkingDir + "/";
			strLocalFile += pszFile;
			
			String strTempFile = ndir;
			
			if (strTempFile[strTempFile.length() - 1] != '/')
				strTempFile += "/";
			
			strTempFile += pszFile;
			
			char buf[2048];
			
			int localFile = open(strLocalFile.chars(), O_RDONLY);
			
			if (localFile >= 0)
			{
				int tempFile = open(strTempFile.chars(), O_RDWR|O_CREAT, 0644);

				if (tempFile >= 0)
				{
					int fileSize = lseek(localFile, 0, SEEK_END);
					lseek(localFile, 0, SEEK_SET);
					int nRead = 0;
					int r = 0;
					
					while (nRead < fileSize)
					{
						r = read(localFile, buf, 2048);
						write(tempFile, buf, r);
						nRead += r;
					}
					
					close(tempFile);
					
					MakeFileReadOnly(strTempFile);
				}
				close(localFile);
			}

			if (strFile[strFile.length() - 1] != '/')
				strFile += "/";

			strFile += pszFile;

			SetWaitState(0);
		}
	}
	else if (m_pFileDiff->bHaveFileOne)
	{
		// must be file two

		if ((strFile == m_pFileDiff->strFileTwo) &&
			(m_pFileDiff->strFileTwoVersion.empty() ||
			 (m_pFileDiff->strFileTwoVersion == pszVersion)))
		{
			m_pFileDiff->bHaveFileTwo = 1;

			m_pFileDiff->strFileTwo = m_pFileDiff->strFileTwo + "_Version_";
			m_pFileDiff->strFileTwo += pszVersion;
		}
	}
	else
	{
		// is it file one?

		if ((strFile == m_pFileDiff->strFileOne) &&
			(m_pFileDiff->strFileOneVersion.empty() ||
			 (m_pFileDiff->strFileOneVersion == pszVersion)))
		{
			m_pFileDiff->bHaveFileOne = 1;

			m_pFileDiff->strFileOne = m_pFileDiff->strFileOne + "_Version_";
			m_pFileDiff->strFileOne += pszVersion;
		}
		else if ((strFile == m_pFileDiff->strFileTwo) &&
				 (m_pFileDiff->strFileTwoVersion.empty() ||
				  (m_pFileDiff->strFileTwoVersion == pszVersion)))
		{
			m_pFileDiff->bHaveFileTwo = 1;

			m_pFileDiff->strFileTwo = m_pFileDiff->strFileTwo + "_Version_";
			m_pFileDiff->strFileTwo += pszVersion;
		}
	}

	// write the file to disk

	// don't append verion strings for project diff
	if (!m_pFileDiff->bProjectDiff)
	{
		strFile = strFile + "_Version_";
		strFile += pszVersion;
	}

	int newFile = open(strFile.chars(), O_WRONLY | O_CREAT, 0600);

	if (newFile < 0)
	{
		// Error
		String s("Unable to open temp file for diff");
		AddMessage(s);
		return;
	}

	write(newFile, pMessageReceived->m_body, atoi(pszContentLen));

	close(newFile);

	// are we ready to diff?

	if (m_pFileDiff->bHaveFileOne && m_pFileDiff->bHaveFileTwo)
	{
		// create a new thread in which to run windiff
		if (Vermie::m_strDiffProgram.empty())
		{
			CleanupFileDiff();
			SetWaitState(0);
			return;
		}
		int pid = fork();

		if (!pid)
		{
			// child
			char diff[2048];

			sprintf(diff, Vermie::m_strDiffProgram.chars(), m_pFileDiff->strFileOne.chars(), m_pFileDiff->strFileTwo.chars());

			system(diff);

			CleanupFileDiff();

			_exit(0);
		}
		
		// turn off the wait state

		SetWaitState(0);

		// delete the file diff struct member

		delete m_pFileDiff;

		m_pFileDiff = NULL;
	}
}

void CVermFrameWnd::ProjectDiffFinished(void)
{
	int pid = fork();

	if (!pid)
	{
		char diff[2048];
		
		sprintf(diff, Vermie::m_strProjectDiffProgram.chars(), m_pFileDiff->strSourceTmpDir.chars(), m_pFileDiff->strTmpDir.chars());
	
		system(diff);
		
		CleanupFileDiff();
		
		_exit(0);
	}
	
	SetWaitState(0);
	
	delete m_pFileDiff;
	
	m_pFileDiff = NULL;
}

void CVermFrameWnd::SearchResultsDlgGone(void)
{
	if (m_pSearchResultsDlg)
		delete m_pSearchResultsDlg;
	
	m_pSearchResultsDlg = NULL;
}

void CVermFrameWnd::HistoryDlgGone(void)
{
	if (m_pHistoryDlg)
		delete m_pHistoryDlg;
	
	m_pHistoryDlg = NULL;
}

void CVermFrameWnd::ShareDlgGone(void)
{
	if (m_pShareDlg)
		delete m_pShareDlg;

	m_pShareDlg = NULL;
}


void CVermFrameWnd::CleanupFileDiff(void)
{
	if (m_pFileDiff->bProjectDiff)
	{
		TrashDirectory(m_pFileDiff->strTmpDir);
		TrashDirectory(m_pFileDiff->strSourceTmpDir);
		
		return;
	}

	if (m_pFileDiff->bOneIsTempFile)
	{
		if (unlink(m_pFileDiff->strFileOne.chars()))
			cout << "Unable to remove first diff file" << endl;
	}

	if (m_pFileDiff->bTwoIsTempFile)
	{
		if (unlink(m_pFileDiff->strFileTwo.chars()))
			cout << "Unable to remove second diff file" << endl;
	}

	if (rmdir(m_pFileDiff->strTmpDir.chars()))
		cout << "Unable to remove tmp diff directory" << endl;

	delete m_pFileDiff;
	m_pFileDiff = NULL;
}

void CVermFrameWnd::UpdateNodeDisplay(CVermTreeNode* pNode,
									  String& strWorkingDir)
{
	if (!pNode)
		return;

	if (pNode->m_bVisited)
	{
		String strProject;
		GetPathFromTreeItem(m_pProjectView->m_hCurSelItem, strProject);
		
		m_pFileView->UpdateFileInfo(pNode, strProject, m_username,
									strWorkingDir);
	}
}

void CVermFrameWnd::UpdateFileInfo(CVermProtocolMessage* pMessage)
{
	int bFetchFile = FALSE;

	const char* pszProject = pMessage->GetHeaderValue(HEADER_PROJECT);
	const char* pszFile = pMessage->GetHeaderValue(HEADER_FILE);
	const char* pszVersion = pMessage->GetHeaderValue(HEADER_VERSION);
	const char* pszRemoteDate = pMessage->GetHeaderValue(HEADER_DATE);

	CVermTreeNode* pNode = m_pProjectView->GetNodeFromProject(pszProject);

	if (!pNode)
		return;

	String strPSZFile = pszFile;
	CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strPSZFile);

	if (!pLeaf)
		return;

	pLeaf->m_nRemoteVersionNumber = atoi(pszVersion);
	pLeaf->m_pRemoteVersionDate = ParseServerDate((char*)pszRemoteDate);

	// newer version on server?
	if (pLeaf->m_nRemoteVersionNumber < atoi(pszVersion))
		bFetchFile = 1;

	if (bFetchFile)
	{
		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_GET_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pszFile));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

		if (!m_pSocket->SendMessage(pMessage))
			SetWaitState(1);
		else
		{
			delete pMessage;
			return;
		}
	}
}

void CVermFrameWnd::GetProjectListings(GtkWidget* hItem, char *cookie)
{
	if (!hItem)
		return;

	// fetch project listing for this hItem
	if (!m_pSocket)
		return;

	
	GList* children = gtk_container_children(GTK_CONTAINER(hItem));
	
	while (children)
	{
		GtkWidget* hChild = GTK_WIDGET(children->data);

		String dir = GetWorkingDirectory(hChild);

		// if we are doing a recursive op, the directory normally wouldn't be
		// created if there were no files in that directory.  Force it to be
		// created here.
		// don't create the dir if the project is cloaked
		
		String strNode;
		GetPathFromTreeItem(hChild, strNode);

		String strWorkingDir = dir;

		CVermTreeNode* pNode = (CVermTreeNode*) gtk_object_get_user_data(GTK_OBJECT(hChild));
	
		if (!pNode->m_bCloaked)
		{
			CreateDirectory(dir);

			if (FetchEntireProject(strWorkingDir) && strcmp(cookie, CHECKIN_FILES) && strncmp(cookie, "Undo Checkout", 13))
			{
				CVermProtocolMessage* pMessage = new CVermProtocolMessage();
				
				String strMessage("Checking out project: ");

				if (!strcmp(cookie, FETCH_FILES) || !strcmp(cookie, FETCH_FILES_MAKE_WRITABLE))
				{
					pMessage->m_number = PM_GET_PROJECT;
					strMessage = "Fetching project: ";
				}
				else if (!strcmp(cookie, CHECKOUT_FILES))
					pMessage->m_number = PM_CHECKOUT_PROJECT;
				
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strNode));	

				pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, cookie));

				strMessage += strNode;

				AddMessage(strMessage);
				
				if (!m_pSocket->SendMessage(pMessage))
					SetWaitState(1);
				else
				{
					delete pMessage;
					return;
				}
			}
			else
			{
				CVermProtocolMessage* pMessage = new CVermProtocolMessage();
				pMessage->m_number = PM_GET_FILE_LIST;
				
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strNode));
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, cookie));
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

				if (Vermie::m_bShowShareIcon)
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_SHARE_INFO, ""));
				
				if (Vermie::m_bDisplayPinStatus)
					pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_PIN_STATUS, ""));
				
				String strMessage = "Updating file list: ";
				strMessage += strNode;
				
				AddMessage(strMessage);
				
				if (!m_pSocket->SendMessage(pMessage))
					SetWaitState(1);
				else
				{
					delete pMessage;
					return;
				}
			}

			if (GTK_TREE_ITEM(hChild)->subtree)
			{
				GetProjectListings(GTK_TREE_ITEM(hChild)->subtree, cookie);
			}
		}

		children = g_list_remove_link(children, children);
	}
}

void CVermFrameWnd::CheckinFilesForNode(CVermTreeNode* pNode,
										String& strWorkingDir,
										String& strProject,
										String& strComment,
										int bRemoveLocalCopy,
										int bKeepCheckedOut)
{
	if (!pNode || !pNode->m_pLeafs)
		return;
	
	VermTreeListNode* pTmp = pNode->m_pLeafs->m_pHead;
	
	while (pTmp)
	{
		CVermTreeLeaf* pLeaf = pTmp->m_pLeaf;

		int nLockStatus = IsFileLocked(pLeaf, strProject, m_username);

		if (nLockStatus > 0)
		{
			CheckInFile(pNode, strProject, strWorkingDir, pLeaf->m_name,
						strComment, bRemoveLocalCopy, bKeepCheckedOut);
		}

		pTmp = pTmp->m_pNext;
	}
}

int CVermFrameWnd::FetchEntireProject(String& strWorkingDir)
{
	// if the working dir isn't set, don't fetch the project
	if (strWorkingDir.empty())
		return 0;

	DIR* d = opendir(strWorkingDir.chars());
	struct dirent* dent;
	
	if (!d)
		return 1;
	
	while ((dent = readdir(d)))
	{
		if (!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
			continue;
		
		closedir(d);
		return 0;
	
}
	closedir(d);
	return 1;
}

void CVermFrameWnd::FetchNodeFiles(CVermTreeNode* pNode,
								   String& strWorkingDir,
								   const char* pszProject,
								   int bLockFiles, int bDontOverwrite, int bMakeWritable)
{
	if (!pNode || !pNode->m_pLeafs)
		return;
	
	VermTreeListNode* pTmp = pNode->m_pLeafs->m_pHead;

	CVermTreeLeaf* pLeaf;

	while (pTmp)
	{
		pLeaf = pTmp->m_pLeaf;

		struct stat sb;
		memset(&sb, 0, sizeof(struct stat));

		String strFile = strWorkingDir + "/" + pLeaf->m_name;
		
		String strProject = pszProject;
		
		int ret = stat(strFile.chars(), &sb);
		int bOverride = 0;
		
		if (!bDontOverwrite && (ret || ShouldFileBeFetched(pLeaf, strProject, m_username,
																		  sb, m_modState, m_lockState, m_writeState, bOverride)))
		{
			if (!bLockFiles || ret)
			{
				CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();
				
				pNewMessage->m_number = PM_GET_FILE;
				
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pLeaf->m_name));
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

				if (bMakeWritable)
					pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, MAKE_WRITABLE));
				else
					pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, MAKE_READONLY));

				if (!m_strComment.empty())
					pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, m_strComment.chars()));
				
				if (!m_pSocket->SendMessage(pNewMessage))
					SetWaitState(1);
				else
				{
					delete pNewMessage;
					return;
				}
			}
		}

		if ((m_modState == IDCANCEL) || (m_lockState == IDCANCEL) || (m_writeState == IDCANCEL))
			return;

		int nLockStatus = IsFileLocked(pLeaf, strProject, m_username);

		if (!bOverride && bLockFiles && !nLockStatus)
		{
			CVermProtocolMessage* pNewMessage = new CVermProtocolMessage();

			pNewMessage->m_number = PM_LOCK_FILE;

			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pLeaf->m_name));
			pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

			if (bDontOverwrite)
				pNewMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, CHECKOUT_FILES_DONT_OVERWRITE));

			if (!m_pSocket->SendMessage(pNewMessage))
				SetWaitState(1);
			else
			{
				delete pNewMessage;
				return;
			}
		}

		if ((m_modState == IDCANCEL) || (m_lockState == IDCANCEL) || (m_writeState == IDCANCEL))
			return;

		pTmp = pTmp->m_pNext;
	}
}

void CVermFrameWnd::ServerShutdown(String& strMessage)
{
	m_pSocket->Close();

	m_bConnected = 0;

	if (strMessage != "")
		AfxMessageBox(strMessage);
	m_nWaitCount = 0;		// reset wait count
	SetWaitState(FALSE);

	ResetToolbar(FALSE);
	gtk_widget_set_sensitive(m_pStop, FALSE);
	gtk_widget_set_sensitive(m_pMStop, FALSE);
}

void CVermFrameWnd::SavePrefs(void)
{
	if (m_bPrefsSaved)
		return;

	m_bPrefsSaved = 1;
	
	FILE* file;
	
	String strPrefsFile = Vermie::m_strPrefsDir + "/prefs";

	file = fopen(strPrefsFile.chars(), "w");

	if (!file)
	{
		String s("Unable to save preferences information");
		AfxMessageBox(s);
		return;
	}
	
	fprintf(file, "eol:%s\n", Vermie::m_strFileEOL.chars());
	fprintf(file, "projectundocheckout:%i\n", Vermie::m_bProjectUndoCheckout);
	fprintf(file, "projectgetlatestversion:%i\n", Vermie::m_bProjectGetLatestVersion);
	fprintf(file, "projectcheckout:%i\n", Vermie::m_bProjectCheckout);
	fprintf(file, "projectdeploy:%i\n", Vermie::m_bProjectDeploy);
	fprintf(file, "filegetlatestversion:%i\n", Vermie::m_bFileGetLatestVersion);
	fprintf(file, "filecheckout:%i\n", Vermie::m_bFileCheckout);
	fprintf(file, "fileundocheckout:%i\n", Vermie::m_bFileUndoCheckout);
	fprintf(file, "fileeditandview:%i\n", Vermie::m_bFileEditAndView);
	
	fprintf(file, "displaypinstatus:%i\n", Vermie::m_bDisplayPinStatus);
	fprintf(file, "usechecksums:%i\n", Vermie::m_bUseChecksums);
	fprintf(file, "showshareicon:%i\n", Vermie::m_bShowShareIcon);
	
	fprintf(file, "cloakedcolor:%i,%i,%i\n", Vermie::m_pCloakedStyle->bg[0].red, Vermie::m_pCloakedStyle->bg[0].green, Vermie::m_pCloakedStyle->bg[0].blue);
	fprintf(file, "deploycolor:%i,%i,%i\n", Vermie::m_pWebDeployStyle->bg[0].red, Vermie::m_pWebDeployStyle->bg[0].green, Vermie::m_pWebDeployStyle->bg[0].blue);
	fprintf(file, "cloakeddeploycolor:%i,%i,%i\n", Vermie::m_pCloakedDeployStyle->bg[0].red, Vermie::m_pCloakedDeployStyle->bg[0].green, Vermie::m_pCloakedDeployStyle->bg[0].blue);
	
	fprintf(file, "sendkeepalive:%i\n", Vermie::m_bSendKeepAlive);

	if (!Vermie::m_strTimeToKeepAlive.empty())
		fprintf(file, "keepalivetime:%s\n", Vermie::m_strTimeToKeepAlive.chars());
			
	fwrite("updatefilelist:", sizeof(char), strlen("updatefilelist:"), file);
	if (Vermie::m_bUpdateFileList)
		fwrite("1\n", sizeof(char), 2, file);
	else
		fwrite("0\n", sizeof(char), 2, file);

	fprintf(file, "serverlist:%s\n", Vermie::m_strServerList.chars());
	fprintf(file, "portlist:%s\n", Vermie::m_strPortList.chars());
	fprintf(file, "lastserver:%s\n", Vermie::m_strLastServer.chars());
	fprintf(file, "lastport:%s\n", Vermie::m_strLastPort.chars());
	
	fprintf(file, "linestodisplay:%d\n", Vermie::m_nLinesToDisplay);
	
	fwrite("sortcolumn:", sizeof(char), strlen("sortcolumn:"), file);
	char column[10];
	sprintf(column, "%i\n", Vermie::m_nSortColumn);
	fwrite(column, sizeof(char), strlen(column), file);

	if (!Vermie::m_strTimeToRefresh.empty())
	{
		fwrite("refreshtime:", sizeof(char), strlen("refreshtime:"), file);
		fwrite(Vermie::m_strTimeToRefresh.chars(), sizeof(char),
			   Vermie::m_strTimeToRefresh.length(), file);
		fwrite("\n", sizeof(char), 1, file);
	}
	
	if (!Vermie::m_strDiffProgram.empty())
	{
		fwrite("diffprogram:", sizeof(char), strlen("diffprogram:"), file);
		fwrite(Vermie::m_strDiffProgram.chars(), sizeof(char),
			   Vermie::m_strDiffProgram.length(), file);
		fwrite("\n", sizeof(char), 1, file);
	}

	if (!Vermie::m_strMergeProgram.empty())
	{
		fprintf(file, "mergeprogram:%s\n", Vermie::m_strMergeProgram.chars());
	}
	
	if (!Vermie::m_strProjectDiffProgram.empty())
	{
		fprintf(file, "projectdiffprogram:%s\n", Vermie::m_strProjectDiffProgram.chars());
	}
	
	fprintf(file, "windowwidth:%i\n", m_nWidth);
	fprintf(file, "windowheight:%i\n", m_nHeight);

	m_pFileView->GetColumnWidths(Vermie::m_nColumnWidths);
	
	fprintf(file, "column1width:%i\n", Vermie::m_nColumnWidths[0]);
	fprintf(file, "column2width:%i\n", Vermie::m_nColumnWidths[1]);
	fprintf(file, "column3width:%i\n", Vermie::m_nColumnWidths[2]);
	fprintf(file, "column4width:%i\n", Vermie::m_nColumnWidths[3]);
	fprintf(file, "column5width:%i\n", Vermie::m_nColumnWidths[4]);
	
	
	fprintf(file, "usehttpproxy:%i\n", Vermie::m_bUseHTTPProxy);
	fprintf(file, "proxyserver:%s\n", Vermie::m_strProxyName.chars());
	fprintf(file, "proxyport:%i\n", Vermie::m_ProxyPort);
	
	fprintf(file, "pviewwidth:%i\n",
			GTK_WIDGET(m_pProjectView->m_pWidget)->allocation.width);
	fprintf(file, "pviewheight:%i\n",
			GTK_WIDGET(m_pProjectView->m_pWidget)->allocation.height);
	fprintf(file, "fviewwidth:%i\n",
			GTK_WIDGET(m_pFileView->m_pScrolledWin)->allocation.width);
	fprintf(file, "fviewheight:%i\n",
			GTK_WIDGET(m_pFileView->m_pScrolledWin)->allocation.height);
	fprintf(file, "mviewwidth:%i\n",
			GTK_WIDGET(m_pMessageView->m_pWidget)->allocation.width);
	fprintf(file, "mviewheight:%i\n",
			GTK_WIDGET(m_pMessageView->m_pWidget)->allocation.height);

	fprintf(file, "dontfetchoncheckout:%i\n",
			Vermie::m_bDontFetchFileOnCheckout);
	fprintf(file, "alwayskeepcheckedout:%i\n",
			Vermie::m_bAlwaysKeepFilesCheckedOut);
	fprintf(file, "recurseonprojects:%i\n",	Vermie::m_bRecurseOnProjects);
	fprintf(file, "removelocalcopy:%i\n", Vermie::m_bRemoveLocalCopy);
	fprintf(file, "makefilesreadonly:%i\n", Vermie::m_bMakeFilesReadOnly);

	if (!Vermie::m_strViewProgram.empty())
		fprintf(file, "viewprogram:%s\n", Vermie::m_strViewProgram.chars());

	fprintf(file, "filedoubleclick:%s\n", Vermie::m_strFileDoubleClick.chars());
	
	fclose(file);
}


int CVermFrameWnd::CheckOutFile(CVermTreeNode* pNode, String& strProject, String& strWorkingDir,
								 String& strFile, int bEditFile)
{
	String strCompleteFile = strFile + strWorkingDir;
	
	int nModifiedStatus = IsFileModified(pNode, strCompleteFile, strWorkingDir);

	if ((nModifiedStatus != FILE_NOT_MODIFIED) &&
		!(Vermie::m_bDontFetchFileOnCheckout))
	{			
		// if it's missing, just fetch it, but if it's been modified, ask ...

		if (nModifiedStatus == FILE_MODIFIED && (m_modState != IDYESTOALL && m_modState != IDNOTOALL))
		{
			String s = strFile + " may have been modified, lose changes?";
			CYesNoDlg dlg(s, MB_OK);
			m_modState = dlg.DoModal();
		}

		// bail out if they hit cancel

		if (m_modState == IDCANCEL)
			return 0;

		// skip this file if they hit no

		if (m_modState == IDNO || m_modState == IDNOTOALL)
			return 1;

		// get the file

		SetWaitState(1);

		CVermProtocolMessage* pMessage = new CVermProtocolMessage();

		pMessage->m_number = PM_GET_FILE;

		pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

		m_pSocket->SendMessage(pMessage);
	}

	SetWaitState(1);

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_LOCK_FILE;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_SEND_CHECKOUT_PROJECT, ""));

	if (bEditFile)
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_CLIENT_COOKIE, EDIT_FILE));

	m_pSocket->SendMessage(pMessage);

	return TRUE;
}

void CVermFrameWnd::GetEditFile(FileEdit* pFileEdit, const char* pszProject)
{
	if (!pFileEdit)
		return;

	m_pFileEdit = pFileEdit;

	CVermProtocolMessage* pMessage;

	SetWaitState(1);

	pMessage = new CVermProtocolMessage();

	pMessage->m_number = PM_GET_FILE;

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, pszProject));

	int nSlashIndex = pFileEdit->strFile.ReverseFind('/');

	String strFile = pFileEdit->strFile.after(nSlashIndex);

	pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

	m_pSocket->SendMessage(pMessage);
}

/*
 *  If we have no file information about this node (nFiles == 0), check to
 *     see if we have a working directory.  If there are files in the
 *     working dir, request Checksum info
 *
 *  If we have file info, and we have any files that we have no version information
 *     on, request Checksum info.
 */

int CVermFrameWnd::RequestChecksums(String& strProject)
{
	// if the user doesn't want to use checksums, don't send the header
	if (Vermie::m_bUseChecksums == 0)
		return 0;
	
	GtkWidget* hItem = GetTreeItemFromProject(m_pProjectView->GetRootItem(), strProject);

	CVermTreeNode* pNode = (CVermTreeNode*)gtk_object_get_user_data(GTK_OBJECT(hItem));

	CVermTreeLeafList* pLeafs = pNode->m_pLeafs;
	
	if (pLeafs == NULL || pLeafs->m_pHead == NULL)
	{
		String strDir = GetWorkingDirectory(hItem);

		if (strDir != "")
		{
			// The logic of FetchEntireProject() is the reverse of what we want
			// so if it returns true, we don't want to use checksums.
			if (!FetchEntireProject(strDir))
				return 1;
			else
				return 0;
		}
		else
		{
			return 0;
		}
	}

	VermTreeListNode* pLeafList = pLeafs->m_pHead;
	
	while (pLeafList)
	{
		CVermTreeLeaf* pLeaf = pLeafList->m_pLeaf;

		if (pLeaf->m_nLocalVersionNumber == -1)
			return 1;

		pLeafList = pLeafList->m_pNext;
	}

	return 0;
}

void CVermFrameWnd::GetDiffFilesForProject(CVermTreeNode* pNode, String& strDir, String& strProject)
{
	String wdir("");

	if (strProject.length() > m_pFileDiff->strProject.length())
		wdir = strProject.after(m_pFileDiff->strProject.length());

		// create the VSS source dir
	String ndir = m_pFileDiff->strTmpDir + wdir;
		
	CreateDirectory(ndir);
		
	// Now create the local copy directory and dump the local file into it ...
	ndir = m_pFileDiff->strSourceTmpDir + wdir;
		
	CreateDirectory(ndir);

	DIR* dir = opendir(strDir.chars());

	struct dirent* d;

	while ((d = readdir(dir)))
	{
		if (!strcmp(d->d_name, ".") || !strcmp(d->d_name, ".."))
			continue;

		String strFile = d->d_name;

		CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

		String strLocalFile = strDir + "/";
		strLocalFile += strFile;
		
		struct stat st;
		memset(&st, 0, sizeof(struct stat));

		if (stat(strLocalFile.chars(), &st))
			continue;

		if (S_ISDIR(st.st_mode))
			continue;
		
		String strTempFile = ndir;
		
		if (strTempFile[strTempFile.length() - 1] != '/')
			strTempFile += "/";
		
		strTempFile += strFile;

		// local file doesn't exist in VSS, copy it to the tmp dir
		if (!pLeaf)
		{
			CopyFile(strLocalFile, strTempFile);
		}		
		else
		{
			pLeaf->m_bChecked = 1;
			
			// File exists both locally & in the database - check to see if we need to compare
			if (IsFileModified(pNode, strFile, strDir) != FILE_NOT_MODIFIED)
			{
				CopyFile(strLocalFile, strTempFile);

				CVermProtocolMessage* pMessage = new CVermProtocolMessage();
				pMessage->m_number = PM_GET_FILE;

				pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
				pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

				m_strLastProjectForProjectDiff = strProject;
				m_strLastFileForProjectDiff = pLeaf->m_name;
				
				if (!m_pSocket->SendMessage(pMessage))
					SetWaitState(1);
				else
				{
					delete pMessage;
					return;
				}
			}
		}
	}

	VermTreeListNode* pTmp = pNode->m_pLeafs->m_pHead;

	while (pTmp)
	{
		// node wasn't hit - so it exists in the DB but not locally - request it
		if (!pTmp->m_pLeaf->m_bChecked)
		{
			CVermProtocolMessage* pMessage = new CVermProtocolMessage();
			pMessage->m_number = PM_GET_FILE;
			
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
			pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, pTmp->m_pLeaf->m_name));
			
			m_strLastProjectForProjectDiff = strProject;
			m_strLastFileForProjectDiff = pTmp->m_pLeaf->m_name;
			
			if (!m_pSocket->SendMessage(pMessage))
				SetWaitState(1);
			else
			{
				delete pMessage;
				return;
			}
		}

		pTmp->m_pLeaf->m_bChecked = 0;
		pTmp = pTmp->m_pNext;
	}
}

void CVermFrameWnd::CopyFile(String& strLocalFile, String& strTempFile)
{
	char buf[2048];
	
	int localFile = open(strLocalFile.chars(), O_RDONLY);
	
	if (localFile >= 0)
	{
		int tempFile = open(strTempFile.chars(), O_RDWR|O_CREAT, 0644);
		
		if (tempFile >= 0)
		{
			int fileSize = lseek(localFile, 0, SEEK_END);
			lseek(localFile, 0, SEEK_SET);
			int nRead = 0;
			int r = 0;
			
			while (nRead < fileSize)
			{
				r = read(localFile, buf, 2048);
				write(tempFile, buf, r);
				nRead += r;
			}
			
			close(tempFile);
			
			MakeFileReadOnly(strTempFile);
		}
		close(localFile);
	}
}

void CVermFrameWnd::BranchFile(String& strProject, String& strFile, String& strWorkingDir, String& strComment)
{
	CVermProtocolMessage* pMessage = new CVermProtocolMessage();
	pMessage->m_number = PM_BRANCH;
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));

	if (strComment != "")
		pMessage->AddHeader(new CVermProtocolHeader(HEADER_LOG, strComment));

	if (m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}

	String s("Branching File: ");
	s += strProject;
	s += "/";
	s += strFile;
	
	AddMessage(s);

	int modstate = 0, lockstate = 0, writestate = 0;
	int bOverride = FALSE;

	CVermTreeNode* pNode = GetTreeNodeFromProject(m_pProjectView->GetRootItem(), strProject.chars());
	CVermTreeLeaf* pLeaf = pNode->GetLeafByName(strFile);

	if (strWorkingDir != "")
	{
		struct stat st;
		memset(&st, 0, sizeof(struct stat));

		String sFile = strWorkingDir;
		sFile += "/";
		sFile += strFile;
		
		if (stat(sFile.chars(), &st))
		{
			if (ShouldFileBeFetched(pLeaf, strProject, m_username, st, modstate, lockstate, writestate, bOverride, 0))
			{
				CVermProtocolMessage* pFileMsg = new CVermProtocolMessage();
				pFileMsg->m_number = PM_GET_FILE;
				pFileMsg->AddHeader(new CVermProtocolHeader(HEADER_PROJECT, strProject));
				pFileMsg->AddHeader(new CVermProtocolHeader(HEADER_FILE, strFile));
				
				if (m_pSocket->SendMessage(pFileMsg))
					SetWaitState(1);
				else
				{
					delete pFileMsg;
					return;
				}
			}
		}
	}
}

void CVermFrameWnd::BranchFilesForNode(String& strProject)
{
	GtkWidget* pRoot = m_pProjectView->GetRootItem();
	String strWorkingDir = GetWorkingDirectory(pRoot, strProject.chars());
	CVermTreeNode* pNode = GetTreeNodeFromProject(pRoot, strProject.chars());

	VermTreeListNode* pList = pNode->m_pLeafs->m_pHead;

	while (pList)
	{
		if (pList->m_pLeaf->m_bShared)
			BranchFile(strProject, pList->m_pLeaf->m_name, strWorkingDir, m_strComment);

		pList = pList->m_pNext;
	}
}

void CVermFrameWnd::OnToolsChangePassword(void)
{
	CPasswordDlg dlg;
	String strNewPassword;
	
	while (1)
	{
		int ret = dlg.DoModal();

		if (ret == IDCANCEL)
			return;
		
		if (ret == IDOK)
		{
			String strOldPassword = gtk_entry_get_text(GTK_ENTRY(dlg.oldPassword));
			
			if (strOldPassword == m_password)
			{
				strNewPassword = gtk_entry_get_text(GTK_ENTRY(dlg.newPassword));
				String strRepeat = gtk_entry_get_text(GTK_ENTRY(dlg.newPasswordRepeat));
			
				if (strNewPassword != strRepeat)
				{
					String s("Your new password does not match.");
					AfxMessageBox(s);
				}
				else
					break;
			}
			else
			{
				String s("Your old password does not match.");
				
				AfxMessageBox(s);
			}
		}
	}

	CVermProtocolMessage* pMessage = new CVermProtocolMessage();
	pMessage->m_number = PM_CHANGE_PASSWORD;
	pMessage->AddHeader(new CVermProtocolHeader(HEADER_PASSWORD, strNewPassword));
	
	if (!m_pSocket->SendMessage(pMessage))
		SetWaitState(1);
	else
	{
		delete pMessage;
		return;
	}
}
